<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

/**
* Accounting class to perform the majority of accounting functions within ILance.
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class accounting
{
        /**
        * Function for processing a valid ILance transaction.
        *
        * @param       integer      subscription id (optional)
        * @param       integer      project id (optional)
        * @param       integer      buy now order id (optional)
        * @param       integer      user id
        * @param       integer      provider to buyer identification (which would be the service provider's id)
        * @param       integer      store id (optional)
        * @param       integer      buy now escrow order id (optional)
        * @param       integer      transaction description
        * @param       string       transaction amount
        * @param       string       transaction amount paid
        * @param       string       transaction status
        * @param       string       transaction type
        * @param       string       transaction payment method
        * @param       string       transaction create date
        * @param       string       transaction due date
        * @param       string       transaction paid date
        * @param       string       custom transaction message (optional) may also be used for payment gateway transaction ids
        * @param       bool         archive this invoice (optional)
        * @param       bool         defines if this transaction is a purchase order (default is no)
        * @param       bool         defines if this function should return the newly created invoice id
        * @param       string       custom transaction id (optional)
        * @param       bool         defines if this transaction is deposit related (default is no)
        * @param       bool         defines if this transaction is withdraw related (default is no)
        * @param       bool         defines if we should force the non-processing of taxes even if we have any
        *
        * @return      integer      returns the newly generated invoice id (if parameter is set)
        */
        function insert_transaction($subscriptionid = 0, $projectid = 0, $buynowid = 0, $user_id = 0, $p2b_user_id = 0, $storeid = 0, $orderid = 0, $description = '', $amount, $paid, $status, $invoicetype, $paymethod, $createdate, $duedate, $paiddate, $custommessage, $archive, $ispurchaseorder = 0, $returnid = 0, $transactionidx = '', $isdeposit = 0, $iswithdraw = 0, $dontprocesstax = 0)
        {
                global $ilance, $myapi;
                
                $subscriptionid  = isset($subscriptionid)           ? intval($subscriptionid)               : '0';
                $projectid       = isset($projectid)                ? intval($projectid)                    : '0';
                $buynowid        = isset($buynowid)                 ? intval($buynowid)                     : '0';
                $user_id         = isset($user_id)                  ? intval($user_id)                      : '0';
                $p2b_user_id     = isset($p2b_user_id)              ? intval($p2b_user_id)                  : '0';
                $storeid         = isset($storeid)                  ? intval($storeid)                      : '0';
                $orderid         = isset($orderid)                  ? intval($orderid)                      : '0';
                $description     = isset($description)              ? $description                          : 'No transaction description provided';
                $amount          = isset($amount)                   ? $amount                               : '0.00';
                $paid            = isset($paid)                     ? $paid                                 : '0.00';
                $status          = isset($status)                   ? $status                               : 'unpaid';
                $invoicetype     = isset($invoicetype)              ? $invoicetype                          : 'debit';
                $paymethod       = isset($paymethod)                ? $paymethod                            : 'account';
                $ipaddress       = IPADDRESS;
                $referer         = REFERRER;
                $createdate      = DATETIME24H;
                $duedate         = isset($duedate)                  ? $duedate                              : DATETIME24H;
                $paiddate        = isset($paiddate)                 ? $paiddate                             : '';
                $custommessage   = isset($custommessage)            ? $custommessage                        : 'No memo or administrative comments';
                $archive         = isset($archive)                  ? intval($archive)                      : '0';
                $ispurchaseorder = isset($ispurchaseorder)          ? $ispurchaseorder                      : '0';
        
                // withdraw and deposit related transactions
                $iswithdraw 	 = isset($iswithdraw)  	            ? intval($iswithdraw)                   : '0';
                $isdeposit 	 = isset($isdeposit)  	            ? intval($isdeposit)                    : '0';
                $totalamount     = '0.00';
                
                if (isset($transactionidx) AND !empty($transactionidx))
                {
                        $transactionid = $transactionidx;
                }
                else
                {
                        $transactionid = construct_transaction_id();
                }
                
                $ilance->db->query("
                        INSERT INTO " . DB_PREFIX . "invoices
                        (invoiceid, subscriptionid, projectid, buynowid, user_id, p2b_user_id, storeid, orderid, description, amount, paid, totalamount, status, paymethod, ipaddress, referer, createdate, duedate, paiddate, custommessage, transactionid, archive, ispurchaseorder, isdeposit, iswithdraw)
                        VALUES(
                        NULL,
                        '" . intval($subscriptionid) . "',
                        '" . intval($projectid) . "',
                        '" . intval($buynowid) . "',
                        '" . intval($user_id) . "',
                        '" . intval($p2b_user_id) . "',
                        '" . intval($storeid) . "',
                        '" . intval($orderid) . "',
                        '" . $ilance->db->escape_string($description) . "',
                        '" . $ilance->db->escape_string($amount) . "',
                        '" . $ilance->db->escape_string($paid) . "',
                        '" . $ilance->db->escape_string($totalamount) . "',
                        '" . $ilance->db->escape_string($status) . "',
                        '" . $ilance->db->escape_string($paymethod) . "',
                        '" . $ilance->db->escape_string($ipaddress) . "',
                        '" . $ilance->db->escape_string($referer) . "',
                        '" . $ilance->db->escape_string($createdate) . "',
                        '" . $ilance->db->escape_string($duedate) . "',
                        '" . $ilance->db->escape_string($paiddate) . "',
                        '" . $ilance->db->escape_string($custommessage) . "',
                        '" . $ilance->db->escape_string($transactionid) . "',
                        '" . $ilance->db->escape_string($archive) . "',
                        '" . intval($ispurchaseorder) . "',
                        '" . intval($isdeposit) . "',
                        '" . intval($iswithdraw) . "')
                ", 0, null, __FILE__, __LINE__); 
                
                // fetch new last invoice id
                $invoiceid = $ilance->db->insert_id();
                
                // do we skip the taxation support for this transaction?
                // we do this in some situations where the tax needs to be applied before the txn is created
                // for situations like escrow fees that we must already know how much to charge the customer for taxes
                // if we don't do this then the txn may have double taxes added to the overall amount
                // and situations like this usually mean that an unpaid transaction is being created (and tax) is
                // auto-applied
                
                // taxation support: is user taxable for this invoice type?
                // this code block will run if a transaction being created is unpaid waiting for payment from the customer
                $ilance->tax = construct_object('api.tax');
                if ($ilance->tax->is_taxable($user_id, $invoicetype) AND isset($dontprocesstax) AND $dontprocesstax == 0)
                {
                        // fetch tax amount to charge for this invoice type
                        $taxamount = $ilance->tax->fetch_amount($user_id, $amount, $invoicetype, 0);
                        
                        // fetch total amount to hold within the "totalamount" field
                        $totalamount = ($amount + $taxamount);
                        
                        // fetch tax bit to display when outputing tax infos
                        $taxinfo = $ilance->tax->fetch_amount($user_id, $amount, $invoicetype, 1);
                        
                        // portfolio invoicetypes are actually debit payments so treat it like so
                        if ($invoicetype == 'portfolio')
                        {
                                $invoicetype = 'debit';
                        }
                        
                        // in cases where an escrow payment is being made, and taxes are involved for commission fees,
                        // we will update our paid amount to the (total amount w/taxes) if our total amount is not the same
                        // as the amount we're paying. we do this because the invoice overview menu will show something like:
                        // Amount Paid: $250.00 but the Total Amount is $300.00 (taxes already applied and paid via escrow)
                        $extra = '';
                        if ($totalamount != $paid AND $totalamount > 0 AND $status == 'paid')
                        {
                                $extra = "paid = '" . $totalamount . "',";
                        }
                        
                        // member is taxable for this invoice type
                        $ilance->db->query("
                            UPDATE " . DB_PREFIX . "invoices
                            SET istaxable = '1',
                            $extra
                            totalamount = '" . sprintf("%01.2f", $totalamount) . "',
                            taxamount = '" . sprintf("%01.2f", $taxamount) . "',
                            taxinfo = '" . $ilance->db->escape_string($taxinfo) . "',
                            invoicetype = '" . $invoicetype . "'
                            WHERE invoiceid = '" . intval($invoiceid) . "'
                                AND user_id = '" . intval($user_id) . "'
                            ", 0, null, __FILE__, __LINE__);
                }
                else
                {
                        // portfolio invoicetypes are actually debit payments so treat it like so
                        if ($invoicetype == 'portfolio')
                        {
                                $invoicetype = 'debit';
                        }
                        
                        // in cases where an escrow payment is being made, and taxes are involved for commission fees,
                        // we will update our paid amount to the (total amount w/taxes) if our total amount is not the same
                        // as the amount we're paying. we do this because the invoice overview menu will show something like:
                        // Amount Paid: $250.00 but the Total Amount is $300.00 (taxes already applied and paid via escrow)
                        $extra = '';
                        if ($totalamount != $paid AND $totalamount > 0 AND $status == 'paid')
                        {
                                $extra = "paid = '".$totalamount."',";
                        }
                        
                        // customer not taxable > update totalamount value
                        $ilance->db->query("
                            UPDATE " . DB_PREFIX . "invoices
                            SET totalamount = '" . sprintf("%01.2f", $amount) . "',
                            $extra
                            invoicetype = '" . $invoicetype . "'
                            WHERE invoiceid = '" . intval($invoiceid) . "'
                                AND user_id = '" . intval($user_id) . "'
                            ", 0, null, __FILE__, __LINE__);
                }    
                    
                if (isset($returnid) AND $returnid > 0)
                {
                        return intval($invoiceid);
                }
        }
        
        
        /**
        * Function for processing a regular debit or commission related transaction.
        *
        * @param       integer      user id
        * @param       integer      invoice id
        * @param       string       invoice type (subscription, escrow, debit, commission, p2b)
        * @param       string       amount to process
        * @param       string       method of payment (ipn/account/creditcard)
        * @param       string       name of gateway processing this transaction
        * @param       string       gateway transaction id
        *
        * @return      mixed        for ipn processing, boolean is used, others will use a print_notice() function to end user.
        */
        function process_debit_payment($userid = 0, $invoiceid = 0, $invoicetype = 'subscription', $amount = 0, $method = 'account', $gateway = '', $gatewaytxn = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                // #### INSTANT PAYMENT NOTIFICATION HANDLER ###########################
                if ($method == 'ipn')
                {
                        $sql = $ilance->db->query("
                                SELECT invoiceid, invoicetype, description, amount, paid, duedate, paiddate, createdate, isif, isfvf, projectid, buynowid
                                FROM " . DB_PREFIX . "invoices
                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                    AND user_id = '" . intval($userid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res_invoice = $ilance->db->fetch_array($sql);
                                
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "invoices
                                        SET paid = '" . $ilance->db->escape_string($amount) . "',
                                        status = 'paid',
                                        paiddate = '" . DATETIME24H . "',
                                        paymethod = '" . $ilance->db->escape_string($gateway) . "',
                                        custommessage = '" . $ilance->db->escape_string($gatewaytxn) . "'
                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                            AND user_id = '" . intval($userid) . "'
                                ", 0, null, __FILE__, __LINE__);
                                
                                if ($res_invoice['isif'])
                                {
                                        // this is an insertion fee.. update auction listing table
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "projects
                                                SET isifpaid = '1'
                                                WHERE project_id = '" . $res_invoice['projectid'] . "'
                                        ", 0, null, __FILE__, __LINE__);
                                }
                                if ($res_invoice['isfvf'])
                                {
                                        // this is a final value fee.. update auction listing table
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "projects
                                                SET isfvfpaid = '1'
                                                WHERE project_id = '" . $res_invoice['projectid'] . "'
                                        ", 0, null, __FILE__, __LINE__);
                                }
                                
                                // this could also be a payment from the "seller" for an unpaid "buy now" escrow fee OR unpaid "buy now" fvf.
                                // let's check the buynow order table to see if we have a matching invoice to update as "ispaid"..
                                // this scenerio would kick in once a buyer or seller deposits funds, this script runs and tries to pay the unpaid fees automatically..
                                // at the same time we need to update the buy now order table so the presentation layer knows what's paid, what's not.
                                $buynowcheck = $ilance->db->query("
                                        SELECT escrowfeeinvoiceid, escrowfeebuyerinvoiceid, fvfinvoiceid, fvfbuyerinvoiceid
                                        FROM " . DB_PREFIX . "buynow_orders
                                        WHERE project_id = '" . $res_invoice['projectid'] . "'
                                                AND orderid = '" . $res_invoice['buynowid'] . "'
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($buynowcheck) > 0)
                                {
                                        $resbuynow = $ilance->db->fetch_array($buynowcheck);
                                        if ($res_invoice['invoiceid'] == $resbuynow['escrowfeeinvoiceid'])
                                        {
                                                // invoice being paid is from seller paying a buy now escrow fee
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                        SET isescrowfeepaid = '1'
                                                        WHERE orderid = '" . $res_invoice['buynowid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                        else if ($res_invoice['invoiceid'] == $resbuynow['escrowfeebuyerinvoiceid'])
                                        {
                                                // invoice being paid is from buyer paying a buy now escrow fee
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                        SET isescrowfeebuyerpaid = '1'
                                                        WHERE orderid = '" . $res_invoice['buynowid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                        else if ($res_invoice['invoiceid'] == $resbuynow['fvfinvoiceid'])
                                        {
                                                // invoice being paid is from seller paying a buy now fvf
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                        SET isfvfpaid = '1'
                                                        WHERE orderid = '" . $res_invoice['buynowid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                        else if ($res_invoice['invoiceid'] == $resbuynow['fvfbuyerinvoiceid'])
                                        {
                                                // invoice being paid is from buyer paying a buy now fvf
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                        SET isfvfbuyerpaid = '1'
                                                        WHERE orderid = '" . $res_invoice['buynowid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                }
                                
                                // track income spent
                                insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                
                                $ilance->email = construct_dm_object('email', $ilance);
                                
                                $ilance->email->mail = SITE_EMAIL;
                                $ilance->email->slng = fetch_site_slng();
                                
                                // email admin
                                $ilance->email->get('debit_fee_paid_online_account_admin');		
                                $ilance->email->set(array(
                                        '{{provider}}' => fetch_user('username', intval($userid)),
                                        '{{invoice_id}}' => intval($invoiceid),
                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                ));
                                
                                $ilance->email->send();
                                
                                // email user
                                $ilance->email->get('debit_fee_paid_online_account');		
                                $ilance->email->set(array(
                                        '{{provider}}' => fetch_user('username', intval($userid)),
                                        '{{invoice_id}}' => intval($invoiceid),
                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                ));
                                
                                $ilance->email->mail = fetch_user('email', intval($userid));
                                $ilance->email->slng = fetch_user_slng(intval($userid));
                                $ilance->email->send();
                                
                                return 1;
                        }
                        else
                        {
                                return 0;
                        }    
                }
                
                // #### ONLINE ACCOUNT HANDLER #########################################
                else if ($method == 'account')
                {
                        $sql_balance = $ilance->db->query("
                            SELECT available_balance, total_balance
                            FROM " . DB_PREFIX . "users
                            WHERE user_id = '" . intval($userid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql_balance) > 0)
                        {
                                $res_balance = $ilance->db->fetch_array($sql_balance);
                                if ($res_balance['available_balance'] >= $amount)
                                {
                                        $avail_balance = $res_balance['available_balance'];
                                        $total_balance = $res_balance['total_balance'];
                                        
                                        $avail_balance_after = ($avail_balance - $amount);
                                        $total_balance_after = ($total_balance - $amount);
                                        
                                        $sql_invoice = $ilance->db->query("
                                                SELECT invoiceid, invoicetype, description, amount, paid, duedate, paiddate, createdate, isif, isfvf, projectid, buynowid
                                                FROM " . DB_PREFIX . "invoices
                                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                                    AND user_id = '" . intval($userid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        if ($ilance->db->num_rows($sql_invoice) > 0)
                                        {
                                                $res_invoice = $ilance->db->fetch_array($sql_invoice);
                                                
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET paid = '" . $ilance->db->escape_string($amount) . "',
                                                        status = 'paid',
                                                        paiddate = '" . DATETIME24H . "'
                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                            AND user_id = '" . intval($userid) . "'
                                                            AND invoicetype = '" . $ilance->db->escape_string($invoicetype) . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET available_balance = '" . $ilance->db->escape_string($avail_balance_after) . "',
                                                        total_balance = '" . $ilance->db->escape_string($total_balance_after) . "'
                                                        WHERE user_id = '" . intval($userid) . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                if ($res_invoice['isif'])
                                                {
                                                        // this is an insertion fee.. update auction listing table
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "projects
                                                                SET isifpaid = '1'
                                                                WHERE project_id = '" . $res_invoice['projectid'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                }
                                                
                                                if ($res_invoice['isfvf'])
                                                {
                                                        // this is a final value fee.. update auction listing table
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "projects
                                                                SET isfvfpaid = '1'
                                                                WHERE project_id = '" . $res_invoice['projectid'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                }
                                                
                                                // this could also be a payment from the "seller" for an unpaid "buy now" escrow fee OR unpaid "buy now" fvf.
                                                // let's check the buynow order table to see if we have a matching invoice to update as "ispaid"..
                                                // this scenerio would kick in once a buyer or seller deposits funds, this script runs and tries to pay the unpaid fees automatically..
                                                // at the same time we need to update the buy now order table so the presentation layer knows what's paid, what's not.
                                                $buynowcheck = $ilance->db->query("
                                                        SELECT orderid, escrowfeeinvoiceid, escrowfeebuyerinvoiceid, fvfinvoiceid, fvfbuyerinvoiceid
                                                        FROM " . DB_PREFIX . "buynow_orders
                                                        WHERE project_id = '" . $res_invoice['projectid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($buynowcheck) > 0)
                                                {
                                                        while ($resbuynow = $ilance->db->fetch_array($buynowcheck))
                                                        {
                                                                if ($res_invoice['invoiceid'] == $resbuynow['escrowfeeinvoiceid'])
                                                                {
                                                                        // invoice being paid is from seller paying a buy now escrow fee
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "buynow_orders
                                                                                SET isescrowfeepaid = '1'
                                                                                WHERE orderid = '" . $resbuynow['orderid'] . "'
                                                                        ", 0, null, __FILE__, __LINE__);
                                                                }
                                                                else if ($res_invoice['invoiceid'] == $resbuynow['escrowfeebuyerinvoiceid'])
                                                                {
                                                                        // invoice being paid is from buyer paying a buy now escrow fee
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "buynow_orders
                                                                                SET isescrowfeebuyerpaid = '1'
                                                                                WHERE orderid = '" . $resbuynow['orderid'] . "'
                                                                        ", 0, null, __FILE__, __LINE__);
                                                                }
                                                                else if ($res_invoice['invoiceid'] == $resbuynow['fvfinvoiceid'])
                                                                {
                                                                        // invoice being paid is from seller paying a buy now fvf
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "buynow_orders
                                                                                SET isfvfpaid = '1'
                                                                                WHERE orderid = '" . $resbuynow['orderid'] . "'
                                                                        ", 0, null, __FILE__, __LINE__);
                                                                }
                                                                else if ($res_invoice['invoiceid'] == $resbuynow['fvfbuyerinvoiceid'])
                                                                {
                                                                        // invoice being paid is from buyer paying a buy now fvf
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "buynow_orders
                                                                                SET isfvfbuyerpaid = '1'
                                                                                WHERE orderid = '" . $resbuynow['orderid'] . "'
                                                                        ", 0, null, __FILE__, __LINE__);
                                                                }
                                                        }
                                                }
                                                
                                                // track income spent
                                                insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                                
                                                $ilance->email = construct_dm_object('email', $ilance);
                                                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                        
                                                // email admin
                                                $ilance->email->get('debit_fee_paid_online_account_admin');		
                                                $ilance->email->set(array(
                                                        '{{provider}}' => $_SESSION['ilancedata']['user']['username'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                // email user
                                                $ilance->email->get('debit_fee_paid_online_account');		
                                                $ilance->email->set(array(
                                                        '{{provider}}' => $_SESSION['ilancedata']['user']['username'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->mail = $_SESSION['ilancedata']['user']['email'];
                                                $ilance->email->slng = $_SESSION['ilancedata']['user']['slng'];
                                                $ilance->email->send();
                                                
                                                $area_title = $phrase['_invoice_payment_complete_menu'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_complete_menu'];
                                                
                                                print_notice($phrase['_invoice_payment_complete'], $phrase['_your_invoice_has_been_paid_in_full'], $ilpage['accounting'], $phrase['_my_account']);
                                                exit();
                                        }
                                        else
                                        {
                                                $area_title = $phrase['_invoice_payment_menu_denied_payment_does_not_belong_to_user'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_menu_denied_payment_does_not_belong_to_user'];
                                                
                                                print_notice($phrase['_invoice_error'], $phrase['_were_sorry_this_invoice_does_not_exist']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                                exit();
                                        }
                                }
                                else
                                {
                                        $area_title = $phrase['_funds_not_available'];
                                        $page_title = SITE_NAME . ' - ' . $phrase['_funds_not_available'];
                                        
                                        print_notice($phrase['_invoice_payment_warning_insufficient_funds'], $phrase['_were_sorry_this_invoice_can_not_be_paid_due_to_insufficient_funds']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                        exit();
                                }
                        }
                }
        }
        
        /**
        * Function for processing a provider generated (to buyer) transaction type being paid by the actual buyer.
        *
        * @param       integer      user id
        * @param       integer      invoice id
        * @param       string       invoice type
        * @param       string       amount to process
        * @param       string       method of payment (ipn/account/creditcard)
        * @param       string       name of gateway processing this transaction
        * @param       string       gateway transaction id
        *
        * @return      mixed        for ipn processing, boolean is used, others will use a print_notice() function to end user.
        */
        function process_p2b_payment($userid = 0, $invoiceid = 0, $invoicetype = 'p2b', $amount = 0, $method = 'account', $gateway = '', $gatewaytxn = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                // #### INSTANT PAYMENT NOTIFICATION ###################################
                if ($method == 'ipn')
                {
                        $sql = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "invoices
                                WHERE invoiceid = '" . intval($invoiceid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res_invoice = $ilance->db->fetch_array($sql);
                                
                                // track income spent
                                insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "invoices 
                                        SET paid = '" . sprintf("%01.2f", $amount) . "',
                                        status = 'paid',
                                        paiddate = '" . DATETIME24H . "',
                                        paymethod = '" . $ilance->db->escape_string($gateway) . "',
                                        custommessage = '".$ilance->db->escape_string($gatewaytxn)."'
                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                            AND user_id = '" . intval($userid) . "'
                                            AND p2b_user_id = '" . $res_invoice['p2b_user_id'] . "'
                                            AND projectid = '" . $res_invoice['projectid'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                
                                // track income reported for provider
                                insert_income_reported($res_invoice['p2b_user_id'], sprintf("%01.2f", $amount), 'credit');
                                
                                $ilance->email = construct_dm_object('email', $ilance);
                                
                                $ilance->email->mail = fetch_user('email', $res_invoice['p2b_user_id']);
                                $ilance->email->slng = fetch_user_slng($res_invoice['p2b_user_id']);
                                
                                $ilance->email->get('buyer_payment_online_account_provider');		
                                $ilance->email->set(array(
                                        '{{buyer}}' => fetch_user('username', $userid),
                                        '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                        '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                        '{{invoiceid}}' => intval($invoiceid),
                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                ));
                                
                                $ilance->email->send();
                                
                                // fetch service providers available account balance
                                $sql_sellerbalance = $ilance->db->query("
                                        SELECT available_balance, total_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . $res_invoice['p2b_user_id'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($sql_sellerbalance) > 0)
                                {
                                        $res_sellerbalance = $ilance->db->fetch_array($sql_sellerbalance);
                                        
                                        $seller_available_balance = $res_sellerbalance['available_balance'];
                                        $seller_total_balance = $res_sellerbalance['total_balance'];
                                        
                                        // readjust new account balance for provider
                                        $new_seller_available_balance = ($seller_available_balance + $amount);
                                        $new_seller_total_balance = ($seller_total_balance + $amount);
                                        
                                        // add funds processed by buyer into service providers online account balance
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET available_balance = '" . sprintf("%01.2f", $new_seller_available_balance) . "',
                                                total_balance = '" . sprintf("%01.2f", $new_seller_total_balance) . "'
                                                WHERE user_id = '" . $res_invoice['p2b_user_id'] . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        
                                        $ilance->email->get('buyer_payment_online_account_admin');		
                                        $ilance->email->set(array(
                                                '{{buyer}}' => fetch_user('username', $userid),
                                                '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                                '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                                '{{invoiceid}}' => intval($invoiceid),
                                                '{{invoice_amount}}' => $ilance->currency->format($amount),
                                        ));
                                        
                                        $ilance->email->mail = SITE_EMAIL;
                                        $ilance->email->slng = fetch_site_slng();
                                        $ilance->email->send();
                                        
                                        $ilance->email->get('buyer_payment_online_account_buyer');		
                                        $ilance->email->set(array(
                                                '{{buyer}}' => fetch_user('username', $userid),
                                                '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                                '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                                '{{invoiceid}}' => intval($invoiceid),
                                                '{{invoice_amount}}' => $ilance->currency->format($amount),
                                        ));
                                        
                                        $ilance->email->mail = fetch_user('email', $userid);
                                        $ilance->email->slng = fetch_user_slng(intval($userid));
                                        $ilance->email->send();
                                        
                                        return 1;
                                }
                        }
                        
                        return 0;
                }
                
                // #### ONLINE ACCOUNT BALANCE #########################################
                else if ($method == 'account')
                {
                        $sql_buyerbalance = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . intval($userid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql_buyerbalance) > 0)
                        {
                                $res_buyerbalance = $ilance->db->fetch_array($sql_buyerbalance);
                                
                                $buyer_available_balance = $res_buyerbalance['available_balance'];
                                $buyer_total_balance = $res_buyerbalance['total_balance'];
                                if ($buyer_available_balance < $amount)
                                {
                                        $area_title = $phrase['_no_funds_available_in_online_account'];
                                        $page_title = SITE_NAME . ' - ' . $phrase['_no_funds_available_in_online_account'];
                                        
                                        print_notice($phrase['_invoice_payment_warning_insufficient_funds'], $phrase['_were_sorry_this_invoice_can_not_be_paid_due_to_insufficient_funds']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                        exit();
                                }
                                else
                                {
                                        $sql_invoice = $ilance->db->query("
                                                SELECT *
                                                FROM " . DB_PREFIX . "invoices
                                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        if ($ilance->db->num_rows($sql_invoice) > 0)
                                        {
                                                $res_invoice = $ilance->db->fetch_array($sql_invoice);
                                                
                                                $new_buyer_avail_balance = ($buyer_available_balance - $amount);
                                                $new_buyer_total_balance = ($buyer_total_balance - $amount);
                                                
                                                ###################################################################################
                                                ## RE-UPDATE BUYERS ACCOUNT TABLE WITH NEW TOTAL AND AVAIL BALANCE - ACCOUNT DEBIT!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET available_balance = '" . sprintf("%01.2f", $new_buyer_avail_balance) . "',
                                                        total_balance = '" . sprintf("%01.2f", $new_buyer_total_balance) . "'
                                                        WHERE user_id = '" . intval($userid) . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                // track income spent
                                                insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                                
                                                ##################################
                                                ## UPDATE PROVIDER INVOICE TO PAID
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices 
                                                        SET paid = '" . sprintf("%01.2f", $amount) . "',
                                                        status = 'paid',
                                                        paiddate = '" . DATETIME24H . "'
                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                            AND user_id = '" . intval($userid) . "'
                                                            AND p2b_user_id = '" . $res_invoice['p2b_user_id'] . "'
                                                            AND projectid = '" . $res_invoice['projectid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                // track income reported for provider
                                                insert_income_reported($res_invoice['p2b_user_id'], sprintf("%01.2f", $amount), 'credit');
                                                
                                                $ilance->email = construct_dm_object('email', $ilance);
                                                
                                                $ilance->email->mail = fetch_user('email', $res_invoice['p2b_user_id']);
                                                $ilance->email->slng = fetch_user_slng($res_invoice['p2b_user_id']);
                                        
                                                $ilance->email->get('buyer_payment_online_account_provider');		
                                                $ilance->email->set(array(
                                                        '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                                        '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                                        '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                                        '{{invoiceid}}' => intval($invoiceid),
                                                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                ###########################################
                                                ## FETCH PROVIDER AVAILABLE ACCOUNT BALANCE
                                                $sql_sellerbalance = $ilance->db->query("
                                                    SELECT available_balance, total_balance
                                                    FROM " . DB_PREFIX . "users
                                                    WHERE user_id = '" . $res_invoice['p2b_user_id'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($sql_sellerbalance) > 0)
                                                {
                                                        $res_sellerbalance = $ilance->db->fetch_array($sql_sellerbalance);
                                                        
                                                        $seller_available_balance = $res_sellerbalance['available_balance'];
                                                        $seller_total_balance = $res_sellerbalance['total_balance'];
                                                        
                                                        $new_seller_available_balance = ($seller_available_balance + $amount);
                                                        $new_seller_total_balance = ($seller_total_balance + $amount);
                                                        
                                                        // add funds processed by buyer to providers online account
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET available_balance = '" . sprintf("%01.2f", $new_seller_available_balance) . "',
                                                                total_balance = '" . sprintf("%01.2f", $new_seller_total_balance) . "'
                                                                WHERE user_id = '" . $res_invoice['p2b_user_id'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                        
                                                        $ilance->email->get('buyer_payment_online_account_admin');		
                                                        $ilance->email->set(array(
                                                                '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                                                '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                                                '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                                                '{{invoiceid}}' => intval($invoiceid),
                                                                '{{invoice_amount}}' => $ilance->currency->format($amount),
                                                        ));
                                                        
                                                        $ilance->email->mail = SITE_EMAIL;
                                                        $ilance->email->slng = fetch_site_slng();
                                                        $ilance->email->send();
                                                        
                                                        $ilance->email->get('buyer_payment_online_account_buyer');		
                                                        $ilance->email->set(array(
                                                                '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                                                '{{provider}}' => fetch_user('username', $res_invoice['p2b_user_id']),
                                                                '{{project_title}}' => fetch_auction('project_title', $res_invoice['projectid']),
                                                                '{{invoiceid}}' => intval($invoiceid),
                                                                '{{invoice_amount}}' => $ilance->currency->format($amount),
                                                        ));
                                                        
                                                        $ilance->email->mail = $_SESSION['ilancedata']['user']['email'];
                                                        $ilance->email->slng = fetch_user_slng(intval($userid));
                                                        $ilance->email->send();
                                                        
                                                        $area_title = $phrase['_generated_invoice_payment_complete_menu'];
                                                        $page_title = SITE_NAME . ' - ' . $phrase['_generated_invoice_payment_complete_menu'];
                                                        
                                                        print_notice($phrase['_invoice_payment_complete'], $phrase['_your_invoice_has_been_paid_in_full']."<br /><br />", $ilpage['accounting'], $phrase['_my_account']);
                                                        exit();
                                                }
                                                else
                                                {
                                                        $area_title = $phrase['_invoice_payment_menu_denied_subscription_payment_does_not_belong_to_user'];
                                                        $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_menu_denied_subscription_payment'];
                                                        
                                                        print_notice($phrase['_invoice_error'], $phrase['_were_sorry_this_invoice_does_not_exist']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                                        exit();
                                                }
                                        }
                                        else
                                        {
                                                $area_title = $phrase['_invoice_payment_menu_denied_subscription_payment_does_not_belong_to_user'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_menu_denied_subscription_payment'];
                                                
                                                print_notice($phrase['_invoice_error'], $phrase['_were_sorry_this_invoice_does_not_exist']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                                exit();
                                        }
                                }
                        }
                        else
                        {
                                $area_title = $phrase['_invoice_payment_menu_denied_subscription_payment_does_not_belong_to_user'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_menu_denied_subscription_payment'];
                                
                                print_notice($phrase['_invoice_error'], $phrase['_were_sorry_this_invoice_does_not_exist']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                exit();
                        }
                }
        }
                
        /**
        * Function for processing a credential verification payment request.
        *
        * @param       integer      user id
        * @param       string       payment method
        * @param       integer      answer id
        * @param       integer      question id
        * @param       string       actual value
        * @param       string       contact name of reference
        * @param       string       contact phone of reference
        * @param       string       contact notes
        *
        * @return      nothing
        */
        function process_credential_payment($userid = 0, $method = 'account', $answerid = 0, $questionid = 0, $answer = '', $contactname = '', $contactnumber = '', $contactnotes = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                $sql = $ilance->db->query("
                        SELECT verifycost, question
                        FROM " . DB_PREFIX . "profile_questions
                        WHERE questionid = '" . intval($questionid) . "'
                            AND canverify = '1'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $resamount = $ilance->db->fetch_array($sql);
                        $question = stripslashes($resamount['question']);
                        
                        // fetch amount
                        $amount = $resamount['verifycost'];
                        $total = $amount;
                        
                        // does tax apply?
                        $ilance->tax = construct_object('api.tax');
                        if ($ilance->tax->is_taxable(intval($userid), 'credential') AND $amount > 0)
                        {
                                $taxamount = $ilance->tax->fetch_amount(intval($userid), $amount, 'credential', 0);
                                $total = ($amount+$taxamount);
                        }
                        
                        // payment process via online account balance
                        if ($method == 'account')
                        {
                                $sel_balance = $ilance->db->query("
                                        SELECT available_balance, total_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . intval($userid) . "'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($sel_balance) > 0)
                                {
                                        $res_balance = $ilance->db->fetch_array($sel_balance);
                                        if ($res_balance['available_balance'] < $total)
                                        {
                                                $area_title = $phrase['_no_funds_available_in_online_account'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_no_funds_available_in_online_account'];
                                                
                                                print_notice($phrase['_invoice_payment_warning_insufficient_funds'], $phrase['_were_sorry_this_invoice_can_not_be_paid_due_to_insufficient_funds'] . '<br /><br />' . $phrase['_please_contact_customer_support'], $ilpage['selling'] . '?cmd=profile', $phrase['_selling_profile']);
                                                exit();
                                        }
                                        else
                                        {
                                                $area_title = $phrase['_profile_verification_payment_via_online_account'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_profile_verification_payment_via_online_account'];
                                                
                                                $transactionid = construct_transaction_id();
                                                
                                                $newinvoiceid = $this->insert_transaction(
                                                        0,
                                                        0,
                                                        0,
                                                        intval($userid),
                                                        0,
                                                        0,
                                                        0,
                                                        $phrase['_profile_verification_fee_question'] . ' ' . $ilance->db->escape_string($question),
                                                        sprintf("%01.2f", $amount),
                                                        sprintf("%01.2f", $total),
                                                        'paid',
                                                        'credential',
                                                        'account',
                                                        DATETIME24H,
                                                        DATEINVOICEDUE,
                                                        DATETIME24H,
                                                        $phrase['_verification_requested_on'] . ' ' . DATETIME24H,
                                                        0,
                                                        0,
                                                        1,
                                                        $transactionid
                                                );
                                                    
                                                $new_total = ($res_balance['total_balance'] - $total);
                                                $new_avail = ($res_balance['available_balance'] - $total);
                    
                                                // update account data                            
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET available_balance = '" . sprintf("%01.2f", $new_avail) . "',
                                                        total_balance = '" . sprintf("%01.2f", $new_total) . "'
                                                        WHERE user_id = '" . intval($userid) . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                // track income spent
                                                insert_income_spent(intval($userid), sprintf("%01.2f", $total), 'credit');
                                                
                                                // #### REFERRAL SYSTEM TRACKER ############################
                                                update_referral_action('credential', intval($userid));
                                                
                                                $isverified = '0';
                                                if ($ilconfig['verificationmoderation'] == 0)
                                                {
                                                        $isverified = '1';
                                                }
                                                
                                                $expiry = gmdate('Y-m-d H:i:s', mktime(gmdate('H', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])), gmdate('i', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])), gmdate('s', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])), gmdate('m', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])), gmdate('d', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])) + $ilconfig['verificationlength'], gmdate('Y', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst']))));
                                                
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "profile_answers
                                                        SET invoiceid = '" . intval($newinvoiceid) . "',
                                                        contactname = '" . $ilance->db->escape_string($contactname) . "',
                                                        contactnumber = '" . $ilance->db->escape_string($contactnumber) . "',
                                                        contactnotes = '" . $ilance->db->escape_string($contactnotes) . "',
                                                        isverified = '" . $isverified . "',
                                                        verifyexpiry = '" . $expiry . "'
                                                        WHERE questionid = '" . intval($questionid) . "'
                                                            AND answerid = '" . intval($answerid) . "'
                                                            AND user_id = '" . intval($userid) . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                $ilance->email = construct_dm_object('email', $ilance);
                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                $ilance->email->get('profile_verification_pending_admin');		
                                                $ilance->email->set(array(
                                                        '{{contactname}}' => $contactname,
                                                        '{{contactnumber}}' => $contactnumber,
                                                        '{{contactnotes}}' => $contactnotes,
                                                        '{{expiry}}' => $expiry,
                                                        '{{customer}}' => $_SESSION['ilancedata']['user']['username'],
                                                        '{{transactionid}}' => $transactionid,
                                                        '{{question}}' => $question,
                                                        '{{answer}}' => $answer,
                                                        '{{total_amount_formatted}}' => $ilance->currency->format($total)
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = $_SESSION['ilancedata']['user']['email'];
                                                $ilance->email->slng = $_SESSION['ilancedata']['user']['slng'];
                                                
                                                $ilance->email->get('profile_verification_pending');		
                                                $ilance->email->set(array(
                                                        '{{contactname}}' => $contactname,
                                                        '{{contactnumber}}' => $contactnumber,
                                                        '{{contactnotes}}' => $contactnotes,
                                                        '{{expiry}}' => $expiry,
                                                        '{{customer}}' => $_SESSION['ilancedata']['user']['username'],
                                                        '{{transactionid}}' => $transactionid,
                                                        '{{question}}' => $question,
                                                        '{{answer}}' => $answer,
                                                        '{{total_amount_formatted}}' => $ilance->currency->format($total)
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                print_notice($phrase['_invoice_payment_complete'], $phrase['_your_invoice_has_been_paid_in_full'] . '<br /><br />', $ilpage['selling'] . '?cmd=profile', $phrase['_selling_profile']);
                                                exit();
                                        }
                                }
                        }            
                }
                else
                {
                        print_notice($phrase['_access_denied'], $phrase['_sorry_your_profile_verification_process_could_not_be_completed'], 'javascript:history.back(1);', $phrase['_back']);
                        exit();
                }
        }
        
        /**
        * Functions for processing a deposit payment from a credit card payment
        *
        * @param       integer          credit card id
        * @param       integer          user id
        * @param       string           deposit amount to process
        * @param       string           deposit account credit amount
        * @param       array            custom array argument (used for sending credit card details of user's supplied card info)
        *
        * @return      mixed            Returns 1 for true, 2 for card not authenticated via account menu or false for gateway transaction error response
        */
        function process_creditcard_deposit($ccid = 0, $userid = 0, $deposit = 0, $credit = 0, $custom = array())
        {
                global $ilance, $ilconfig, $ilpage, $phrase, $myapi;
                
                if ($userid > 0 AND $ccid > 0)
                {
                        // use customer's credit card in the databasefile
                        
                        $sql = $ilance->db->query("
                                SELECT user_id, creditcard_type, creditcard_number, name_on_card, card_billing_address1, card_billing_address2, card_city, card_state, card_postalzip, card_country, phone_of_cardowner, email_of_cardowner, cc_id, creditcard_expiry, cvv2
                                FROM " . DB_PREFIX . "creditcards
                                WHERE user_id = '" . intval($userid) . "'
                                    AND cc_id = '" . intval($ccid) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $resultcc = $ilance->db->fetch_array($sql);
                                
                                if ($result['authorized'] == 'no')
                                {
                                        return '2';
                                }
                                
                                $resultcc['card_country'] = stripslashes($ilance->db->fetch_field(DB_PREFIX . "locations", "locationid = " . $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id = '" . intval($ccid) . "'", "card_country"), "location_" . fetch_user_slng(intval($userid))));
                                $resultcc['creditcard_number'] = $ilance->crypt->three_layer_decrypt($resultcc['creditcard_number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                        }
                }
                else if ($userid > 0 AND $ccid == 'ccform' AND is_array($custom))
                {
                        // use custom supplied card details from the form just submitted
                        
                        $resultcc['card_country'] = $_SESSION['ilancedata']['user']['country'];
                        $resultcc['creditcard_number'] = $custom['creditcard_number'];
                        $resultcc['name_on_card'] = $custom['creditcard_name'];
                        $resultcc['card_billing_address1'] = $custom['creditcard_billing'];
                        $resultcc['card_billing_address2'] = '';
                        $resultcc['card_postalzip'] = $custom['creditcard_postal'];
                        $resultcc['creditcard_type'] = $custom['creditcard_type'];
                        $resultcc['cvv2'] = $custom['creditcard_cvv2'];
                        $resultcc['creditcard_expiry'] = $custom['creditcard_month'] . $custom['creditcard_year'];
                        $resultcc['phone_of_cardowner'] = fetch_user('phone', intval($userid));
                        $resultcc['card_city'] = fetch_user('city', intval($userid));
                        $resultcc['card_state'] = fetch_user('state', intval($userid));
                }
                
                $ccnum_hidden = substr_replace($resultcc['creditcard_number'], '', 0, (mb_strlen($resultcc['creditcard_number']) - 4));
                $nameoncard = ucwords(stripslashes($resultcc['name_on_card']));
                
                $namesplit = explode(' ', $nameoncard);
                if (empty($namesplit[0]))
                {
                        $namesplit[0] = '';
                }
                if (empty($namesplit[1]))
                {
                        $namesplit[1] = '';
                }
                
                $transa = rand(100,999) . rand(100,999) . rand(100,999);
                
                $ilance->paymentgateway = construct_object('api.paymentgateway', $ilconfig['use_internal_gateway']);
                
                $ilance->paymentgateway->set_user(
                        $ilconfig['cc_login'],
                        $ilconfig['cc_password'],
                        $ilconfig['cc_key'],
                        SITE_EMAIL
                );
                
                $ilance->paymentgateway->set_customer(
                        fetch_user('username', intval($userid)),
                        $resultcc['phone_of_cardowner'],
                        $namesplit[0],
                        $namesplit[1],
                        stripslashes($resultcc['card_billing_address1']) . ' ' . stripslashes($resultcc['card_billing_address2']),
                        ucfirst($resultcc['card_city']),
                        ucfirst($resultcc['card_state']),
                        mb_strtoupper($resultcc['card_postalzip']),
                        $resultcc['card_country']
                );
                
                $ilance->paymentgateway->set_ccard(
                        $resultcc['name_on_card'],
                        $resultcc['creditcard_type'],
                        $resultcc['creditcard_number'],
                        mb_substr($resultcc['creditcard_expiry'], 0, 2),
                        mb_substr($resultcc['creditcard_expiry'], -4),
                        $resultcc['cvv2']
                );
                
                $ilance->paymentgateway->set_currency(
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['code'],
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['symbol_left']
                );
                
                $ilance->paymentgateway->set_order(
                        $deposit,
                        $transa,
                        $phrase['_account_deposit_credit_via'] . ' ' . ucfirst($resultcc['creditcard_type']) . ' : #' . $ccnum_hidden . ' ' . $phrase['_into_online_account'],
                        $ilconfig['authentication_capture'],
                        null,
                        null,
                        null
                );
                
                $gatewayinfo = array();
                $ilance->paymentgateway->set_extra($gatewayinfo);
                
                // #### PROCESS CC FOR DEPOSIT TRANSACTION #####################
                
                if ($ilance->paymentgateway->process())
                {
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "users
                                SET available_balance = available_balance + $credit,
                                total_balance = total_balance + $credit
                                WHERE user_id = '" . intval($userid) . "'
                        ", 0, null, __FILE__, __LINE__);
        
                        // insert deposit transaction for customer
                        $ilance->accounting = construct_object('api.accounting');
                        
                        $deposit_invoice_id = $ilance->accounting->insert_transaction(
                                0,
                                0,
                                0,
                                $userid,
                                0,
                                0,
                                0,
                                $phrase['_account_deposit_credit_via'] . ' ' . ucfirst($resultcc['creditcard_type']) . ' : #' . $ccnum_hidden . ' ' . $phrase['_into_online_account'] . ': ' . $ilance->currency->format($credit),
                                $deposit,
                                $deposit,
                                'paid',
                                'credit',
                                $resultcc['creditcard_type'],
                                DATETIME24H,
                                DATETIME24H,
                                DATETIME24H,
                                $transa,
                                0,
                                0,
                                1,
                                '',
                                1,
                                0
                        );
                        
                        // update the transaction with the acual amount we're crediting this user for
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "invoices
                                SET depositcreditamount = '" . sprintf("%01.2f", $credit) . "',
                                paymentgateway = '" . $ilance->db->escape_string($ilconfig['use_internal_gateway']) . "'
                                WHERE user_id = '" . intval($userid) . "'
                                        AND invoiceid = '" . intval($deposit_invoice_id) . "'
                        ");
                        
                        $ilance->email = construct_dm_object('email', $ilance);
        
                        $ilance->email->mail = fetch_user('email', intval($userid));
                        $ilance->email->slng = fetch_user_slng(intval($userid));
                        
                        $ilance->email->get('member_deposit_funds_creditcard');		
                        $ilance->email->set(array(
                                '{{username}}' => fetch_user('username', intval($userid)),
                                '{{ip}}' => IPADDRESS,
                                '{{amount}}' => $ilance->currency->format($credit),
                                '{{cost}}' => $ilance->currency->format($deposit),
                                '{{invoiceid}}' => $deposit_invoice_id,
                                '{{paymethod}}' => mb_strtoupper($resultcc['creditcard_type']) . ' : #' . $ccnum_hidden,
                                '{{gateway}}' => ucwords($ilconfig['use_internal_gateway']),
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('member_deposit_funds_creditcard_admin');		
                        $ilance->email->set(array(
                                '{{username}}' => fetch_user('username', intval($userid)),
                                '{{ip}}' => IPADDRESS,
                                '{{amount}}' => $ilance->currency->format($credit),
                                '{{cost}}' => $ilance->currency->format($deposit),
                                '{{invoiceid}}' => $deposit_invoice_id,
                                '{{paymethod}}' => mb_strtoupper($resultcc['creditcard_type']) . ' : #' . $ccnum_hidden,
                                '{{gateway}}' => ucwords($ilconfig['use_internal_gateway']),
                        ));
                        
                        $ilance->email->send();
                        
                        return '1';
                }
                
                return false;
        }
        
        /**
        * Function for processing a credit card authentication step 1 through a major creditcard gateway.
        *
        * This function will connect to the pre-configured payment gateway, send the card details, charge the credit card 2 amounts both under 2.00 each.  The user
        * would have to wait for the billing statement to complete this process (or they can view online debits using their merchant web site to learn amounts) which
        * is then handled in the creditcard_authentication_step_two() function.
        *
        * @param       integer      user id
        * @param       integer      credit card id
        * @param       string       customer first name
        * @param       string       customer last name
        * @param       string       customer address
        * @param       string       customer city
        * @param       string       customer state
        * @param       string       customer zip / postal code
        * @param       integer      customer country code
        *
        * @return      bool         returns true or false
        */
        function creditcard_authentication_step_one($userid = 0, $v3customer_ccid = 0, $v3customer_fname = '', $v3customer_lname = '', $v3customer_address = '', $v3customer_city = '', $v3customer_state = '', $v3customer_zip = '', $v3customer_country = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage, $SCRIPT_URL;
                
                $sql_cc_url = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "creditcards
                        WHERE user_id = '" . intval($userid) . "'
                            AND cc_id = '".intval($v3customer_ccid)."'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql_cc_url) > 0)
                {
                        $sql_cc_arr = $ilance->db->fetch_array($sql_cc_url);
                }
                else
                {
                        return false;
                }
        
                // generate two random amounts both under 2.00
                $amount1 = (rand(1, 199)/100);
                $amount2 = (rand(1, 199)/100);
        
                // some payment gateways require a transaction id / order id
                $transa1 = (rand(100, 999) . rand(100, 999) . rand(100, 999));
                $transa2 = (rand(100, 999) . rand(100, 999) . rand(100, 999));
        
                // decrypt credit card
                $decrypted_card_no = $ilance->crypt->three_layer_decrypt($sql_cc_arr['creditcard_number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                $decrypted_card_no = str_replace(' ', '', $decrypted_card_no);
        
                // hide credit card using unique string
                $decrypted_card_no_hidden = str_replace(' ', '', $decrypted_card_no);
                $ccnum_hidden = substr_replace($decrypted_card_no_hidden, 'XX XXXX XXXX ', 2, (mb_strlen($decrypted_card_no_hidden) - 6));
        
                // authentication amount 1
                $ilance->paymentgateway = construct_object('api.paymentgateway', $ilconfig['use_internal_gateway']);
                
                $ilance->paymentgateway->set_user(
                        $ilconfig['cc_login'],
                        $ilconfig['cc_password'],
                        $ilconfig['cc_key'],
                        SITE_EMAIL
                );
                
                $ilance->paymentgateway->set_customer(
                        fetch_user('username', $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id = '".$v3customer_ccid."'", "user_id")),
                        $sql_cc_arr['phone_of_cardowner'],
                        $v3customer_fname,
                        $v3customer_lname,
                        $v3customer_address,
                        $v3customer_city,
                        $v3customer_state,
                        $v3customer_zip,
                        $v3customer_country
                );
                
                $ilance->paymentgateway->set_ccard(
                        $sql_cc_arr['name_on_card'],
                        $sql_cc_arr['creditcard_type'],
                        $decrypted_card_no,
                        mb_substr($sql_cc_arr['creditcard_expiry'], 0, 2),
                        mb_substr($sql_cc_arr['creditcard_expiry'], -4),
                        $sql_cc_arr['cvv2']
                );
                
                $ilance->paymentgateway->set_currency(
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['code'],
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['symbol_left']
                );
                
                $ilance->paymentgateway->set_order(
                        $amount1,
                        $transa1,
                        $phrase['_credit_card_authentication_amount_one'],
                        $ilconfig['authentication_capture'],
                        null,
                        null,
                        null
                );
                
                $gatewayinfo = array();
                $ilance->paymentgateway->set_extra($gatewayinfo);
                
                if (!$ilance->paymentgateway->process())
                {
                        $v3process_success = 0;
                        $transaction_messageT = $ilance->paymentgateway->get_response_message();
                        $date_timeT = DATETIME24H;
            
                        global $transaction_message, $date_time;
                        $transaction_message = $transaction_messageT;
                        $date_time = $date_timeT;
            
                        $area_title = $area_title;
                        $page_title = $page_title;
                        $date_time = DATETIME24H;
                        if (empty($transaction_message))
                        {
                                $transaction_message = 'Could not communicate with merchant server / no response.';
                        }
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_processing_error');		
                        $ilance->email->set(array(
                                '{{gatewayresponse}}' => $ilance->paymentgateway->get_answer(),
                                '{{gatewaymessage}}' => $ilance->paymentgateway->get_response_message(),
                                '{{ipaddress}}' =>IPADDRESS,
                                '{{location}}' => LOCATION,
                                '{{scripturi}}' => SCRIPT_URI,
                                '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                        $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                        $ilance->template->parse_if_blocks('main');
                        $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                        exit();
                }
                else
                {
                        $v3process_success = 1;
                        
                        // gateway transaction id
                        $gatewaytransact1 = $ilance->paymentgateway->get_transact_code();
                }
        
                // authentication amount 2
                $ilance->paymentgateway2 = construct_object('api.paymentgateway', $ilconfig['use_internal_gateway']);
                
                $ilance->paymentgateway2->set_user(
                        $ilconfig['cc_login'],
                        $ilconfig['cc_password'],
                        $ilconfig['cc_key'],
                        SITE_EMAIL
                );
                
                $ilance->paymentgateway2->set_customer(
                        $_SESSION['ilancedata']['user']['username'],
                        $sql_cc_arr['phone_of_cardowner'],
                        $v3customer_fname,
                        $v3customer_lname,
                        $v3customer_address,
                        $v3customer_city,
                        $v3customer_state,
                        $v3customer_zip,
                        $v3customer_country
                );
                
                $ilance->paymentgateway2->set_ccard(
                        $sql_cc_arr['name_on_card'],
                        $sql_cc_arr['creditcard_type'],
                        $decrypted_card_no,
                        mb_substr($sql_cc_arr['creditcard_expiry'], 0, 2),
                        mb_substr($sql_cc_arr['creditcard_expiry'], -4),
                        $sql_cc_arr['cvv2']
                );
                
                $ilance->paymentgateway2->set_currency(
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['code'],
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['symbol_left']
                );
                
                $ilance->paymentgateway2->set_order(
                        $amount2,
                        $transa2,
                        $phrase['_credit_card_authentication_amount_two'],
                        $ilconfig['authentication_capture'],
                        NULL,
                        NULL,
                        NULL
                );// the last is an extra field for some gateways (mode)
                
                $gatewayinfo = array();
                $ilance->paymentgateway2->set_extra($gatewayinfo);
                
                if (!$ilance->paymentgateway2->process())
                {
                        $v3process_success = 0;
                        $transaction_messageT = $ilance->paymentgateway2->get_response_message();
                        $date_timeT = DATETIME24H;
            
                        global $transaction_message, $date_time;
                        $transaction_message = $transaction_messageT;
                        $date_time = $date_timeT;
            
                        $area_title = $area_title;
                        $page_title = $page_title;
            
                        $date_time = DATETIME24H;
                        if (empty($transaction_message))
                        {
                                $transaction_message = 'Could not communicate with merchant server / no response.';
                        }
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_processing_error');		
                        $ilance->email->set(array(
                                '{{gatewayresponse}}' => $ilance->paymentgateway2->get_answer(),
                                '{{gatewaymessage}}' => $ilance->paymentgateway2->get_response_message(),
                                '{{ipaddress}}' =>IPADDRESS,
                                '{{location}}' => LOCATION,
                                '{{scripturi}}' => SCRIPT_URI,
                                '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                        $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                        $ilance->template->parse_if_blocks('main');
                        $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                        exit();
                }
                else
                {
                        // add authentication 1 amounts secretly into the members card table
                        // also -- set this card as default card to use
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "creditcards
                                SET auth_amount1 = '" . $ilance->db->escape_string($amount1) . "',
                                trans1_id = '" . $ilance->db->escape_string($gatewaytransact1) . "',
                                default_card = 'yes'
                                WHERE user_id = '" . intval($userid) . "'
                                    AND cc_id = '" . intval($v3customer_ccid) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);    
                }
        
                if ($v3process_success)
                {
                        $gatewaytransact2 = $ilance->paymentgateway2->get_transact_code();
                        
                        // set all current cards as no default card (reset)
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "creditcards
                                SET default_card = 'no'
                                WHERE user_id = '" . intval($userid) . "'
                        ", 0, null, __FILE__, __LINE__);
            
                        // and set credit card as the default card to use and add these amounts in the users card table
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "creditcards
                                SET auth_amount2 = '" . $ilance->db->escape_string($amount2) . "',
                                trans2_id = '" . $ilance->db->escape_string($gatewaytransact2) . "',
                                default_card = 'yes'
                                WHERE user_id = '" . intval($userid) . "'
                                        AND cc_id = '" . intval($v3customer_ccid) . "'
                        ", 0, null, __FILE__, __LINE__);
            
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_authentication_admin');		
                        $ilance->email->set(array(
                                '{{provider}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{amount1}}' => $amount1,
                                '{{amount2}}' => $amount2,
                                '{{trans1}}' => $gatewaytransact1,
                                '{{trans2}}' => $gatewaytransact2,
                                '{{cc_id}}' => $v3customer_ccid,
                        ));
                        
                        $ilance->email->send();
            
                        return true;
                }
                else
                {
                        $v3process_success = 0;
                        $transaction_messageT = $ilance->paymentgateway2->get_response_message();
                        $date_timeT = DATETIME24H;
            
                        global $transaction_message, $date_time;
                        $transaction_message = $transaction_messageT;
                        $date_time = $date_timeT;
            
                        $area_title = $area_title;
                        $page_title = $page_title;
                        if (empty($transaction_message))
                        {
                                $transaction_message = 'Could not communicate with merchant server / no response.';
                        }
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_processing_error');		
                        $ilance->email->set(array(
                                '{{gatewayresponse}}' => $ilance->paymentgateway2->get_answer(),
                                '{{gatewaymessage}}' => $ilance->paymentgateway2->get_response_message(),
                                '{{ipaddress}}' =>IPADDRESS,
                                '{{location}}' => LOCATION,
                                '{{scripturi}}' => SCRIPT_URI,
                                '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                        $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                        $ilance->template->parse_if_blocks('main');
                        $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                        exit();
                }
        }
        
        /**
        * Function to update a customers credit card in the database as active based on the authentication amounts taken during the authentication process step 1.
        *
        * @param       integer      authentication amount sum
        * @param       integer      credit card id
        * @param       string       customer first name
        * @param       string       customer last name
        * @param       string       customer address
        * @param       string       customer city
        * @param       string       customer state
        * @param       string       customer zip / postal code
        * @param       integer      customer country code
        *
        * @return      bool         returns true or false
        */
        function creditcard_authentication_step_two($input_auth = '', $v3customer_ccid = '', $v3customer_fname = '', $v3customer_lname = '', $v3customer_address = '', $v3customer_city = '', $v3customer_state = '', $v3customer_zip = '', $v3customer_country = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
        
                $sql_cc_url = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "creditcards
                        WHERE user_id = '" . $_SESSION['ilancedata']['user']['userid'] . "'
                            AND cc_id = '" . intval($v3customer_ccid) . "'
                ", 0, null, __FILE__, __LINE__);
                $sql_cc_arr = $ilance->db->fetch_array($sql_cc_url);
        
                // decrypt credit card number
                $decrypted_card_no = $ilance->crypt->three_layer_decrypt($sql_cc_arr['creditcard_number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                $decrypted_card_no = str_replace(' ', '', $decrypted_card_no);
        
                // hide cc number
                $decrypted_card_no_hidden = str_replace(' ', '', $decrypted_card_no);
                $ccnum_hidden = substr_replace($decrypted_card_no_hidden, 'XX XXXX XXXX ', 2 , (mb_strlen($decrypted_card_no_hidden) - 6));
        
                $server_sum = ($sql_cc_arr['auth_amount1'] + $sql_cc_arr['auth_amount2']);
                if ($server_sum == $input_auth)
                {
                        // amounts match!
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "creditcards
                                SET authorized = 'yes',
                                creditcard_status = 'active'
                                WHERE cc_id = '" . intval($v3customer_ccid) . "'
                                        AND user_id = '" . $_SESSION['ilancedata']['user']['userid'] . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
            
                        // refund mode?
                        if ($ilconfig['authentication_refund_on_max_cc_attempts'])
                        {
                                $v3customer_fname = stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid,"name_on_card"));
                                $v3customer_lname = '';
                                $v3customer_address = stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid,"card_billing_address1")) . " " . stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid, "card_billing_address2"));
                                $v3customer_city = stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid,"card_city"));
                                $v3customer_state = stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid,"card_state"));
                                $v3customer_zip = stripslashes($ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid,"card_postalzip"));
                                $v3customer_country = stripslashes($ilance->db->fetch_field(DB_PREFIX . "locations", "locationid=" . $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid, "card_country"), "location_" . $_SESSION['ilancedata']['user']['slng']));
                                $input_auth = $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid, "auth_amount1") + $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid, "auth_amount2");
                                
                                $refundsuccess = $this->creditcard_authentication_refund($server_sum, $v3customer_ccid, $v3customer_fname, $v3customer_lname, $v3customer_address, $v3customer_city, $v3customer_state, $v3customer_zip, $v3customer_country);
                                if ($refundsuccess)
                                {
                                        return true;
                                }
                                else
                                {
                                        $header_text = $phrase['_credit_card_refund_to_card_failure'];
                                        $body_text = $phrase['_were_sorry_there_seems_to_have_been_a_problem_during_the_recredit_operation_to_your_credit_card'];
                                        
                                        print_notice($header_text, $body_text, $ilpage['accounting'], $phrase['_accounting']);
                                }
                        }
                        else
                        {
                                $account_balance_res = $ilance->db->query("
                                        SELECT total_balance, available_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . $_SESSION['ilancedata']['user']['userid'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                $account_balance_array = $ilance->db->fetch_array($account_balance_res);
                
                                // deposit refund into online account
                                $new_tbalance = $account_balance_array['total_balance'] + $sql_cc_arr['auth_amount1'];
                                $new_abalance = $account_balance_array['available_balance'] + $sql_cc_arr['auth_amount1'];
                                $new_tbalance += $sql_cc_arr['auth_amount2'];
                                $new_abalance += $sql_cc_arr['auth_amount2'];
                
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "invoices
                                        (invoiceid, user_id, description, amount, paid, totalamount, status, invoicetype, paymethod, referer, ipaddress, createdate, paiddate, custommessage, transactionid, archive)
                                        VALUES(
                                        NULL,
                                        '" . $_SESSION['ilancedata']['user']['userid'] . "',
                                        '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_credit_into_online_account']) . "',
                                        '" . ($sql_cc_arr['auth_amount1']+$sql_cc_arr['auth_amount2']) . "',
                                        '" . ($sql_cc_arr['auth_amount1']+$sql_cc_arr['auth_amount2']) . "',
                                        '" . ($sql_cc_arr['auth_amount1']+$sql_cc_arr['auth_amount2']) . "',
                                        'paid',
                                        'credit',
                                        'account',
                                        '" . $ilance->db->escape_string(REFERRER) . "',
                                        '" . $ilance->db->escape_string(IPADDRESS) . "',
                                        '" . DATETIME24H . "',
                                        '" . DATETIME24H . "',
                                        '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_credit_into_online_account']) . "',
                                        '" . construct_transaction_id() . "',
                                        '0')
                                ", 0, null, __FILE__, __LINE__);
                
                                // update account balances
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "users
                                        SET total_balance = '" . $new_tbalance . "',
                                        available_balance = '" . $new_abalance . "'
                                        WHERE user_id = '" . $_SESSION['ilancedata']['user']['userid'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                
                                return true;
                        }
                }
                
                return false;
        }        
        
        /**
        * Function for processing a credit card refund for the credit card authentication step process.
        *
        * @param       string       amount to refund
        * @param       integer      credit card id
        * @param       string       credit card first name
        * @param       string       credit card last name
        * @param       string       customer address
        * @param       string       customer city
        * @param       string       customer state
        * @param       string       customer zip code
        * @param       string       customer country
        * 
        * @return      string       Returns formatted template with success or failure message.
        */
        function creditcard_authentication_refund($input_auth = '', $v3customer_ccid = '', $v3customer_fname = '', $v3customer_lname = '', $v3customer_address = '', $v3customer_city = '', $v3customer_state = '', $v3customer_zip = '', $v3customer_country = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                $sql = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "creditcards
                        WHERE cc_id = '" . intval($v3customer_ccid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql_cc_url) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                }
                else
                {
                        return false;
                }
        
                $decrypted_card_no = $ilance->crypt->three_layer_decrypt($res['creditcard_number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                $decrypted_card_no = str_replace(' ', '', $decrypted_card_no);
        
                $ilance->paymentgateway = construct_object('api.paymentgateway', $ilconfig['use_internal_gateway']);
                
                $ilance->paymentgateway->set_user(
                        $ilconfig['cc_login'],
                        $ilconfig['cc_password'],
                        $ilconfig['cc_key'],
                        SITE_EMAIL
                );
                
                $ilance->paymentgateway->set_customer(
                        fetch_user('username', $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id = '".$v3customer_ccid."'", "user_id")),
                        $res['phone_of_cardowner'],
                        $v3customer_fname,
                        $v3customer_lname,
                        $v3customer_address,
                        $v3customer_city,
                        $v3customer_state,
                        $v3customer_zip,
                        $v3customer_country
                );
                
                $ilance->paymentgateway->set_ccard(
                        $res['name_on_card'],
                        $res['creditcard_type'],
                        $decrypted_card_no,
                        mb_substr($res['creditcard_expiry'], 0, 2),
                        mb_substr($res['creditcard_expiry'], -4),
                        $res['cvv2']
                );
                
                $ilance->paymentgateway->set_currency(
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['code'],
                        $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['symbol_left']
                );
                
                $ilance->paymentgateway->set_order(
                        $res['auth_amount1'],
                        $res['trans1_id'],
                        $phrase['_credit_card_authentication_failure_refund_one_of_two'],
                        $ilconfig['authentication_refund'],
                        $ilconfig['authentication_refund']
                );
                
                $gatewayinfo = array();
                $ilance->paymentgateway->set_extra($gatewayinfo);
                
                if (!$ilance->paymentgateway->process())
                {
                        $v3process_success = 0;
                        $transaction_messageT = $ilance->paymentgateway->get_response_message();
                        $date_timeT = DATETIME24H;
            
                        global $transaction_message, $date_time;
                        $transaction_message = $transaction_messageT;
                        $date_time = $date_timeT;
            
                        $date_time = DATETIME24H;
            
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_processing_error');		
                        $ilance->email->set(array(
                                '{{gatewayresponse}}' => $ilance->paymentgateway->get_answer(),
                                '{{gatewaymessage}}' => $ilance->paymentgateway->get_response_message(),
                                '{{ipaddress}}' =>IPADDRESS,
                                '{{location}}' => LOCATION,
                                '{{scripturi}}' => SCRIPT_URI,
                                '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                        ));
                        
                        $ilance->email->send();
                        
                        if (defined('LOCATION') AND LOCATION == 'cron')
                        {
                                return false;    
                        }
                        
                        $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                        $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                        $ilance->template->parse_if_blocks('main');
                        $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                        exit();
                }
                else
                {
                        $v3process_success = 1;
                        $ilance->db->query("
                                INSERT INTO " . DB_PREFIX . "invoices
                                (invoiceid, user_id, description, amount, paid, totalamount, status, invoicetype, paymethod, paymentgateway, referer, ipaddress, createdate, duedate, paiddate, custommessage, transactionid, archive)
                                VALUES(
                                NULL,
                                '" . $ilance->db->fetch_field(DB_PREFIX . "creditcards","cc_id=" . $v3customer_ccid,"user_id") . "',
                                '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_one_of_two']) . "',
                                '" . $res['auth_amount1'] . "',
                                '" . $res['auth_amount1'] . "',
                                '" . $res['auth_amount1'] . ",
                                'paid',
                                'credit',
                                '" . $ilance->db->escape_string($res['creditcard_type']) . "',
                                '" . $ilance->db->escape_string($ilconfig['use_internal_gateway']) . "',
                                '" . $ilance->db->escape_string(REFERRER) . "',
                                '" . $ilance->db->escape_string(IPADDRESS) . "',
                                '" . DATETIME24H . "',
                                '" . DATEINVOICEDUE . "',
                                '" . DATETIME24H . "',
                                '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_credit_to_credit_card']) . "',
                                '" . construct_transaction_id() . "',
                                '0')
                        ", 0, null, __FILE__, __LINE__);
                }
        
                if ($v3process_success)
                {
                        $ilance->paymentgateway = construct_object('api.paymentgateway', $ilconfig['use_internal_gateway']);
                        
                        $ilance->paymentgateway->set_user(
                                $ilconfig['cc_login'],
                                $ilconfig['cc_password'],
                                $ilconfig['cc_key'],
                                SITE_EMAIL
                        );
                        
                        $ilance->paymentgateway->set_customer(
                                fetch_user('username', $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id = '".$v3customer_ccid."'", "user_id")),
                                $res['phone_of_cardowner'],
                                $v3customer_fname,
                                $v3customer_lname,
                                $v3customer_address,
                                $v3customer_city,
                                $v3customer_state,
                                $v3customer_zip,
                                $v3customer_country
                        );
                        
                        $ilance->paymentgateway->set_ccard(
                                $res['name_on_card'],
                                $res['creditcard_type'],
                                $decrypted_card_no,
                                mb_substr($res['creditcard_expiry'], 0, 2),
                                mb_substr($res['creditcard_expiry'], -4),
                                $res['cvv2']
                        );
                        
                        $ilance->paymentgateway->set_currency(
                                $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['code'],
                                $ilance->currency->currencies[$ilconfig['globalserverlocale_defaultcurrency']]['symbol_left']
                        );
                        
                        $ilance->paymentgateway->set_order(
                                $res['auth_amount2'],
                                $res['trans2_id'],
                                $phrase['_credit_card_authentication_failure_refund_two_of_two'],
                                $ilconfig['authentication_refund'],
                                $ilconfig['authentication_refund']
                        );
                        
                        $gatewayinfo = array();
                        $ilance->paymentgateway->set_extra($gatewayinfo);
                        
                        if (!$ilance->paymentgateway->process())
                        {
                                $v3process_success = 0;
                                $transaction_messageT = $ilance->paymentgateway->get_response_message();
                                $date_timeT = DATETIME24H;
                
                                global $transaction_message, $date_time;
                                
                                $transaction_message = $transaction_messageT;
                                $date_time = $date_timeT;
                                $date_time = DATETIME24H;
                
                                $ilance->email = construct_dm_object('email', $ilance);
                
                                $ilance->email->mail = SITE_EMAIL;
                                $ilance->email->slng = fetch_site_slng();
                                
                                $ilance->email->get('creditcard_processing_error');		
                                $ilance->email->set(array(
                                        '{{gatewayresponse}}' => $ilance->paymentgateway->get_answer(),
                                        '{{gatewaymessage}}' => $ilance->paymentgateway->get_response_message(),
                                        '{{ipaddress}}' =>IPADDRESS,
                                        '{{location}}' => LOCATION,
                                        '{{scripturi}}' => SCRIPT_URI,
                                        '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                        '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                        '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                                ));
                                
                                $ilance->email->send();
                                
                                if (defined('LOCATION') AND LOCATION == 'cron')
                                {
                                        return false;    
                                }
                                
                                $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                                $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                                $ilance->template->parse_if_blocks('main');
                                $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                                exit();
                        }
                        else
                        {
                                $v3process_success = 1;
                                
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "invoices
                                        (invoiceid, user_id, description, amount, paid, totalamount, status, invoicetype, paymethod, paymentgateway, referer, ipaddress, createdate, paiddate, custommessage, transactionid, archive)
                                        VALUES(
                                        NULL,
                                        '" . $ilance->db->fetch_field(DB_PREFIX . "creditcards", "cc_id=" . $v3customer_ccid, "user_id") . "',
                                        '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_two_of_two']) . "',
                                        '" . $ilance->db->escape_string($res['auth_amount2']) . "',
                                        '" . $ilance->db->escape_string($res['auth_amount2']) . "',
                                        '" . $ilance->db->escape_string($res['auth_amount2']) . "',
                                        'paid',
                                        'credit',
                                        '" . $ilance->db->escape_string($res['creditcard_type']) . "',
                                        '" . $ilance->db->escape_string($ilconfig['use_internal_gateway']) . "',
                                        '" . $ilance->db->escape_string(REFERRER) . "',
                                        '" . $ilance->db->escape_string(IPADDRESS) . "',
                                        '" . DATETIME24H . "',
                                        '" . DATETIME24H . "',
                                        '" . $ilance->db->escape_string($phrase['_credit_card_authentication_refund_credit_to_credit_card']) . "',
                                        '" . construct_transaction_id() . "',
                                        '0')
                                ", 0, null, __FILE__, __LINE__);
                                
                                return true;
                        }
                }
                else
                {
                        $v3process_success = 0;
                        $transaction_messageT = $ilance->paymentgateway->get_response_message();
                        $date_timeT = DATETIME24H;
            
                        global $transaction_message, $date_time;
                        $transaction_message = $transaction_messageT;
                        $date_time = $date_timeT;
            
                        $area_title = $area_title;
                        $page_title = $page_title;
                        $date_time = DATETIME24H;
            
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('creditcard_processing_error');		
                        $ilance->email->set(array(
                                '{{gatewayresponse}}' => $ilance->paymentgateway->get_answer(),
                                '{{gatewaymessage}}' => $ilance->paymentgateway->get_response_message(),
                                '{{ipaddress}}' =>IPADDRESS,
                                '{{location}}' => LOCATION,
                                '{{scripturi}}' => SCRIPT_URI,
                                '{{gateway}}' => $ilconfig['use_internal_gateway'],
                                '{{member}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{memberemail}}' => $_SESSION['ilancedata']['user']['email'],
                        ));
                        
                        $ilance->email->send();
                        
                        if (defined('LOCATION') AND LOCATION == 'cron')
                        {
                                return false;    
                        }
                        
                        $ilance->template->load_file('main', 'print_notice_payment_gateway.html');
                        $ilance->template->parse_hash('main', array('ilpage' => $ilpage));
                        $ilance->template->parse_if_blocks('main');
                        $ilance->template->pprint('main', array('date_time','transaction_message','transaction_code','login_include','headinclude','onload','area_title','page_title','site_name','https_server','http_server','lanceads_header','lanceads_footer'));
                        exit();
                }
        }
        
        /**
        * Function to generate a credit card type pulldown menu.
        *
        * @param       string       selected card type (optional)
        * @param       string       select menu field name (default is card[type])
        *
        * @return      string       returns HTML representation of the pulldown selection menu
        */
        function creditcard_type_pulldown($selected = '', $fieldname = 'form[type]')
        {
                $html = '<select name="' . $fieldname . '" style="font-family: Verdana">';
                
                $html .= '<option value="visa"';
                if ($selected == 'visa')
                {
                        $html .= ' selected="selected"';
                }
                $html .= '>Visa</option>';
                $html .= '<option value="mc"';
                if ($selected == 'mc')
                {
                        $html .= ' selected="selected"';
                }
                $html .= '>Mastercard</option>';
                $html .= '<option value="amex"';
                if ($selected == 'amex')
                {
                        $html .= ' selected="selected"';
                }
                $html .= '>American Express</option>';
                $html .= '<option value="disc"';
                if ($selected == 'disc')
                {
                        $html .= ' selected="selected"';
                }
                $html .= '>Discover</option>';
                $html .= '</select>';
                
                return $html;
        }
            
        /**
        * Function to generate a credit card month pulldown menu.
        *
        * @param       string       selected month (optional)
        * @param       string       select menu field name (default is card[expmon])
        * 
        * @return      string       returns HTML representation of the pulldown selection menu
        */
        function creditcard_month_pulldown($selected = '', $fieldname = 'form[expmon]')
        {
                $html  = '<select name="' . $fieldname . '" style="font-family: Verdana">';
                for($i = 1; $i < 13; $i++)
                {
                        $html  .= '<option value="';
                        if ($i < 10)
                        {
                                $html  .= '0' . $i;
                                if ($selected == '0' . $i)
                                {
                                        $html .= '" selected="selected">0' . $i;
                                }
                                else
                                {
                                        $html .= '">0' . $i;
                                }
                        }
                        else
                        {
                                $html .= $i;
                                if ($selected == $i)
                                {
                                        $html .= '" selected="selected">' . $i;
                                }
                                else
                                {
                                        $html .= '">' . $i;
                                }
                                $html .= '</option>';
                        }
                }
                $html .= '</select>';
                
                return $html;
        }
            
        /**
        * Function to generate a credit card year pulldown menu.
        *
        * @param       string       selected year (optional)
        * @param       string       select menu field name (default is card[expyear])
        *
        * @return      string       returns HTML representation of the pulldown selection menu
        */
        function creditcard_year_pulldown($selected = '', $fieldname = 'form[expyear]')
        {
                $html = '<select name="' . $fieldname . '" style="font-family: Verdana">';
                
                for ($i = date('Y'); $i < date('Y')+10; $i++)
                {
                        if ($selected == mb_substr("$i", -2))
                        {
                                $html .= '<option value="' . mb_substr("$i", -2) . '" selected="selected">';
                        }
                        else
                        {
                                $html .= '<option value="' . mb_substr("$i", -2) . '">';
                        }
                        $html .= $i;
                        $html .= '</option>';
                }
                
                $html .= '</select>';
                
                return $html;
        }
            
        /**
        * Function to generate a credit card country pulldown menu.
        *
        * @param       string       selected year (optional)
        * @param       string       short language identifier (default eng)
        * @param       string       pulldown menu fieldname (default form[countryid])
        *
        * @return      string       returns HTML representation of the pulldown selection menu
        */
        function creditcard_country_pulldown($countryid, $slng = 'eng', $fieldname = 'form[countryid]')
        {
                global $ilance, $myapi, $ilconfig, $phrase;
                
                $html = '<select name="' . $fieldname . '" style="font-family: Verdana">';
                
                $sql = $ilance->db->query("
                        SELECT locationid, location_$slng AS location
                        FROM " . DB_PREFIX . "locations
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                $html .= '<option value="' . $res['locationid'] . '"';
                                if ($res['locationid'] == $countryid)
                                {
                                        $html .= 'selected="selected"';
                                }
                                $html .= '>' . ucwords(stripslashes($res['location'])) . '</option>';
                        }
                }
                
                $html .= '</select>';
                
                return $html;
        }
        
        /**
        * Function to print the destination currency pulldown menu
        *
        * @param       string       selected pulldown option (optional)
        * @param       string       actual form field name
        * @param       boolean      show currency names in pulldown menu in uppercase? (default false)
        *
        * @return      string       Returns HTML representation of the destination currency pulldown types
        */
        function print_destination_currency_pulldown($selected = '', $fieldname = 'form[destination_currency_id]', $uppercase = false)
        {
                global $ilance, $myapi, $ilconfig, $phrase;
                
                $html = '<select name="' . $fieldname . '" style="font-family: verdana">';
                
                $sql = $ilance->db->query("
                        SELECT currency_id, currency_name
                        FROM " . DB_PREFIX . "currency
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                $html .= '<option value="' . $res['currency_id'] . '"';
                                
                                if (empty($selected) AND $res['currency_id'] == $_SESSION['ilancedata']['user']['currencyid'])
                                {
                                        $html .= ' selected="selected"';
                                }
                                else if (!empty($selected) AND $res['currency_id'] == $selected)
                                {
                                        $html .= ' selected="selected"';
                                }
                                
                                if ($uppercase)
                                {
                                        $html .= '>' . mb_strtoupper($res['currency_name']) . '</option>';
                                }
                                else
                                {
                                        $html .= '>' . $res['currency_name'] . '</option>';
                                }
                        }
                }
                
                $html .= '</select>';
                
                return $html;
        }
        
        /**
        * Function to print a bank account type pulldown menu
        *
        * @param       string       selected pulldown option (optional)
        * @param       string       actual form field name
        *
        * @return      string       Returns HTML representation of the bank account pulldown types
        */
        function print_bank_account_type_pulldown($selected = '', $fieldname = 'form[bank_account_type]')
        {
                global $ilance, $myapi, $ilconfig, $phrase;
                
                $options = array('checking', 'savings');
                
                $html = '<select name="' . $fieldname . '" style="font-family: verdana">';
                foreach ($options AS $type)
                {
                        if (isset($selected) AND $selected == $type)
                        {
                                $html .= '<option value="' . $type . '" selected="selected">' . $phrase["_$type"] . '</option>';   
                        }
                        else
                        {
                                $html .= '<option value="' . $type . '">' . $phrase["_$type"] . '</option>';
                        }
                }
                $html .= '</select>';
                
                return $html;
        }
        
        /**
        * Function to verify mod-10 characteristics of a credit card number.
        *
        * @param       string       credit card number
        *
        * @return      bool         true or false valid mod10
        */
        function verify_creditcard_mod10($strccno = '')
        {
                if (empty($strccno))
                {
                        return false;
                }
                $len = mb_strlen($strccno);
                if ($len < 13 OR $len > 16)
                {
                        return false;
                }
                $checkdig = (int)$strccno[--$len];
                for ($i=--$len, $sum = 0, $dou = true; $i >= 0; $i--, $dou =! $dou)
                {
                        $curdig = (int)$strccno[$i];
                        if ($dou)
                        {
                                $curdig *= 2;
                                if ($curdig > 9) $curdig-=9;
                        }
                        $sum += $curdig;
                }
                if (($checkdig+$sum)%10 == 0)
                {
                        return true;
                }
                else
                {
                        return false;
                }
        }
        
        /**
        * Function to create a new bank account deposit account for a registered member.
        *
        * @param       array        array holding all $form['xx'] field values
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function insert_bank_account($form = array(), $userid = 0)
        {
                global $ilance, $ilconfig, $phrase, $ilpage, $myapi;
                
                $ilance->db->query("
                        INSERT INTO " . DB_PREFIX . "bankaccounts
                        (bank_id, user_id, beneficiary_account_name, destination_currency_id, beneficiary_bank_name,
                        beneficiary_account_number, beneficiary_bank_routing_number_swift, bank_account_type, beneficiary_bank_address_1,
                        beneficiary_bank_address_2, beneficiary_bank_city, beneficiary_bank_state, beneficiary_bank_zipcode, beneficiary_bank_country_id) VALUES(
                        NULL,
                        '" . intval($userid) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_account_name']) . "',
                        '" . intval($form['destination_currency_id']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_name']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_account_number']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_routing_number_swift']) . "',
                        '" . $ilance->db->escape_string($form['bank_account_type']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_address_1']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_address_2']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_city']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_state']) . "',
                        '" . $ilance->db->escape_string($form['beneficiary_bank_zipcode']) . "',
                        '" . intval($form['beneficiary_bank_country_id']) . "')
                ", 0, null, __FILE__, __LINE__);
                $bank_id = $ilance->db->insert_id();
                
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', intval($userid));
                $ilance->email->slng = fetch_user_slng(intval($userid));
                
                $ilance->email->get('member_added_bankaccount');		
                $ilance->email->set(array(
                        '{{username}}' => fetch_user('username', intval($userid)),
                ));
                
                $ilance->email->send();                    
                
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
                $ilance->email->get('member_added_bankaccount_admin');		
                $ilance->email->set(array(
                        '{{username}}' => fetch_user('username', intval($userid)),
                        '{{account_name}}' => $form['beneficiary_account_name'],
                        '{{bank_name}}' => $form['beneficiary_bank_name'],
                        '{{account_type}}' => $form['bank_account_type'],
                        '{{bank_city}}' => $form['beneficiary_bank_city'],
                        '{{bank_zippostal}}' => $form['beneficiary_bank_zipcode'],
                ));
                
                $ilance->email->send();
                
                return true;
        }
        
        /**
        * Function to create a new credit card account for a registered member.
        *
        * @param       array        array holding all $form['xx'] field values
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function insert_creditcard($form = array(), $userid = 0)
        {
                global $ilance, $ilconfig, $phrase, $ilpage, $myapi;
                
                $form['number_encrypted'] = $ilance->crypt->three_layer_encrypt($form['number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                $form['authorized'] = 'yes';
                $form['creditcard_status'] = 'active';
                $form['default_card'] = 'yes';
                
                if ($ilconfig['creditcard_authentication'])
                {
                        $form['authorized'] = 'no';
                }
                
                $ilance->db->query("
                        INSERT INTO " . DB_PREFIX . "creditcards
                        (cc_id, date_added, date_updated, user_id, creditcard_number, creditcard_expiry, cvv2, name_on_card, phone_of_cardowner, email_of_cardowner, card_billing_address1, card_billing_address2, card_city, card_state, card_postalzip, card_country, creditcard_status, default_card, creditcard_type, authorized) 
                        VALUES(
                        NULL,
                        '" . DATETIME24H . "',
                        '" . DATETIME24H . "',
                        '" . intval($userid) . "',
                        '" . $ilance->db->escape_string($form['number_encrypted']) . "',
                        '" . $ilance->db->escape_string($form['expmon'] . $form['expyear']) . "',
                        '" . intval($form['cvv2']) . "',
                        '" . $ilance->db->escape_string($form['first_name'] . " " . $form['last_name']) . "',
                        '" . $ilance->db->escape_string($form['phone']) . "',
                        '" . $ilance->db->escape_string($form['email']) . "',
                        '" . $ilance->db->escape_string($form['address1']) . "',
                        '" . $ilance->db->escape_string($form['address2']) . "',
                        '" . $ilance->db->escape_string($form['city']) . "',
                        '" . $ilance->db->escape_string($form['state']) . "',
                        '" . $ilance->db->escape_string($form['postalzip']) . "',
                        '" . $ilance->db->escape_string($form['countryid']) . "',
                        '" . $ilance->db->escape_string($form['creditcard_status']) . "',
                        '" . $ilance->db->escape_string($form['default_card']) . "',
                        '" . $ilance->db->escape_string($form['type']) . "',
                        '" . $ilance->db->escape_string($form['authorized']) . "')
                ", 0, null, __FILE__, __LINE__);
                $cc_id = $ilance->db->insert_id();    
                
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', $userid);
                $ilance->email->slng = fetch_user_slng(intval($userid));
                
                $ilance->email->get('member_added_new_card');		
                $ilance->email->set(array(
                        '{{username}}' => fetch_user('username', intval($userid)),
                ));
                
                $ilance->email->send();
                
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
                $ilance->email->get('member_added_new_card_admin');		
                $ilance->email->set(array(
                        '{{username}}' => fetch_user('username', intval($userid)),
                ));
                
                $ilance->email->send();
                                        
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "subscription_user
                        SET paymethod = 'account'
                        WHERE user_id = '" . intval($userid) . "'
                ", 0, null, __FILE__, __LINE__);
                
                return true;
        }
        
        /**
        * Function to update an existing bank account for a registered member.
        *
        * @param       array        array holding all $form['xx'] field values
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function update_bank_account($form = array(), $userid = 0)
        {
                global $ilance, $ilconfig, $phrase, $ilpage, $myapi;
                
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "bankaccounts
                        SET beneficiary_account_name = '" . $ilance->db->escape_string($form['beneficiary_account_name']) . "',
                        destination_currency_id = '" . intval($form['destination_currency_id']) . "',
                        beneficiary_bank_name = '" . $ilance->db->escape_string($form['beneficiary_bank_name']) . "',
                        beneficiary_account_number = '" . $ilance->db->escape_string($form['beneficiary_account_number']) . "',
                        beneficiary_bank_routing_number_swift = '" . $ilance->db->escape_string($form['beneficiary_bank_routing_number_swift']) . "',
                        bank_account_type = '" . $ilance->db->escape_string($form['bank_account_type']) . "',
                        beneficiary_bank_address_1 = '" . $ilance->db->escape_string($form['beneficiary_bank_address_1']) . "',
                        beneficiary_bank_address_2 = '" . $ilance->db->escape_string($form['beneficiary_bank_address_2']) . "',
                        beneficiary_bank_city = '" . $ilance->db->escape_string($form['beneficiary_bank_city']) . "',
                        beneficiary_bank_state = '" . $ilance->db->escape_string($form['beneficiary_bank_state']) . "',
                        beneficiary_bank_zipcode = '" . $ilance->db->escape_string($form['beneficiary_bank_zipcode']) . "',
                        beneficiary_bank_country_id = '" . intval($form['beneficiary_bank_country_id']) . "'
                        WHERE bank_id = '".intval($form['bankid'])."'
                        AND user_id = '" . intval($userid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                
                return true;
        }
            
        /**
        * Function to update an existing credit card account for a registered member.
        *
        * @param       array        array holding all $form['xx'] field values
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function update_creditcard($form = array(), $userid = 0)
        {
                global $ilance, $ilconfig, $phrase, $ilpage, $myapi;
                
                $form['number_encrypted'] = $ilance->crypt->three_layer_encrypt($form['number'], $ilconfig['key1'], $ilconfig['key2'], $ilconfig['key3']);
                $form['authorized'] = 'yes';
                
                if ($ilconfig['creditcard_authentication'])
                {
                        $form['authorized'] = 'no';    
                }
                
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "creditcards 
                        SET creditcard_number = '" . $ilance->db->escape_string($form['number_encrypted']) . "',
                        creditcard_type = '" . mb_strtolower($form['type']) . "',
                        date_updated = '" . DATETIME24H . "',
                        creditcard_expiry = '" . $form['expmon'] . $form['expyear'] . "',
                        cvv2 = '" . $ilance->db->escape_string($form['cvv2']) . "',
                        name_on_card = '" . $ilance->db->escape_string($form['first_name'] . " " . $form['last_name']) . "',
                        phone_of_cardowner = '" . $ilance->db->escape_string($form['phone']) . "',
                        email_of_cardowner = '" . $ilance->db->escape_string($form['email']) . "',
                        card_billing_address1 = '" . $ilance->db->escape_string($form['address1']) . "',
                        card_billing_address2 = '" . $ilance->db->escape_string($form['address2']) . "',
                        card_city = '" . $ilance->db->escape_string($form['city']) . "',
                        card_state = '" . $ilance->db->escape_string($form['state']) . "',
                        card_postalzip = '" . $ilance->db->escape_string($form['postalzip']) . "',
                        card_country = '" . $ilance->db->escape_string($form['countryid']) . "',
                        authorized = '" . $ilance->db->escape_string($form['authorized']) . "'
                        WHERE user_id = '" . intval($userid) . "'
                        AND cc_id = '" . $ilance->db->escape_string($form['cc_id']) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
        
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', intval($userid));
                $ilance->email->slng = fetch_user_slng(intval($userid));
                
                $ilance->email->get('member_updated_creditcard');		
                $ilance->email->set(array(
                        '{{member}}' => fetch_user('username', intval($userid)),
                ));
                
                $ilance->email->send();
                
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
                $ilance->email->get('member_updated_creditcard_admin');		
                $ilance->email->set(array(
                        '{{member}}' => fetch_user('username', intval($userid)),
                ));
                
                $ilance->email->send();
                
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "subscription_user
                        SET paymethod = 'account'
                        WHERE user_id = '" . intval($userid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                
                return true;
        }
        
        /**
        * Function to remove a bank account account for a registered member.
        *
        * @param       integer      bank account id number
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function remove_bank_account($bankid = 0, $userid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase;
                
                $ilance->db->query("
                        DELETE FROM " . DB_PREFIX . "bankaccounts
                        WHERE bank_id = '" . intval($bankid) . "'
                            AND user_id = '" . intval($userid) . "'
                ", 0, null, __FILE__, __LINE__);
        
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', intval($userid));
                $ilance->email->slng = fetch_user_slng(intval($userid));
                
		$ilance->email->get('member_removed_bank_account');		
		$ilance->email->set(array(
			'{{member}}' => fetch_user('username', intval($userid)),
		));
		
		$ilance->email->send();
                
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
		$ilance->email->get('member_removed_bank_account_admin');		
		$ilance->email->set(array(
			'{{member}}' => fetch_user('username', intval($userid)),
		));
		
		$ilance->email->send();
                
                return true;
        }
        
        /**
        * Function to remove a credit card account for a registered member.
        *
        * @param       integer      credit card id number
        * @param       integer      user id
        *
        * @return      bool         true on success
        */
        function remove_creditcard($ccid = 0, $userid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase;
                $ilance->db->query("
                        DELETE FROM " . DB_PREFIX . "creditcards
                        WHERE cc_id = '" . intval($ccid) . "'
                            AND user_id = '" . intval($userid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
        
                // change paymethod to online account
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "subscription_user
                        SET paymethod = 'account'
                        WHERE user_id = '" . intval($userid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
        
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', intval($userid));
                $ilance->email->slng = fetch_user_slng(intval($userid));
                
		$ilance->email->get('member_removed_creditcard');		
		$ilance->email->set(array(
			'{{member}}' => fetch_user('username', intval($userid)),
		));
		
		$ilance->email->send();
                
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
		$ilance->email->get('member_removed_creditcard_admin');		
		$ilance->email->set(array(
			'{{member}}' => fetch_user('username', intval($userid)),
		));
		
		$ilance->email->send();
                
                return true;
        }
        
        /**
        * Functions for creating a final value fee based on a bid id, category id and project id.
        * 
        * This function will take the bid id, cat id and project id to determine the final value
        * fee to charge (or refund) to a user based on an awarded auction they accepted to
        * complete within this particular category.  Note: The buyer has ability to unaward this
        * awarded bid, if this happens the service provider or merchant still owes the final value
        * fee even if the project is unawarded.  Note 2: This function will now check to see if
        * funds exist in online account and will auto-debit the fvf if possible.
        *
        * @param       integer          bid id
        * @param       integer          category id
        * @param       integer          project id
        * @param       string           final value fee creation mode (charge or refund)
        * @param       string           category type (service or product)
        *
        * @return      bool             Returns true or false based on the creation of the final value fee or refund
        */
        function construct_final_value_fee($bidid = 0, $cid = 0, $pid = 0, $mode = '', $cattype = '')
        {
                global $ilance, $myapi, $ilconfig, $phrase;
                
                $ilance->subscription = construct_object('api.subscription');
                
                $usetier = $tiers = $price = $total = $remaining = $fvf = 0;
                
                ($apihook = $ilance->api('construct_final_value_fee_start')) ? eval($apihook) : false;
                
                // fetch awarded bid amount
                $project = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($pid) . "'
                            AND bid_id = '" . intval($bidid) . "'
                        LIMIT 1
                ");
                if ($ilance->db->num_rows($project) > 0)
                {
                        $resproject = $ilance->db->fetch_array($project);
                        
                        // awarded bid amount
                        if ($resproject['bidamounttype'] == 'entire' OR $resproject['bidamounttype'] == 'item' OR $resproject['bidamounttype'] == 'lot' OR $resproject['bidamounttype'] == 'weight')
                        {
                                $bidamount = ($resproject['bidamount'] * $resproject['qty']);
                        }
                        else
                        {
                                $bidamount = ($resproject['bidamount'] * $resproject['estimate_days']);
                        }
                        
                        // fvf commission logic
                        if ($cattype == 'product')
                        {
                                // seller of product auction is charged FVF
                                $bidderid = $resproject['project_user_id'];
                        }
                        else if ($cattype == 'service')
                        {
                                // service provider is charged FVF
                                $bidderid = $resproject['user_id'];
                        }
                        
                        // #### ARE WE USING FIXED CATEGORY FEES? ##############################
                        // first check if admin uses fixed fees in this category
                        // admin defines fixed fees within AdminCP > Distribution > Categories > (edit mode)
                        $ilance->categories->build_array($cattype, $_SESSION['ilancedata']['user']['slng'], 0, true);
                        if ($ilance->categories->fetch[$_SESSION['ilancedata']['user']['slng']][$cattype][intval($cid)]['usefixedfees'] AND !empty($resproject['bidamounttype']))
                        {
                                // let's output our fixed commission amount
                                $fvf = $ilance->categories->fetch[$_SESSION['ilancedata']['user']['slng']][$cattype][intval($cid)]['fixedfeeamount'];
                                $usetier = 1;
                        }
                        
                        // #### NO FIXED CATEGORIES FEES > CHECK FINAL VALUE GROUP #############
                        else
                        {
                                // fetch final value group for this category
                                // we are at this point because the admin has not defined fixed fees
                                $categories = $ilance->db->query("
                                        SELECT finalvaluegroup
                                        FROM " . DB_PREFIX . "categories
                                        WHERE cid = '" . intval($cid) . "'
                                ");
                                if ($ilance->db->num_rows($categories) > 0)
                                {
                                        $cats = $ilance->db->fetch_array($categories);
                                        if (!empty($cats['finalvaluegroup']))
                                        {
                                                $finalvalues = $ilance->db->query("
                                                        SELECT *
                                                        FROM " . DB_PREFIX . "finalvalue
                                                        WHERE groupname = '" . trim($cats['finalvaluegroup']) . "'
                                                            AND state = '" . $ilance->db->escape_string($cattype) . "'
                                                        ORDER BY tierid ASC
                                                ");
                                                $totaltiers = (int)$ilance->db->num_rows($finalvalues);
                                                if ($totaltiers == 1)
                                                {
                                                        // #### SINGLE FVF TIER LOGIC ##############################
                                                        $fees = $ilance->db->fetch_array($finalvalues);
                                                        if ($bidamount >= $fees['finalvalue_from'])
                                                        {
                                                                $usetier = $fees['tierid'];
                                                                if ($fees['amountfixed'] > 0)
                                                                {
                                                                        $fvf += $fees['amountfixed'];
                                                                        $fv = $fees['amountfixed'];
                                                                }
                                                                else
                                                                {
                                                                        $fvf += ($bidamount * $fees['amountpercent'] / 100);
                                                                        $fv = ($bidamount * $fees['amountpercent'] / 100);
                                                                }
                                                        }
                                                }
                                                else
                                                {
                                                        // #### MULTIPLE FVF TIER LOGIC ############################
                                                        if ($totaltiers > 0)
                                                        {
                                                                while ($fees = $ilance->db->fetch_array($finalvalues))
                                                                {
                                                                        $tiers++;
                                                                        if ($fees['finalvalue_to'] >= 0.00)
                                                                        {
                                                                                if ($bidamount >= $fees['finalvalue_from'] AND $bidamount <= $fees['finalvalue_to'])
                                                                                {
                                                                                        $usetier = $fees['tierid'];
                                                                                        $bid = ($bidamount-($fees['finalvalue_to'] - $fees['finalvalue_from'])); 
                                                                                        if ($tiers == 1)
                                                                                        {
                                                                                                if ($fees['amountfixed'] > 0)
                                                                                                {
                                                                                                        // fixed
                                                                                                        $fvf += $fees['amountfixed'];
                                                                                                        $fv = $fees['amountfixed'];
                                                                                                }
                                                                                                else
                                                                                                {
                                                                                                        // percentage
                                                                                                        $fvf += ($bidamount * $fees['amountpercent'] / 100);
                                                                                                        $fv = ($bidamount * $fees['amountpercent'] / 100);
                                                                                                }
                                                                                        }
                                                                                        else
                                                                                        {
                                                                                                if ($fees['amountfixed'] > 0)
                                                                                                {
                                                                                                        // fixed
                                                                                                        $fvf += $fees['amountfixed'];
                                                                                                        $fv = $fees['amountfixed'];
                                                                                                }
                                                                                                else
                                                                                                {
                                                                                                        // percent
                                                                                                        $fvf += ($remaining * $fees['amountpercent'] / 100);
                                                                                                        $fv = ($remaining * $fees['amountpercent'] / 100);    
                                                                                                }
                                                                                        }
                                                                                        break;
                                                                                }
                                                                                else
                                                                                {
                                                                                        // the fees must go on! .-)
                                                                                        if ($fees['amountfixed'] > 0)
                                                                                        {
                                                                                                // fixed
                                                                                                $fvf += $fees['amountfixed'];
                                                                                                $fv = $fees['amountfixed'];
                                                                                        }
                                                                                        else
                                                                                        {
                                                                                                // percent
                                                                                                $fvf += (($fees['finalvalue_to'] - $fees['finalvalue_from']) * $fees['amountpercent'] / 100);
                                                                                                $fv = (($fees['finalvalue_to'] - $fees['finalvalue_from']) * $fees['amountpercent'] / 100);
                                                                                        }
                                                                                        // calculate remaining bid amount for next tier
                                                                                        $bid = ($bidamount - ($fees['finalvalue_to'] - $fees['finalvalue_from']));
                                                                                        $remaining = ($bid - $fees['finalvalue_from']);
                                                                                }
                                                                        }
                                                                        else
                                                                        {
                                                                                // ie: 1000.01 to -1 denotes 1000.01 - (and above)
                                                                                if ($bidamount >= $fees['finalvalue_from'])
                                                                                {
                                                                                        $usetier = $fees['tierid'];
                                                                                        if ($fees['amountfixed'] > 0)
                                                                                        {
                                                                                                $fvf += $fees['amountfixed'];
                                                                                                $fv = $fees['amountfixed'];
                                                                                        }
                                                                                        else
                                                                                        {
                                                                                                $fvf += ($remaining * $fees['amountpercent'] / 100);
                                                                                                $fv = ($remaining * $fees['amountpercent'] / 100);    
                                                                                        }
                                                                                        // calculate remaining bid amount for next tier
                                                                                        $bid = ($bidamount - $fees['finalvalue_from']);
                                                                                        $remaining = ($bid - $fees['finalvalue_from']);
                                                                                }
                                                                        }
                                                                }
                                                        }
                                                }
                                        }
                                }    
                        }
                        
                        // check if we're exempt from final value fees
                        if (!empty($bidderid) AND $bidderid > 0 AND $ilance->subscription->check_access($bidderid, 'fvfexempt') == 'yes')
                        {
                                $fee = $usetier = 0;
                        }
                        
                        if (isset($usetier) AND $usetier > 0)
                        {
                                if (isset($fvf) AND $fvf > 0)
                                {
                                        ($apihook = $ilance->api('construct_final_value_fee_end')) ? eval($apihook) : false;
                                        
                                        // #### CHARGE FVF LOGIC #######################################
                                        if ($mode == 'charge')
                                        {
                                                // do we have funds in online account?
                                                $account = $ilance->db->query("
                                                        SELECT available_balance, total_balance
                                                        FROM " . DB_PREFIX . "users
                                                        WHERE user_id = '" . intval($bidderid) . "'
                                                ");
                                                if ($ilance->db->num_rows($account) > 0)
                                                {
                                                        $res = $ilance->db->fetch_array($account);
                                                        
                                                        $avail = $res['available_balance'];
                                                        $total = $res['total_balance'];
                                                        
                                                        if ($total >= $fvf AND fetch_user('autopayment', intval($bidderid)))
                                                        {
                                                                // create a paid final value fee to the service provider
                                                                $invoiceid = $this->insert_transaction(
                                                                        0,
                                                                        intval($pid),
                                                                        0,
                                                                        intval($bidderid),
                                                                        0,
                                                                        0,
                                                                        0,
                                                                        $phrase['_final_value_fee_for_auction'] . ' #' . intval($pid),
                                                                        sprintf("%01.2f", $fvf),
                                                                        sprintf("%01.2f", $fvf),
                                                                        'paid',
                                                                        'debit',
                                                                        'account',
                                                                        DATETIME24H,
                                                                        DATETIME24H,
                                                                        DATETIME24H,
                                                                        $phrase['_auto_debit_from_online_account_balance'],
                                                                        0,
                                                                        0,
                                                                        1
                                                                );
                                                                
                                                                // update invoice mark as final value fee invoice type
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                        SET isfvf = '1'
                                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                                        LIMIT 1
                                                                ");
                                                                
                                                                // update final value fee field in bid table & project table for awarded amount
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "project_bids
                                                                        SET fvf = '" . sprintf("%01.2f", $fvf) . "'
                                                                        WHERE bid_id = '" . intval($bidid) . "'
                                                                            AND project_id = '" . intval($pid) . "'
                                                                        LIMIT 1
                                                                ");
                                                                
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "projects
                                                                        SET fvf = '" . sprintf("%01.2f", $fvf) . "',
                                                                        isfvfpaid = '1',
                                                                        fvfinvoiceid = '" . intval($invoiceid) . "'
                                                                        WHERE project_id = '" . intval($pid) . "'
                                                                ");
                                                                
                                                                // update account balance
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "users
                                                                        SET available_balance = available_balance - " . sprintf("%01.2f", $fvf) . ",
                                                                        total_balance = total_balance - " . sprintf("%01.2f", $fvf) . "
                                                                        WHERE user_id = '" . intval($bidderid) . "'
                                                                ");
                                                                
                                                                // track income history
                                                                insert_income_spent(intval($bidderid), sprintf("%01.2f", $fvf), 'credit');
                                                                
                                                                // #### REFERRAL SYSTEM TRACKER ############################
                                                                update_referral_action('fvf', intval($bidderid));                                                                                            
                                                        }
                                                        else
                                                        {
                                                                // create an unpaid final value fee to the service provider
                                                                $invoiceid = $this->insert_transaction(
                                                                        0,
                                                                        intval($pid),
                                                                        0,
                                                                        intval($bidderid),
                                                                        0,
                                                                        0,
                                                                        0,
                                                                        $phrase['_final_value_fee_for_auction'] . ' #' . intval($pid),
                                                                        sprintf("%01.2f", $fvf),
                                                                        '',
                                                                        'unpaid',
                                                                        'debit',
                                                                        'account',
                                                                        DATETIME24H,
                                                                        DATEINVOICEDUE,
                                                                        '',
                                                                        $phrase['_please_pay_this_invoice_soon_as_possible'],
                                                                        0,
                                                                        0,
                                                                        1
                                                                );
                                                            
                                                                // update invoice mark as final value fee invoice type
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                        SET isfvf = '1'
                                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                                        LIMIT 1
                                                                ");
                                                                
                                                                // update final value fee field in bid & project table for awarded amount
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "project_bids
                                                                        SET fvf = '" . sprintf("%01.2f", $fvf) . "'
                                                                        WHERE bid_id = '" . intval($bidid) . "'
                                                                            AND project_id = '" . intval($pid) . "'
                                                                        LIMIT 1
                                                                ");
                                                                
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "projects
                                                                        SET fvf = '" . sprintf("%01.2f", $fvf) . "',
                                                                        isfvfpaid = '0',
                                                                        fvfinvoiceid = '" . intval($invoiceid) . "'
                                                                        WHERE project_id = '" . intval($pid) . "'
                                                                ");
                                                        }
                                                }
                                        }
                                        
                                        // #### REFUND FVF LOGIC #######################################
                                        else if ($mode == 'refund')
                                        {
                                                // let's refund this final value fee due to an unaward by the buyer
                                                // find out if the provider paid this fvf or learn if it's still unpaid
                                                
                                                // fetch the most recent fvf for this particular project id
                                                $maxinvoicesql = $ilance->db->query("
                                                        SELECT MAX(invoiceid) AS maxinvoiceid
                                                        FROM " . DB_PREFIX . "invoices
                                                        WHERE projectid = '" . intval($pid) . "'
                                                            AND user_id = '" . intval($bidderid) . "'
                                                            AND isfvf = '1'
                                                        LIMIT 1
                                                ");
                                                if ($ilance->db->num_rows($maxinvoicesql) > 0)
                                                { 
                                                        $maxid = $ilance->db->fetch_array($maxinvoicesql);
                                                        $invsql = $ilance->db->query("
                                                                SELECT invoiceid, status, paid
                                                                FROM " . DB_PREFIX . "invoices
                                                                WHERE projectid = '" . intval($pid) . "'
                                                                    AND user_id = '" . intval($bidderid) . "'
                                                                    AND isfvf = '1'
                                                                    AND invoiceid = '" . $maxid['maxinvoiceid'] . "'
                                                                LIMIT 1
                                                        ");
                                                        if ($ilance->db->num_rows($invsql) > 0)
                                                        {
                                                                $invres = $ilance->db->fetch_array($invsql);
                                                                
                                                                // #### UNPAID FVF HANDLER #####################
                                                                if ($invres['status'] == 'unpaid')
                                                                {
                                                                        // provider hasn't paid final value fee yet! let's cancel this invoice
                                                                        // so they do not see any pending/unpaid invoices for this project
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "invoices
                                                                                SET status = 'cancelled',
                                                                                custommessage = '" . $ilance->db->escape_string($phrase['_awarded_bid_was_unawarded_by_the_owner_of_this_project_invoice_cancelled']) . "'
                                                                                WHERE invoiceid = '" . $invres['invoiceid'] . "'
                                                                                LIMIT 1
                                                                        ");
                                                                        
                                                                        // reset final value fees for this project
                                                                        $ilance->db->query("
                                                                                UPDATE " . DB_PREFIX . "projects
                                                                                SET fvf = '0.00',
                                                                                isfvfpaid = '0',
                                                                                fvfinvoiceid = '0'
                                                                                WHERE project_id = '" . intval($pid) . "'
                                                                        ");
                                                                        // perhaps an email gets dispatched informing the provider
                                                                        // that the final value fee has been cancelled
                                                                }
                                                                
                                                                // #### PAID FVF HANDLER #######################
                                                                else if ($invres['status'] == 'paid')
                                                                {
                                                                        // provider already paid the site for the final value fee 
                                                                        // so let's refund this amount and update the providers account balance
                                                                        if ($invres['paid'] > 0)
                                                                        {
                                                                                // create a final value fee credit to the service provider
                                                                                $refundinvoiceid = $this->insert_transaction(
                                                                                        0,
                                                                                        intval($pid),
                                                                                        0,
                                                                                        intval($bidderid),
                                                                                        0,
                                                                                        0,
                                                                                        0,
                                                                                        $phrase['_final_value_fee_refund_credit_for_auction'] . ' #' . intval($pid),
                                                                                        sprintf("%01.2f", $invres['paid']),
                                                                                        sprintf("%01.2f", $invres['paid']),
                                                                                        'paid',
                                                                                        'credit',
                                                                                        'account',
                                                                                        DATETIME24H,
                                                                                        DATETIME24H,
                                                                                        DATETIME24H,
                                                                                        '',
                                                                                        0,
                                                                                        0,
                                                                                        1
                                                                                );    
                                                                                
                                                                                // track income history
                                                                                insert_income_spent($bidderid, sprintf("%01.2f", $invres['paid']), 'debit');
                                                                                
                                                                                // re credit the provider online account
                                                                                $ilance->db->query("
                                                                                        UPDATE " . DB_PREFIX . "users
                                                                                        SET available_balance = available_balance + " . $invres['paid'] . ",
                                                                                        total_balance = total_balance + " . $invres['paid'] . "
                                                                                        WHERE user_id = '" . intval($bidderid) . "'
                                                                                ");
                                                                                
                                                                                // we should also update the bid and project table fvf fields back to 0.00
                                                                                $ilance->db->query("
                                                                                        UPDATE " . DB_PREFIX . "project_bids
                                                                                        SET fvf = '0.00'
                                                                                        WHERE project_id = '" . intval($pid) . "'
                                                                                            AND user_id = '" . intval($bidderid) . "'
                                                                                            AND fvf > 0
                                                                                ");
                                                                                
                                                                                $ilance->db->query("
                                                                                        UPDATE " . DB_PREFIX . "projects
                                                                                        SET fvf = '0.00',
                                                                                        isfvfpaid = '0',
                                                                                        fvfinvoiceid = '0'
                                                                                        WHERE project_id = '" . intval($pid) . "'
                                                                                ");
                                                                                
                                                                                // perhaps an email gets dispatched informing the provider
                                                                                // that the final value fee has been refunded
                                                                                
                                                                                // additionally, another email to admin advising the loss of FVF funds.
                                                                        }
                                                                }    
                                                        }
                                                }
                                        }
                                }
                                
                                return 1;    
                        }
                }
                
                return 0;
        }
        
        /**
        * Function for constructing a provider to buyer generated transaction to be paid through the marketplace.
        *
        * Basically, we have an invoice type of "p2b" where the provider can generate an unpaid invoice through the marketplace officially.
        * As a result, provider to buyer transactions can be viewed by the buyers on a per-project basis and multiple invoices can be generated
        * based on a single auction.  Allowing the service such as this, it is also possible to pre-configure "p2b fees" where you can set a rate
        * of 5% (for example) to let the provider generate an invoice to his/her buyer.  Upon the buyer's payment, the fee will be debitted from that
        * service provider.
        *
        * @param       string       amount to process
        * @param       integer      service provider id
        * @param       integer      service buyer id
        * @param       integer      associated project id
        * @param       string       associated comments
        * @param       string       transaction fee amount (if applicable)
        * @param       bool         defines if instant payment (from account balance) was selected by payer
        * @param       string       defines the providers selected payment status of this newly generated transaction (unpaid or paid) unpaid is used by default
        * @param       string       defines the providers preferred payment method when being viewed on the transaction page
        *
        * @return      nothing      The data inserted into datastore
        */
        function construct_p2b_transaction($amount = 0, $sellerid = 0, $buyerid = 0, $projectid = 0, $comments = '', $txnfee = 0, $instantpay = 0, $paymentstatus = 'unpaid', $paymentmethod = '')
        {
                global $ilance, $myapi, $phrase, $iltemplate, $page_title, $area_title, $ilconfig, $ilpage;
                
                if (!empty($instantpay) AND $instantpay > 0)
                {
                        $instantpay = 1;
                }
                else
                {
                        $instantpay = 0;
                }
        
                $area_title = $phrase['_generating_invoice_to_buyer'];
                $page_title = SITE_NAME . ' - ' . $phrase['_generating_invoice_to_buyer'];
        
                // invoice due dates
                $inv_due_date = gmdate('Y-m-d', mktime(0, 0, 0, gmdate('m', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])), gmdate('d', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst'])) + $ilconfig['invoicesystem_maximumpaymentdays'], gmdate('Y', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst']))));
                $inv_due_time = gmdate('H:i:s', time() + 3600 * ($ilconfig['globalserverlocale_officialtimezone'] + $ilconfig['globalserverlocale_officialtimezonedst']));
                $invoice_due_date = $inv_due_date . ' ' . $inv_due_time;
        
                // is a transaction / commission fee set?
                if (isset($txnfee) AND $txnfee > 0)
                {
                        $new_txn_transaction = construct_transaction_id();
            
                        // construct unpaid transaction fee invoice to the customer generating the invoice
                        $txn_description = $phrase['_transaction_fee_for_rfp'] . ' #' . intval($projectid) . ': ' . fetch_auction('project_title', intval($projectid)) . ': ' . $phrase['_invoice_generation_to'] . ' ' . fetch_user('username', intval($buyerid));
                        
                        $txnfee_invoice_id = $this->insert_transaction(
                                0,
                                intval($projectid),
                                0,
                                $sellerid,
                                0,
                                0,
                                0,
                                $txn_description,
                                sprintf("%01.2f", $txnfee),
                                '',
                                'unpaid',
                                'debit',
                                'account',
                                DATETIME24H,
                                $invoice_due_date,
                                '',
                                '',
                                0,
                                0,
                                1,
                                $new_txn_transaction
                        );
                        
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "invoices
                                SET isp2bfee = '1'
                                WHERE invoiceid = '" . intval($txnfee_invoice_id) . "'
                        ", 0, null, __FILE__, __LINE__);
            
                        $transaction_url = HTTPS_SERVER . $ilpage['invoicepayment'] . '?id=' . $txnfee_invoice_id;
            
                        // does customer wish to pay fees now?
                        if ($instantpay OR fetch_user('autopayment', $sellerid))
                        {
                                $sqlgetacc = $ilance->db->query("
                                        SELECT total_balance, available_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . intval($sellerid) . "'
                                ", 0, null, __FILE__, __LINE__);
                                $resgetacc = $ilance->db->fetch_array($sqlgetacc);
                
                                if ($resgetacc['available_balance'] >= $txnfee)
                                {
                                        $new_total = sprintf("%01.2f", $resgetacc['total_balance'] - $txnfee);
                                        $new_avail = sprintf("%01.2f", $resgetacc['available_balance'] - $txnfee);
                    
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET available_balance = '" . $new_avail . "',
                                                total_balance = '" . $new_total . "'
                                                WHERE user_id = '" . intval($sellerid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                    
                                        // track income spent
                                        insert_income_spent($sellerid, sprintf("%01.2f", $txnfee), 'credit');
                    
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "invoices
                                                SET paid = '" . sprintf("%01.2f", $txnfee) . "',
                                                status = 'paid',
                                                paiddate = '" . DATETIME24H . "'
                                                WHERE user_id = '" . intval($sellerid) . "'
                                                    AND invoiceid = '" . intval($txnfee_invoice_id) . "'
                                        ", 0, null, __FILE__, __LINE__);
                    
                                        $ilance->email = construct_dm_object('email', $ilance);
                
                                        $ilance->email->mail = fetch_user('email', $sellerid);
                                        $ilance->email->slng = fetch_user_slng($sellerid);
                                        
                                        $ilance->email->get('transaction_payment_complete');		
                                        $ilance->email->set(array(
                                                '{{provider}}' => fetch_user('username', $sellerid),
                                                '{{invoiceid}}' => $txnfee_invoice_id,
                                                '{{new_txn_transaction}}' => $new_txn_transaction,
                                                '{{invoice_amount}}' => $ilance->currency->format($txnfee),
                                                '{{description}}' => $txn_description,
                                                '{{datepaid}}' => DATETIME24H,
                                        ));
                                
                                        $ilance->email->send();
                                        
                                        $ilance->email->mail = SITE_EMAIL;
                                        $ilance->email->slng = fetch_site_slng();
                                        
                                        $ilance->email->get('transaction_payment_complete_admin');		
                                        $ilance->email->set(array(
                                                '{{provider}}' => fetch_user('username', $sellerid),
                                                '{{invoiceid}}' => $txnfee_invoice_id,
                                                '{{new_txn_transaction}}' => $new_txn_transaction,
                                                '{{invoice_amount}}' => $ilance->currency->format($txnfee),
                                                '{{description}}' => $txn_description,
                                                '{{datepaid}}' => DATETIME24H,
                                        ));
                                        
                                        $ilance->email->send();
                                }
                        }
                }
                else
                {
                        $new_txn_transaction = '--';
                        $transaction_url = '';
                        $txn_fee = '0.00';
                }
        
                $new_inv_transaction = construct_transaction_id();
                
                $p2b_invoice_id = $this->insert_transaction(
                        0,
                        intval($projectid),
                        0,
                        intval($buyerid),
                        intval($sellerid),
                        0,
                        0,
                        $phrase['_invoice_generated_for_rfp'] . ' #' . intval($projectid) . ': ' . fetch_auction('project_title', intval($projectid)),
                        sprintf("%01.2f", $amount),
                        '',
                        mb_strtolower($paymentstatus),
                        'p2b',
                        'account',
                        DATETIME24H,
                        DATEINVOICEDUE,
                        '',
                        $comments,
                        0,
                        0,
                        1,
                        $new_inv_transaction
                );
                
                if (isset($paymentstatus) AND $paymentstatus == 'paid')
                {
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "invoices
                                SET p2b_markedaspaid = '1'
                                WHERE invoiceid = '" . intval($p2b_invoice_id) . "'
                        ");
                }
                
                if (isset($paymentmethod) AND !empty($paymentmethod))
                {
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "invoices
                                SET p2b_paymethod = '" . $ilance->db->escape_string($paymentmethod) . "'
                                WHERE invoiceid = '" . intval($p2b_invoice_id) . "'
                        ");
                }
        
                $ilance->email = construct_dm_object('email', $ilance);
                
                $ilance->email->mail = fetch_user('email', intval($buyerid));
                $ilance->email->slng = fetch_user_slng(intval($buyerid));
                
                $ilance->email->get('provider_invoiced_buyer_buyer');		
                $ilance->email->set(array(
                        '{{buyer}}' => fetch_user('username', $buyerid),
                        '{{provider}}' => fetch_user('username', $sellerid),
                        '{{project_title}}' => fetch_auction('project_title', intval($projectid)),
                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                        '{{paymentstatus}}' => ucwords($paymentstatus),
                        '{{paymentmethod}}' => $paymentmethod,
                        '{{invoiceid}}' => $p2b_invoice_id,
                        '{{new_inv_transaction}}' => $new_inv_transaction,
                        '{{invoice_notes}}' => $comments,
                        '{{url}}' => HTTPS_SERVER . $ilpage['invoicepayment'] . '?id=' . $p2b_invoice_id,
                ));
                
                $ilance->email->send();
        
                $ilance->email->mail = fetch_user('email', $sellerid);
                $ilance->email->slng = fetch_user_slng($sellerid);
                
                $ilance->email->get('provider_invoiced_buyer');		
                $ilance->email->set(array(
                        '{{buyer}}' => fetch_user('username', $buyerid),
                        '{{provider}}' => fetch_user('username', $sellerid),
                        '{{project_title}}' => fetch_auction('project_title', intval($projectid)),
                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                        '{{paymentstatus}}' => ucwords($paymentstatus),
                        '{{paymentmethod}}' => $paymentmethod,
                        '{{invoiceid}}' => $p2b_invoice_id,
                        '{{new_inv_transaction}}' => $new_inv_transaction,
                        '{{new_txn_transaction}}' => $new_txn_transaction,
                        '{{invoice_notes}}' => $comments,
                        '{{url}}' => $transaction_url,
                        '{{transaction_fee}}' => $ilance->currency->format($txnfee)
                ));
                
                $ilance->email->send();
        
                $ilance->email->mail = SITE_EMAIL;
                $ilance->email->slng = fetch_site_slng();
                
                $ilance->email->get('provider_invoiced_buyer_admin');		
                $ilance->email->set(array(
                        '{{buyer}}' => fetch_user('username', $buyerid),
                        '{{provider}}' => fetch_user('username', $sellerid),
                        '{{project_title}}' => fetch_auction('project_title', intval($projectid)),
                        '{{invoice_amount}}' => $ilance->currency->format($amount),
                        '{{paymentstatus}}' => ucwords($paymentstatus),
                        '{{paymentmethod}}' => $paymentmethod,
                        '{{invoiceid}}' => $p2b_invoice_id,
                        '{{new_inv_transaction}}' => $new_inv_transaction,
                        '{{new_txn_transaction}}' => $new_txn_transaction,
                        '{{invoice_notes}}' => $comments,
                        '{{url}}' => $transaction_url,
                        '{{transaction_fee}}' => $ilance->currency->format($txnfee)
                ));
                
                $ilance->email->send();
        }
        
        /**
        * Function designed to send out commission and credential invoice reminder notices based on an admin defined email dispatch frequency
        * This function is called from cron.reminders.php
        */
        function send_unpaid_invoice_frequency_reminders()
        {
                global $ilance, $phrase, $ilconfig;
                
                $ilance->email = construct_dm_object('email', $ilance);
                
                $cronlog = '';
                $count = 0;
                $remindfrequency = $ilance->datetime->fetch_date_fromnow($ilconfig['invoicesystem_resendfrequency']);
                
                $expiry = $ilance->db->query("
                        SELECT user_id, invoiceid, projectid, buynowid, description, paiddate, invoicetype, duedate, createdate, amount, paid, totalamount, transactionid, isif, isfvf
                        FROM " . DB_PREFIX . "invoices
                        WHERE (invoicetype = 'commission' OR invoicetype = 'credential' OR invoicetype = 'debit')
                            AND isdeposit = '0'
                            AND iswithdraw = '0'
                            AND ispurchaseorder = '0'
                            AND (status = 'unpaid' OR status = 'scheduled')
                            AND amount > 0
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($expiry) > 0)
                {
                        while ($reminder = $ilance->db->fetch_array($expiry))
                        {
                                $user = $ilance->db->query("
                                        SELECT *
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . $reminder['user_id'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($user) > 0)
                                {
                                        $res_user = $ilance->db->fetch_array($user);
                                        
                                        // does user have sufficient funds within online account to cover invoice
                                        // before we send an unpaid reminder?
                                        if ($res_user['available_balance'] >= $reminder['totalamount'] AND fetch_user('autopayment', $res_user['user_id']))
                                        {
                                                // pay invoice using funds available via online account
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'paid',
                                                        paid = '" . $reminder['totalamount'] . "',
                                                        paymethod = 'account',
                                                        paiddate = '" . DATETIME24H . "',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_automated_debit_from_account_balance_via_billing_and_payments']) . "'
                                                        WHERE user_id = '" . $res_user['user_id'] . "'
                                                            AND invoiceid = '" . $reminder['invoiceid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                // adjust customers online account balances
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET total_balance = total_balance - $reminder[totalamount],
                                                        available_balance = available_balance - $reminder[totalamount]
                                                        WHERE user_id = '" . $res_user['user_id'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                
                                                // handle the insertion fee and/or final value fee automatic payment settlement and set the fields in the auction table to paid
                                                if ($reminder['isif'])
                                                {
                                                        // this is an insertion fee.. update auction listing table
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "projects
                                                                SET isifpaid = '1'
                                                                WHERE project_id = '" . $reminder['projectid'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                }
                                                if ($reminder['isfvf'])
                                                {
                                                        // this is a final value fee.. update auction listing table
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "projects
                                                                SET isfvfpaid = '1'
                                                                WHERE project_id = '" . $reminder['projectid'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                }
                                                
                                                // this could also be a payment from the "seller" for an unpaid "buy now" escrow fee OR unpaid "buy now" fvf.
                                                // let's check the buynow order table to see if we have a matching invoice to update as "ispaid"..
                                                // this scenerio would kick in once a buyer or seller deposits funds, this script runs and tries to pay the unpaid fees automatically..
                                                // at the same time we need to update the buy now order table so the presentation layer knows what's paid, what's not.
                                                $buynowcheck = $ilance->db->query("
                                                        SELECT escrowfeeinvoiceid, escrowfeebuyerinvoiceid, fvfinvoiceid, fvfbuyerinvoiceid
                                                        FROM " . DB_PREFIX . "buynow_orders
                                                        WHERE project_id = '" . $reminder['projectid'] . "'
                                                                AND orderid = '" . $reminder['buynowid'] . "'
                                                        LIMIT 1
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($buynowcheck) > 0)
                                                {
                                                        $resbuynow = $ilance->db->fetch_array($buynowcheck);
                                                        
                                                        if ($reminder['invoiceid'] == $resbuynow['escrowfeeinvoiceid'])
                                                        {
                                                                // invoice being paid is from seller paying a buy now escrow fee
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                                        SET isescrowfeepaid = '1'
                                                                        WHERE orderid = '" . $reminder['buynowid'] . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                        }
                                                        else if ($reminder['invoiceid'] == $resbuynow['escrowfeebuyerinvoiceid'])
                                                        {
                                                                // invoice being paid is from buyer paying a buy now escrow fee
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                                        SET isescrowfeebuyerpaid = '1'
                                                                        WHERE orderid = '" . $reminder['buynowid'] . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                        }
                                                        else if ($reminder['invoiceid'] == $resbuynow['fvfinvoiceid'])
                                                        {
                                                                // invoice being paid is from seller paying a buy now fvf
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                                        SET isfvfpaid = '1'
                                                                        WHERE orderid = '" . $reminder['buynowid'] . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                        }
                                                        else if ($reminder['invoiceid'] == $resbuynow['fvfbuyerinvoiceid'])
                                                        {
                                                                // invoice being paid is from buyer paying a buy now fvf
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "buynow_orders
                                                                        SET isfvfbuyerpaid = '1'
                                                                        WHERE orderid = '" . $reminder['buynowid'] . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                        }
                                                }
                                                
                                                // adjust members total amount paid for subscription plan
                                                insert_income_spent($res_user['user_id'], sprintf("%01.2f", $reminder['totalamount']), 'credit');
                                                
                                                // remove old invoice logs now that it's paid in full
                                                $ilance->db->query("
                                                        DELETE FROM " . DB_PREFIX . "invoicelog
                                                        WHERE invoiceid = '" . $reminder['invoiceid'] . "'
                                                ");
                                                
                                                // email user
                                                $ilance->email->mail = $res_user['email'];
                                                $ilance->email->slng = fetch_user_slng($res_user['user_id']);
                                                
                                                $ilance->email->get('transaction_payment_complete');		
                                                $ilance->email->set(array(
                                                        '{{provider}}' => $res_user['username'],
                                                        '{{invoiceid}}' => $reminder['invoiceid'],
                                                        '{{new_txn_transaction}}' => $reminder['transactionid'],
                                                        '{{invoice_amount}}' => $ilance->currency->format($reminder['totalamount']),
                                                        '{{description}}' => $reminder['description'],
                                                        '{{datepaid}}' => $reminder['paiddate'],
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                // email admin
                                                /*
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                $ilance->email->get('transaction_payment_complete_admin');		
                                                $ilance->email->set(array(
                                                        '{{provider}}' => $res_user['username'],
                                                        '{{invoiceid}}' => $reminder['invoiceid'],
                                                        '{{new_txn_transaction}}' => $reminder['transactionid'],
                                                        '{{invoice_amount}}' => $ilance->currency->format($reminder['totalamount']),
                                                        '{{description}}' => $reminder['description'],
                                                        '{{datepaid}}' => $reminder['paiddate'],
                                                ));
                                                
                                                $ilance->email->send();
                                                */
                                        }
                                        
                                        // insufficient funds
                                        else 
                                        {
                                                // unpaid invoice reminder for this customer
                                                $logs = $ilance->db->query("
                                                        SELECT invoicelogid, date_sent, date_remind
                                                        FROM " . DB_PREFIX . "invoicelog
                                                        WHERE user_id = '" . $reminder['user_id'] . "'
                                                            AND invoiceid = '" . $reminder['invoiceid'] . "'
                                                        ORDER BY invoicelogid DESC
                                                        LIMIT 1
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($logs) == 0)
                                                {
                                                        $ilance->db->query("
                                                                 INSERT INTO " . DB_PREFIX . "invoicelog
                                                                 (invoicelogid, user_id, invoiceid, invoicetype, date_sent, date_remind)
                                                                 VALUES(
                                                                 NULL,
                                                                 '" . $res_user['user_id'] . "',
                                                                 '" . $reminder['invoiceid'] . "',
                                                                 '" . $reminder['invoicetype'] . "',
                                                                 '" . DATETODAY . "',
                                                                 '" . $remindfrequency . "')
                                                        ", 0, null, __FILE__, __LINE__);
                                                        
                                                        // email user
                                                        $ilance->email->mail = $res_user['email'];
                                                        $ilance->email->slng = fetch_user_slng($res_user['user_id']);
                                                            
                                                        $ilance->email->get('cron_expired_subscription_invoice_reminder');		
                                                        $ilance->email->set(array(
                                                                '{{username}}' => $res_user['username'],
                                                                '{{firstname}}' => $res_user['first_name'],
                                                                '{{description}}' => $reminder['description'],
                                                                '{{transactionid}}' => $reminder['transactionid'],
                                                                '{{amount}}' => $ilance->currency->format($reminder['amount']),
                                                                '{{total}}' => $ilance->currency->format($reminder['totalamount']),
                                                                '{{paid}}' => $ilance->currency->format($reminder['paid']),
                                                                '{{duedate}}' => $reminder['duedate'],
                                                                '{{invoiceid}}' => $reminder['invoiceid'],
                                                                '{{reminddate}}' => $remindfrequency,
                                                        ));
                                                        
                                                        if ($ilconfig['invoicesystem_unpaidreminders'])
                                                        {
                                                                $ilance->email->send();
                                                                $count++;
                                                        }                                                        
                                                }
                                                else if ($ilance->db->num_rows($logs) > 0)
                                                {
                                                        // it appears we have a log for this invoice id ..
                                                        $reslogs = $ilance->db->fetch_array($logs);
                                                        
                                                        // time to send an update to this user for this invoice
                                                        // make sure we didn't already send one today
                                                        if ($reslogs['date_remind'] == DATETODAY AND $reslogs['date_sent'] == DATETODAY)
                                                        {
                                                                // we've sent a reminder to this user for this invoice today already.. do nothing until next reminder frequency        
                                                        }
                                                        else if ($reslogs['date_remind'] == DATETODAY AND $reslogs['date_sent'] != DATETODAY)
                                                        {
                                                                // time to send a new frequency reminder.. update table with new email sent date as today
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "invoicelog
                                                                        SET date_sent = '" . DATETODAY . "',
                                                                        date_remind = '" . $remindfrequency . "'
                                                                        WHERE invoiceid = '" . $reminder['invoiceid'] . "'
                                                                                AND user_id = '" . $reminder['user_id'] . "'
                                                                ");
                                                                
                                                                // email user
                                                                $ilance->email->mail = $res_user['email'];
                                                                $ilance->email->slng = fetch_user_slng($res_user['user_id']);
                                                                    
                                                                $ilance->email->get('cron_expired_subscription_invoice_reminder');		
                                                                $ilance->email->set(array(
                                                                        '{{username}}' => $res_user['username'],
                                                                        '{{firstname}}' => $res_user['first_name'],
                                                                        '{{description}}' => $reminder['description'],
                                                                        '{{transactionid}}' => $reminder['transactionid'],
                                                                        '{{amount}}' => $ilance->currency->format($reminder['amount']),
                                                                        '{{total}}' => $ilance->currency->format($reminder['totalamount']),
                                                                        '{{paid}}' => $ilance->currency->format($reminder['paid']),
                                                                        '{{duedate}}' => $reminder['duedate'],
                                                                        '{{invoiceid}}' => $reminder['invoiceid'],
                                                                        '{{reminddate}}' => $remindfrequency,
                                                                ));
                                                                
                                                                if ($ilconfig['invoicesystem_unpaidreminders'])
                                                                {
                                                                        $ilance->email->send();
                                                                        $count++;
                                                                }
                                                        }
                                                }
                                        }
                                }
                        }
                }
                
                return $cronlog;
        }
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.accounting.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>