<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

/**
* Bid class to perform the majority of bidding functions within ILance
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class bid
{
        /**
        * Function to determine the actual "win type" for a listing.  For example, if we're viewing feedback for this entry, and the user won a product auction,
        * show the win type as "bid", if for example the product listing had buy now available, and the user won via buy now the win type would show "buynow".
        * Additionally, if the winning type was a service auction, the wintype would be "awarded".
        * 
        * This function accepts both service and product logic.
        *
        * @param      integer      project id
        * @param      string       project state (service or product)
        * @param      string       auction type (regular or fixed)
        * @param      integer      seller id
        * @param      integer      buyer id
        *
        * @return     string       win type (highestbid, buynow, awarded or unknown)
        */
        function fetch_auction_win_type($project_id = 0, $project_state = '', $auctiontype = '', $project_details = '', $seller_id = 0, $buyer_id = 0)
        {
                global $ilance, $ilconfig, $phrase;
                
                $html = 'unknown';

                if ($project_state == 'service')
                {
                        $html = 'awarded';        
                }
                else if ($project_state == 'product')
                {
                        if ($auctiontype == 'regular')
                        {
                                $sql = $ilance->db->query("
                                        SELECT orderid
                                        FROM " . DB_PREFIX . "buynow_orders
                                        WHERE project_id = '" . intval($project_id) . "'
                                        LIMIT 1
                                ");
                                if ($ilance->db->num_rows($sql) > 0)
                                {
                                        $html = 'buynow';
                                }
                                else
                                {
                                        if ($project_details == 'unique')
                                        {
                                                $html = 'lowestunique';       
                                        }
                                        else
                                        {
                                                $html = 'highestbid';
                                        }
                                }
                        }
                        else if ($auctiontype == 'fixed')
                        {
                                $html = 'buynow';        
                        }
                }
                                
                return $html;
        }
        
        /**
        * Function for returning the raw winning amount from an auction that has ended with a winner.  This function accepts both service and product logic
        * and will calculate service, product and buy now winnings.  Even if shipping is used, it will not be added within this function.
        *
        * @param      integer      project id
        * @param      integer      seller id
        * @param      integer      buyer id
        *
        * @return     string       raw auction winning cost
        */
        function fetch_auction_win_amount($project_id = 0, $seller_id = 0, $buyer_id = 0)
        {
                global $ilance, $ilconfig, $show;
                
                $totalamount = 0;
                
                $project_state = fetch_auction('project_state', $project_id);
                $filtered_auctiontype = fetch_auction('filtered_auctiontype', $project_id);
                
                if ($project_state == 'product' AND ($filtered_auctiontype == 'fixed' OR $filtered_auctiontype == 'regular'))
                {
                        // this item may be a buynow or regular fixed priced auction
                        // do we have any buynow orders for this user? also, we'll select the most recent order in case
                        // this buyer has already purchased 1 or more qty of this exact same item in the past to ensure
                        // we don't grab the value from the original buy now order purchase
                        $sql = $ilance->db->query("
                                SELECT qty
                                FROM " . DB_PREFIX . "buynow_orders
                                WHERE project_id = '" . intval($project_id) . "'
                                    AND (owner_id = '" . intval($seller_id) . "' AND buyer_id = '" . intval($buyer_id) . "' OR owner_id = '" . intval($buyer_id) . "' AND buyer_id = '" . intval($seller_id) . "')
                                ORDER BY orderid DESC
                                LIMIT 1
                        ");
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                // we do so lets find out the actual cost of this item for the qty ordered .. and do not include shipping!
                                $res = $ilance->db->fetch_array($sql);
                                
                                if ($res['qty'] <= 0)
                                {
                                        $res['qty'] = 1;
                                }
                                
                                $buynowprice = fetch_auction('buynow_price', $project_id);
                                $totalamount = sprintf("%01.2f", ($buynowprice * $res['qty']));
                        }
                        else
                        {
                                $totalamount = $this->fetch_awarded_bid_amount($project_id);
                        }
                }
                else
                {
                        $totalamount = $this->fetch_awarded_bid_amount($project_id);
                }
                
                return $totalamount;
        }
        
        /**
        * Function for returning awarded bid amount for a particular auction event.
        * This function now supports lowest unique bid awarded amounts as well.
        *
        * @param      integer      project id
        *
        * @return     string       awarded bid amount
        */
        function fetch_awarded_bid_amount($projectid = 0)
        {
                global $ilance, $myapi;
                
                $project_details = fetch_auction('project_details', $projectid);
                $amount = 0;
                
                if ($project_details == 'unique')
                {
                        $sql = $ilance->db->query("
                                SELECT uniquebid
                                FROM " . DB_PREFIX . "projects_uniquebids
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND status = 'lowestunique'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $amount = sprintf("%.02f", $res['uniquebid']);
                        }       
                }
                else
                {
                        $sql = $ilance->db->query("
                                SELECT bidamount, bidamounttype, estimate_days
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND bidstatus = 'awarded'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                if ($res['bidamounttype'] == 'entire' OR $res['bidamounttype'] == 'lot' OR $res['bidamounttype'] == 'weight')
                                {
                                        $amount = sprintf("%.02f", $res['bidamount']);
                                }
                                else
                                {
                                        $amount = sprintf("%.02f", ($res['bidamount'] * $res['estimate_days']));
                                }
                        }        
                }
                
                return $amount;
        }
    
        /**
        * Function for returning the current bid price for a particular auction event.
        *
        * @param      integer      project id
        * @param      integer      no format argument to determine if the amount should be formatted or not
        *
        * @return     string       amount of awarded bid
        */
        function fetch_current_bid($projectid = 0, $noformat = 0)
        {
                global $ilance, $myapi, $ilconfig;
                
                $sql = $ilance->db->query("
                        SELECT startprice, cid
                        FROM " . DB_PREFIX . "projects
                        WHERE project_id = '" . intval($projectid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                $res = $ilance->db->fetch_array($sql);
                if ($res['startprice'] > 0)
                {
                        // do we have more than 1 bid?
                        $bidders = $ilance->db->query("
                                SELECT bidamount
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($projectid) . "'
                                ORDER BY bidamount DESC
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($bidders) > 0)
                        {
                                while ($resbidders = $ilance->db->fetch_array($bidders))
                                {
                                        $bidamount = $resbidders['bidamount'];
                                }
                                if (isset($noformat) AND $noformat)
                                {
                                        return $bidamount;
                                }
                                return $ilance->currency->format($bidamount);
                        }
                        else
                        {
                                if (isset($noformat) AND $noformat)
                                {
                                        return $res['startprice'];
                                }
                                return $ilance->currency->format($res['startprice']);
                        }
                }
                else
                {
                        return 0;
                }
        }
    
        /**
        * Function for determining if a particular auction event has any bids placed.
        *
        * @param      integer      project id
        *
        * @return     bool         true or false
        */
        function has_bids($projectid = 0)
        {
                global $ilance, $myapi, $ilconfig;
                $bidders = $ilance->db->query("
                        SELECT COUNT(*) AS bids
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($bidders) > 0)
                {
                        $res = $ilance->db->fetch_array($bidders);
                        return (int)$res['bids'];
                }
                else 
                {
                        return 0;
                }
        }
        
        /**
        * Function for determining if a particular auction event has any bid filters and if so this
        * function will print out the appropriate error response based on the credentials of the logged in
        * member.  Additionally, this function has been updated to also detect the new profile auction filter
        * option where admin can define say "weight" and the auction poster defines a "weight" range for the
        * auction (from: 100 lbs to: 200 lbs) type format (question is ultimately answered by the bidder from
        * his/her profile menu)
        *
        * @param       integer      project id
        *
        * @return     string       HTML representation of a particular bid permission error (if applicable)
        */
        function bid_filter_checkup($projectid = 0)
        {
                global $ilance, $myapi, $ilpage, $phrase, $ilconfig;
                
                $sql = $ilance->db->query("
                        SELECT *, UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, UNIX_TIMESTAMP(date_starts) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS starttime
                        FROM " . DB_PREFIX . "projects
                        WHERE project_id = '" . intval($projectid) . "'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                $row = $ilance->db->fetch_array($sql);
                if ($row['mytime'] < 0)
                {
                        $row['mytime'] = - $row['mytime'];
                        $sign = '-';
                }
                else
                {
                        $sign = '+';
                }
                if ($sign == '-')
                {
                        $area_title = $phrase['_this_rfp_has_expired_bidding_is_over'];
                        $page_title = SITE_NAME . ' - ' . $phrase['_this_rfp_has_expired_bidding_is_over'];
                        
                        $filtered_message = $phrase['_this_rfp_has_expired_bidding_is_over'];
                        $return_url = $ilpage['main'];
                        
                        print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                        exit();
                }
                if ($row['filter_rating'])
                {
                        $ilance->feedback = construct_object('api.feedback');
                        
                        $memberinfo = array();
                        $memberinfo = $ilance->feedback->datastore($_SESSION['ilancedata']['user']['userid']);
                        if ($memberinfo['rating'] < intval($row['filtered_rating']))
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_requires_rating_of_at_least'] . " " . $row['filtered_rating'] . " " . $phrase['_stars'];
                                $return_url = $ilpage['main'];
                                
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
                if ($row['filter_country'])
                {
                        $cfiltered = mb_strtolower(stripslashes($row['filtered_country']));
                        $countryname = mb_strtolower(stripslashes(print_user_country($_SESSION['ilancedata']['user']['userid'], $_SESSION['ilancedata']['user']['slng'])));
                        if (!empty($_SESSION['ilancedata']['user']['countryid']) AND $countryname != $cfiltered)
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_country_must_be_located_in'] . " " . stripslashes($row['filtered_country']);
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $ilpage['main'], $phrase['_main_menu']);
                                exit();
                        }
                }
                if ($row['filter_state'])
                {
                        $sfiltered = mb_strtolower(stripslashes($row['filtered_state']));
                        $cstate = mb_strtolower(stripslashes($_SESSION['ilancedata']['user']['state']));
                        if ($cstate != $sfiltered)
                        {                
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_state_province_must_be_located_in'] . " " . ucfirst($sfiltered);
                                $return_url = $ilpage['main'];
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
                if ($row['filter_city'])
                {
                        $cityfiltered = mb_strtolower(stripslashes($row['filtered_city']));
                        $ccity = mb_strtolower(stripslashes($_SESSION['ilancedata']['user']['city']));
                        if ($ccity != $cityfiltered AND !empty($cityfiltered))
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_city_must_be_located_in'] . " " . ucfirst($cityfiltered);
                                $return_url = $ilpage['main'];
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
                if ($row['filter_zip'])
                {
                        $zipfiltered = mb_strtolower($row['filtered_zip']);
                        $czip = mb_strtolower($_SESSION['ilancedata']['user']['postalzip']);
                        if ($czip != $zipfiltered)
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_zip_postal_code_must_be_located_in'] . " " . mb_strtoupper($zipfiltered);
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $ilpage['main'], $phrase['_main_menu']);
                                exit();
                        }
                }
        }
        
        /**
        * Function for inserting a new service bid proposal on a service auction event.
        * If a service provider has already placed a bid on this particular project
        * this function will update that previous bid to the new amount specified.
        * Additionally, if the previous bid was declined this new bid will be inserted
        * vs being updated.
        *
        * @param       integer      bidder id
        * @param       string       bid proposal message
        * @param       integer      low bid notify filter (optional)
        * @param       integer      last hour notify filter (optional)
        * @param       integer      project id
        * @param       integer      project owner id
        * @param       string       bid amount
        * @param       integer      estimated number of days
        * @param       string       bid state status
        * @param       string       bid amount type
        * @param       string       custom argument
        * @param       array        bid field answers
        * @param       boolean      show error messages (disable if you want to call this function via API to hide html error messages; this will then only return true or false) - default true
        */
        function insert_service_bid($bidderid = 0, $proposal = '', $lowbidnotify = 0, $lasthournotify = 0, $subscribed = 0, $id = 0, $project_user_id = 0, $bidamount = 0, $estimate_days = 0, $bidstate = '', $bidamounttype = '', $bidcustom = '', $bidfieldanswers = '', $showerrormessages = true)
        {
                global $ilance, $myapi, $ilpage, $phrase, $ilconfig, $area_title, $page_title;

                $ilance->subscription = construct_object('api.subscription');
                $ilance->watchlist = construct_object('api.watchlist');
                $ilance->email = construct_dm_object('email', $ilance);
                $ilance->bid_fields = construct_object('api.bid_fields');

                if ($ilance->subscription->check_access(intval($bidderid), 'servicebid') == 'no')
                {
                        $area_title = $phrase['_buying_menu_denied_upgrade_subscription'];
                        $page_title = SITE_NAME . ' - ' . $phrase['_buying_menu_denied_upgrade_subscription'];
                        
                        if ($showerrormessages)
                        {
                                print_notice($phrase['_access_denied'], $phrase['_your_current_subscription_level_does_not_permit_you_to_use_this_marketplace_resource']." <a href='".$ilpage['subscription']."'><strong>".$phrase['_click_here']."</strong></a>", $ilpage['subscription'], ucwords($phrase['_click_here']), fetch_permission_name('servicebid'));
                                exit();
                        }
                        else
                        {
                                return false;
                        }        
                }
                
                $area_title = $phrase['_submitting_bid_proposal'];
                $page_title = SITE_NAME . ' ' . $phrase['_submitting_bid_proposal'];
                
                $sqlexpiry = $ilance->db->query("
                        SELECT UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, status
                        FROM " . DB_PREFIX . "projects 
                        WHERE project_id = '" . intval($id) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sqlexpiry) > 0)
                {
                        $resexpiry = $ilance->db->fetch_array($sqlexpiry);
                        
                        if ($resexpiry['mytime'] < 0 OR $resexpiry['status'] != 'open')
                        {
                                $area_title = $phrase['_this_rfp_has_expired_bidding_is_over'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_this_rfp_has_expired_bidding_is_over'];
                                
                                if ($showerrormessages)
                                {
                                        print_notice($area_title, $phrase['_this_rfp_has_expired_bidding_is_over'], $ilpage['main'], $phrase['_main_menu']);
                                        exit();
                                }
                                else
                                {
                                        return false;
                                }
                        }
                }
                unset($sqlexpiry, $resexpiry);
                
                // #### add project to watchlist if applicable #########
                $ilance->watchlist->insert_item(intval($bidderid), $id, 'auction', 'n/a', $lowbidnotify, 0, $lasthournotify, $subscribed);
                
                if (empty($bidcustom))
                {
                        $bidcustom = '';
                }
    
                if (empty($proposal))
                {
                        $proposal = '';	
                }
    
                // determine if listing is realtime auction
                $project_details = fetch_auction('project_details', intval($id));
    
                // did we already place a bid on this project?
                $sql = $ilance->db->query("
                        SELECT bid_id, bidstatus, bidstate
                        FROM " . DB_PREFIX . "project_bids
                        WHERE user_id = '" . intval($bidderid) . "'
                            AND project_id = '" . intval($id) . "'
                        ORDER BY bid_id DESC
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        if ($res['bidstatus'] == 'declined' OR $res['bidstate'] == 'retracted')
                        {
                                // insert bid proposal
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "project_bids
                                        (bid_id, user_id, project_id, project_user_id, proposal, bidamount, estimate_days, date_added, bidstatus, bidstate, bidamounttype, bidcustom, state)
                                        VALUES(
                                        NULL,
                                        '" . intval($bidderid) . "',
                                        '" . intval($id) . "',
                                        '" . intval($project_user_id) . "',
                                        '" . $ilance->db->escape_string($proposal) . "',
                                        '" . sprintf("%01.2f", $bidamount) . "',
                                        '" . intval($estimate_days) . "',
                                        '" . DATETIME24H . "',
                                        'placed',
                                        '" . $ilance->db->escape_string($bidstate) . "',
                                        '" . $ilance->db->escape_string($bidamounttype) . "',
                                        '" . $ilance->db->escape_string($bidcustom) . "',
                                        'service')
                                ", 0, null, __FILE__, __LINE__);
                                $this_bid_id = $ilance->db->insert_id();
                                
                                // insert realtime bid proposal
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "project_realtimebids
                                        (id, bid_id, user_id, project_id, project_user_id, proposal, bidamount, estimate_days, date_added, bidstatus, bidstate, bidamounttype, bidcustom, state)
                                        VALUES(
                                        NULL,
                                        '" . intval($this_bid_id) . "',
                                        '" . intval($bidderid) . "',
                                        '" . intval($id) . "',
                                        '" . intval($project_user_id) . "',
                                        '" . $ilance->db->escape_string($proposal) . "',
                                        '" . sprintf("%01.2f", $bidamount) . "',
                                        '" . intval($estimate_days) . "',
                                        '" . DATETIME24H . "',
                                        'placed',
                                        '" . $ilance->db->escape_string($bidstate) . "',
                                        '" . $ilance->db->escape_string($bidamounttype) . "',
                                        '" . $ilance->db->escape_string($bidcustom) . "',
                                        'service')
                                ", 0, null, __FILE__, __LINE__);    
                                
                                // update bid count for auction
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects
                                        SET bids = bids + 1
                                        WHERE project_id = '" . intval($id) . "'
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                                
                                // will increase bidstoday and bidsthismonth for the user placing a bid
                                $this->set_bid_counters(intval($bidderid), 'increase');
                        }
                        else
                        {
                                // update/revise existing bid amount placed
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "project_bids
                                        SET proposal = '" . $ilance->db->escape_string($proposal) . "',
                                        bidamount = '" . sprintf("%01.2f", $bidamount) . "',
                                        estimate_days = '" . intval($estimate_days) . "',
                                        bidamounttype = '" . $ilance->db->escape_string($bidamounttype) . "',
                                        bidcustom = '" . $ilance->db->escape_string($bidcustom) . "'
                                        WHERE bid_id = '" . $res['bid_id'] . "'
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                                
                                // make sure our realtime applet has some live bid history info
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "project_realtimebids
                                        (id, bid_id, user_id, project_id, project_user_id, proposal, bidamount, estimate_days, date_added, bidstatus, bidstate, bidamounttype, bidcustom, state)
                                        VALUES(
                                        NULL,
                                        '" . $res['bid_id'] . "',
                                        '" . intval($bidderid) . "',
                                        '" . intval($id) . "',
                                        '" . intval($project_user_id) . "',
                                        '" . $ilance->db->escape_string($proposal) . "',
                                        '" . sprintf("%01.2f", $bidamount) . "',
                                        '" . intval($estimate_days) . "',
                                        '" . DATETIME24H . "',
                                        'placed',
                                        '".$ilance->db->escape_string($bidstate) . "',
                                        '".$ilance->db->escape_string($bidamounttype) . "',
                                        '".$ilance->db->escape_string($bidcustom) . "',
                                        'service')
                                ", 0, null, __FILE__, __LINE__);    
                                
                                $this_bid_id = $res['bid_id'];
                        }
                }
                else
                {               
                        // insert bid proposal
                        $ilance->db->query("
                                INSERT INTO " . DB_PREFIX . "project_bids
                                (bid_id, user_id, project_id, project_user_id, proposal, bidamount, estimate_days, date_added,
                                bidstatus, bidstate, bidamounttype, bidcustom, state)
                                VALUES(
                                NULL,
                                '" . intval($bidderid) . "',
                                '" . intval($id) . "',
                                '" . intval($project_user_id) . "',
                                '" . $ilance->db->escape_string($proposal) . "',
                                '" . sprintf("%01.2f", $bidamount) . "',
                                '" . intval($estimate_days) . "',
                                '" . DATETIME24H . "',
                                'placed',
                                '" . $ilance->db->escape_string($bidstate) . "',
                                '" . $ilance->db->escape_string($bidamounttype) . "',
                                '" . $ilance->db->escape_string($bidcustom) . "',
                                'service')
                        ", 0, null, __FILE__, __LINE__);
                        $this_bid_id = $ilance->db->insert_id();
                        
                        // make sure our realtime applet has some live bid history info
                        $ilance->db->query("
                                INSERT INTO " . DB_PREFIX . "project_realtimebids
                                (id, bid_id, user_id, project_id, project_user_id, proposal, bidamount, estimate_days, date_added, bidstatus, bidstate, bidamounttype, bidcustom, state)
                                VALUES(
                                NULL,
                                '" . $this_bid_id . "',
                                '" . intval($bidderid) . "',
                                '" . intval($id) . "',
                                '" . intval($project_user_id) . "',
                                '" . $ilance->db->escape_string($proposal) . "',
                                '" . sprintf("%01.2f", $bidamount) . "',
                                '" . intval($estimate_days) . "',
                                '" . DATETIME24H . "',
                                'placed',
                                '" . $ilance->db->escape_string($bidstate) . "',
                                '" . $ilance->db->escape_string($bidamounttype) . "',
                                '" . $ilance->db->escape_string($bidcustom) . "',
                                'service')
                        ", 0, null, __FILE__, __LINE__);    
                        
                        // update bid count for auction
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "projects
                                SET bids = bids + 1
                                WHERE project_id = '" . intval($id) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        
                        // will increase bidstoday and bidsthismonth
                        $this->set_bid_counters(intval($bidderid), 'increase');
                        
                        // was this service provider invited?
                        $sql_invites = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "project_invitations
                                WHERE project_id = '" . intval($id) . "'
                                    AND seller_user_id = '" . intval($bidderid) . "'
                                    AND bid_placed = 'no'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql_invites) > 0)
                        {
                                // update invitations with bid placed for invited service provider
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "project_invitations
                                        SET bid_placed = 'yes',
                                        date_of_bid = '" . DATETIME24H . "'
                                        WHERE seller_user_id = '" . intval($bidderid) . "'
                                            AND project_id = '" . intval($id) . "'
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                
                                $url = HTTPS_SERVER . $ilpage['rfp'] . '?id=' . intval($id);
                                
                                // email user
                                $ilance->email->mail = fetch_user('email', $project_user_id);
                                $ilance->email->slng = fetch_user_slng($project_user_id);
                                
                                $ilance->email->get('invited_bid_placed_buyer');		
                                $ilance->email->set(array(
                                        '{{buyer}}' => fetch_user('username', $project_user_id),
                                        '{{vendor}}' => fetch_user('username', $bidderid),
                                        '{{rfp_title}}' => fetch_auction('project_title', intval($id)),
                                        '{{project_id}}' => intval($id),
                                        '{{url}}' => $url,
                                ));
                                
                                $ilance->email->send();
                        }
                }
    
                // #### capture custom bid fields ######################
                $ilance->bid_fields->process_custom_bid_fields($bidfieldanswers, intval($id), $this_bid_id);
                
                // #### lower bid notification bulk email sender #######
                $ilance->watchlist->send_notification(intval($bidderid), 'lowbidnotify', intval($id), $bidamount);
    
                $ilance->email->mail = fetch_user('email', $project_user_id);
                $ilance->email->slng = fetch_user_slng($project_user_id);
                
                $ilance->email->get('bid_notification_alert');		
                $ilance->email->set(array(
                        '{{provider}}' => fetch_user('username', $bidderid),
                        '{{price}}' => $ilance->currency->format($bidamount),
                        '{{p_id}}' => intval($id),
                ));
                
                $ilance->email->send();
                
                if ($showerrormessages)
                {
                        // todo: detect seo
                        refresh($ilpage['rfp'] . '?id=' . intval($id));
                        exit();
                }
                else
                {
                        return true;
                }
        }
        
        /**
        * Function to fetch the highest bidder "user_id" for a reverse or forward auction event.
        *
        * @param       integer      project id
        * 
        * @return      integer      bidder id
        */
        function fetch_highest_bidder($projectid = 0)
        {
                global $ilance, $myapi, $ilconfig;

                $project_state = fetch_auction('project_state', $projectid);
                
                if ($project_state == 'product')
                {
                        $proxycid = fetch_auction('cid', $projectid);
                        if ($ilconfig['productbid_enableproxybid'] AND $ilance->categories->useproxybid($_SESSION['ilancedata']['user']['slng'], 'product', $proxycid))
                        {
                                $highbid = $ilance->db->query("
                                        SELECT b.user_id
                                        FROM " . DB_PREFIX . "project_bids AS b,
                                        " . DB_PREFIX . "proxybid AS p
                                        WHERE b.project_id = '" . intval($projectid) . "'
                                            AND b.project_id = p.project_id
                                            AND b.user_id = p.user_id
                                        ORDER BY b.bidamount DESC, p.date_added ASC
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($highbid) > 0)
                                {
                                        $res = $ilance->db->fetch_array($highbid);
                                        return $res['user_id'];
                                }
                                else
                                {
                                        $highbid = $ilance->db->query("
                                                SELECT user_id
                                                FROM " . DB_PREFIX . "project_bids
                                                WHERE project_id = '" . intval($projectid) . "'
                                                ORDER BY bidamount DESC, date_added ASC
                                                LIMIT 1
                                        ", 0, null, __FILE__, __LINE__);
                                        if ($ilance->db->num_rows($highbid) > 0)
                                        {
                                                $res = $ilance->db->fetch_array($highbid);
                                                return $res['user_id'];
                                        }
                                        else
                                        {
                                                return 0;
                                        }
                                }
                        }
                        else
                        {
                                $highbid = $ilance->db->query("
                                        SELECT user_id
                                        FROM " . DB_PREFIX . "project_bids
                                        WHERE project_id = '" . intval($projectid) . "'
                                        ORDER BY bidamount DESC, date_added ASC
                                        LIMIT 1
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($highbid) > 0)
                                {
                                        $res = $ilance->db->fetch_array($highbid);
                                        return $res['user_id'];
                                }
                                else
                                {
                                        return 0;
                                }
                        }        
                }
                else
                {
                        $highbid = $ilance->db->query("
                                SELECT user_id
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($projectid) . "'
                                        AND bidstatus != 'declined'
                                        AND bidstate != 'retracted'
                                ORDER BY bidamount DESC, date_added ASC
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($highbid) > 0)
                        {
                                $res = $ilance->db->fetch_array($highbid);
                                return $res['user_id'];
                        }
                        else
                        {
                                return 0;
                        }                        
                }

        }
        
        /**
        * Function to fetch the highest bid amount for a particular auction event.  This function does not care about the highest proxy bid, only current bids within the bids table.
        *
        * @param       integer      project id
        * 
        * @return      string       bid amount
        */
        function fetch_highest_bid($projectid = 0)
        {
                global $ilance, $myapi;
                
                $highestbid = 0;
                $sql = $ilance->db->query("
                        SELECT bidamount
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                        ORDER BY bidamount DESC, date_added ASC
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        $highestbid = sprintf("%.02f", $res['bidamount']);
                }
                
                DEBUG("fetch_highest_bid() return: $highestbid", 'NOTICE');
                
                return $highestbid;
        }
        
        /**
        * Function to fetch the minimum bid amount to place for a particular auction event.  This function take into consideration any increments to apply within a specific category.
        *
        * @param       string       highest bid amount
        * @param       integer      category id
        * 
        * @return      string       bid amount
        */
        function fetch_minimum_bid($highestbid = 0, $cid = 0)
        {
                global $ilance, $myapi, $ilconfig;
                
                $increment = 0;
                $cid = intval($cid);
                $minbidamount = sprintf("%.02f", $highestbid);
                
                $incrementgroup = $ilance->db->fetch_field(DB_PREFIX . "categories", "cid = '" . $cid . "'", "incrementgroup");
                
                $sql = $ilance->db->query("
                        SELECT amount
                        FROM " . DB_PREFIX . "increments
                        WHERE ((increment_from <= $highestbid AND increment_to >= $highestbid) OR (increment_from < $highestbid AND increment_to < $highestbid))
                            AND groupname = '" . $ilance->db->escape_string($incrementgroup) . "'
                        ORDER BY amount DESC
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        $minbidamount = sprintf("%.02f", $highestbid + $res['amount']);
                        
                        DEBUG("fetch_minimum_bid() category increment amount: $res[amount]", 'NOTICE');
                }
                
                DEBUG("fetch_minimum_bid() return: $minbidamount", 'NOTICE');
                
                return $minbidamount;
        }
        
        /**
        * Function for inserting a new product bid on a product auction event.  Additionally, this function will
        * detect if another bidder just placed a bid before this bid is inserted.  This will allow the system to
        * generate an error message back to the current bidder informing them to bid higher due to another bidder
        * placing a bid first.
        *
        * @param       integer      higher bid notify filter (optional)
        * @param       integer      last hour notify filter (optional)
        * @param       integer      project id
        * @param       integer      project owner id
        * @param       string       bid amount
        * @param       integer      qty
        * @param       integer      bidder id
        * @param       bool         is proxy bid?
        * @param       string       minimum bid amount
        * @param       string       reserve price amount
        * @param       string       custom argument for live bidding (future)
        * @param       boolean      show error messages (disable if you want to call this function via API to hide html error messages; this will then only return true or false) - default true
        */
        function insert_product_bid($highbidnotify = 0, $lasthournotify = 0, $subscribed = 0, $id = 0, $project_user_id = 0, $bidamount = 0, $qty = 1, $bidderid = 0, $isproxy, $minimumbid, $reserveprice, $showerrormessages = true)
        {
                global $ilance, $myapi, $ilpage, $phrase, $ilconfig;
                   
                $ilance->watchlist = construct_object('api.watchlist');
                $ilance->subscription = construct_object('api.subscription');
                $ilance->bid_proxy = construct_object('api.bid_proxy');
                
                if ($ilance->subscription->check_access($bidderid, 'productbid') == 'no')
                {
                        $area_title = $phrase['_buying_menu_denied_upgrade_subscription'];
                        $page_title = SITE_NAME . ' - ' . $phrase['_buying_menu_denied_upgrade_subscription'];
                        
                        if ($showerrormessages)
                        {
                                print_notice($phrase['_access_denied'], $phrase['_your_current_subscription_level_does_not_permit_you_to_use_this_marketplace_resource']." <a href='".$ilpage['subscription']."'><strong>".$phrase['_click_here']."</strong></a>", $ilpage['subscription'], ucwords($phrase['_click_here']), fetch_permission_name('productbid'));
                                exit();
                        }
                        else
                        {
                                return false;
                        }       
                }
                
                $area_title = $phrase['_submitting_bid_proposal'];
                $page_title = SITE_NAME . ' ' . $phrase['_submitting_bid_proposal'];
                
                $sqlexpiry = $ilance->db->query("
                        SELECT UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, status
                        FROM " . DB_PREFIX . "projects 
                        WHERE project_id = '" . intval($id) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sqlexpiry) > 0)
                {
                        $resexpiry = $ilance->db->fetch_array($sqlexpiry);
                        if ($resexpiry['mytime'] < 0 OR $resexpiry['status'] != 'open')
                        {
                                $area_title = $phrase['_this_rfp_has_expired_bidding_is_over'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_this_rfp_has_expired_bidding_is_over'];
                                
                                if ($showerrormessages)
                                {
                                        print_notice($area_title, $phrase['_this_rfp_has_expired_bidding_is_over'], $ilpage['main'], $phrase['_main_menu']);
                                        exit();
                                }
                                else
                                {
                                        return false;
                                }
                        }
                }
                unset($sqlexpiry, $resexpiry);
                
                $ilance->watchlist->insert_item(intval($bidderid), $id, 'auction', 'n/a', 0, $highbidnotify, $lasthournotify, $subscribed);
                
                // is proxy bid (and does category permit proxy bidding usage)?
                $proxycid = fetch_auction('cid', $id);
                if ($ilconfig['productbid_enableproxybid'] AND isset($isproxy) AND $isproxy AND $ilance->categories->useproxybid($_SESSION['ilancedata']['user']['slng'], 'product', $proxycid))
                {
                        DEBUG("------------------------------------------", 'NOTICE');
                        DEBUG("Proxy Bid Enabled for Category ID $proxycid", 'NOTICE');
                        DEBUG("------------------------------------------", 'NOTICE');
                        DEBUG("User ID: $bidderid is placing a proxy bid amount: $bidamount, minimum bid including category increment is currently: $minimumbid", 'NOTICE');

                        $res = $ilance->bid_proxy->fetch_first_highest_proxybid($id);
                        $highestproxy = $res[0];
                        $highestproxyuserid = $res[1];
                        unset($res);
                        
                        $res = $ilance->bid_proxy->fetch_second_highest_proxybid($id);
                        $secondhighestproxy = $res[0];
                        $secondhighestproxyuserid = $res[1];
                        unset($res);
                        
                        $highestproxybiduser = $ilance->bid_proxy->fetch_highest_proxy_bid($id, $bidderid);
                        $secondhighestproxybiduser = $ilance->bid_proxy->fetch_second_highest_proxy_bid($id, $bidderid);

                        DEBUG("------------------------------------------", 'NOTICE');                                
                        DEBUG("Highest proxy bid is currently: $highestproxy by user id: $highestproxyuserid", 'NOTICE');
                        DEBUG("------------------------------------------", 'NOTICE');
                        DEBUG("Second Highest proxy bid is currently $secondhighestproxy by user id: $secondhighestproxyuserid", 'NOTICE');
                        DEBUG("------------------------------------------", 'NOTICE');
                        
                        // did this bidder already place a proxy bid for this auction?
                        $sql = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "proxybid
                                WHERE user_id = '" . intval($bidderid) . "'
                                    AND project_id = '" . intval($id) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                // this bidder already placed a proxy bid at some point .. is this new proxy bid higher? if so, place it!
                                $resproxy = $ilance->db->fetch_array($sql);
                                if ($resproxy['maxamount'] < $bidamount)
                                {
                                        DEBUG("SQL: UPDATE " . DB_PREFIX . "proxybid with proxy bid amount $bidamount for User ID: $bidderid", 'NOTICE');
                                        DEBUG("------------------------------------------", 'NOTICE');
                                        
                                        // update existing proxybid
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "proxybid
                                                SET maxamount = '" . sprintf("%01.2f", $bidamount)."',
                                                date_added = '" . DATETIME24H."'
                                                WHERE user_id = '" . intval($bidderid) . "'
                                                    AND project_id = '" . intval($id) . "'
                                                LIMIT 1
                                        ", 0, null, __FILE__, __LINE__);
                                }
                                else
                                {
                                        // inform bidder of proxy bid error (lower proxy bid being placed for same auction)
                                        if ($showerrormessages)
                                        {
                                                print_notice($phrase['_cannot_bid_lower_than_original_proxy_amount'] . ' ' . $ilance->currency->format($resproxy['maxamount']), $phrase['_it_has_been_detected_that_you_have_already_placed_a_higher_proxy_bid'], $ilpage['rfp'].'?cmd=bid&amp;id='.$id.'&amp;state=product', $phrase['_back']);
                                                exit();
                                        }
                                        else
                                        {
                                                return false;
                                        }
                                }
                        }
                        else
                        {
                                DEBUG("SQL: INSERT INTO " . DB_PREFIX . "proxybid with proxy bid amount $bidamount for User ID: $bidderid", 'NOTICE');
                                DEBUG("------------------------------------------", 'NOTICE');
                                
                                // bidder wishes to enter a new maximum highest bid for proxy
                                $ilance->db->query("
                                        INSERT INTO " . DB_PREFIX . "proxybid
                                        (id, project_id, user_id, maxamount, date_added)
                                        VALUES(
                                        NULL,
                                        '" . intval($id) . "',
                                        '" . intval($bidderid) . "',
                                        '" . sprintf("%01.2f", $bidamount) . "',
                                        '" . DATETIME24H . "')
                                ", 0, null, __FILE__, __LINE__);
                                $proxybidid = $ilance->db->insert_id();
                        }
                        
                        $sqlbids = $ilance->db->query("
                                SELECT COUNT(*) AS bids
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($id) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sqlbids) > 0)
                        {
                                $resbids = $ilance->db->fetch_array($sqlbids);
                                if ($resbids['bids'] == 0)
                                {
                                        // #### NO BIDS PLACED YET #############################
                                        DEBUG("No bids currently placed for this auction", 'NOTICE');
                                        
                                        // two scenerios:
                                        
                                        // #### RESERVE PRICE ENABLED
                                        if ($reserveprice > 0)
                                        {
                                                DEBUG("Reserve price exists: $reserveprice", 'NOTICE');
                                                
                                                // when enabled, any bid placed that is lower than the reserve price
                                                // will be placed at the rate of the actual bid 
                                                // this will get the bids rolling up to the reserve amount bypassing the increment logic.
                                                // side note: in this situation, this is the first bid being placed.
                                                if ($bidamount >= $reserveprice)
                                                {
                                                        DEBUG("Reserve price has been met ($reserveprice); set the bid amount price ($bidamount) as reserve: $reserveprice", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        DEBUG("BID: $reserveprice", 'NOTICE');
                                                        
                                                        $bidamount = $reserveprice;
                                                }
                                                else
                                                {
                                                        DEBUG("Reserve price: $reserveprice has not been met", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        DEBUG("BID: $bidamount", 'NOTICE');
                                                }
                                        }
                                        
                                        // #### RESERVE PRICE DISABLED
                                        else 
                                        {
                                                DEBUG("Reserve price disabled", 'NOTICE');
                                                DEBUG("------------------------------------------", 'NOTICE');
                                                DEBUG("BID: $minimumbid", 'NOTICE');
                                                
                                                // when reserve price is inactive, the bid placed will adhere to the admin defined increment logic for the bids
                                                // which is basically the minimum bid amount passed to this function
                                                // side note: in this situation, this is the first bid being placed.
                                                $bidamount = $minimumbid;
                                        }
                                }
                                else if ($resbids['bids'] > 0)
                                {
                                        // #### BIDS HAVE BEEN PLACED 
                                        DEBUG("Auction has $resbids[bids] bids currently placed", 'NOTICE');
                                        DEBUG("------------------------------------------", 'NOTICE');
                                        
                                        // do we have a reserve price?
                                        if ($reserveprice > 0)
                                        {
                                                // #### RESERVE PRICE ENABLED 
                                                DEBUG("Reserve price exists: $reserveprice", 'NOTICE');
                                                DEBUG("------------------------------------------", 'NOTICE');
                                                
                                                if ($bidamount == $reserveprice)
                                                {
                                                        DEBUG("Reserve price has been met: $reserveprice; set bid amount: $bidamount as reserve price: $reserveprice", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        
                                                        $bidamount = $reserveprice;
                                                        DEBUG("BID: $bidamount", 'NOTICE');
                                                }
                                                else if ($bidamount > $reserveprice)
                                                {
                                                        DEBUG("Reserve price has been met; bid being placed: $bidamount is higher than our reserve price ($reserveprice)", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        
                                                        $cid = fetch_auction('cid', $id);
                                                        
                                                        // if we have a high proxy bid, and if our bid amount is higher than the proxy bid, and if the highest proxy + increment <= bid amount
                                                        //     310.01     > 0 and 360.00     > 310.01        and              320.01                           <= 360.00
                                                        //     360.00     > 0 and 330.01     > 360.00        and              370.00                           <= 330.01
                                                        if ($highestproxy > 0 AND $bidamount > $highestproxy AND $this->fetch_minimum_bid($highestproxy, $cid) <= $bidamount)
                                                        {
                                                                $nextamount = $this->fetch_minimum_bid($highestproxy, $cid);
                                                                
                                                                DEBUG("Next Bid + Increment $nextamount is <= proxy bid $bidamount", 'NOTICE');
                                                                DEBUG("------------------------------------------", 'NOTICE');
                                                                $bidamount = $nextamount;
                                                                DEBUG("BID: $bidamount", 'NOTICE');
                                                        }
                                                        else
                                                        {
                                                                DEBUG("Higher proxy bid exists $highestproxy by user id: $highestproxyuserid, and this bid: $bidamount by user id: $bidderid", 'NOTICE');
                                                                DEBUG("------------------------------------------", 'NOTICE');
                                                                
                                                                if ($highestproxyuserid != $bidderid)
                                                                {
                                                                        DEBUG("Highest proxy bid user id: $highestproxyuserid ($highestproxy) is not the same bidder as user id: $bidderid ($bidamount) so we'll place the bid without the increment logic as it's much greater and will get the bids moving faster", 'NOTICE');
                                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                                }
                                                                else
                                                                {
                                                                        DEBUG("Highest proxy bid user id: $highestproxyuserid ($highestproxy) is not the same bidder as user id: $bidder ($bidamount) so we'll place the bid a minimum bid instead", 'NOTICE');
                                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                                        $bidamount = $minimumbid;
                                                                }
                                                                
                                                                        
                                                                DEBUG("BID: $bidamount", 'NOTICE');
                                                        }
                                                }
                                                else
                                                {
                                                        DEBUG("Reserve price has not been met", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        DEBUG("BID: $bidamount", 'NOTICE');
                                                }
                                        }
                                        else 
                                        {
                                                // #### RESERVE PRICE DISABLED #####################
                                                DEBUG("Reserve price is disabled", 'NOTICE');
                                                DEBUG("------------------------------------------", 'NOTICE');
                                                
                                                if ($bidamount > $highestproxy)
                                                {
                                                        DEBUG("Proxy bid amount being placed: $bidamount is greater than highest proxy: $highestproxy", 'NOTICE');
                                                        
                                                        $cid = fetch_auction('cid', $id);
                                                        
                                                        DEBUG("Category ID: #$cid", 'NOTICE');
                                                        DEBUG("------------------------------------------", 'NOTICE');
                                                        
                                                        $nextamount = $this->fetch_minimum_bid($highestproxy, $cid);
                                                        
                                                        if ($nextamount >= $bidamount)
                                                        {
                                                                DEBUG("Next Bid + Bid Increment (in this category) $nextamount is >= to proxy bid $bidamount", 'NOTICE');
                                                                DEBUG("------------------------------------------", 'NOTICE');
                                                        }
                                                        else
                                                        {
                                                                DEBUG("Next Bid + Increment $nextamount is < proxy bid $bidamount", 'NOTICE');
                                                                DEBUG("------------------------------------------", 'NOTICE');
                                                                $bidamount = $nextamount;
                                                        }
                                                }
                                        }
                                }
                        }
                }
    
                DEBUG("SQL: INSERT INTO " . DB_PREFIX . "project_bids bid amount: $bidamount for bidder id $bidderid", 'NOTICE');
                
                // insert the next minimum bid for the bidder
                $ilance->db->query("
                        INSERT INTO " . DB_PREFIX . "project_bids
                        (bid_id, user_id, project_id, project_user_id, bidamount, qty, date_added, bidstatus, bidstate, state)
                        VALUES(
                        NULL,
                        '" . intval($bidderid) . "',
                        '" . intval($id) . "',
                        '" . intval($project_user_id) . "',
                        '" . sprintf("%01.2f", $bidamount) . "',
                        '" . intval($qty) . "',
                        '" . DATETIME24H . "',
                        'placed',
                        '',
                        'product')
                ", 0, null, __FILE__, __LINE__);
                $this_bid_id = $ilance->db->insert_id();
                
                $ilance->db->query("
                        INSERT INTO " . DB_PREFIX . "project_realtimebids
                        (id, bid_id, user_id, project_id, project_user_id, bidamount, qty, date_added, bidstatus, bidstate, state)
                        VALUES(
                        NULL,
                        '" . intval($this_bid_id) . "',
                        '" . intval($bidderid) . "',
                        '" . intval($id) . "',
                        '" . intval($project_user_id) . "',
                        '" . sprintf("%01.2f", $bidamount) . "',
                        '" . intval($qty) . "',
                        '" . DATETIME24H . "',
                        'placed',
                        '',
                        'product')
                ", 0, null, __FILE__, __LINE__);
    
                DEBUG("------------------------------------------", 'NOTICE');
                DEBUG("set_bid_counters() - Set bid counters for this bidder (increases bidstoday + bidsthismonth)", 'NOTICE');
                
                // will increase bidstoday and bidsthismonth
                $this->set_bid_counters($bidderid, 'increase');
                
                DEBUG("------------------------------------------", 'NOTICE');
                DEBUG("SQL UPDATE project table bids + 1", 'NOTICE');
                
                // update bid count
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "projects
                        SET bids = bids + 1,
                        currentprice = '" . sprintf("%01.2f", $bidamount) . "'
                        WHERE project_id = '" . intval($id) . "'
                            AND project_state = 'product'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                
                // was this buyer invited?
                $sql_invites = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "project_invitations
                        WHERE project_id = '" . intval($id) . "'
                            AND buyer_user_id = '" . intval($bidderid) . "'
                            AND bid_placed = 'no'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql_invites) > 0)
                {
                        DEBUG("------------------------------------------", 'NOTICE');
                        DEBUG("It appears this bidder was invited by the seller, update invite table to bid_placed = yes", 'NOTICE');
                
                        // update invite table 
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "project_invitations
                                SET bid_placed = 'yes',
                                date_of_bid = '" . DATETIME24H . "'
                                WHERE buyer_user_id = '" . intval($bidderid) . "'
                                    AND project_id = '" . intval($id) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
        
                        $url = HTTPS_SERVER . $ilpage['merch'] . '?id=' . intval($id);
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                                            
                        // email owner
                        $ilance->email->mail = fetch_user('email', $project_user_id);
                        $ilance->email->slng = fetch_user_slng($project_user_id);
                        
                        $ilance->email->get('invited_bid_placed_buyer');		
                        $ilance->email->set(array(
                                '{{buyer}}' => fetch_user('username', $project_user_id),
                                '{{vendor}}' => fetch_user('username', $bidderid),
                                '{{rfp_title}}' => fetch_auction('project_title', intval($id)),
                                '{{project_id}}' => $id,
                                '{{url}}' => $url,
                        ));
                        
                        $ilance->email->send();
                }
                
                // #### AUTOMATED PROXY BIDDER ENGINE ##################
                // now supports proxy bidding on a per category basis
                if ($ilconfig['productbid_enableproxybid'] AND isset($isproxy) AND $isproxy AND $ilance->categories->useproxybid($_SESSION['ilancedata']['user']['slng'], 'product', $proxycid))
                {
                        DEBUG("---------------------------------------------", 'NOTICE');
                        DEBUG("PROXY: ILance Proxy Bid Engine " . ILANCEVERSION . "", 'NOTICE');
                        DEBUG("---------------------------------------------", 'NOTICE');
                        // background proxy bidder init for this last bidder
                        // this is where the proxy automation comes into action
                        $ilance->bid_proxy->do_proxy_bidder(intval($id), $bidderid, $project_user_id, $skipbidder = 1, $bidderid);
                        
                        DEBUG("---------------------------------------------", 'NOTICE');
                        DEBUG("PROXY: Finished", 'NOTICE');
                        DEBUG("---------------------------------------------", 'NOTICE');
                }
                
                // for debug purposes only
                //print_r($GLOBALS['DEBUG']); exit;
                
                if ($showerrormessages)
                {
                        refresh($ilpage['merch'] . '?id=' . $id);
                        exit();
                }
                else
                {
                        return true;
                }
        }
        
        /**
        * Function to determine if the viewing bidder is the highest bidder for a particular auction event.
        *
        * @param       integer      bidder id
        * @param       integer      project id
        * 
        * @return      bool         true or false
        */
        function is_highest_bidder($bidderid = 0, $projectid = 0)
        {
                global $ilance, $myapi;
                
                $sql = $ilance->db->query("
                        SELECT bidamount, user_id
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                        ORDER BY bidamount DESC
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        if ($res['user_id'] == $bidderid)
                        {
                                return 1;
                        }
                }
                
                return 0;
        }
        
        /**
        * Function to determine if a particular auction event has an active highest bidder present
        *
        * @param       integer      project id
        * 
        * @return      bool         true or false
        */
        function has_highest_bidder($projectid = 0)
        {
                global $ilance, $myapi;
                
                $sql = $ilance->db->query("
                        SELECT user_id
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                        ORDER BY bidamount DESC
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        return true;
                }
                
                return false;
        }
    
        /**
        * Function to determine if the viewing bidder is the winner for a particular auction event.
        * This function will scan both, project bids (for awarded bids) or buynow_orders to determine if
        * a buy now purchase order was made for the item.
        *
        * @param       integer      bidder id
        * @param       integer      project id
        * 
        * @return      bool         true or false
        */
        function is_winner($bidderid = 0, $projectid = 0)
        {
                global $ilance, $myapi, $show;
                
                $show['wonbyauction'] = $show['wonbypurchase'] = false;
                
                // was item won by a high bidder?
                $sql = $ilance->db->query("
                        SELECT user_id
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                            AND bidstatus = 'awarded'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        if ($res['user_id'] == $bidderid)
                        {
                                $show['wonbyauction'] = true;
                                return true;
                        }
                }
                unset($res);
                
                // was item won via buy now option?
                $sql = $ilance->db->query("
                        SELECT buyer_id
                        FROM " . DB_PREFIX . "buynow_orders
                        WHERE project_id = '" . intval($projectid) . "'
                                AND status != 'cancelled'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        if ($res['buyer_id'] == $bidderid)
                        {
                                $show['wonbypurchase'] = true;
                                return true;
                        }
                }
                unset($res);
                
                
                return false;
        }
        
        /**
        * Function to determine if a particular product listing has a winning bidder.
        *
        * @param       integer      project id
        * 
        * @return      bool         true or false
        */
        function has_winning_bidder($projectid = 0)
        {
                global $ilance, $show;
                
                $show['soldbyauction'] = false;
                
                $sql = $ilance->db->query("
                        SELECT user_id
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                            AND bidstatus = 'awarded'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $show['soldbyauction'] = true;
                        return true;
                }
                
                return false;
        }
    
        /**
        * Function to determine if the viewing bidder has been outbid for a particular auction event.
        *
        * @param       integer      bidder id
        * @param       integer      project id
        * 
        * @return      bool         true or false
        */
        function is_outbidded($bidderid = 0, $projectid = 0)
        {
                global $ilance, $myapi;
                
                $exist = $ilance->db->query("
                        SELECT bidamount
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                            AND user_id = '" . intval($bidderid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($exist) > 0)
                {
                        $sql = $ilance->db->query("
                                SELECT bidamount, user_id
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($projectid) . "'
                                ORDER BY bidamount DESC, bid_id DESC
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                // user has placed a bid in the auction.  have we been outbid?
                                while ($res = $ilance->db->fetch_array($sql))
                                {
                                        if ($res['user_id'] == $bidderid)
                                        {
                                                return 0;
                                        }
                                        else
                                        {
                                                // we are outbid: double check if the auction is expired
                                                $sql2 = $ilance->db->query("
                                                        SELECT status
                                                        FROM " . DB_PREFIX . "projects
                                                        WHERE project_id = '" . intval($projectid) . "'
                                                        LIMIT 1
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($sql2) > 0)
                                                {
                                                        $res2 = $ilance->db->fetch_array($sql2);
                                                        if ($res2['status'] != 'open')
                                                        {
                                                                // auction not open: hide outbid html bit
                                                                return 0;
                                                        }
                                                        else
                                                        {
                                                                // auction still open
                                                                return 1;
                                                        }
                                                }
                                        }
                                }
                        }
                }
                
                return 3;
        }
        
        /**
        * Function to determine if the viewing auction has any bid filters.
        *
        * @param       integer      project id
        * 
        * @return      string       error message if any
        */
        function product_bid_filter_checkup($id = 0)
        {
                global $ilance, $myapi, $ilpage, $phrase, $page_title, $area_title, $ilconfig;
                
                $sql_filterprojects = $ilance->db->query("
                        SELECT *, UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, UNIX_TIMESTAMP(date_starts) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS starttime
                        FROM " . DB_PREFIX . "projects
                        WHERE project_id = '" . intval($id) . "'
                            AND project_state = 'product'
                            AND visible = '1'
                ", 0, null, __FILE__, __LINE__);
                $row = $ilance->db->fetch_array($sql_filterprojects);
        
                $sign = '+';
                if ($row['mytime'] < 0)
                {
                        $row['mytime'] = - $row['mytime'];
                        $sign = '-';
                }
                
                if ($sign == '-')
                {
                        $area_title = $phrase['_this_rfp_has_expired_bidding_is_over'];
                        $page_title = SITE_NAME . ' - ' . $phrase['_this_rfp_has_expired_bidding_is_over'];
                        
                        $filtered_message = $phrase['_this_rfp_has_expired_bidding_is_over'];
                        print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $ilpage['main'], $phrase['_main_menu']);
                        exit();
                }
                
                if ($row['filter_rating'])
                {
                        $ilance->feedback = construct_object('api.feedback');
                        
                        $memberinfo = array();
                        $memberinfo = $ilance->feedback->datastore($_SESSION['ilancedata']['user']['userid']);
                        if ($memberinfo['rating'] < $row['filtered_rating'])
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_messageT = $phrase['_requires_rating_of_at_least'] . " " . $row['filtered_rating'] . " " . $phrase['_stars'];
                                global $filtered_message;
                                $filtered_message = $filtered_messageT;
                                
                                print_notice($phrase['_bid_filter_restriction']." ".$filtered_message, "".$phrase['_sorry_this_merchant_has_set_bid_filtering_permissions_on_their_auction']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\")>".$phrase['_click_here']."</a>.", 'javascript:history.back(1);', $phrase['_back']);
                                exit();
                        }
                }
                
                if ($row['filter_country'])
                {
                        $cfiltered = fetch_country_id($row['filtered_country'], $_SESSION['ilancedata']['user']['slng']);
                        if ($_SESSION['ilancedata']['user']['countryid'] == $cfiltered)
                        {
                                $sql_cname = $ilance->db->query("SELECT location_".$_SESSION['ilancedata']['user']['slng']." FROM " . DB_PREFIX . "locations WHERE locationid = '".$cfiltered."'", 0, null, __FILE__, __LINE__);
                                $res_cname = $ilance->db->fetch_array($sql_cname);
                        }
                        else
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_country_must_be_located_in']." ".stripslashes($res_cname['location_'.$_SESSION['ilancedata']['user']['slng']]);
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $ilpage['main'], $phrase['_main_menu']);
                                exit();
                        }
                }
                
                if ($row['filter_state'])
                {
                        $sfiltered = mb_strtolower($row['filtered_state']);
                        $sql_state = $ilance->db->query("SELECT state FROM " . DB_PREFIX . "users WHERE user_id = '".$_SESSION['ilancedata']['user']['userid']."'", 0, null, __FILE__, __LINE__);
                        $res_state = $ilance->db->fetch_array($sql_state);
                        
                        $cstate = mb_strtolower($res_state['state']);
                        if ($cstate != $sfiltered)
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_state_province_must_be_located_in']." ".ucfirst($sfiltered);
                                $return_url = $ilpage['main'];
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
                
                if ($row['filter_city'])
                {
                        $cityfiltered = mb_strtolower($row['filtered_city']);
                        $sql_city = $ilance->db->query("SELECT city FROM " . DB_PREFIX . "users WHERE user_id='".$_SESSION['ilancedata']['user']['userid']."'", 0, null, __FILE__, __LINE__);
                        $res_city = $ilance->db->fetch_array($sql_city);
                        
                        $ccity = mb_strtolower($res_city['city']);
                        if ($ccity != $cityfiltered AND !empty($cityfiltered))
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_city_must_be_located_in']." ".ucfirst($cityfiltered);
                                $return_url = $ilpage['main'];
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
                
                if ($row['filter_zip'])
                {
                        $zipfiltered = mb_strtolower($row['filtered_zip']);
                        $sql_zip = $ilance->db->query("SELECT zip_code FROM " . DB_PREFIX . "users WHERE user_id = '".$_SESSION['ilancedata']['user']['userid']."'", 0, null, __FILE__, __LINE__);
                        $res_zip = $ilance->db->fetch_array($sql_zip);
                        $czip = mb_strtolower($res_zip['zip_code']);
                        if ($czip != $zipfiltered)
                        {
                                $area_title = $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                $page_title = SITE_NAME . ' - ' . $phrase['_placing_bid_invalid_bid_filter_permissions'];
                                
                                $filtered_message = $phrase['_bidders_zip_postal_code_must_be_located_in']." ".mb_strtoupper($zipfiltered);
                                $return_url = $ilpage['main'];
                                print_notice($phrase['_bid_filter_restriction'] . "&nbsp;" . $filtered_message, $phrase['_were_sorry_this_project_owner_has_set_bid_filtering_permissions_on_their_project']."<br /><br />".$phrase['_bid_filtering_allows_the_buyer_to_filter_various_aspects_of_their_project']."<br /><br />".$phrase['_to_learn_more_about_bid_filter_permissions']." <a href=\"javascript:void(0)\" onClick=Attach(\"".$ilpage['rfp']."?msg=bid-permissions\") >".$phrase['_click_here']."</a>", $return_url, $phrase['_main_menu']);
                                exit();
                        }
                }
        }
    
        /**
        * Function for printing any bid filter permissions.
        *
        * @param       string       filter type
        * @param       integer      project id
        */
        function construct_bidfilter_permissions($filtertype = '', $id = 0)
        {
                global $ilance, $myapi, $ilconfig, $phrase, $iltemplate, $area_title, $page_title, $parts, $official_time, $phrase, $ilpage;
                
                if ($filtertype == 'service')
                {
                        $result_bidtop = $ilance->db->query("
                                SELECT *, UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, UNIX_TIMESTAMP(date_starts) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS starttime
                                FROM " . DB_PREFIX . "projects
                                WHERE project_id = '" . intval($id) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($result_bidtop) > 0)
                        {
                                $filter_permissions = '';
                                
                                $ilance->feedback = construct_object('api.feedback');
                                while ($row = $ilance->db->fetch_array($result_bidtop))
                                {
                                        if (!empty($_SESSION['ilancedata']['user']['userid']))
                                        {
                                                if ($row['filter_rating'])
                                                {
                                                        $memberinfo = array();
                                                        $memberinfo = $ilance->feedback->datastore($_SESSION['ilancedata']['user']['userid']);
                                                        if ($memberinfo['rating'] >= intval($row['filtered_rating']))
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_overall_rating_of_at_least'].' <strong>'.$row['filtered_rating'].' '.$phrase['_stars'].'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_overall_rating_of_at_least'].' <strong>'.$row['filtered_rating'].' '.$phrase['_stars'].'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_country'] AND !empty($row['filtered_country']))
                                                {
                                                        $cfiltered = mb_strtolower(stripslashes($row['filtered_country']));
                                                        $countryname = mb_strtolower(print_user_country($_SESSION['ilancedata']['user']['userid'], $_SESSION['ilancedata']['user']['slng']));
                                                        if ($cfiltered == $countryname)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_country_must_be_located_in'].' <strong>'.ucwords($cfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_country_must_be_located_in'].' <strong>'.ucwords($cfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_state'] AND !empty($row['filtered_state']))
                                                {
                                                        $sfiltered = mb_strtolower($row['filtered_state']);
                                                        $cstate = mb_strtolower($_SESSION['ilancedata']['user']['state']);
                                                        if ($cstate == $sfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_state_province_must_be_located_in'].' <strong>'.ucwords($sfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_state_province_must_be_located_in'].' <strong>'.ucwords($sfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_city'] AND !empty($row['filtered_city']))
                                                {
                                                        $cityfiltered = mb_strtolower($row['filtered_city']);
                                                        $ccity = mb_strtolower($_SESSION['ilancedata']['user']['city']);
                                                        if ($ccity == $cityfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_city_must_be_located_in'].' <strong>'.ucwords($cityfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_city_must_be_located_in'].' <strong>'.ucwords($cityfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_zip'] AND !empty($row['filtered_zip']))
                                                {
                                                        $zipfiltered = mb_strtolower($row['filtered_zip']);
                                                        $czip = mb_strtolower($_SESSION['ilancedata']['user']['postalzip']);
                                                        if ($czip == $zipfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_zip_postal_code_must_be_located_in'].' <strong>'.mb_strtoupper($zipfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_zip_postal_code_must_be_located_in'].' <strong>'.mb_strtoupper($zipfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="" /></div>';
                                                        }
                                                }
                                                if ($row['filter_underage'])
                                                {
                                                        
                                                }
                                                if ($row['filter_businessnumber'])
                                                {
                                                        
                                                }
                                        }
                                }
                        }
                }
                else if ($filtertype == 'product')
                {
                        $result_bidtop = $ilance->db->query("
                                SELECT *, UNIX_TIMESTAMP(date_end) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS mytime, UNIX_TIMESTAMP(date_starts) - UNIX_TIMESTAMP('" . DATETIME24H . "') AS starttime
                                FROM " . DB_PREFIX . "projects
                                WHERE project_id = '".intval($id)."'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($result_bidtop) > 0)
                        {
                                $filter_permissions = '';
                                
                                $ilance->feedback = construct_object('api.feedback');
                                while ($row = $ilance->db->fetch_array($result_bidtop))
                                {
                                        if (!empty($_SESSION['ilancedata']['user']['userid']))
                                        {
                                                if ($row['filter_rating'])
                                                {
                                                        $memberinfo = array();
                                                        $memberinfo = $ilance->feedback->datastore($_SESSION['ilancedata']['user']['userid']);
                                                        if ($memberinfo['rating'] >= intval($row['filtered_rating']))
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_overall_rating_of_at_least'].' <strong>'.$row['filtered_rating'].' '.$phrase['_stars'].'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_overall_rating_of_at_least'].' <strong>'.$row['filtered_rating'].' '.$phrase['_stars'].'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_country'] AND !empty($row['filtered_country']))
                                                {
                                                        $cfiltered = mb_strtolower(stripslashes($row['filtered_country']));
                                                        $countryname = mb_strtolower(print_user_country($_SESSION['ilancedata']['user']['userid'], $_SESSION['ilancedata']['user']['slng']));
                                                        if ($cfiltered == $countryname)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_country_must_be_located_in'].' <strong>'.ucwords($cfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_country_must_be_located_in'].' <strong>'.ucwords($cfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_state'] AND !empty($row['filtered_state']))
                                                {
                                                        $sfiltered = mb_strtolower($row['filtered_state']);
                                                        $cstate = mb_strtolower($_SESSION['ilancedata']['user']['state']);
                                                        if ($cstate == $sfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_state_province_must_be_located_in'].' <strong>'.ucwords($sfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_state_province_must_be_located_in'].' <strong>'.ucwords($sfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_city'] AND !empty($row['filtered_city']))
                                                {
                                                        $cityfiltered = mb_strtolower($row['filtered_city']);
                                                        $ccity = mb_strtolower($_SESSION['ilancedata']['user']['city']);
                                                        if ($ccity == $cityfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_city_must_be_located_in'].' <strong>'.ucwords($cityfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_city_must_be_located_in'].' <strong>'.ucwords($cityfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_you_do_not_meet_this_requirement'] . '" /></div>';
                                                        }
                                                }
                                                if ($row['filter_zip'] AND !empty($row['filtered_zip']))
                                                {
                                                        $zipfiltered = mb_strtolower($row['filtered_zip']);
                                                        $czip = mb_strtolower($_SESSION['ilancedata']['user']['postalzip']);
                                                        if ($czip == $zipfiltered)
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_zip_postal_code_must_be_located_in'].' <strong>'.mb_strtoupper($zipfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_you_meet_this_requirement'] . '" /></div>';
                                                        }
                                                        else
                                                        {
                                                                $filter_permissions .= '<div>' . $phrase['_bidders_zip_postal_code_must_be_located_in'].' <strong>'.mb_strtoupper($zipfiltered).'</strong> <img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="" /></div>';
                                                        }
                                                }
                                                if ($row['filter_underage'])
                                                {
                                                        
                                                }
                                                if ($row['filter_businessnumber'])
                                                {
                                                        
                                                }
                                        }
                                }
                        }
                }
                
                // profile bid permissions
                
                
                if (!empty($filter_permissions))
                {
                        return $filter_permissions;
                }
        }
        
        /**
        * Function for printing a bid privacy details pulldown menu.
        *
        * @param       string       selected value (optional)
        *
        * @return     string       HTML representation of the pulldown menu
        */
        function construct_bid_details_pulldown($selected = '')
        {
                global $ilance, $myapi, $phrase;
                
                $html = '<select name="bid_details" style="font-family: verdana">';
                $html .= '<option value="open"'; if ($selected == "open") { $html .= ' selected="selected"'; } $html .= '>' . $phrase['_open'] . '</option>';
                $html .= '<option value="sealed"'; if ($selected == "sealed") { $html .= ' selected="selected"'; } $html .= '>' . $phrase['_sealed'] . '</option>';
                $html .= '<option value="blind"'; if ($selected == "blind") { $html .= ' selected="selected"'; } $html .= '>' . $phrase['_blind'] . '</option>';
                $html .= '<option value="full"'; if ($selected == "full") { $html .= ' selected="selected"'; } $html .= '>' . $phrasr['_full'] . '</option>';
                $html .= '</select>';
                
                return $html;
        }
        
        /**
        * Function for returning a boolean value if a particular bid amount range for a project is valid
        *
        * @param       integer      project id
        * @param       string       bid amount
        * @param       string       filtered bid type id
        *
        * @return     string       HTML representation of the lowest unique bidder and their bid
        */
        function is_valid_bid_range($projectid = 0, $bidamount = 0, $filtered_bidtype = '')
        {
                global $ilance, $myapi;
                
                $bidamount = $ilance->currency->string_to_number($bidamount);
                
                if (isset($filtered_bidtype) AND $filtered_bidtype == 'hourly')
                {
                        // lets allow the service provider to enter an hourly rate that
                        // does not take the bid range system into consideration
                        // eg: bid range = $500 - $1000
                        // service provider bids: $50 per hour - we let it through.
                        return 1;    
                }
                else
                {
                        $sql = $ilance->db->query("
                                SELECT filter_budget, filtered_budgetid
                                FROM " . DB_PREFIX . "projects
                                WHERE project_id = '" . intval($projectid) . "'
                        ");
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                if ($res['filter_budget'] == 1)
                                {
                                        // buyer chooses a bid amount range
                                        // select the budget id range from the budget table
                                        $sql2 = $ilance->db->query("
                                                SELECT budgetfrom, budgetto
                                                FROM " . DB_PREFIX . "budget
                                                WHERE budgetid = '" . $res['filtered_budgetid'] . "'
                                        ");
                                        if ($ilance->db->num_rows($sql2) > 0)
                                        {
                                                $res2 = $ilance->db->fetch_array($sql2);
                                                // are we bidding to low or too high?
                                                if ($bidamount < $res2['budgetfrom'] OR $bidamount > $res2['budgetto'])
                                                {
                                                        return 0;    
                                                }
                                                // we've made it past the budget restrictions
                                                return 1;
                                        }
                                        else
                                        {
                                                // admin or someone else removed the budgetid associated
                                                // with this project .. treat as an okay bid.. (just in case)
                                                return 1;
                                        }
                                }
                                else 
                                {
                                        //  buyer wishes to let any bid amount type
                                        return 1;
                                }
                        }
                }
                
                return 1;
        }
        
        /**
        * Function for setting member bid count history (bidstoday and bidsthismonth) within the user database table.
        *
        * @param       integer      user id
        * @param       string       action to perform (increase or decrease)
        *
        * @return      nothing
        */
        function set_bid_counters($userid = 0, $dowhat = '')
	{
		global $ilance, $myapi;
                
		if (!empty($dowhat))
		{
			switch ($dowhat)
			{
				case 'increase':
                                {
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET bidstoday = bidstoday + 1,
                                                bidsthismonth = bidsthismonth + 1
                                                WHERE user_id = '" . intval($userid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        break;
                                }			    
				case 'decrease':
                                {
                                        $sql = $ilance->db->query("
                                                SELECT bidstoday, bidsthismonth
                                                FROM " . DB_PREFIX . "users
                                                WHERE user_id = '" . intval($userid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        $res = $ilance->db->fetch_array($sql);
                                        $bidstoday = $res['bidstoday'];
                                        $bidsthismonth = $res['bidsthismonth'];            
                                        $newtoday = ($bidstoday - 1);
                                        $newthismonth = ($bidsthismonth - 1);
                                        if ($newtoday < 0)
                                        {
                                                $newtoday = 0;
                                        }
                                        if ($newthismonth < 0)
                                        {
                                                $newthismonth = 0;
                                        }
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET bidstoday = '".$newtoday."',
                                                bidsthismonth = '".$newthismonth."'
                                                WHERE user_id = '" . intval($userid) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        break;
                                }
			}    
		}
	}
        
        /**
        * Function to fetch the average bid amount price for a particular service auction event.
        *
        * @param       integer        project id
        * @param       boolean        force no privacy (ie: will not show sealed if it is sealed)
        * @param       string         bid details to do the checkup on
        *
        * @return      string         Returns formatted average bid price amount
        */
        function fetch_average_bid($projectid = 0, $noprivacy = true, $bid_details = '', $noformatting = false)
        {
                global $ilance, $phrase, $myapi;
                
                $sql = $ilance->db->query("
                        SELECT AVG(bidamount) AS average
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($projectid) . "'
                                AND bidstate != 'retracted'
                                AND bidstatus != 'declined'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        
                        if ($noprivacy)
                        {
                                if ($noformatting)
                                {
                                        return $res['average'];        
                                }
                                else
                                {
                                        return print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res['average']);
                                }
                        }
                        else
                        {
                                if (!empty($bid_details) AND $bid_details == 'open' OR $bid_details == 'blind')
                                {
                                        if ($noformatting)
                                        {
                                                return $res['average'];        
                                        }
                                        else
                                        {
                                                return print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res['average']);
                                        }
                                }
                                else
                                {
                                        return '= ' . $phrase['_sealed'] . ' =';
                                }
                        }
                }
                
                if ($noformatting)
                {
                        return 0;       
                }
                else
                {
                        return print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], 0);
                }
        }
        
        /**
        * Function to fetch the declined bids count for a particular service auction event.
        *
        * @param       integer        project id
        *
        * @return      string         Returns delicned bids count
        */
        function fetch_declined_bids($projectid = 0)
        {
                global $ilance, $myapi;
                
                $declined = 0;
                
                $sql = $ilance->db->query("
                        SELECT bidsdeclined
                        FROM " . DB_PREFIX . "projects
                        WHERE project_id = '" . intval($projectid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        $declined = $res['bidsdeclined'];
                }
                
                return (int)$declined;
        }
        
        /**
        * Function to fetch the declined bids count for a particular service auction event.
        *
        * @param       integer        project id
        *
        * @return      string         Returns delicned bids count
        */
        function fetch_retracted_bids($projectid = 0)
        {
                global $ilance, $myapi;
                
                $retracted = 0;
                
                $sql = $ilance->db->query("
                        SELECT bidsretracted
                        FROM " . DB_PREFIX . "projects
                        WHERE project_id = '" . intval($projectid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        $retracted = $res['bidsretracted'];
                }
                
                return (int)$retracted;
        }
        
        /**
        * Function to fetch the declined bids count for a particular service auction event.
        *
        * @param       integer        project id
        * @param       integer        buyer id
        *
        * @return      string         Returns delicned bids count
        */
        function fetch_shortlisted_bids($projectid = 0, $buyerid = 0)
        {
                global $ilance, $myapi;
                
                $shortlisted = 0;
                if (isset($buyerid) AND $buyerid > 0)
                {
                        $sql = $ilance->db->query("
                                SELECT bidsshortlisted
                                FROM " . DB_PREFIX . "projects
                                WHERE project_id = '" . intval($projectid) . "'
                                        AND user_id = '" . intval($buyerid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $shortlisted = $res['bidsshortlisted'];
                        }
                }
                
                return (int)$shortlisted;
        }
        
        /**
        * Function to find any awarded projects and unaward them due to expiry of awarding set by admin
        *
        * @param       integer        days to determine if the expiry has  been met
        *
        * @return      string         Returns true or false
        */
        function wait_approval_unaward_cron()
        {
                global $ilance, $ilconfig, $phrase;
                
                $html = '';
                
                $sql = $ilance->db->query("
                        SELECT project_id, user_id
                        FROM " . DB_PREFIX . "projects
                        WHERE status = 'wait_approval'
                                AND close_date != '0000-00-00 00:00:00'
                ");
                if ($ilance->db->num_rows($sql) > 0)
                {
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                // determine if this project has waited more than "admin defined days" if so, unaward the providers bid automatically
                                
                                $date1split = explode(' ', $res['close_date']);
                                $date2split = explode('-', $date1split[0]);
                                
                                $duration = $ilance->datetime->fetch_days_between($date2split[1], $date2split[2], $date2split[0], gmdate('m'), gmdate('d'), gmdate('Y'));
                                if ($duration == 0)
                                {
                                        $duration = 1;
                                }
                                
                                // we'll only unaward if for example the day this logic is running on: Monday through Friday.
                                // if the function cannot run on Friday (6 day of the 7 total for example) then it will be caught up again on the following Monday
                                // because this will be 9 days of 7 total and it will expire if it hasn't been awarded over the weekend.
                                $isbusinessday = $ilance->datetime->is_business_day();
                                
                                if ($duration >= $ilconfig['servicebid_awardwaitperiod'] AND $isbusinessday)
                                {
                                        // we're ready to unaward this provider's bid
                                        // find out who's taking so long.....
                                        
                                        $sqlbid = $ilance->db->query("
                                                SELECT bid_id
                                                FROM " . DB_PREFIX . "project_bids
                                                WHERE project_id = '" . $res['project_id'] . "'
                                                        AND bidstatus = 'placed'
                                                        AND bidstate = 'wait_approval'
                                                        AND state = 'service'
                                                        AND date_awarded != '0000-00-00 00:00:00'
                                                        AND project_user_id = '" . $res['user_id'] . "'
                                                LIMIT 1
                                        ");
                                        if ($ilance->db->num_rows($sqlbid) > 0)
                                        {
                                                $resbid = $ilance->db->fetch_array($sqlbid);
                                                
                                                $ilance->auction = construct_object('api.auction');
                                                $ilance->auction_award = construct_object('api.auction_award');
                                                $ilance->auction_award->unaward_service_auction($resbid['bid_id'], $silentmode = true);
                                                
                                                $html .= 'Automatic provider unaward for project ' . $res['project_id'] . ' - provider did not accept buyers award in ' . $days . ' days time, ';
                                        }
                                }
                        }
                }
                
                return $html;
        }

        /**
        * Function for determining if a user can bid based on the auction posters filtered profile answer logic.
        * For example, if a profile question is "Hair Color" and the user placing a bid answered this previously saying "Red"
        * then the auction poster can also filter his auction bids based on users only having the hair color of "Red" which they can place a bid.
        * If the bidder has "Black" hair he / she will not be able to bid.
        *
        * @param       integer      user id
        * @param       integer      project id
        * @param       boolean      show stop template? (default true)
        *
        * @return      string       Returns true or false
        */        
        function user_can_bid($userid = 0, $projectid = 0, $showtemplate = true)
        {
                global $ilance, $myapi, $phrase, $ilpage;
                
                $note = '';
                
                $sql = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "profile_filter_auction_answers
                        WHERE project_id = '" . intval($projectid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $n = 1;
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                $sql1 = $ilance->db->query("
                                        SELECT *
                                        FROM " . DB_PREFIX . "profile_answers
                                        WHERE user_id = '" . intval($userid) . "'
                                        AND questionid = '" . intval($res['questionid']) . "'
                                        AND visible = 1
                                ", 0, null, __FILE__, __LINE__);
                                
                                // #### RANGE (from/to) ########################
                                if ($res['filtertype'] == 'range')
                                {
                                        $res1 = $ilance->db->fetch_array($sql1);
                                        
                                        // answer would be formatted like 1|5
                                        $choices = explode('|', $res['answer']);
                                        if (!($res1['answer'] >= $choices[0] AND $res1['answer'] <= $choices[1]))
                                        {
                                                $title = $ilance->db->fetch_field(DB_PREFIX . "profile_questions", "questionid = '" . $res1['questionid'] . "'", "question");
                                                $desc = $ilance->db->fetch_field(DB_PREFIX . "profile_questions", "questionid = '" . $res1['questionid'] . "'", "description");
                                                $gid = $ilance->db->fetch_field(DB_PREFIX . "profile_questions", "questionid = '" . $res1['questionid'] . "'", "groupid");
                                                $cid = $ilance->db->fetch_field(DB_PREFIX . "profile_groups", "groupid = '" . $gid . "'", "cid");
                                                
                                                $note .= '<div>' . $n . '. <strong>' . $title . '</strong> (' . $desc . '): <span class="gray">' . $phrase['_between_upper'] . ' ' . $choices[0] . ' ' . $phrase['_and'] . ' ' . $choices[1] . '.</span></div>';
                                                $n++;
                                        }
                                }
                                // #### CHECKBOX (pulldown, multiple choice ####
                                else if ($res['filtertype'] == 'checkbox')
                                {
                                        $res1 = $ilance->db->fetch_array($sql1);
                                        
                                        $choices = explode('|', $res['answer']);
                                        if (!in_array($res1['answer'], $choices))
                                        {
                                                $title = $ilance->db->fetch_field(DB_PREFIX . "profile_questions", "questionid = '" . $res1['questionid'] . "'", "question");
                                                $gid = $ilance->db->fetch_field(DB_PREFIX . "profile_questions", "questionid = '" . $res1['questionid'] . "'", "groupid");
                                                $cid = $ilance->db->fetch_field(DB_PREFIX . "profile_groups", "groupid = '" . $gid . "'", "cid");
                                                
                                                $note .= '<div>' . $n . '. <strong>' . $title . '</strong>: <span class="gray">' . str_replace('|', ', ', $res['answer']) . '.</span></div>';
                                                $n++;
                                        }
                                }
                        }
                        
                        if (!empty($note))
                        {
                                if ($showtemplate)
                                {
                                        $area_title = $phrase['_access_to_bid_is_denied'];
                                        $page_title = SITE_NAME . ' - ' . $phrase['_access_to_bid_is_denied'];
                                        
                                        print_notice($phrase['_access_denied'], $phrase['_sorry_but_you_did_not_answer_specific_profile_questions_or_your_answer_does_not_match_with_poster_requirements'] . '<br /><br />' . $note . '<br />' . $phrase['_please_visit_your_selling_profile_area_to_answer_various_questions_relating_to_this_posters_requirement'], $ilpage['selling'] . '?cmd=profile&cid=' . $cid . '#categories' , $phrase['_selling_profile']);
                                        exit();
                                }
                                else
                                {
                                        return false;
                                }
                        }
                        
                        return true;
                }
	}
        
        /**
        * Function to print out the bid transaction status for a listing that is won by a winning bidder.
        *
        * @param      integer      project id
        * @param      boolean      show short form phrase only? (default false)
        * @param      boolean      show icon only (default false)
        * @param      boolean      show invoice id with link to transaction page (to make payment for invoice IF unpaid)? (default false)
        *
        * @return     string       HTML formatted representation of the response
        */
        function fetch_transaction_status($projectid = 0, $shortform = false, $showicononly = false, $showlinktopayment = false)
        {
                global $ilance, $myapi, $ilpage, $phrase, $show, $ilconfig;
                
                $html = '';
                
                // check if invoice is paid
                $bidderid = $ilance->db->fetch_field(DB_PREFIX . "projects", "project_id = '" . intval($projectid) . "'", "winner_user_id");
                if (isset($bidderid) AND $bidderid > 0)
                {
                        $status = $ilance->db->fetch_field(DB_PREFIX . "invoices", "projectid = '" . intval($projectid) . "' AND user_id = '" . intval($bidderid) . "'", "status");
                        $invoiceid = (int)$ilance->db->fetch_field(DB_PREFIX . "invoices", "projectid = '" . intval($projectid) . "' AND user_id = '" . intval($bidderid) . "'", "invoiceid");
                        
                        if (isset($status) AND $status == 'paid' AND isset($invoiceid) AND $invoiceid > 0)
                        {
                                $paiddate = $ilance->db->fetch_field(DB_PREFIX . "invoices", "projectid = '" . intval($projectid) . "' AND user_id = '" . intval($bidderid) . "'", "paiddate");
                                if ($showicononly)
                                {
                                        $html .= '<img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="' . $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_marked_as_paid_on'] . ' ' . print_date($paiddate) . '" />';        
                                }
                                else if ($shortform)
                                {
                                        $html .= $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_marked_as_paid_on'] . ' ' . print_date($paiddate);
                                }
                                else
                                {
                                        $html .= $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_the_transaction_associated_with_this_listing_was_marked_as_paid_on'] . ' ' . print_date($paiddate);
                                }
                        }
                        else if (isset($status) AND $status == 'unpaid' AND isset($invoiceid) AND $invoiceid > 0)
                        {
                                if ($showicononly)
                                {
                                        $html .= '<img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/delete.gif" border="0" alt="' . $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_the_transaction_associated_with_this_listing_has_not_been_paid'] . '" />';        
                                }
                                else if ($shortform)
                                {
                                        $html .= $phrase['_pending_payment'];        
                                }
                                else
                                {
                                        $html .= $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_the_transaction_associated_with_this_listing_has_not_been_paid'];
                                }
                                
                                if ($showlinktopayment)
                                {
                                        $html = '<div class="smaller gray">[ <span class="blue"><a href="' . $ilpage['invoicepayment'] . '?cmd=view&amp;id=' . $invoiceid . '" title="' . $phrase['_invoice_id'] . ' #' . $invoiceid . ' - ' . $phrase['_the_transaction_associated_with_this_listing_has_not_been_paid'] . '">' . $phrase['_unpaid'] . '</a></span> ]</div>';
                                }
                        }
                }
                
                return $html;
        }
        
        /**
        * Function to print out the bid amount taking into consideration sealed and blind bidding privacy filters
        *
        * @param      string       bid privacy details
        * @param      integer      bid amount
        * @param      integer      bidder user id
        * @param      integer      owner user id
        *
        * @return     string       HTML formatted representation of the amount
        */
        function fetch_bid_amount($bid_details = '', $bidamount = 0, $bidderid = 0, $ownerid = 0)
        {
                global $ilance, $phrase;
                
                // no bidding privacy
                if ($bid_details == 'open' OR $bid_details == 'blind')
                {
                        if (!empty($_SESSION['ilancedata']['user']['userid']) AND $ilance->subscription->check_access($_SESSION['ilancedata']['user']['userid'], 'enablecurrencyconversion') == 'yes')
                        {
                                $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                        }
                        else
                        {
                                $bidamount = $ilance->currency->format($bidamount);
                        }
                }
                // sealed bidding: guest
                else if ($bid_details == 'sealed' AND empty($_SESSION['ilancedata']['user']['userid']))
                {
                        $bidamount = '= ' . $phrase['_sealed'] . ' =';
                }
                // sealed bidding: member
                else if ($bid_details == 'sealed' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] != $bidderid AND $_SESSION['ilancedata']['user']['userid'] != $ownerid)
                {
                        $bidamount = '= ' . $phrase['_sealed'] . ' =';
                }
                // sealed bidding: actual bidder
                else if ($bid_details == 'sealed' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $bidderid AND $_SESSION['ilancedata']['user']['userid'] != $ownerid)
                {
                        if ($ilance->subscription->check_access($_SESSION['ilancedata']['user']['userid'], 'enablecurrencyconversion') == 'yes')
                        {
                                $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                        }
                        else
                        {
                                $bidamount = $ilance->currency->format($bidamount);
                        }
                }
                // sealed bidding: owner
                else if ($bid_details == 'sealed' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $ownerid)
                {
                        $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                }
                // sealed bidding: admin
                else if ($bid_details == 'sealed' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['isadmin'] == '1')
                {
                        $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                }
                // full privacy: guests
                else if ($bid_details == 'full' AND empty($_SESSION['ilancedata']['user']['userid']))
                {
                        $bidamount = '= ' . $phrase['_sealed'] . ' =';
                }
                // full privacy: members
                else if ($bid_details == 'full' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] != $bidderid AND $_SESSION['ilancedata']['user']['userid'] != $ownerid)
                {
                        $bidamount = '= ' . $phrase['_sealed'] . ' =';
                }
                // full privacy: actual bidder
                else if ($bid_details == 'full' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $bidderid)
                {
                        $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                }
                // full privacy: owner
                else if ($bid_details == 'full' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $ownerid)
                {
                        $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                }
                // full privacy: admin
                else if ($bid_details == 'full' AND !empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['isadmin'] == '1')
                {
                        $bidamount = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $bidamount);
                }
                
                return $bidamount;
        }
        
        /**
        * Function to fetch and return an array of the average, lowest and highest bid amounts placed on a listing event
        *
        * @param      string       bid privacy details
        * @param      integer      listing id
        * @param      integer      owner user id
        *
        * @return     array        Mixed array of amounts requested
        */
        function fetch_average_lowest_highest_bid_amounts($bid_details = '', $project_id = 0, $ownerid = 0)
        {
                global $ilance, $phrase;
                
                // format average bid amount
                $sel_bids_av = $ilance->db->query("
                        SELECT AVG(bidamount) AS average, MIN(bidamount) AS lowest, MAX(bidamount) AS highest 
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($project_id) . "'
                                AND bidstate != 'retracted'
                                AND bidstatus != 'declined'
                        LIMIT 1
                ", 0, null, __FILE__, __LINE__);
                $res_bids_av = $ilance->db->fetch_array($sel_bids_av);
                
                // #### BID PRIVACY ####################################
                if ($bid_details == 'open')
                {
                        $bidprivacy = $phrase['_no_privacy'];
                        
                        $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                        $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                        $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);
                }
                // #### BLIND BIDS #####################################
                else if ($bid_details == 'blind')
                {
                        $bidprivacy = $phrase['_blind_bidding'];
                        
                        $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                        $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                        $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);        
                }
                // #### SEALED BIDS ####################################
                else if ($bid_details == 'sealed')
                {
                        $bidprivacy = $phrase['_sealed_bidding'];
                        
                        // #### OWNER VIEWING ##########################
                        if (!empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $ownerid)
                        {
                                $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                                $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                                $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);
                        }
                        // #### ADMIN VIEWING ##########################
                        else if (!empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['isadmin'] == '1')
                        {
                                $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                                $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                                $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);
                        }
                        // #### EVERYONE ELSE ##########################
                        else
                        {
                                $average = '= ' . $phrase['_sealed'] . ' =';
                                $lowest = '= ' . $phrase['_sealed'] . ' =';
                                $highest = '= ' . $phrase['_sealed'] . ' =';
                        }
                }
                // #### FULL PRIVACY (SEALED + BLIND) ##################
                else if ($bid_details == 'full')
                {
                        $bidprivacy = $phrase['_full_privacy'];
                        
                        // #### OWNER VIEWING ##########################
                        if (!empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['userid'] == $ownerid)
                        {
                                $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                                $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                                $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);
                        }
                        // #### ADMIN VIEWING ##########################
                        else if (!empty($_SESSION['ilancedata']['user']['userid']) AND $_SESSION['ilancedata']['user']['isadmin'] == '1')
                        {
                                $average = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['average']);
                                $lowest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['lowest']);
                                $highest = print_currency_conversion($_SESSION['ilancedata']['user']['currencyid'], $res_bids_av['highest']);
                        }
                        // #### EVERYONE ELSE ##########################
                        else
                        {
                                $average = '= ' . $phrase['_sealed'] . ' =';
                                $lowest = '= ' . $phrase['_sealed'] . ' =';
                                $highest = '= ' . $phrase['_sealed'] . ' =';
                        }
                }
                
                return array(
                        'bidprivacy' => $bidprivacy,
                        'average' => $average,
                        'lowest' => $lowest,
                        'highest' => $highest
                );
        }
        
        /**
        * Function to fetch and return an array of the lowest bidder details for a particular listing event
        *
        * @param      integer      listing id
        * @param      integer      owner user id
        * @param      string       bid privacy details
        *
        * @return     array        Mixed array of amounts requested
        */
        function fetch_lowest_bidder_info($project_id = 0, $user_id = 0, $bid_details = '')
        {
                global $ilance, $ilconfig, $show, $phrase;
                
                $lowbidder = $lowbidderid = '';
                $show['lowbidder_active'] = false;
                
                $sql_lowest_bidder = $ilance->db->query("
                        SELECT user_id
                        FROM " . DB_PREFIX . "project_bids
                        WHERE project_id = '" . intval($project_id) . "'
                        ORDER BY bidamount ASC
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql_lowest_bidder) > 0)
                {
                        $res_lowest_bidder = $ilance->db->fetch_array($sql_lowest_bidder);
                        
                        // format lowest, average and highest bid amounts
                        $sel_bids_av = $ilance->db->query("
                                SELECT AVG(bidamount) AS average, MIN(bidamount) AS lowest, MAX(bidamount) AS highest 
                                FROM " . DB_PREFIX . "project_bids
                                WHERE project_id = '" . intval($project_id) . "'
                                        AND bidstate != 'retracted'
                                        AND bidstatus != 'declined'
                                ORDER BY lowest 
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sel_bids_av) > 0)
                        {
                                $res_bids_av = $ilance->db->fetch_array($sel_bids_av);
                                
                                $show['lowbidder_active'] = true;
                                
                                $lowbidderid = $res_lowest_bidder['user_id'];
                                $lowbidder = print_username($lowbidderid, 'href');
                                
                                // fetch lowest bidder name
                                if ($bid_details == 'blind' OR $bid_details == 'full')
                                {
                                        if (!empty($_SESSION['ilancedata']['user']['userid']))
                                        {
                                                if ($user_id != $_SESSION['ilancedata']['user']['userid'])
                                                {
                                                        $lowbidder = '= ' . $phrase['_blind'] . ' =';
                                                }        
                                        }
                                        else
                                        {
                                                $lowbidder = '= ' . $phrase['_blind'] . ' =';
                                        }
                                }
                                
                                return array(
                                        'lowbidderid' => $lowbidderid,
                                        'lowbidder' => $lowbidder
                                );
                        }
                }
                
                return array(
                        'lowbidderid' => '',
                        'lowbidder' => ''
                );
        }
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.bid.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>