<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

/**
* Category class to perform the majority of category functions in ILance.
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class categories
{
        var $cats = array();
        var $fetch = array();
        var $subs = null;
        var $cat_map = array();
        var $buildarray = false;
        
        /**
        * Constructor
        *
        * @param       bool         build the array on load? default = false
        * @param       string       category type (service, product, experts, portfolio, serviceprovider)
        * @param       string       short language identifier? default = user session lang.
        * @param       integer      category mode (0 = all,  1 = portfolio, 2 = rss, 3 = newsletters)
        * @param       boolean      enable proper category sorting (builds $this->cat & $this->fetch internally) (default true)
        */
        function categories($buildarray = false, $cattype = 'service', $slng = 'eng', $categorymode = 0, $propersort = true)
        {
                if ($buildarray)
                {
                        $this->build_array($cattype, $slng, $categorymode, $propersort);
                }
        }
        
        /**
        * Function to fetch and build the array of the category structure.  This will internally build our $ilance->categories->fetch[] array.
        * Additionally this function can sort the array using an internal sorting method if required.
        *
        * @param       string       category type (service/product)
        * @param       string       short language identifier (default eng)
        * @param       string       category mode (0 = all,  1 = portfolio, 2 = rss, 3 = newsletters)
        * @param       bool         enable proper category/parent/child sorting on the fly? (default yes)
        *
        * @return      array        Returns category array structure
        */
        function build_array($cattype = 'service', $slng = 'eng', $categorymode = 0, $propersort = true, $extra = '', $extra2 = '')
        {
                global $ilance;

                $ilance->timer->start();
                
                // let other scripts know we've built the array cache already
                $this->buildarray = true;

                if ($cattype == 'stores')
                {
                        $query = $ilance->db->query("
                                SELECT cid, storeid, category_name AS title, parentid, type, canpost, views, itemcount AS auctioncount, visible, sort
                                FROM " . DB_PREFIX . "stores_category
                                WHERE (storeid = '" . intval($extra) . "' OR cid = '" . intval($extra2) . "')        
                                ORDER BY cid, sort ASC
                        ", 0, null, __FILE__, __LINE__);
                        while ($categories = $ilance->db->fetch_array($query, DB_ASSOC))
                        {
                                //$this->fetch["$slng"]["$cattype"]["$categories[cid]"] = $categories;
                                $this->fetch["$slng"]["$cattype"][] = $categories;
                        }
                        unset($categories);
                        //print_r($this->fetch); exit;
                }
                else if ($cattype == 'storesmain')
                {
                        $query = $ilance->db->query("
                                SELECT cid, storeid, category_name AS title, parentid, type, canpost, views, itemcount AS auctioncount, visible, sort
                                FROM " . DB_PREFIX . "stores_category
                                WHERE storeid = '-1'
                                ORDER BY sort ASC
                        ", 0, null, __FILE__, __LINE__);
                        while ($categories = $ilance->db->fetch_array($query, DB_ASSOC))
                        {
                                //$this->fetch["$slng"]["$cattype"]["$categories[cid]"] = $categories;
                                $this->fetch["$slng"]["stores"][] = $categories;
                        }
                        unset($categories);
                        //print_r($this->fetch); exit;
                }
                else
                {
                        $cattype = ($cattype == 'service' OR $cattype == 'servicecatmap' OR $cattype == 'experts' OR $cattype == 'portfolio' OR $cattype == 'serviceprovider') ? 'service' : 'product';
                        
                        $query = $ilance->db->query("
                                SELECT *, title_$slng AS title, description_$slng AS description
                                FROM " . DB_PREFIX . "categories
                                ORDER BY sort ASC
                        ", 0, null, __FILE__, __LINE__);
                        while ($categories = $ilance->db->fetch_array($query, DB_ASSOC))
                        {
                                $this->fetch["$slng"]["$categories[cattype]"]["$categories[cid]"] = $categories;
                        }                
                        unset($categories);  
                }
                
                if ($propersort)
                {
                        $ilance->timer->stop();
                        DEBUG("build_array(\$slng = $slng, \$cattype = $cattype, \$categorymode = $categorymode) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                        return $this->propersort($slng, $cattype, $categorymode);
                }
                else
                {
                        return $this->fetch;
                }

                $ilance->timer->stop();
                DEBUG("build_array(\$slng = $slng, \$cattype = $cattype, \$categorymode = $categorymode) in " . $ilance->timer->get() . " seconds", 'FUNCTION');                        
        }
        
        /**
        * Function to fetch the entire category cache and make it a proper formatted result set for various areas within ILance
        *
        * @param       string       short language identifier
        * @param       string       category type (service or product)
        * @param       integer      category mode (0 = all,  1 = portfolio, 2 = rss, 3 = newsletters)
        */
        function propersort($slng = 'eng', $cattype = '', $categorymode = 0)
        {
                global $ilance;
                
                $ilance->timer->start();
                $arr = array();
                
                if (!empty($this->fetch["$slng"]["$cattype"]))
                {
                        foreach ($this->fetch["$slng"]["$cattype"] AS $cid => $array)
                        {
                                if (isset($categorymode))
                                {
                                        // portfolio
                                        if ($categorymode == '1')
                                        {
                                                if ($array['portfolio'])
                                                {
                                                        $arr[] = $array;                
                                                }
                                        }
                                        // rss feeds
                                        else if ($categorymode == '2')
                                        {
                                                if ($array['xml'])
                                                {
                                                        $arr[] = $array;                
                                                }       
                                        }
                                        // newsletters
                                        else if ($categorymode == '3')
                                        {
                                                if ($array['newsletter'])
                                                {
                                                        $arr[] = $array;                
                                                }         
                                        }
                                        else
                                        {
                                                $arr[] = $array;
                                        }
                                }
                                else
                                {
                                        $arr[] = $array;
                                }
                        }
                        
                        // sort results properly and creates $this->cats or $ilance->categores->cats
                        $this->get_cats($arr, $parentid = 0, $level = 1);
                }

                $ilance->timer->stop();
                DEBUG("propersort(\$slng = $slng, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $arr;
        }
        
        /**
        * Function to fetch the title of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      mixed        Returns category array structure (or All Categories) text otherwise
        */
        function title($slng = 'eng', $cattype = 'service', $cid = 0)
        {
                global $phrase;
                
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]) OR !empty($this->fetch["$slng"]["$cattype"]["$cid"]) AND $this->fetch["$slng"]["$cattype"]["$cid"] != '0')
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['title'];
                }
                
                return $phrase['_unknown'];
        }
        
        /**
        * Function to fetch the visibility of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      mixed        Returns true or false based on visibility
        */
        function visible($slng = 'eng', $cattype = 'service', $cid = 0)
        {
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]) OR !empty($this->fetch["$slng"]["$cattype"]["$cid"]) AND $this->fetch["$slng"]["$cattype"]["$cid"] != '0')
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['visible'];
                }
                
                return 1;
        }
        
        /**
        * Function to fetch the true level of a category within the category structure.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      mixed        Returns category level for the selected category
        */
        function level($slng = 'eng', $cattype = '', $cid = 0)
        {
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]))
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['level'];
                }
                
                return 0;
        }
        
        /**
        * Function to determine if a specific product category has proxy bidding enabled.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      mixed        Returns category array structure (or All Categories) text otherwise
        */
        function useproxybid($slng = 'eng', $cattype = '', $cid = 0)
        {
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]) OR !empty($this->fetch["$slng"]["$cattype"]["$cid"]) AND $this->fetch["$slng"]["$cattype"]["$cid"] != '0')
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['useproxybid'];
                }
                
                return 0;
        }
        
        /**
        * Function to fetch the description text of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      mixed        Returns category array structure (or All Categories) text otherwise
        */
        function description($slng = 'eng', $cattype = '', $cid = 0)
        {
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]) OR !empty($this->fetch["$slng"]["$cattype"]["$cid"]) AND $this->fetch["$slng"]["$cattype"]["$cid"] != '0')
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['description'];
                }
                
                return '';
        }
        
        /**
        * Function to fetch the meta tag keywords text of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        * @param       boolean      add comma after? (default false)
        * @param       boolean      show input keywords? (default false)
        *
        * @return      mixed        Returns category array structure (or All Categories) text otherwise
        */
        function keywords($slng = 'eng', $cattype = '', $cid = 0, $commaafter = false, $showinputkeywords = false)
        {
                global $ilance;
                
                $keywordbit = $text = $bit = '';
                
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]) OR !empty($this->fetch["$slng"]["$cattype"]["$cid"]) AND $this->fetch["$slng"]["$cattype"]["$cid"] != '0')
                {
                        if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]['keywords']))
                        {
                                if ($commaafter)
                                {
                                        $bit = ', ';
                                }
                                $text = $this->fetch["$slng"]["$cattype"]["$cid"]['keywords'] . $bit;
                        }
                }
                if ($showinputkeywords)
                {
                        if (!empty($ilance->GPC['q']))
                        {
                                $keywordbit = htmlspecialchars($ilance->GPC['q']) . ', ';
                        }
                }
                if (!empty($text) AND $commaafter)
                {
                        $text = mb_substr($text, 0, -2);
                }
                
                return $keywordbit . $text;
        }
        
        /**
        * Function to fetch the parentid of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      integer      Returns parentid of a category or 0 otherwise
        */
        function parentid($slng = 'eng', $cattype = '', $cid = 0)
        {
                global $ilance, $myapi;
                
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]))
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['parentid'];
                }
                
                return 0;
        }
        
        /**
        * Function to fetch the category type of a category.
        *
        * @param       string       short language identifier (default eng)
        * @param       integer      category id
        *
        * @return      string        Returns the category type (service/product)
        */
        function cattype($slng = 'eng', $cid = 0)
        {
                if (!empty($this->fetch["$slng"]))
                {
                        foreach ($this->fetch as $slng)
                        {
                                foreach ($slng as $cattype)
                                {
                                        if (!empty($cattype[$cid]['cid']) AND $cattype[$cid]['cid'] == $cid)
                                        {
                                                return $cattype[$cid]['cattype'];
                                                break;
                                        }
                                }                       
                        }
                }
        }
        
        /**
        * Function to determine if a category is a postable auction category.
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      string       Returns true or false response
        */
        function can_post($slng = 'eng', $cattype = '', $cid = 0)
        {
                global $ilance, $myapi;
                
                ($apihook = $ilance->api('categories_can_post_start')) ? eval($apihook) : false;
                
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]))
                {
                        return $this->fetch["$slng"]["$cattype"]["$cid"]['canpost'];
                }
                
                return false;
        }
        
        /**
        * Function to determine if a category is proxy bid ready (if proxy bidding is enabled)
        *
        * @param       string       short language identifier (default eng)
        * @param       string       category type (service/product)
        * @param       integer      category id
        *
        * @return      string       Returns true or false response
        */
        function proxy_bid_ready($slng = 'eng', $cattype = '', $cid = 0)
        {
                global $ilance, $myapi;
                
                $useproxybid = false;
                $incrementgroup = '';
                
                if (!empty($this->fetch["$slng"]["$cattype"]["$cid"]))
                {
                        $useproxybid = $this->fetch["$slng"]["$cattype"]["$cid"]['useproxybid'];
                }
                
                if ($useproxybid)
                {
                        $incrementgroup = $this->fetch["$slng"]["$cattype"]["$cid"]['incrementgroup'];
                        if ($incrementgroup != '0' AND !empty($incrementgroup) AND $incrementgroup != '')
                        {
                                // increments exist .. all good!
                                return true;
                        }
                        else
                        {
                                return false;
                        }
                }
                
                return true;
        }
        
        /**
        * Function to fetch a category count based on a set of search options per specific category.
        * For example, say Web Design normally by itself has 300 auctions.  If a user enters a search for keyword
        * "template" and 35 results were found, "35" would be that category's counter: Web Design (35) even though
        * the current category selected is "Programming".  This can be considered a mini-internal search engine for
        * category listing counts.
        *
        * @param       string       category id
        * @param       string       category type (service/product/serviceprovider)
        * @param       integer      search sql query (array)
        *
        * @return      integer      Returns category counter number
        */
        function bestmatch_auction_count($cid = 0, $cattype = 'service', $sqlquery = array())
        {
                global $ilance;
                
                $ilance->timer->start();
                
                $sqlquery['cidfield'] = 'p.cid';
                
                $count = 0;
                if (is_array($sqlquery))
                {
                        $cids = $this->fetch_children_ids($cid, $cattype);
                        if (!empty($cids))
                        {
                                $subcategorylist = $cid . ',' . $cids;
                                $sqlquery['categories'] = "AND (FIND_IN_SET($sqlquery[cidfield], '" . $subcategorylist . "'))";
                        }
                        else
                        {
                                $sqlquery['categories'] = "AND (FIND_IN_SET($sqlquery[cidfield], '" . $cid . "'))";        
                        }                        
                        
                        if ($cattype == 'serviceprovider')
                        {
                                $sql = "$sqlquery[select] $sqlquery[keywords] $sqlquery[categories] $sqlquery[options] $sqlquery[location] $sqlquery[radius] $sqlquery[userquery] $sqlquery[hidequery] $sqlquery[pricerange] $sqlquery[genrequery] $sqlquery[profileanswersquery] $sqlquery[skillsquery] $sqlquery[groupby] $sqlquery[orderby]";        
                        }
                        else
                        {
                                $sql = "$sqlquery[select] $sqlquery[timestamp] $sqlquery[projectstatus] $sqlquery[keywords] $sqlquery[categories] $sqlquery[projectdetails] $sqlquery[projectstate] $sqlquery[options] $sqlquery[pricerange] $sqlquery[location] $sqlquery[radius] $sqlquery[userquery] $sqlquery[hidequery] $sqlquery[genrequery] $sqlquery[groupby] $sqlquery[orderby]";
                        }
                        
                        $rows = $ilance->db->query($sql);
                        $count = $ilance->db->num_rows($rows);
                }
                
                $ilance->timer->stop();
                DEBUG("bestmatch_auction_count(\$cid = $cid, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $count;
        }
        
        /**
        * Function to print a recursive category breadcrumb trail.
        *
        * @param       integer      category id
        * @param       string       category type (service/product)
        * @param       string       short language identifier (default eng)
        * @param       boolean      no urls flag (default off)
        * @param       string       custom url (optional)
        * @param       boolean      enable seo urls (default off)
        *
        * @return      string       Returns HTML formatted version of the breadcrumb trail
        */
        function recursive($cid = 0, $cattype = 'service', $slng = 'eng', $nourls = 0, $customurl = '', $seourls = 0)
        {
                global $ilance, $myapi, $ilconfig, $navcrumb, $ilpage, $phrase, $htmlx, $show, $storeid;
                
                $ilance->timer->start();
                
                $sid = isset($ilance->GPC['cid']) ? intval($ilance->GPC['cid']) : 0;
                
                if ((empty($cattype) OR $cattype == '') OR ($cattype == 'service' OR $cattype == 'product') OR ($cattype == 'servicecatmap' OR $cattype == 'productcatmap'))
                {
                        $dbtable = DB_PREFIX . "categories";
                        $cidname = 'cid';
                        $titlefield = "title_$slng";    
                }
                
                ($apihook = $ilance->api('categories_recursive_start')) ? eval($apihook) : false;
                
                $htmlx = '';
                $sql = $ilance->db->query("
                        SELECT $cidname AS cid, $titlefield AS title, parentid
                        FROM $dbtable
                        WHERE $cidname = '" . intval($cid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        $this->recursive($res['parentid'], $cattype, $slng, $nourls, $customurl, $seourls);
                        
                        if ($cattype == 'service' OR $cattype == 'servicecatmap')
                        {
                                if ($nourls)
                                {
                                        $htmlx .= stripslashes($res['title']). ' > ';
                                }
                                else 
                                {
                                        // we using custom url?
                                        if (!empty($customurl))
                                        {
                                                if ($sid == $cid)
                                                {
                                                        $htmlx .= stripslashes($res['title']).' > ';  					
                                                }
                                                else 
                                                {
                                                        $htmlx .= '<a href="' . $customurl . '&amp;cid=' . $res['cid'] . '">' . stripslashes($res['title']) . '</a> > ';  										
                                                }
                                        }
                                        else 
                                        {
                                                if ($seourls)
                                                {
                                                        if ($sid == $cid)
                                                        {
                                                                $htmlx .= stripslashes($res['title']) . ' > ';
                                                        }
                                                        else 
                                                        {
                                                                if ($cattype == 'service')
                                                                {
                                                                        $seotype = 'servicecat';
                                                                }
                                                                else if ($cattype == 'servicecatmap')
                                                                {
                                                                        $seotype = 'servicecatmap';    
                                                                }
                                                                
                                                                $htmlx .= construct_seo_url($seotype, $res['cid'], $auctionid = 0, stripslashes($res['title']), $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0) . ' > ';
                                                        }
                                                                   
                                                }
                                                else
                                                {                                        
                                                        if ($sid == $cid)
                                                        {
                                                                $htmlx .= stripslashes($res['title']) . ' > ';
                                                        }
                                                        else 
                                                        {
                                                                $htmlx .= '<a href="' . $ilpage['rfp'] . '?cid=' . $res['cid'] . '">' . stripslashes($res['title']) . '</a> > ';
                                                        }
                                                }
                                        }
                                }
                        }
                        
                        if ($cattype == 'product' OR $cattype == 'productcatmap')
                        {
                                if ($nourls)
                                {
                                        $htmlx .= stripslashes($res['title']) . ' > ';
                                }
                                else 
                                {
                                        if (!empty($customurl))
                                        {
                                                if ($sid == $cid)
                                                {
                                                        $htmlx .= stripslashes($res['title']) . ' > ';  					
                                                }
                                                else 
                                                {
                                                        $htmlx .= '<a href="' . $customurl . '&amp;cid=' . $res['cid'] . '">' . stripslashes($res['title']) . '</a> > ';  					
                                                }
                                        }
                                        else 
                                        {
                                                if ($seourls)
                                                {
                                                        if ($sid == $cid)
                                                        {
                                                                $htmlx .= stripslashes($res['title']) . ' > ';
                                                        }
                                                        else 
                                                        {
                                                                if ($cattype == 'product')
                                                                {
                                                                        $seotype = 'productcat';
                                                                }
                                                                else if ($cattype == 'productcatmap')
                                                                {
                                                                        $seotype = 'productcatmap';    
                                                                }
                                                                
                                                                $htmlx .= construct_seo_url($seotype, $res['cid'], $auctionid = 0, stripslashes($res['title']), $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0) . ' > ';
                                                        }
                                                }
                                                else
                                                {
                                                        if ($sid == $cid)
                                                        {
                                                                $htmlx .= stripslashes($res['title']) . ' > ';
                                                        }
                                                        else 
                                                        {
                                                                $htmlx .= '<a href="' . $ilpage['merch'] . '?cid=' . $res['cid'] . '">' . stripslashes($res['title']) . '</a> > ';
                                                        }   
                                                }
                                        }
                                }
                        }
                        
                        ($apihook = $ilance->api('categories_recursive_conditions')) ? eval($apihook) : false;
                        
                }
                
                $html = mb_substr($htmlx, 0, -3);
                
                // can we show category checkmark icon (used in category selection menus)?
                if (isset($show['submit']) AND $show['submit'])
                {
                        $html .= ' <img vspace="-2" hspace="3" src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/checkmark.gif" border="0" alt="" width="16" height="16" />';
                }
                
                ($apihook = $ilance->api('categories_recursive_end')) ? eval($apihook) : false;
                
                $ilance->timer->stop();
                DEBUG("recursive(\$cid = $cid, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $html;
        }
                
        /**
        * Function to generate the main header breadcrumb category trail.
        *
        * @param       integer      category id
        * @param       string       category type (service/product)
        * @param       string       short language identifier (default eng)
        * @param       array        category cache (prevents hitting the db again)
        *
        * @return      array        Returns array $navcrumb breadcrumb trail
        */        
        function breadcrumb($cid = 0, $cattype = 'service', $slng = 'eng', $categorycache = array())
        {
                global $ilance, $ilconfig, $myapi, $navcrumb, $ilpage, $phrase;
                
                $ilance->timer->start();
                
                /*
                 Array
                (
                    [0] => Array
                        (
                            [cid] => 1
                            [parentid] => 0
                */
                
                ($apihook = $ilance->api('breadcrumb_start')) ? eval($apihook) : false;
                
                $results = $categorycache;
                $resultscount = count($results);
                for ($i = 0; $i < $resultscount; $i++)
                {
                        if ($results[$i]['cid'] == $cid)
                        {
                                $this->breadcrumb($results[$i]['parentid'], $cattype, $slng, $results);
                                
                                if ($cid == 0 AND defined('LOCATION') AND LOCATION == 'search')
                                {
                                        if (!empty($ilance->GPC['q']))
                                        {
                                                $navcrumb["$ilpage[search]?q=" . htmlspecialchars($ilance->GPC['q'])] = $phrase['_keywords'] . ': ' . htmlspecialchars($ilance->GPC['q']);
                                        }
                                        
                                        return $navcrumb;
                                }
                                
                                if ($cattype == 'service')
                                {
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        
                                        if ($ilconfig['globalauctionsettings_seourls'])
                                        {
                                                $url = construct_seo_url('servicecatplain', $cid, $auctionid = 0, $title, $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0);
                                                $navcrumb["$url"] = $title;
                                                unset($url);
                                        }
                                        else
                                        {
                                                $navcrumb["$ilpage[rfp]?cid=" . $cid] = $title;   
                                        }
                                }
                                else if ($cattype == 'servicecatmap')
                                {
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        
                                        if ($ilconfig['globalauctionsettings_seourls'])
                                        {
                                                $url = construct_seo_url('servicecatmapplain', $cid, $auctionid = 0, $title, $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0);
                                                $catmap = print_seo_url($ilconfig['servicecatmapidentifier']);
                                                $catmap2 = print_seo_url($ilconfig['categoryidentifier']);
                                                
                                                $navcrumb[HTTP_SERVER . $catmap2] = $phrase['_categories'];
                                                $navcrumb["$catmap"] = $phrase['_browse'];
                                                $navcrumb["$url"] = $title;
                                                unset($catmap, $catmap2, $url);
                                        }
                                        else
                                        {
                                                $navcrumb["$ilpage[main]?cmd=categories"] = $phrase['_categories'];
                                                $navcrumb["$ilpage[rfp]?cmd=listings"] = $phrase['_browse'];
                                                $navcrumb["$ilpage[rfp]?cmd=listings&amp;cid=" . $cid] = $title;  
                                        }
                                }
                                else if ($cattype == 'product')
                                {
                                        $title = $this->title($slng, 'product', $cid);
                                        $parentid = $this->parentid($slng, 'product', $cid);
                                        
                                        if ($ilconfig['globalauctionsettings_seourls'])
                                        {
                                                $url = construct_seo_url('productcatplain', $cid, $auctionid = 0, $title, $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0);
                                                $navcrumb["$url"] = $title;
                                                unset($url);
                                        }
                                        else
                                        {
                                                $navcrumb["$ilpage[merch]?cid=" . $cid] = $title;
                                        }
                                }
                                else if ($cattype == 'productcatmap')
                                {
                                        $title = $this->title($slng, 'product', $cid);
                                        $parentid = $this->parentid($slng, 'product', $cid);
                                        
                                        if ($ilconfig['globalauctionsettings_seourls'])
                                        {
                                                $url = construct_seo_url('productcatmapplain', $cid, $auctionid = 0, $title, $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0);
                                                $catmap = print_seo_url($ilconfig['productcatmapidentifier']);
                                                $catmap2 = print_seo_url($ilconfig['categoryidentifier']);
                                                
                                                $navcrumb[HTTP_SERVER . $catmap2] = $phrase['_categories'];
                                                $navcrumb["$catmap"] = $phrase['_buy'];
                                                $navcrumb["$url"] = $title;
                                                unset($catmap, $catmap2, $url);
                                        }
                                        else
                                        {
                                                $navcrumb["$ilpage[main]?cmd=categories"] = $phrase['_categories'];
                                                $navcrumb["$ilpage[merch]?cmd=listings"] = $phrase['_buy'];
                                                $navcrumb["$ilpage[merch]?cmd=listings&amp;cid=" . $cid] = $title;
                                        }
                                }
                                else if ($cattype == 'experts')
                                {
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        
                                        if ($ilconfig['globalauctionsettings_seourls'])
                                        {
                                                $url = construct_seo_url('serviceprovidercatplain', $cid, $auctionid = 0, $title, $customlink = '', $bold = 0, $searchquestion = '', $questionid = 0, $answerid = 0);
                                                $navcrumb["$ilpage[search]?mode=experts"] = $phrase['_browse'] . ' ' . $phrase['_experts'];
                                                $navcrumb["$url"] = $title;
                                                unset($url);
                                        }
                                        else
                                        {
                                                $navcrumb["$ilpage[search]?mode=experts"] = $phrase['_browse'] . ' ' . $phrase['_experts'];
                                                $navcrumb["$ilpage[search]?mode=experts&cid=" . $cid] = $title;   
                                        }
                                }
                                else if ($cattype == 'servicerfp')
                                {
                                        // used to lead nav crumb to buying services category selection menus
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        $navcrumb["$ilpage[buying]?cmd=rfp&amp;cid=" . $cid] = $title;
                                }
                                else if ($cattype == 'productrfp')
                                {
                                        // used to lead nav crumb to selling products category selection menus
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        $navcrumb["$ilpage[buying]?cmd=rfp&amp;state=product&amp;cid=" . $cid] = $title;
                                }
                                else if ($cattype == 'portfolio')
                                {
                                        // used to take customer to specific rfp category menu
                                        $title = $this->title($slng, 'service', $cid);
                                        $parentid = $this->parentid($slng, 'service', $cid);
                                        $navcrumb["$ilpage[portfolio]?cid=" . $cid] = $title;
                                }
                        
                        }
                }
                
                ($apihook = $ilance->api('breadcrumb_end')) ? eval($apihook) : false;
                
                $ilance->timer->stop();
                DEBUG("breadcrumb(\$cid = $cid, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
        }        
        
        /**
        * Function to fetch the profile verification count for a user within a particular category.
        *
        * @param       integer      category id
        * @param       integer      user id
        *
        * @return      integer      Returns xxx
        */
        function fetch_profile_verification_count($cid = 0, $userid = 0)
        {
                global $ilance, $myapi;
                
                $count = 0;
                $groups = $ilance->db->query("
                        SELECT groupid
                        FROM " . DB_PREFIX . "profile_groups
                        WHERE cid = '".intval($cid)."'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($groups) > 0)
                {
                        // for every group, fetch questionid's
                        while ($group = $ilance->db->fetch_array($groups))
                        {
                                $questions = $ilance->db->query("
                                        SELECT questionid
                                        FROM " . DB_PREFIX . "profile_questions
                                        WHERE groupid = '".$group['groupid']."'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($questions) > 0)
                                {
                                        // for every questionid in this group, count answers
                                        while ($res = $ilance->db->fetch_array($questions))
                                        {
                                                $verified = $ilance->db->query("
                                                        SELECT answerid
                                                        FROM " . DB_PREFIX . "profile_answers
                                                        WHERE questionid = '".$res['questionid']."'
                                                            AND user_id = '".intval($userid)."'
                                                            AND isverified = '1'
                                                            AND visible = '1'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($verified) > 0)
                                                {
                                                        $count += 1;
                                                }
                                        }
                                }
                        }
                }
                return $count;
        }
        
        /**
        * Function to fetch the entire profile count within any category.
        *
        * @param       integer      category id
        *
        * @return      integer      Returns xxx
        */
        function fetch_profile_question_count($cid = 0)
        {
                global $ilance, $myapi;
                
                $count = 0;
                $groups = $ilance->db->query("
                        SELECT groupid, cid
                        FROM " . DB_PREFIX . "profile_groups
                        WHERE cid = '".intval($cid)."'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($groups) > 0)
                {
                        while ($group = $ilance->db->fetch_array($groups))
                        {
                                // count questions in this profile group
                                $questions = $ilance->db->query("
                                        SELECT COUNT(*) AS count
                                        FROM " . DB_PREFIX . "profile_questions
                                        WHERE groupid = '".$group['groupid']."'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($questions) > 0)
                                {
                                        $res = $ilance->db->fetch_array($questions);
                                        $count += $res['count'];
                                }
                                
                        }
                }
                
                return $count;
        }
        
        /**
        * Function to fetch the number of portfolio items within a particular category
        *
        * @param       integer      category id
        *
        * @return      integer      Returns xxx
        */
        function fetch_portfolio_count($cid = 0)
        {
                global $ilance, $myapi;
                
                $count = 0;
                $items = $ilance->db->query("
                        SELECT COUNT(*) AS count
                        FROM " . DB_PREFIX . "attachment
                        WHERE category_id = '".intval($cid)."'
                            AND attachtype = 'portfolio'
                            AND visible = '1'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($items) > 0)
                {
                        $item = $ilance->db->fetch_array($items);
                        $count = intval($item['count']);
                }
                
                return $count;
        }
        
        /**
        * Function to fetch the answer count for the profile question within a category for a particular user.
        *
        * @param       integer      category id
        * @param       integer      user id
        *
        * @return      integer      Returns xxx
        */
        function fetch_profile_answer_count($cid = 0, $userid = 0)
        {
                global $ilance, $myapi;
                
                $count = 0;
                
                // select groupid's this category id belongs to
                $groups = $ilance->db->query("
                        SELECT groupid
                        FROM " . DB_PREFIX . "profile_groups
                        WHERE cid = '".intval($cid)."'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($groups) > 0)
                {
                        // for every group, fetch questionid's
                        while ($group = $ilance->db->fetch_array($groups))
                        {
                                $questions = $ilance->db->query("
                                        SELECT questionid
                                        FROM " . DB_PREFIX . "profile_questions
                                        WHERE groupid = '".$group['groupid']."'
                                ", 0, null, __FILE__, __LINE__);
                                if ($ilance->db->num_rows($questions) > 0)
                                {
                                        // for every questionid in this group, count answers
                                        while ($res = $ilance->db->fetch_array($questions))
                                        {
                                                $answers = $ilance->db->query("
                                                        SELECT answerid
                                                        FROM " . DB_PREFIX . "profile_answers
                                                        WHERE questionid = '".$res['questionid']."'
                                                            AND user_id = '".intval($userid)."'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($answers) > 0)
                                                {
                                                        $count += 1;
                                                }
                                        }
                                }
                        }
                }
                
                return $count;
        }
        
        /**
        * Function to process and fetch categories recusively
        *
        * @param       array        category results array
        * @param       integer      parent id
        * @param       integer      category level
        *
        * @return      nothing
        */
        function get_cats_recursive($result, $parentid = 0, $level = 1)
        {
                global $ilance;
                //$ilance->timer->start();
                
                /*
                $result = Array
                (
                    [0] => Array
                        (
                            [cid] => 3
                            [parentid] => 0
                            [etc....]
                        )
                */
                
                $count = count($result);
                for ($i = 0; $i < $count; $i++)
                {
                        if ($result[$i]['parentid'] == $parentid)
                        {
                                $result[$i]['level'] = $level;
                                $this->cats[] = $result[$i];
                                $this->get_cats_recursive($result, $result[$i]['cid'], $level + 1);
                        }
                }
                
                //$ilance->timer->stop();
                //DEBUG("get_cats_recursive(\$result = $result, \$parentid = $parentid, \$level = $level) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
        }
        
        /**
        * Function to process and fetch categories
        *
        * @param       array        category results array
        * @param       integer      parent id
        * @param       integer      category level
        *
        * @return      nothing
        */
        function get_cats($result, $parentid = 0, $level = 1)
        {
                global $ilance;
                //$ilance->timer->start();
                
                $this->cats = array();
                $this->tmp_cats = array();
                $this->get_cats_recursive($result, $parentid, $level);
                
                //$ilance->timer->stop();
                //DEBUG("get_cats(\$parentid = $parentid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
        }
        
        /**
        * Function to count subcategories within a given result set including a parent id
        *
        * @param       integer      xxx
        * @param       array        xxx 
        *
        * @return      nothing
        */
        function count_subs($parentid, $result)
        {
                $this->tmp_cats = array();
                $this->subs = null;
                $this->get_cats($result, $parentid, $level = 1);
                $this->subs = count($this->tmp_cats);
        }
        
        /**
        * Function to scan through the category map recursively
        *
        * @param       integer      xxx
        * @param       array        xxx 
        *
        * @return      nothing
        */
        function cat_map_recursive($cid, $result)
        {
                global $ilance;
                $ilance->timer->start();
                
                $count = count($result);
                for ($i = 0; $i < $count; $i++)
                {
                        $cats_result_[$result[$i]['cid']] = $result[$i];
                }    
                while (list($a, $b) = @each($cats_result_))
                {
                        if ($cats_result_[$cid]['parentid'] > 0 AND $cats_result_[$cid]['parentid'] == $cats_result_[$a]['cid'])
                        {
                                $this->cat_map[] = $cats_result_[$a];
                                if ($cats_result_[$a]['parentid'] > 0)
                                {
                                        $this->cat_map_recursive($cats_result_[$a]['cid'], $result);
                                }
                        }
                }
                
                $ilance->timer->stop();
                DEBUG("cat_map_recursive(\$cid = $cid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
        }
        
        /**
        * Function to build a category map based on a result set
        *
        * @param       integer      xxx
        * @param       array        xxx 
        *
        * @return      nothing
        */
        function cat_map($id, $result)
        {
                global $ilance;
                $ilance->timer->start();
                
                $this->cat_map = array();
                $this->tmp_cat_map = array();
                $this->cat_map_recursive($id, $result);
                $this->cat_map = @array_reverse($this->cat_map);
                
                $ilance->timer->stop();
                DEBUG("cat_map(\$id = $id) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
        }
    
        /**
        * Function to print the searchable category questions auction count sum.
        * For example, if a seller posted an item under DVD and answered "Horror" as the
        * question, Horror (1) would be shown.  Additionally, this counter would
        * be affected if and when the searcher enters existing search patterns such as
        * selecting another category question at the same time or keywords being used.
        *
        * @param       integer      question id
        * @param       string       choice
        * @param       string       category type (service/product)
        * @param       array        current search sql info (optional)
        *
        * @return      integer      Returns item count
        */
        function searchable_question_count($qid = 0, $choice = '', $cattype = '', $sqlquery = array())
        {
                global $ilance, $ilconfig, $myapi;
                
                $ilance->timer->start();
                
                $supported = array('service','product');
                
                if (isset($cattype) AND !in_array($cattype, $supported) OR !isset($cattype) OR empty($cattype))
                {
                        return 0;
                }
                
                $table = ($cattype == 'service') ? 'project_answers' : 'product_answers';
                $sqlcount = count($sqlquery);
                
                $sqlquery['genrequery'] = '';
                if (!empty($ilance->GPC['qid']) AND is_array($ilance->GPC['qid']))
                {
                        foreach ($ilance->GPC['qid'] AS $keyquestionid => $keyanswerid)
                        {
                                if (isset($keyanswerid) AND !empty($keyanswerid)) 
                                {
                                        // this condition above prevents qid[x]= from passing blank values causing a search engine bug based on below
                                        $sqlquery['genrequery'] .= "AND (" . fetch_searchable_sql_condition(intval($keyquestionid), $keyanswerid, $cattype, 'p.project_id') . ") ";
                                }
                        }
                }
                
                $count = 0;
                
                $extraquery = "";
                if ($sqlcount > 0)
                {
                        $extraquery = "
                                $sqlquery[hidequery]
                                $sqlquery[userquery]
                                $sqlquery[projectstate]
                                $sqlquery[projectdetails]
                                $sqlquery[keywords]
                                $sqlquery[categories]
                                $sqlquery[options]
                                $sqlquery[pricerange]
                                $sqlquery[radius]
                                $sqlquery[location]
                                $sqlquery[timestamp]
                                $sqlquery[projectstatus]
                        ";
                }
                
                $sql = $ilance->db->query("
                        SELECT a.questionid, a.answer, a.project_id, p.status
                        FROM " . DB_PREFIX . $table . " a
                        LEFT JOIN " . DB_PREFIX . "projects p ON (a.project_id = p.project_id)
                        LEFT JOIN " . DB_PREFIX . "users u ON (p.user_id = u.user_id)
                        WHERE questionid = '" . intval($qid) . "'
                            AND answer != ''
                            " . (($ilconfig['globalauctionsettings_payperpost']) ? "AND p.visible = '1' AND (p.insertionfee = 0 OR (p.insertionfee > 0 AND p.ifinvoiceid > 0 AND p.isifpaid = '1'))" : "AND p.visible = '1'") . "
                            $sqlquery[genrequery]
                            $extraquery
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $choice = ilance_htmlentities($choice);
                        while ($answers = $ilance->db->fetch_array($sql))
                        {
                                if (is_serialized($answers['answer']))
                                {
                                        $decrypted = unserialize($answers['answer']);
                                }
                                else
                                {
                                        $decrypted = $answers['answer'];
                                }
                                
                                if (isset($decrypted) AND is_array($decrypted))
                                {
                                        foreach ($decrypted AS $answervalue)
                                        {
                                                //$answervalue = htmlentities($answervalue, ENT_QUOTES);
                                                $answervalue = ilance_htmlentities($answervalue);
                                                if (mb_strtolower($answervalue) == mb_strtolower($choice))
                                                {
                                                        if ($answers['status'] == 'open')
                                                        {
                                                                $count++;
                                                        }
                                                }
                                        }
                                }
                        }
                }    
                
                $ilance->timer->stop();
                DEBUG("searchable_question_count(\$qid = $qid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $count;
        }
        
        /**
        * Function to print an html formatted representation of the searchable question category genres under a specific category
        * when viewing the search results
        *
        * @param       integer      category id
        * @param       bool         show category counters
        * @param       integer      current level
        * @param       integer      current show view level
        * @param       boolean      force showing of links for category maps (default false)
        * @param       string       category type (optional if forcing links)
        * @param       string       category title (optional if forcing links)
        *
        * @return      string       Returns HTML formatted searchable question output
        */
        function print_searchable_questions($cid = 0, $showcount = 1, $level = 0, $viewlevel = -1, $forcelinks = false, $cattype = '', $title = '')
        {
                global $ilance, $myapi, $ilconfig, $ilpage, $phrase, $ilcollapse, $sqlquery, $show;
                
                $ilance->timer->start();
                
                if ($cid == 0)
                {
                        return false;
                }
                
                $paddingtop = 5;
                
                if ($viewlevel == -1)
                {
                        $level++;
                }
                else
                {
                        $level = $viewlevel;
                }
                
                $res = array();
                if ($forcelinks)
                {
                        // forcing links via category map
                        $res['cattype'] = $cattype;
                        $res['title'] = $title;
                }
                else
                {
                        $res['cattype'] = $this->cattype($_SESSION['ilancedata']['user']['slng'], $cid);
                        $res['title'] = $this->title($_SESSION['ilancedata']['user']['slng'], $res['cattype'], $cid);
                }
                
                $table = ($res['cattype'] == 'service') ? 'project_questions' : 'product_questions';
                $seotype = ($res['cattype'] == 'service') ? 'servicesearchquestion' : 'productsearchquestion';
                $pagetype = ($res['cattype'] == 'service') ? HTTP_SERVER . $ilpage['rfp'] : HTTP_SERVER . $ilpage['merch'];
                $formattedhtml = $catcounter = '';
                
                $questioncount = 0;
                $questions = $ilance->db->query("
                        SELECT questionid, inputtype, multiplechoice, question_" . $_SESSION['ilancedata']['user']['slng'] . " AS question
                        FROM " . DB_PREFIX . $table . "
                        WHERE (cid = '" . intval($cid) . "' OR cid = '-1')
                            AND cansearch = '1'
                            AND visible = '1'
                        ORDER BY sort ASC
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($questions) > 0)
                {
                        $count = 0;
                        while ($question = $ilance->db->fetch_array($questions))
                        {
                                if (!empty($question['multiplechoice']))
                                {
                                        $questioncount++;
                                        $html[$count]['html'] = '';
                                        $html[$count]['htmlhead'] = '';
                                        
                                        $choices = explode('|', $question['multiplechoice']);
                                        $i = 0;
                                        foreach ($choices AS $choice)
                                        {
                                                if (!empty($choice))
                                                {
                                                        $i++;
                                                        // this will additionally attach itself to the existing search pattern being used and determine
                                                        // any new item counting logic based on for example a keyword being used, a price being selected
                                                        // or any other "category questions" being currently selected at the same time
                                                        if ($ilconfig['globalfilters_enablecategorycount'] AND isset($showcount) AND $showcount)
                                                        {
                                                                $itemcount = $this->searchable_question_count($question['questionid'], $choice, $res['cattype'], $sqlquery);
                                                                $catcounter = (($itemcount > 0)
                                                                               ? ' <span class="smaller gray">(' . $itemcount . ')</span>'
                                                                               : '');
                                                        }
                                                        
                                                        if ($ilconfig['globalauctionsettings_seourls'])
                                                        {
                                                                if (!empty($ilance->GPC['qid'][$question['questionid']]) AND $ilance->GPC['qid'][$question['questionid']] == $i)
                                                                {
                                                                        // #### currently selected genre question (non-clickable)
                                                                        $html[$count]['html'] .= '<div style="padding-left:' . ($this->fetch_level_padding($viewlevel) + 16) . 'px; padding-top:3px"><span class="gray"><strong>' . ilance_htmlentities($choice) . '</strong></span>'  . $catcounter . '</div>';
                                                                }
                                                                else
                                                                {
                                                                        // #### unselected genre question (clickable)
                                                                        $html[$count]['html'] .= (($itemcount > 0)
                                                                                                  ? '<div style="padding-left:' . ($this->fetch_level_padding($viewlevel) + 16) . 'px; padding-top:3px"><span class="blue">' . construct_seo_url($seotype, $cid, $auctionid = 0, $res['title'], '', $bold = 0, ilance_htmlentities($choice), $question['questionid'], (int)$i) . '</span>' . $catcounter . '</div>'
                                                                                                  : '');
                                                                }
                                                        }
                                                        else
                                                        {
                                                                if (!empty($ilance->GPC['qid'][$question['questionid']]) AND $ilance->GPC['qid'][$question['questionid']] == $i)
                                                                {
                                                                        $html[$count]['html'] .= '<div style="padding-left:' . ($this->fetch_level_padding($viewlevel) + 16) . 'px; padding-top:3px"><span class="gray"><strong>' . ilance_htmlentities($choice) . '</strong></span>' . $catcounter . '</div>';
                                                                }
                                                                else
                                                                {
                                                                        if (defined('LOCATION') AND (LOCATION == 'rfp' OR LOCATION == 'main' OR LOCATION == 'merch'))
                                                                        {
                                                                                $html[$count]['html'] .= (($itemcount > 0)
                                                                                                          ? '<div style="padding-left:' . ($this->fetch_level_padding($viewlevel) + 10) . 'px; padding-top:3px"><span class="blue"><a href="' . $pagetype . '?cid=' . $cid . print_hidden_fields($string = true, $excluded = array('cid','cmd','page','list'), $questionmarkfirst = false, $prepend_text = '', $append_text = '', $htmlentities = true, $urldecode = true) . '&qid[' . $question['questionid'] . ']=' . (int)$i . '">' . ilance_htmlentities($choice) . '</a></span>' . $catcounter . '</div>'
                                                                                                          : '');
                                                                        }
                                                                        else
                                                                        {
                                                                                if (defined('LOCATION') AND (LOCATION == 'rfp' OR LOCATION == 'search' OR LOCATION == 'merch'))
                                                                                {
                                                                                        $html[$count]['html'] .= (($itemcount > 0)
                                                                                                                  ? '<div style="padding-left:' . ($this->fetch_level_padding($viewlevel) + 10) . 'px; padding-top:3px"><span class="blue"><a href="' . $ilpage['search'] . '?cid=' . $cid . print_hidden_fields($string = true, $excluded = array('cid','cmd','page','list'), $questionmarkfirst = false, $prepend_text = '', $append_text = '', $htmlentities = true, $urldecode = true) . '&qid[' . $question['questionid'] . ']=' . (int)$i . '">' . ilance_htmlentities($choice) . '</a></span>' . $catcounter . '</div>'
                                                                                                                  : '');
                                                                                }
                                                                        }
                                                                }
                                                        }
                                                }
                                                
                                                if (!empty($html[$count]['html']))
                                                {
                                                        $html[$count]['htmlhead'] = '<div style="padding-left:' . $this->fetch_level_padding($viewlevel) . 'px; padding-top:' . $paddingtop . 'px;"><strong>' . $question['question'] . '</strong></div>';
                                                }
                                        }
                                }
                                
                                $count++;                                
                        }
                        
                        $bit['visible'] = $bit['hidden'] = '';
                        
                        $hidden = '<div style="padding-left:' . $this->fetch_level_padding($viewlevel) . 'px; padding-bottom:6px; padding-top:5px" class="smaller"><a href="javascript:void(0)" onclick="toggle_more(\'showmore_' . $cid . '\', \'moretext_' . $cid . '\', \'' . $phrase['_more_options'] . '\', \'' . $phrase['_less_options'] . '\', \'showmoreicon_' . $cid . '\')"><span id="moretext_' . $cid . '" class="gray" style="font-weight:bold; text-decoration:none">' . (!empty($ilcollapse["showmore_$cid"]) ? $phrase['_less_options'] : $phrase['_more_options']) . '</strong></span></a> <img id="showmoreicon_' . $cid . '" src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'icons/' . (!empty($ilcollapse["showmore_$cid"]) ? 'arrowup2.gif' : 'arrowdown2.gif') . '" border="0" alt="" /></div>';
                
                        if (!empty($html) AND is_array($html))
                        {
                                $c = 0;
                                foreach ($html AS $key => $array)
                                {
                                        $c++;
                                        
                                        if ($c <= $ilconfig['globalauctionsettings_catquestiondepth'])
                                        {
                                                $bit['visible'] .= $html[$key]['htmlhead'];
                                                $bit['visible'] .= $html[$key]['html'];
                                        }
                                        else
                                        {
                                                $bit['hidden'] .= $html[$key]['htmlhead'];
                                                $bit['hidden'] .= $html[$key]['html'];
                                        }
                                }
                        }
                        
                        if ($questioncount <= $ilconfig['globalauctionsettings_catquestiondepth'])
                        {
                                $hidden = '';
                        }
                        
                        // rebuild display options
                        $formattedhtml = "$bit[visible] <div id=\"showmore_$cid\" style=\"" . (!empty($ilcollapse["showmore_$cid"]) ? $ilcollapse["showmore_$cid"] : 'display: none;') . "\">$bit[hidden]</div>$hidden";
                        
                        // tell our template we'll be showing category genre questions
                        $show['categoryfinder'] = (empty($bit['visible']) OR $bit['visible'] == '') ? false : true;
                }
                
                $ilance->timer->stop();
                DEBUG("print_searchable_questions(\$cid = $cid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $formattedhtml;
        }
        
        /**
        * Function to fetch the padding based on a particular category level we're currently viewing
        *
        * @param       integer      level
        *
        * @return      string       Returns HTML formatted table with category results
        */
        function fetch_level_padding($level = 0)
        {
                $padding = array(
                        '0' => '0',
                        '1' => '0',
                        '2' => '15',
                        '3' => '30',
                        '4' => '45',
                        '5' => '60',
                        '6' => '75',
                        '7' => '90',
                        '8' => '105',
                );
                
                return $padding["$level"];
        }
        
        /**
        * Function to print a category's "new" icon representing any new auctions posted within that category
        *
        * @param       integer      category id
        * @param       bool         category type (service/product)
        * @param       integer      days (default 7)
        *
        * @return      string       Returns xxx
        */
        function print_category_newicon($cid = 0, $cattype = '', $daysnew = 7)
        {
                global $ilance, $myapi, $ilconfig, $phrase;
                
                $ilance->timer->start();
                
                if ($cattype != 'service' OR $cattype != 'product')
                {
                        return;
                }
                
                $html = false;
                $daysnew = $ilconfig['globalauctionsettings_newicondays'];
                
                $sql = $ilance->db->query("
                        SELECT project_id
                        FROM " . DB_PREFIX . "projects
                        WHERE cid = '" . intval($cid) . "'
                            AND (status = 'open' AND project_state = '" . $ilance->db->escape_string($cattype) . "')
                            AND visible = '1'
                            AND date_added >= DATE_SUB('" . DATETIME24H . "', INTERVAL $daysnew DAY)
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $html = '<img src="' . $ilconfig['template_relativeimagepath'] . $ilconfig['template_imagesfolder'] . 'new2.gif" border="0" alt="' . $phrase['_new_projects_have_been_posted'] . ': ' . $phrase['_past'] . ' ' . $daysnew . ' ' . $phrase['_days_lower'] . '" />';
                }
                
                $ilance->timer->stop();
                DEBUG("print_category_newicon(\$cid = $cid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $html;
        }
            
        /**
        * Function to count the number of ads within a Want Ads category
        *
        * @param       integer      category id
        *
        * @return      string       Returns xxx
        */
        function wantads_in_category($cid = 0)
        {
                global $ilance, $myapi;
                
                $sql = $ilance->db->query("
                    SELECT COUNT(*) AS nr_pr
                    FROM " . DB_PREFIX . "wantads
                    WHERE cid = '".intval($cid)."'
                        AND wantstatus = 'wanted'
                        AND visible = '1'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql)> 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        return intval($res['nr_pr']);
                }
                
                return 0;
        }
        
        /**
        * Function to bridge the category system for usage with Want Ads and other add on products like Stores
        *
        * @param       string       category mode
        * @param       integer      columns
        * @param       integer      category id
        * @param       integer      store id (optional)
        *
        * @return      string       Returns HTML formatted category output display
        */
        function construct_categories($mode, $columns, $cid = 0, $storeid = '')
        {
                global $ilance, $myapi, $phrase, $ilconfig, $ilpage;
                
                ($apihook = $ilance->api('categories_construct_categories_start')) ? eval($apihook) : false;
                ($apihook = $ilance->api('categories_construct_categories_end')) ? eval($apihook) : false;
                
                return $html;
        }
        
        /**
        * Function to
        *
        * @param       integer      parent category id
        * @param       integer      current level
        * @param       string       category type (service/product)
        * @param       string       option groupns
        * @param       string       add spaces (optional)
        *
        * @return      string       Returns xxx
        */
        function display_children($parentid, $level, $cattype = '', $optgroups = '', $addspaces = '')
        {
                global $ilance, $myapi;
                
                $ilance->timer->start();
                
                $html = '';
                
                if ($cattype == 'service' OR $cattype == 'product') 
                {
                        $result = $ilance->db->query("
                                SELECT cid, title_" . $_SESSION['ilancedata']['user']['slng'] . " AS title, canpost
                                FROM " . DB_PREFIX . "categories
                                WHERE parentid = '" . intval($parentid) . "'
                                        AND visible = '1'
                                        AND cattype = '$cattype'
                        ", 0, null, __FILE__, __LINE__);
                }
        
                ($apihook = $ilance->api('categories_display_children_condition')) ? eval($apihook) : false;
        
                // display each child
                while ($row = $ilance->db->fetch_array($result))
                {
                        // indent and display the title of this child
                        if (!empty($row['parentid']) AND $row['parentid'] == '-1')
                        {
                                $html .= '<optgroup label="' . stripslashes($row['title']) . '">';
                        }
                        else 
                        {
                                if ($addspaces)
                                {
                                        $html .= '<option value="' . $row['cid'] . '">' . str_repeat('&nbsp;&nbsp;&nbsp;', $level) . stripslashes($row['title']) . '</option>';
                                }
                                else 
                                {
                                        $html .= '<option value="' . $row['cid'] . '">' . str_repeat('&nbsp;&nbsp;&nbsp;', $level) . stripslashes($row['title']) . '</option>';
                                }
                        }
        
                        // call this function again to display this
                        // child's children
                        if (isset($cattype) AND !empty($cattype))
                        {
                                $html .= $this->display_children($row['cid'], $level + 1, $cattype);
                        }
                        if (!empty($row['parentid']) AND $row['parentid'] == '-1')
                        {
                                $html .= '</optgroup>';
                        }
                }
                
                $ilance->timer->stop();
                DEBUG("display_children(\$parentid = $parentid) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $html;
        }
        
        /**
        * Function to provide add-on products with a useable API allowing the generation of multi-select fields such as categories.
        *
        * @param       string       category type
        * @param       string       selected category
        * @param       bool         single pulldown mode?
        * @param       bool         hide all categories?
        * @param       string       select field name
        * @param       bool         show "Please Select: " option in the pulldown
        *
        * @return      string       Returns HTML formatted pulldown/select menu
        */
        function api_multicategory_select($cattype = '', $selected = '', $singlepulldown = '', $hideallcats = '', $selectname = '', $pleaseselect = '')
        {
                global $ilance, $myapi, $phrase;
                
                $ilance->timer->start();
                
                if (isset($singlepulldown) AND $singlepulldown)
                {
                        $html = '';
                        if (empty($selectname))
                        {
                                $html .= '<select name="subcatid" style="font-family: verdana">';
                        }
                        else 
                        {
                                $html .= '<select name="' . $selectname . '" style="font-family: verdana">';
                        }
                        if ($hideallcats == 0)
                        {
                                $html .= '<option value="0" style="background-color: #ebebeb;">';
                                $html .= $phrase['_all_categories_upper'];
                                $html .= '</option>';
                        }
                        if ($pleaseselect)
                        {
                                $html .= '<option value="" style="background-color: #ebebeb;">';
                                $html .= $phrase['_please_select'];
                                $html .= '</option>';
                        }
                        if ($cattype == 'service')
                        {
                                $html .= $this->display_children(-1, 0, 'service');
                        }
                        if ($cattype == 'product')
                        {
                                $html .= $this->display_children(-1, 0, 'product');
                        }
                        
                        ($apihook = $ilance->api('categories_api_multicategory_select_single_condition')) ? eval($apihook) : false;
                        
                        $html .= '</select>';
                }
                else
                {
                        ($apihook = $ilance->api('categories_api_multicategory_select_multiple_condition')) ? eval($apihook) : false;
                }
                
                $ilance->timer->stop();
                DEBUG("api_multicategory_select(\$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $html;
        }
        
        /**
        * Function to set all the levels in proper format for the main category system
        */
        function set_levels()
        {
                global $ilance;
                
                $sql = $ilance->db->query("
                        SELECT cid, parentid, level
                        FROM " . DB_PREFIX . "categories
                ", 0, null, __FILE__, __LINE__);
                while ($cats = $ilance->db->fetch_array($sql, DB_ASSOC))
                {
                        if ($cats['parentid'] == 0)
                        {    
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "categories
                                        SET level = 1
                                        WHERE cid = '" . $cats['cid'] . "'
                                ", 0, null, __FILE__, __LINE__);                    
                        }
                        else
                        {
                                $level = 1;
                                $this->set_levels_update($cats['cid'], $cats['parentid'], $level);
                        }
                }
        }
        
        /**
        * Function to set all levels and to handle the updating
        *
        * @param       integer        category id
        * @param       integer        parent id
        * @param       integer        level
        * @param       integer        category id to save
        *
        * @return      nothing
        */
        function set_levels_update($cid, $parentid, $level, $cid_save = '')
        {
                global $ilance;
                
                if (empty($cid_save))
                {
                        $cid_save = $cid;
                }
                
                $sql = $ilance->db->query("
                        SELECT cid, parentid, level
                        FROM " . DB_PREFIX . "categories
                        WHERE cid = '" . intval($parentid) . "'
                ", 0, null, __FILE__, __LINE__);
                                 
                $category = $ilance->db->fetch_array($sql);
                
                if ($category['parentid'] == 0)
                {
                        $level = $level + 1;
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "categories
                                SET level = '" . intval($level) . "'
                                WHERE cid = '" . intval($cid_save) . "'
                        ", 0, null, __FILE__, __LINE__);                    
                }
                else
                {
                        $level = $level + 1;
                        $this->set_levels_update($category['cid'], $category['parentid'], $level, $cid_save);
                }                 
        }
        
        /**
        * Function to fetch all children category id numbers recursivly in comma separated values based on a parent category id number.
        * This function is useful because it reads from the cache and does not hit the database.
        *
        * @param       string         category id number (or all)
        * @param       string         category type (service or product)
        *
        * @return      string         Returns category id's in comma separate values (ie: 1,3,4,6)
        */
        function fetch_children_ids($cid = 'all', $cattype = 'service')
        {
                global $ilance, $myapi;
                
                //$ilance->timer->start();
                $c = 0;                
                $ids = '';
                
                if ($cid == 'all')
                {
                        foreach ($this->fetch[$_SESSION['ilancedata']['user']['slng']] as $type => $categories)
                        {
                                if ($type == $cattype)
                                {
                                        $c = 0;
                                        foreach ($categories as $category)
                                        {
                                                if ($category['visible'])
                                                {
                                                        $ids .= $category['cid'] . ',';
                                                        $c++;
                                                }
                                        }
                                }
                        }
                        if ($c > 1)
                        {
                                $ids = mb_substr($ids, 0, -1);
                        }                
                }
                else
                {
                        foreach ($this->fetch[$_SESSION['ilancedata']['user']['slng']] AS $type => $categories)
                        {                                
                                if ($type == $cattype)
                                {
                                        foreach ($categories AS $category)
                                        {
                                                if ($category['visible'])
                                                {
                                                        if ($category['parentid'] == $cid)
                                                        {
                                                                if ($category['cid'] != $cid)
                                                                {
                                                                        $ids .= $category['cid'] . ',' . $this->fetch_children_ids($category['cid'], $cattype);
                                                                }    
                                                        }
                                                }
                                        }
                                }
                        }                
                }
                
                //$ilance->timer->stop();
                //DEBUG("fetch_children_ids(\$cid = $cid, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $ids;
        }
        
        /**
        * Function to fetch all children category id numbers returns in comma separated values.
        *
        * @param       integer        category id number (or all)
        * @param       string         category type (service/product)
        *
        * @return      string         Returns category id's in comma separate values (ie: 1,3,4,6)
        */
        function fetch_children($cid = 0, $cattype = 'service')
        {
                global $ilance, $myapi;
                
                $ilance->timer->start();
                
                $ids = $this->fetch_children_ids($cid, $cattype);
                if (empty($ids))
                {
                        $ids = $cid;
                }
                else 
                {
                        $ids = $cid . ',' . mb_substr($ids, 0, -1);
                }
                
                $ilance->timer->stop();
                DEBUG("fetch_children(\$cid = $cid, \$cattype = $cattype) in " . $ilance->timer->get() . " seconds", 'FUNCTION');
                
                return $ids;
        }
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.categories.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>