<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

/**
* Currency class to perform the majority of currency related functions in ILance
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class currency
{
        var $currencies = array();
        
        /**
        * Constructor
        *
        */
        function currency()
        {
                global $ilance, $myapi;
                
                $this->currencies = array();
                
                $query = $ilance->db->query("
                        SELECT currency_id, currency_abbrev AS code, symbol_left, symbol_right, decimal_point, thousands_point, decimal_places, rate, currency_name, currency_abbrev
                        FROM " . DB_PREFIX . "currency
                ", 0, null, __FILE__, __LINE__);
                while ($currencies = $ilance->db->fetch_array($query))
                {
                        // generate string type values (ie: USD)
                        $this->currencies[$currencies['code']] = array(
                                'symbol_left' => $currencies['symbol_left'],
                                'symbol_right' => $currencies['symbol_right'],
                                'decimal_point' => $currencies['decimal_point'],
                                'thousands_point' => $currencies['thousands_point'],
                                'decimal_places' => $currencies['decimal_places'],
                                'rate' => $currencies['rate'],
                                'currency_id' => $currencies['currency_id'],
                                'currency_name' => $currencies['currency_name'],
                                'currency_abbrev' => $currencies['currency_abbrev']
                        );
                        
                        // generate integer type values (ie: 1)
                        $this->currencies[$currencies['currency_id']] = array(
                                'symbol_left' => $currencies['symbol_left'],
                                'symbol_right' => $currencies['symbol_right'],
                                'decimal_point' => $currencies['decimal_point'],
                                'thousands_point' => $currencies['thousands_point'],
                                'decimal_places' => $currencies['decimal_places'],
                                'rate' => $currencies['rate'],
                                'code' => $currencies['code'],
                                'currency_name' => $currencies['currency_name'],
                                'currency_abbrev' => $currencies['currency_abbrev']
                        );
                }
        }
    
        /**
        * Function to properly format a dollar value based on the database currency settings (symbols, decimal places, thousands place, etc)
        *
        */
        function format($number = 0, $currency_type = '', $hidesymbols = false)
        {
                global $currencies, $ilconfig;
        
                $html = '';
        
                if (empty($currency_type))
                {
                        $currency_type = $ilconfig['globalserverlocale_defaultcurrency'];
                }
                
                // remove user input formatting and get a solid number with only decimals
                $number = $this->string_to_number($number);
                if ($hidesymbols == false)
                {
                        $html .= $this->currencies["$currency_type"]['symbol_left'];
                        $html .= number_format($number, $this->currencies["$currency_type"]['decimal_places'], $this->currencies["$currency_type"]['decimal_point'], $this->currencies["$currency_type"]['thousands_point']);
                        $html .= $this->currencies["$currency_type"]['symbol_right'];
                }
                else
                {
                        $html .= number_format($number, $this->currencies["$currency_type"]['decimal_places'], $this->currencies["$currency_type"]['decimal_point'], $this->currencies["$currency_type"]['thousands_point']);
                }
                
                return $html;
        }
    
        /**
        * Function to fetch the default currency id installed for the marketplace
        *
        */
        function fetch_default_currencyid()
        {
                global $ilance, $myapi, $ilconfig;
                (isset($ilconfig['globalserverlocale_defaultcurrency']) ? $cid = $ilconfig['globalserverlocale_defaultcurrency'] : $cid = '1');
                return $cid;
        }
        
        /**
        * Function to fetch a user's default currency setup when they registered or edited their profile
        *
        */
        function fetch_user_currency($userid = 0)
        {
                global $ilance, $myapi;
        
                $sql = $ilance->db->query("
                        SELECT currencyid
                        FROM " . DB_PREFIX . "users
                        WHERE user_id = '".intval($userid)."'
                ", 0, null, __FILE__, __LINE__);        
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $cur = $ilance->db->fetch_array($sql);
                        return $cur['currencyid'];
                }
                else
                {
                        return '1';
                }
        }
        
        /**
        * Function to build a currency selector pulldown menu element
        *
        */
        function pulldown($inputtype = '', $variableinfo = '')
        {
                global $ilance, $myapi, $ilconfig;
        
                $sql = $ilance->db->query("
                        SELECT currency_id, currency_name, currency_abbrev
                        FROM " . DB_PREFIX . "currency
                ", 0, null, __FILE__, __LINE__);
                if ($inputtype == 'admin')
                {
                        $html = '<select name="config[' . $variableinfo . ']" style="font-family: verdana">';
                        
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                $html .= '<option value="' . $res['currency_id'] . '"';
                                if ($res['currency_id'] == $ilconfig['globalserverlocale_defaultcurrency'])
                                {
                                        $html .= ' selected="selected"';
                                }
                                $html .= '>' . $res['currency_abbrev'] . ' ' . stripslashes($res['currency_name']) . '</option>';
                        }
                }
                else
                {
                        $html = '<select name="currencyid" style="font-family: verdana">';
                        
                        while ($res = $ilance->db->fetch_array($sql))
                        {
                                if (!empty($_SESSION['ilancedata']['user']['userid']))
                                {
                                        $sqlprefs = $ilance->db->query("
                                                SELECT currencyid
                                                FROM " . DB_PREFIX . "users
                                                WHERE user_id = '" . intval($_SESSION['ilancedata']['user']['userid']) . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        $prefs = $ilance->db->fetch_array($sqlprefs);
                                }
                                else
                                {
                                        $prefs['currencyid'] = $ilconfig['globalserverlocale_defaultcurrency'];
                                }
                
                                $html .= '<option value="' . $res['currency_id'] . '"';
                                if ($res['currency_id'] == $prefs['currencyid'])
                                {
                                        $html .= ' selected="selected"';
                                }
                                $html .= '>' . $res['currency_abbrev'] . ' ' . stripslashes($res['currency_name']) . '</option>';
                        }
                }
                
                $html .= '</select>';
                
                return $html;
        }
        
        function string_to_number($price)
        {
                $price = trim($price);
        
                if (preg_match("~^([0-9]+|(?:(?:[0-9]{1,3}([.,' ]))+[0-9]{3})+)(([.,])[0-9]{1,2})?$~", $price, $r))
                {
                        if (!empty($r['2']))
                        {
                                $pre = preg_replace("~[" . $r['2'] . "]~", "", $r['1']);
                        }
                        else
                        {
                                $pre = $r['1'];
                        }
                        
                        if (!empty($r['4']))
                        {
                                $post = "." . preg_replace("~[" . $r['4'] . "]~", "", $r['3']);
                        }
                        else
                        {
                                $post = false;
                        }
                        
                        $form_price = $pre . $post;
                        
                        return $form_price;
                }
                
                return $price;
        }
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.currency.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>