<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

define('DB_ASSOC', 1);
define('DB_NUM', 2);
define('DB_BOTH', 3);

/**
* ILance database class to perform the majority of database caching in ILance
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class ilance_database
{
	/**
	* Cache to database timeout in sections
	*
	* @var	    $cache_timeo
	*/
	var $cache_timeo;
    
	/**
	* Cache to Database
	*
	* @var	    $cache_to_db
	*/
	var $cache_to_db = false;
	var $cache_db_data_at_once;
	var $cache_db_data_at_once_loaded;

	/**
	* The ILance registry object
	*
	* @var	    $registry
	*/
        var $registry = null;
        
        /**
	* Debug Mode
	*
	* @var	    $debug
	*/
	var $debug = false;
        
        /**
	* Email Error Reporting
	*
	* @var	    $email_reporting
	*/
        var $error_reporting = true;
	var $email_reporting = true;
        
        /**
	* Timer Variables
	*
	* @var	    $start
	* @var      $end
	* @var      $totaltime
	* @var      $formatted
	*/
	var $start = null;
	var $end = null;
	var $totaltime = null;
	var $formatted = null;
        
        /**
	* Database Connection Parameters
	*
	* @var	    $multiserver
	* @var      $database
	* @var      $explain
	* @var      $querylist
	* @var      $query_count
	* @var      $connection_write
	* @var      $connection_read
	* @var      $connection_link
	*/
        var $multiserver = false;
	var $database = null;
        var $explain = null;
        var $querylist = array();
        var $query_count = 0;
	var $connection_write = null;
	var $connection_read = null;
	var $connection_link = null;
	
	var $error = '';
	var $errno = '';

	/**
	* Constructor
	*
	* @param       integer         cache timeout (in seconds)
	* @param       bool            cache data to database?
	*/
	function ilance_database($cachetimeout = 1, $cachetodatabase = false)
	{
		$this->cache_timeo = $cachetimeout;
		$this->cache_to_db = $cachetodatabase;
		
		// set to true - to load cached data at once from database
		$this->cache_db_data_at_once_loaded = false;	
		$this->cache_db_data_at_once_loaded = false;
                
                if (function_exists($this->functions['real_escape_string']))
		{
			$this->functions['escape_string'] = $this->functions['real_escape_string'];
		}
	}
	
	/**
	* Initialize database connection
	*
	* Connects to a database server
	* 
	* @return	boolean
	*/
	function connect()
	{
		$this->connection_write = $this->db_connect();
		$this->multiserver = false;
		$this->connection_read =& $this->connection_write;
		$this->database = DB_DATABASE;

		if ($this->connection_write)
		{
			$this->select_db($this->database);
		}
	}
	
        /**
        * Connect to the database and return the connection link resource
        *
        * Connects to a database server and physically returns the connection link identifier
        * 
        * @return	boolean
        */
        function db_connect()
        {
                $hostname = DB_SERVER;
                $username = DB_SERVER_USERNAME;
                $password = DB_SERVER_PASSWORD;
                $pconnect = DB_PERSISTANT_MASTER;
                $dbcharset = DB_CHARSET;
                $dbcollate = DB_COLLATE;
                
                $port = DB_SERVER_PORT;
                $port = $port ? $port : 3306;
                
                $link = $this->functions[$pconnect ? 'pconnect' : 'connect']("$hostname:$port", $username, $password);
                
                if (!empty($dbcharset) AND !empty($dbcollate))
                {
                        $this->query("SET CHARACTER SET $dbcharset", 0, null, 'class.database.inc.php: db_connect()', __LINE__, true, $link);
                        $this->query("SET NAMES $dbcharset", 0, null, 'class.database.inc.php: db_connect()', __LINE__, true, $link);
                        $this->query("SET COLLATION_DATABASE $dbcollate", 0, null, 'class.database.inc.php: db_connect()', __LINE__, true, $link);
                        $this->query("SET COLLATION_CONNECTION $dbcollate", 0, null, 'class.database.inc.php: db_connect()', __LINE__, true, $link);
                        
                        // just in case..
                        $this->query("
                                SET character_set_results = '$dbcharset',
                                character_set_client = '$dbcharset',
                                character_set_connection = '$dbcharset',
                                character_set_database = '$dbcharset',
                                character_set_server = '$dbcharset',
                                character_set_system = '$dbcharset'
                        ", 0, null, 'class.database.inc.php: db_connect()', __LINE__, true, $link);
                }
                
                return $link;
        }
	
	/**
	* Selects a database for usage
	*
	* @param	string	 name of the database to use
	*
	* @return	boolean
	*/
        function select_db($database = '')
	{
		if ($database != '')
		{
			$this->database = $database;
		}

		if ($check_write = @$this->select_db_wrapper($this->database, $this->connection_write))
		{
			$this->connection_link =& $this->connection_write;
			return true;
		}
		else
		{
			$this->connection_link =& $this->connection_write;
			$this->dberror('Cannot select database ' . $this->database . ' for usage');
			return false;
		}
	}

	/**
	* Function to select the database with an associated mysql link identifier
	* 
	* @param       string        database name
	* @param       object        database link
	* 
        * @return      nothing
	*/
	function select_db_wrapper($database = '', $link = null)
	{
		return $this->functions['select_db']($database, $link);
	}
	
	/**
	* Function to perform a database specific query
	* 
	* @param       string        sql code
	* @param       bool          hide database errors? default false
	* @param       string        cache to filesystem filename
	* @param       string        script name
	* @param       string        script line number
	* 
        * @return      nothing
	*/
	function query($string = '', $hideerr = 0, $cache_to_file = null, $script = '', $line = '')
	{
		global $pagestarttime, $querytime;
		
		$this->query_count++;
                
		$dblink = preg_match('#(^|\s)SELECT\s#s', $string) ? 'connection_read' : 'connection_write';
                $qtimer = $this->timer();
                
		$query = $this->cached_query($string, $this->connection_link, $cache_to_file);
		if ($this->errno() AND !$hideerr)
		{
			 $this->dberror($string);
			 exit;
		}
                
		$qtime = $this->stop();
		if (defined('DB_EXPLAIN') AND DB_EXPLAIN)
		{
			$this->explain_query($string, $qtime, $script, $line);
		}
                
		$querytime += $this->totaltime;
		$this->remove();
		
		return $query;
	}
	
	/**
	* Function to perform a database specific query and immediately returns the associated array/results
	* 
	* @param       string        sql code
	* @param       bool          hide database errors? default false
	* @param       string        cache to filesystem filename
	* @param       string        script name
	* @param       string        script line number
	* 
        * @return      nothing
	*/
	function query_fetch($string = '', $hideerr = 0, $cache_to_file = null, $script = '', $line = '')
	{
		global $pagestarttime, $querytime;
		
		$qtimer = $this->timer();
                
		$query = $this->cached_query($string, $this->connection_link, $cache_to_file);
		if ($this->errno() AND !$hideerr)
		{
			 $this->dberror($string);
			 exit;
		}
                
		$qtime = $this->stop();

		// explain query output
		if (defined('DB_EXPLAIN') AND DB_EXPLAIN)
		{
			$this->explain_query($string, $qtime, $script, $line);
		}
                
		$querytime += $this->totaltime;
                
		$this->remove();
                
		$this->query_count++;
		
		return $this->fetch_array($query);
	}

	/**
	* Function to perform a database explain query
	* 
	* @param       string        sql code
	* @param       integer       sql query time
	* @param       string        script name
	* @param       string        script line number
	* 
        * @return      nothing
	*/
	function explain_query($string = '', $qtime = '', $script = '', $line = '')
	{
		if (mb_stristr($string, 'SELECT'))
		{
			$query = $this->functions['query']("EXPLAIN $string", $this->connection_link);
			$this->explain .= "<table bgcolor=\"#666666\" width=\"95%\" cellpadding=\"4\" cellspacing=\"1\" align=\"center\">\n".
			"<tr>\n".
			"<td colspan=\"8\" bgcolor=\"#cccccc\"><strong>#".$this->query_count." - Select Query</strong></td>\n".
			"</tr>\n".
			"<tr>\n".
			"<td colspan=\"8\" bgcolor=\"#fefefe\"><span style=\"font-family: Courier; font-size: 14px;\">Script: ".$script.", Line: ".$line."</span></td>\n".
			"</tr>\n".
			"<tr>\n".
			"<td colspan=\"8\" bgcolor=\"#fefefe\"><span style=\"font-family: Courier; font-size: 14px;\">".$string."</span></td>\n".
			"</tr>\n".
			"<tr bgcolor=\"#efefef\">\n".
			"<td><strong>table</strong></td>\n".
			"<td><strong>type</strong></td>\n".
			"<td><strong>possible_keys</strong></td>\n".
			"<td><strong>key</strong></td>\n".
			"<td><strong>key_len</strong></td>\n".
			"<td><strong>ref</strong></td>\n".
			"<td><strong>rows</strong></td>\n".
			"<td><strong>Extra</strong></td>\n".
			"</tr>\n";

			while ($table = $this->functions['fetch_array']($query))
			{
				$this->explain .=
				"<tr bgcolor=\"#ffffff\">\n".
				"<td>".$table['table']."</td>\n".
				"<td>".$table['type']."</td>\n".
				"<td>".$table['possible_keys']."</td>\n".
				"<td>".$table['key']."</td>\n".
				"<td>".$table['key_len']."</td>\n".
				"<td>".$table['ref']."</td>\n".
				"<td>".$table['rows']."</td>\n".
				"<td>".$table['Extra']."</td>\n".
				"</tr>\n";
			}
			
			$this->explain .=
			"<tr>\n".
			"<td colspan=\"8\" bgcolor=\"#ffffff\">Query Time: ".$qtime."</td>\n".
			"</tr>\n".
			"</table>\n".
			"<br />\n";
		}
		else if (mb_stristr($string, 'DELETE'))
		{
			$this->explain .= "<table bgcolor=\"#666666\" width=\"95%\" cellpadding=\"4\" cellspacing=\"1\" align=\"center\">\n".
			"<tr>\n".
			"<td bgcolor=\"#ff0000\"><font color=\"#ffffff\"><strong>#".$this->query_count." - Delete Query</strong></font></td>\n".
			"</tr>\n".
			"<tr bgcolor=\"#fefefe\">\n".
			"<td><span style=\"font-family: Courier; font-size: 14px;\">Script: ".$script.", Line: ".$line."</span></td>\n".
			"</tr>\n".
			"<tr bgcolor=\"#fefefe\">\n".
			"<td><span style=\"font-family: Courier; font-size: 14px;\">".$string."</span></td>\n".
			"</tr>\n".
			"<tr>\n".
			"<td bgcolor=\"#ffffff\">Query Time: ".$qtime."</td>\n".
			"</tr>\n".
			"</table>\n".
			"</table>\n".
			"<br />\n";
		}
		else
		{
			$this->explain .= "<table bgcolor=\"#666666\" width=\"95%\" cellpadding=\"4\" cellspacing=\"1\" align=\"center\">\n".
			"<tr>\n".
			"<td bgcolor=\"#ffee00\"><strong>#".$this->query_count." - Write Query</strong></td>\n".
			"</tr>\n".
			"<tr bgcolor=\"#fefefe\">\n".
			"<td><span style=\"font-family: Courier; font-size: 14px;\">Script: ".$script.", Line: ".$line."</span></td>\n".
			"</tr>\n".
			"<tr bgcolor=\"#fefefe\">\n".
			"<td><span style=\"font-family: Courier; font-size: 14px;\">".$string."</span></td>\n".
			"</tr>\n".
			"<tr>\n".
			"<td bgcolor=\"#ffffff\">Query Time: ".$qtime."</td>\n".
			"</tr>\n".
			"</table>\n".
			"</table>\n".
			"<br />\n";
		}
		
		$this->querylist[$this->query_count]['query'] = $string;
		$this->querylist[$this->query_count]['time'] = $qtime;
	}

	/**
	* Function to perform a database fetch array
	* 
	* @param       string        sql code
	* @param       string        sql result type
	* 
        * @return      nothing
	*/
	function fetch_array(&$query, $type = DB_BOTH)
	{
		return @$this->cached_fetch_array($query, $this->types[$type]);
	}
	
	/**
	* Function to perform a database fetch associative array
	* 
	* @param       string        sql code
	* @param       string        sql result type
	* 
        * @return      nothing
	*/
	function fetch_assoc(&$query, $type = DB_ASSOC)
	{
		return @$this->cached_fetch_array($query, $this->types[$type]);
	}

	/**
	* Function to perform a database fetch row
	* 
	* @param       string        sql code
	* @param       string        sql result type
	* 
        * @return      nothing
	*/
	function fetch_row(&$query, $type = DB_NUM)
	{
		return @$this->cached_fetch_array($query, $this->types["$type"]);
	}
        
        /**
	* Function to perform a database fetch object
	* 
	* @param       string        sql code
	* 
        * @return      nothing
	*/
	function fetch_object(&$query)
        {
                $result = $this->functions['fetch_object']($query);
                return $result;
        }

	/**
	* Function to perform a database result
	* 
	* @param       string        sql code
	* @param       string        table row
	* 
        * @return      nothing
	*/
	function result(&$query, $row = '')
	{
		$result = $this->functions['result']($query, $row);
		return $result;
	}

	/**
	* Function to perform a database num rows
	* 
	* @param       string        sql code
	* 
        * @return      nothing
	*/
	function num_rows($query = '')
	{
		return $this->cached_num_rows($query);
	}

	/**
	* Function to fetch the last insert id for the database connection
	* 
        * @return      nothing
	*/
	function insert_id()
	{
		$id = $this->functions['insert_id']();
		return $id;
	}

	/**
	* Function to close the database connection
	* 
        * @return      nothing
	*/
	function close()
	{
		@$this->functions['close']($this->connection_link);
	}

	/**
	* Function to mimic database error handling
	* 
        * @return      nothing
	*/
	function error()
	{
		if ($this->connection_link === null)
		{
			$this->error = '';
		}
		else
		{
			$this->error = $this->functions['error']($this->connection_link);
		}
		return $this->error;
	}

	/**
	* Function to mimic database error number handling
	* 
        * @return      nothing
	*/
	function errno()
	{
		if ($this->connection_link === null)
		{
			$this->errno = 0;
		}
		else
		{
			$this->errno = $this->functions['errno']($this->connection_link);
		}
                
		return $this->errno;	
	}
	
	/**
	* Function to perform database error handling
	* 
        * @return      nothing
	*/
	function dberror($string = '')
	{
		define('NO_DB', true);
                
		global $ilance, $message, $ilconfig, $site_email, $site_name;
		
		if ($this->error_reporting)
		{
                        $message = '';
                        $messageemail = '';
                        
                        $html = "
" . $string . "

MySQL Error  : " . $this->error() . "
Error Number : " . $this->errno() . "
Date         : " . date('M d, Y, H:i:s') . "
Script       : http://" . $_SERVER['HTTP_HOST'] . "/" . $_SERVER['PHP_SELF'] . "
Class        : " . get_class($this) . "
Location     : " . (defined('LOCATION') ? LOCATION : 'Unknown') . "
Referrer     : " . REFERRER . "
IP Address   : " . IPADDRESS;

                        $messageemail = $html;
			if (defined('DB_DEBUGMODE') AND DB_DEBUGMODE)
			{
				$message = '<textarea style="width:600px; height:130px">'.$html.'</textarea>';
			}
                        else
                        {
                            if (defined('DB_DEBUGMODE_VIEWSOURCE') AND DB_DEBUGMODE_VIEWSOURCE)
                            {
                                $message = "\n\n<!-- DATABASE ERROR\n\n************************\n$html\n************************\n\nEND DATABASE ERROR //-->\n\n";
                            }    
                        }
			if ($this->email_reporting)
			{
				$subject = "Database error on " . date('M d, Y, H:i:s');
				if (defined('SITE_EMAIL'))
				{
					require_once(DIR_FUNCTIONS . 'core/functions.php');
					send_email(SITE_EMAIL, $subject, $messageemail, 'ILance MySQL ReportBot');
				}
			}
                        
                        $template = '
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
        <title>Database Error</title>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <style type="text/css">
        <!--	
        body { background-color: white; color: black; }
        #container { width: 400px; }
        #message   { width: 400px; color: black; background-color: #FFFFCC; }
        #bodytitle { font: 13pt/15pt verdana, arial, sans-serif; height: 35px; vertical-align: top; }
        .bodytext  { font: 8pt/11pt verdana, arial, sans-serif; }
        a:link     { font: 8pt/11pt verdana, arial, sans-serif; color: red; }
        a:visited  { font: 8pt/11pt verdana, arial, sans-serif; color: #4e4e4e; }
        -->
        </style>
</head>
<body>
<table cellpadding="3" cellspacing="5" id="container">
<tr>
        <td id="bodytitle" width="100%">Database error</td>
</tr>
<tr>
        <td class="bodytext" colspan="2">The database has encountered a problem.</td>
</tr>
<tr>
        <td colspan="2"><hr /></td>
</tr>
<tr>
        <td class="bodytext" colspan="2">
                Please try the following:
                <ul>
                        <li>Load the page again by clicking the <a href="#" onclick="window.location = window.location;">Refresh</a> button in your web browser.</li>
                        <li>Click the <a href="javascript:history.back(1)">Back</a> button to try another link.</li>
                </ul>
        </td>
</tr>
<tr>
        <td class="bodytext" colspan="2">The technical staff have been notified of the error.  We apologise for any inconvenience.</td>
</tr>
<tr>
        <td class="bodytext" colspan="2">' . $message . '</td>
</tr>
</table>

</body>
</html>';
                        // tell the search engines that our service is temporarily unavailable to prevent indexing db errors
                        header('HTTP/1.1 503 Service Temporarily Unavailable');
                        header('Status: 503 Service Temporarily Unavailable');
                        header('Retry-After: 3600');
                        
                        echo $template;
			exit();
		}
	}

	/**
	* Function to fetch the total number of affected rows for the connection
	* 
        * @return      nothing
	*/
	function affected_rows()
	{
		return $this->functions['affected_rows']();
	}
        
	/**
	* Function to determine if a field within a table exists
	* 
	* @param       string       field name
	* @param       string       table name
	*
        * @return      boolean      Returns false on no field existing, true on field existing
	*/
        function field_exists($field = '', $table = '')
        {
                $exists = false;
                $columns = $this->query("SHOW COLUMNS FROM $table");
                while ($c = $this->fetch_assoc($columns))
                {
                        if ($c['Field'] == $field)
                        {
                                $exists = true;
                                break;
                        }
                }
                
                return $exists;
        }
	
	/**
	* Function to determine if a database table exists based on the currently selected database
	* 
	* @param       string       table name
	*
	* @return      boolean      Returns false on no table existing, true on table existing
	*/
	function table_exists($table = '')
	{		
		$res = $this->query("SHOW TABLES LIKE '$table'");
		if ($this->num_rows($res) > 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
        /**
        * Function to determine if a field within a table exists, and if not, to automatically add the necessary field column details
        * 
        * @param       string       database table name
        * @param       string       table field name to add (if does not already exist)
        * @param       string       table field attributes to process (ie: VARCHAR(250) NOT NULL)
        * @param       string       table field name that we'll add our new field name after (ie: AFTER `title`)
        *
        * @return      boolean      Returns valid sql string if added, blank string if already exists
        */
        function add_field_if_not_exist($table = '', $column = '', $attributes = '', $addaftercolumn = '', $doquery = true)
        {
                $exists = false;
                $sql = '';
                
                $columns = $this->query("SHOW COLUMNS FROM $table");
                while ($c = $this->fetch_assoc($columns))
                {
                        if (isset($c['Field']) AND !empty($c['Field']) AND $c['Field'] == $column)
                        {
                                $exists = true;
                                break;
                        }
                }
                
                if ($exists == false)
                {
                        if ($doquery)
                        {
                                $sql = "ALTER TABLE `$table` ADD `$column` $attributes $addaftercolumn";
                                $this->query($sql);
                        }
                        else
                        {
                                $sql = "ALTER TABLE `$table` ADD `$column` $attributes $addaftercolumn";
                        }
                }
                
                return $sql;
        }

	/**
	* Function to fetch a field value result from a table
	* 
	* @param       string       table name
	* @param       string       sql condition code
	* @param       string       field name
	* @param       bool         (optional) cache results to file?
	*
        * @return      nothing
	*/
	function fetch_field($tbl = '', $condition = '', $field = '', $cache_to_file = null)
	{
		return $this->cached_fetch_field($tbl, $condition, $field, $cache_to_file);
	}
    
	/**
	* Function to read any data based on a supplied container name and a query as arguments.
	*
	* @param       string        container name
	* @param       string        query
	* 
        * @return      bool          Returns true or false if we can read the data within the supplied container name and associated query
	*/
	function data_read($container_name = '', $query = '', $type = DB_ASSOC)
	{
		global $__db_file_cache;
		
		if (!isset($__db_file_cache))
		{
			$__db_file_cache = array();
		}
		
		if ($this->cache_to_db AND $this->cache_db_data_at_once !== false AND $this->cache_db_data_at_once_loaded === false)
		{			
			$result = $this->functions['query']("
                                SELECT title, data
                                FROM ".DB_PREFIX."cache
                        ");
			while ($row = $this->functions['fetch_array']($result, $this->types[$type]))
			{
				$pos = mb_strrpos($row['data'], '|');
				if ($pos !== false)
				{
					$__db_file_cache[$row['title']] = mb_substr($row['data'], 0, $pos);
				}
			}	
			$this->cache_db_data_at_once_loaded = true;	
		}

		if (isset($__db_file_cache[$container_name]))
		{
			return $__db_file_cache[$container_name];
		}

		if ($this->cache_to_db AND $this->cache_db_data_at_once === false)
		{		
			$container_name_without_addslashed = $container_name;
			$container_name = addslashes($container_name);
                        
			$query = $this->functions['query']("
                                SELECT title, data, datetime
                                FROM ".DB_PREFIX."cache 
                                WHERE title = '".$this->escape_string($container_name)."'
                        ");
			if ($this->functions['num_rows']($query) > 0)
			{
				$__db_file_cache[$container_name_without_addslashed] = $query['data'];
				$res = $this->functions['fetch_array']($query, $this->types[$type]);
                                
				return $res['data'];
			}
			else 
			{
				return false;	
			}
		}

		if (!$this->cache_to_db)
		{
			if (file_exists($container_name))
			{
				// check if cache is not outdated
				if (filemtime($container_name) + ($this->cache_timeo*60) < time())
				{
					return false;
				}
			}
			else 
			{
				return false;	
			}
			$__db_file_cache[$container_name] = file_get_contents(DIR_TMP . $container_name); 
			$crc = mb_strrchr($__db_file_cache[$container_name], '|');
			if ($crc !== '|' . crc32($query))
                        {
				return false;
                        }
                        
			return mb_substr($__db_file_cache[$container_name], 0, mb_strlen($__db_file_cache[$container_name]) - mb_strlen($crc));
		}
		
		return false;
	}
	
	/**
	* Function to write any data based on a supplied container name and a query as argument.
	* Notes: its important to do this, so duplicated queries get read from memory
	* rather than from db in "load at once" db mode in other modes data will be read "from memory" instead of "from cache"
	* 
	* @param       string        container name
	* @param       string        data to write
	* @param       string        query
	* 
        * @return      nothing
	*/
	function data_write($container_name = '', $data = '', $query = '')
	{
		global $__db_file_cache;
		
		$__db_file_cache["$container_name"] = $data;
		
		if ($this->cache_to_db)
		{
			$this->functions['query']("
                                INSERT INTO `".DB_PREFIX."cache`
                                (title, data, datetime) 
                                VALUES(
                                '".$this->escape_string($container_name)."',
                                '".$this->escape_string($data . '|' . crc32($query))."',
                                NOW()) 
                                    ON DUPLICATE KEY
                                UPDATE data = '".$this->escape_string($data)."'
                        ") OR die($this->functions['error']());
		}
		else 
		{
			$f = fopen(DIR_TMP . $container_name, 'w+');
			flock($f, LOCK_EX);
			fwrite($f, $data . '|' . crc32($query));
			flock($f, LOCK_UN);
			fclose($f);		
		}
	}
	
	/**
	* Function to read any data based on a supplied container name and a query as argument.
	*
	* @param       string        query
	* @param       string        database link identifier
	* @param       bool          cache to file system?
	* @param       bool          query is buffered?
	* @param       string        database array result type (mysql_both default)
	* 
        * @return      array         Returns array of cached query requested
	*/
	function cached_query($query = '', $link = null, $cache_to_file = null, $buffered = true, $type = DB_BOTH)
	{		
		if ($cache_to_file === true)
		{
			$cache_to_file = md5($query);
		}
		if ($cache_to_file !== null)
		{
			$cache_to_file = $cache_to_file . '.sqlcache';
		}
		if (isset($cache_to_file))
		{
			$data = $this->data_read($cache_to_file, $query);
			if ($data !== false)
			{		
				if (($data = unserialize($data)) === false)
				{
					// data corruption - very unlikable because crc is valid	
					$data = false;
				}
				else 
				{
					return array($data, 0, 0);
				}
			}
			if ($data === false);
			{		
				if ($link === null)
				{
					$result = $this->functions[$buffered ? 'query' : 'query_unbuffered']($query);
				}
				else 
				{
					$result = $this->functions[$buffered ? 'query' : 'query_unbuffered']($query, $link);
				}
				
				$ret = array();
				while ($row = $this->functions['fetch_array']($result, $this->types[$type]))
				{
					$ret[] = $row;
				}
				
				$this->data_write($cache_to_file, serialize($ret), $query);	
				
				// move back to beginning
				$this->functions['data_seek']($result, 0);
				
				return $result;
			}
		}

		return $this->functions[$buffered ? 'query' : 'query_unbuffered']($query);
	}

	/**
	* Cachable version of the mysql_fetch_array() where results can be stored in the filesystem or database "cache" table
	*
	* @param       string        query resource
	* @param       string        database resource type
	* 
        * @return      array         Returns array of cached query requested
	*/
	function cached_fetch_array(&$query_res, $type = DB_BOTH)
	{
		if (is_array($query_res))
		{
			// our cached sql [0] - array with results, [1] - row position, [2] - column position
                        $count = count($query_res[0]);
			if ($query_res[1] < $count)
			{
				// return numeric array
                                if ($type == DB_NUM)
                                {
                                        $ret = $query_res[0][$query_res[1]++];
                                        
                                        $ret2 = array();
                                        foreach ($ret as $key => $value)
                                        {
                                                if (is_numeric($key))
                                                {
                                                        $ret2[intval($key)] = $value;
                                                }
                                        }
                                        
                                        return $ret2;
                                }
                                
                                // return associative array
                                if ($type == DB_ASSOC)
                                {
                                        $ret = $query_res[0][$query_res[1]++];
                                        
                                        $ret2 = array();
                                        foreach ($ret as $key => $value)
                                        {
                                                if (!is_numeric($key))
                                                {
                                                        $ret2["$key"] = $value;
                                                }
                                        }
                                        
                                        return $ret2;
                                }
				
				return $query_res[0][$query_res[1]++];
			}
			else
			{
				return false;
			}
		}
		else
		{
			return @$this->functions['fetch_array']($query_res, $this->types[$type]);
		}
	}

	/**
	* Cachable version of fetch_field where field results can be stored in the filesystem or database "cache" table
	*
	* @param       string        database table
	* @param       string        where condition
	* @param       string        table field name
	* @param       boolean       cache filename (default null)
	* 
        * @return      array         Returns array of cached query requested
	*/
        function cached_fetch_field($tbl = '', $condition = '', $field = '', $cache_to_file = null)
	{
		$result = $this->cached_query("
                        SELECT " . $this->escape_string($field) . "
                        FROM " . $this->escape_string($tbl) . "
                        WHERE " . $condition
                , null, $cache_to_file);
		$row = ($this->cached_fetch_array($result));
		
		return $row["$field"];
	}
	
	/**
	* Cachable version of num rows
	* 
	* @param       string        sql code
	* 
        * @return      nothing
	*/
	function cached_num_rows($result = '')
	{
		if (is_array($result))
		{
			return count($result[0]);
		}
		else 
		{
			return @$this->functions['num_rows']($result);
		}
	}
	
	
	
	/**
	* Timer function
	* 
        * @return      nothing
	*/
	function timer()
	{
		$this->add();
	}

	/**
	* Timer add function
	* 
        * @return      nothing
	*/
	function add()
	{
		if (!$this->start) 
		{
			$mtime1 = explode(" ", microtime());
			$this->start = $mtime1[1] + $mtime1[0];
		}
	}

	/**
	* Get Time from timer() function
	* 
        * @return      nothing
	*/
	function gettime()
	{
		if ($this->end)
		{ // timer has been stopped
			return $this->totaltime;
		}
		else if ($this->start AND !$this->end)
		{ // timer is still going
			$mtime2 = explode(" ", microtime());
			$currenttime = $mtime2[1] + $mtime2[0];
			$totaltime = $currenttime - $this->start;
			return $this->format($totaltime);
		}
		else
		{
			return false;
		}
	}
	
	/**
	* Stop time from timer() function
	* 
        * @return      nothing
	*/
	function stop()
	{
		if ($this->start)
		{
			$mtime2 = explode(" ", microtime());
			$this->end = $mtime2[1] + $mtime2[0];
			$totaltime = $this->end - $this->start;
			$this->totaltime = $totaltime;
			$this->formatted = $this->format($totaltime);
			return $this->formatted;
		}
	}
	
	/**
	* Remove time from timer() function
	* 
        * @return      nothing
	*/
	function remove()
	{
		$this->name = '';
		$this->start = '';
		$this->end = '';
		$this->totaltime = '';
		$this->formatted = '';
	}
	
	/**
	* Format time from timer() function
	* 
        * @return      nothing
	*/
	function format($string = '')
	{
		return number_format($string, 7);
	}
	
	/**
	* Function to execute xxxx_real_escape_string()
	* 
	* @param       string        sql code
	* 
        * @return      nothing
	*/
	function escape_string($query = '')
	{
		if ($this->functions['escape_string'] == $this->functions['real_escape_string'])
		{
			return $this->functions['escape_string']($query, $this->connection_write);
		}
		else
		{
			return $this->functions['escape_string']($query);
		}
	}
}

/**
* MySQL database class to perform the majority of database related functions in ILance
*
* @package      ILance
* @version	$Revision: 1.0.0 $
* @date		$Date: 2007-02-36 09:32:17 -0500 (Wed, 13 Sep 2006) $
*/
class ilance_mysql extends ilance_database
{
	/**
	* MySQL Database Array Resource Types
	*
	* @var	    $types
	*/
        var $types = array(
		DB_NUM => MYSQL_NUM,
		DB_ASSOC => MYSQL_ASSOC,
		DB_BOTH => MYSQL_BOTH
	);
	
	/**
	* MySQL Database Interface Functions
	*
	* @var	    $functions
	*/
	var $functions = array(
		'select_db' => 'mysql_select_db',
		'pconnect' => 'mysql_pconnect',
		'connect' => 'mysql_connect',
		'query' => 'mysql_query',
		'query_unbuffered' => 'mysql_unbuffered_query',
		'fetch_row' => 'mysql_fetch_row',
		'fetch_object' => 'mysql_fetch_object',
		'fetch_array' => 'mysql_fetch_array',
		'fetch_field' => 'mysql_fetch_field',
		'free_result' => 'mysql_free_result',
		'data_seek' => 'mysql_data_seek',
		'error' => 'mysql_error',
		'errno' => 'mysql_errno',
		'affected_rows' => 'mysql_affected_rows',
		'num_rows' => 'mysql_num_rows',
		'num_fields' => 'mysql_num_fields',
		'field_name' => 'mysql_field_name',
		'insert_id' => 'mysql_insert_id',
		'list_tables' => 'mysql_list_tables',
                'list_fields' => 'mysql_list_fields',
		'escape_string' => 'mysql_escape_string',
		'real_escape_string' => 'mysql_real_escape_string',
		'close' => 'mysql_close',
		'client_encoding' => 'mysql_client_encoding',
		'create_db' => 'mysql_create_db',
		'ping' => 'mysql_ping'
	);
	
        /**
	* Constructor
	*
	* @param	object	        ilance registry object
	* @param        integer         cache time out
	* @param        bool            cache results to database within cache table?
	*/
	function ilance_mysql(&$registry, $cachetimeout = 1, $cachetodatabase = true)
	{
                $this->registry =& $registry;
		parent::ilance_database($cachetimeout, $cachetodatabase);
                
                $this->connect();
	}
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.database.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>
