<?php
/*==========================================================================*\
|| ######################################################################## ||
|| # ILance Marketplace Software
|| # -------------------------------------------------------------------- # ||
|| # Copyright ©2000–2009 ILance Inc. All Rights Reserved.	          # ||
|| # This file may not be redistributed in whole or significant part. 	  # ||
|| # ----------------- ILANCE IS NOT FREE SOFTWARE ---------------------- # ||
|| # http://www.ilance.com | http://www.ilance.com/eula	| info@ilance.com # ||
|| # -------------------------------------------------------------------- # ||
|| ######################################################################## ||
\*==========================================================================*/

/**
* Escrow class to perform the majority of escrow and related payment functions in ILance
*
* @package      iLance
* @version	$Revision: 1.0.0 $
* @author       ILance
*/
class escrow
{
        /**
        * Function to process an escrow payment for a particular user for a specific invoice id
        *
        * @param       integer      user id
        * @param       integer      invoice id
        * @param       string       invoice type
        * @param       integer      invoice amount
        * @param       string       payment method
        * @param       string       payment gateway to use
        * @param       string       payment gateway txn
        *
        * @return      bool         Returns true or false if reversal could be completed
        */
        function payment($userid = 0, $invoiceid = 0, $invoicetype = 'escrow', $amount = 0, $method = 'account', $gateway = '', $gatewaytxn = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                // #### IPN MODE ###############################################
                if ($method == 'ipn')
                {
                        if ($ilconfig['escrowsystem_enabled'] == 0)
                        {
                                return 0;
                        }
                        
                        $sql_invoice = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "invoices
                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                LIMIT 1
                        ");
                        if ($ilance->db->num_rows($sql_invoice) > 0)
                        {
                                $res_invoice = $ilance->db->fetch_array($sql_invoice);
                                $userid = $res_invoice['user_id'];
                                
                                $sql_escrow = $ilance->db->query("
                                        SELECT *
                                        FROM " . DB_PREFIX . "projects_escrow
                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                            AND status = 'pending'
                                        LIMIT 1
                                ");
                                if ($ilance->db->num_rows($sql_escrow) > 0)
                                {
                                        $res_escrow = $ilance->db->fetch_array($sql_escrow);
                                        
                                        // #### SERVICE BUYER FUNDING ESCROW ACCOUNT #######
                                        
                                        // first of all, if we are the buyer (owner) of a service auction,
                                        // let's determine if we had any fees associated with this escrow deposit..
                                        // if we do, generate a paid invoice showing the fee as complete just to allow
                                        // admin and user to review from transaction history
                                        if ($res_escrow['project_user_id'] == $userid)
                                        {
                                                // we are the actual service buyer (owner of auction) funding our escrow account
                                                // buyer escrow fees
                                                $fee = 0;
                                                $bidamount = $res_escrow['bidamount'];
                                                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                                {
                                                        // escrow commission fees to auction owner enabled
                                                        // find out how the admin charges and generate unpaid invoice
                                                        if ($ilconfig['escrowsystem_servicebuyerfixedprice'] > 0)
                                                        {
                                                                // fixed escrow cost to buyer
                                                                $fee = $ilconfig['escrowsystem_servicebuyerfixedprice'];
                                                        }
                                                        else
                                                        {
                                                                if ($ilconfig['escrowsystem_servicebuyerpercentrate'] > 0)
                                                                {
                                                                        // percentage rate of total winning bid amount
                                                                        // which would be the same as the amount being forwarded into escrow
                                                                        $fee = ($bidamount * $ilconfig['escrowsystem_servicebuyerpercentrate'] / 100);
                                                                }
                                                        }
                                                        if ($fee > 0)
                                                        {
                                                                // generate paid escrow transaction to the buyer of the project
                                                                // this will allow the admin and buyer to see the commission fee invoice
                                                                // separate for this escrow transaction and since we've already applied
                                                                // any applicable taxes, we'll send a 0 to the ending of this accounting
                                                                // function to disable any tax support
                                                                $ilance->accounting = construct_object('api.accounting');
                                                                $escrowfee_invoice_id = $ilance->accounting->insert_transaction(
                                                                        0,
                                                                        intval($res_escrow['project_id']),
                                                                        0,
                                                                        intval($res_escrow['project_user_id']),
                                                                        0,
                                                                        0,
                                                                        0,
                                                                        $phrase['_escrow_transaction_fee_securing_funds_for_auction'] . ' - #' . $res_escrow['project_id'] . ': ' . fetch_auction('project_title', $res_escrow['project_id']),
                                                                        sprintf("%01.2f", $fee),
                                                                        sprintf("%01.2f", $fee),
                                                                        'paid',
                                                                        'debit',
                                                                        $gateway,
                                                                        DATETIME24H,
                                                                        DATEINVOICEDUE,
                                                                        DATETIME24H,
                                                                        $gatewaytxn,
                                                                        0,
                                                                        0,
                                                                        1
                                                                );
                                                                
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                        SET isescrowfee = '1'
                                                                        WHERE invoiceid = '" . intval($escrowfee_invoice_id) . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                                
                                                                // because we've inserted a separate fee for the escrow transaction
                                                                // let's set our $amount to the bid amount value only
                                                                // since both members will have access to this transaction
                                                                // and it would appear confusing to the "provider" showing
                                                                // total including the buyer fees.
                                                                // (however the buyer will see his fee as a separate transaction)
                                                                $amount = $res_escrow['bidamount'];
                                                                
                                                                // additionally we'll track this fee payment for the total spending amount for the user
                                                                insert_income_spent(intval($userid), sprintf("%01.2f", $fee), 'credit');
                                                        }
                                                }
                                        }
                                        
                                        // #### PRODUCT BIDDER FUNDING ESCROW ACCOUNT BALANCE ######
                                        else if ($res_escrow['user_id'] == $userid)
                                        {
                                                // we are the actual product bidder (winner of auction) funding our escrow account
                                                // bidder escrow fees
                                                $fee = 0;
                                                $bidamount = $res_escrow['bidamount'];
                                                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                                {
                                                        // escrow commission fees to bidder enabled
                                                        // find out how the admin charges and generate unpaid invoice
                                                        if ($ilconfig['escrowsystem_bidderfixedprice'] > 0)
                                                        {
                                                                // fixed escrow cost to buyer
                                                                $fee = $ilconfig['escrowsystem_bidderfixedprice'];
                                                        }
                                                        else
                                                        {
                                                                if ($ilconfig['escrowsystem_bidderpercentrate'] > 0)
                                                                {
                                                                        // percentage rate of total winning bid amount
                                                                        // which would be the same as the amount being forwarded into escrow
                                                                        $fee = ($bidamount * $ilconfig['escrowsystem_bidderpercentrate'] / 100);
                                                                }
                                                        }
                                                        if ($fee > 0)
                                                        {
                                                                // generate paid escrow fee invoice to the winner of the auction
                                                                // this will allow the admin and winner to see the commission fee invoice
                                                                // separate for this escrow transaction and since we've already applied
                                                                // any applicable taxes, we'll send a 0 to the ending of this accounting
                                                                // function to disable any tax support
                                                                $ilance->accounting = construct_object('api.accounting');
                                                                $escrowfee_invoice_id = $ilance->accounting->insert_transaction(
                                                                        0,
                                                                        intval($res_escrow['project_id']),
                                                                        0,
                                                                        intval($res_escrow['user_id']),
                                                                        0,
                                                                        0,
                                                                        0,
                                                                        $phrase['_escrow_transaction_fee_securing_funds_for_auction'] . ' - #' . $res_escrow['project_id'] . ': ' . fetch_auction('project_title', $res_escrow['project_id']),
                                                                        sprintf("%01.2f", $fee),
                                                                        sprintf("%01.2f", $fee),
                                                                        'paid',
                                                                        'debit',
                                                                        $gateway,
                                                                        DATETIME24H,
                                                                        DATEINVOICEDUE,
                                                                        DATETIME24H,
                                                                        $gatewaytxn,
                                                                        0,
                                                                        0,
                                                                        1,
                                                                        '',
                                                                        0,
                                                                        0,
                                                                        1
                                                                );
                                                                
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                        SET isescrowfee = '1'
                                                                        WHERE invoiceid = '" . intval($escrowfee_invoice_id) . "'
                                                                ", 0, null, __FILE__, __LINE__);
                                                                
                                                                // because we've inserted a separate fee for the escrow transaction
                                                                // let's set our $amount to the bid amount + shipping value only
                                                                // since both members will have access to this escrow transaction
                                                                // and it would appear confusing to the "merchant" showing
                                                                // total including the winning bidders fees.
                                                                // (however the bidder will see his fee as a separate transaction)
                                                                $amount = ($res_escrow['bidamount'] + $res_escrow['shipping']);
                                                                
                                                                // additionally we'll track this fee payment for the total spending amount for the user
                                                                insert_income_spent(intval($userid), sprintf("%01.2f", $fee), 'credit');
                                                        }
                                                }
                                        }
                                        
                                        // because we've generated a separate transaction fee for the bidder above (if applicable)
                                        // let's adjust the main escrow transaction to amounts that make sense to both
                                        // parties while viewing the transaction.  ultimately, the bidder will have another
                                        // separate transaction to view the fee and related taxes
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "invoices
                                                SET paid = '" . $ilance->db->escape_string($amount) . "',
                                                amount = '" . $ilance->db->escape_string($amount) . "',
                                                totalamount = '" . $ilance->db->escape_string($amount) . "',
                                                status = 'paid',
                                                paiddate = '" . DATETIME24H . "',
                                                paymethod = '" . $ilance->db->escape_string($gateway) . "',
                                                custommessage = '" . $ilance->db->escape_string($gatewaytxn) . "'
                                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                                AND user_id = '" . intval($userid) . "'
                                                LIMIT 1
                                        ");
                    
                                        // track income spent
                                        insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                        
                                        $escrowprojectid = $ilance->db->fetch_field(DB_PREFIX."projects_escrow", "escrow_id = '" . $res_escrow['escrow_id'] . "'", "project_id");
                                        if (fetch_auction('project_state', $escrowprojectid) == 'service')
                                        {
                                                // remember - we update the "amount in escrow" as the original bid amount
                                                // since the buyer had to pay the full amount + fee so if this was for example
                                                // an awarded bid for $20.00 and the fee was $5.00 the amount the buyer pays is $25.00
                                                // however the amount that the held in escrow is $20.00 to the provider
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "projects_escrow
                                                        SET escrowamount = '" . $res_escrow['bidamount'] . "',
                                                        status = 'confirmed',
                                                        date_paid = '" . DATETIME24H . "'
                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                        AND escrow_id = '" . $res_escrow['escrow_id'] . "'
                                                        LIMIT 1
                                                ");
                        
                                                $ilance->email = construct_dm_object('email', $ilance);
                                                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                        
                                                $ilance->email->get('escrow_payment_forward_admin');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = fetch_user('email', intval($userid));
                                                $ilance->email->slng = fetch_user_slng($userid);
                                                
                                                $ilance->email->get('escrow_payment_forward_provider');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = fetch_user('email', $res_escrow['user_id']);
                                                $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                                                
                                                $ilance->email->get('escrow_payment_forward_buyer');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                ));
                                                
                                                $ilance->email->send();
                                        }
                                        else if (fetch_auction('project_state', $escrowprojectid) == 'product')
                                        {
                                                // remember - we update the "amount in escrow" as the original bid amount + shipping
                                                // since the bidder had to pay the full amount + fee (maybe) so if this was for example
                                                // an winning bid for $20.00 and the fee was $5.00 the amount the bidder pays is $25.00
                                                // however the amount that is held in escrow is $20.00 to the merchant
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "projects_escrow
                                                        SET escrowamount = '" . ($res_escrow['bidamount'] + $res_escrow['shipping']) . "',
                                                        status = 'started',
                                                        date_paid = '" . DATETIME24H . "'
                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                        AND escrow_id = '" . $res_escrow['escrow_id'] . "'
                                                        LIMIT 1
                                                ");
                        
                                                $ilance->email = construct_dm_object('email', $ilance);
                                                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                        
                                                $ilance->email->get('product_escrow_payment_foward_admin');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                        
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = fetch_user('email', $res_escrow['project_user_id']);
                                                $ilance->email->slng = fetch_user_slng($userid);
                                                
                                                $ilance->email->get('product_escrow_payment_forward_merchant');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                        
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = fetch_user('email', $res_escrow['user_id']);
                                                $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                        
                                                $ilance->email->get('product_escrow_payment_forward_bidders');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{invoice_id}}' => intval($invoiceid),
                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                        
                                                ));
                                                
                                                $ilance->email->send();
                                        }
                                        
                                        return 1;
                                }
                        }
                        
                        return 0;
                }
                
                // #### ACCOUNT BALANCE MODE ###################################
                else if ($method == 'account')
                {
                        $sql_balance = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . intval($userid) . "'
                        ");
                        if ($ilance->db->num_rows($sql_balance) > 0)
                        {
                                $res_balance = $ilance->db->fetch_array($sql_balance);
                                if ($res_balance['available_balance'] >= $amount)
                                {
                                        $avail_balance = $res_balance['available_balance'];
                                        $total_balance = $res_balance['total_balance'];
                                        
                                        $avail_balance_after = ($avail_balance - $amount);
                                        $total_balance_after = ($total_balance - $amount);
                                        
                                        $sql_invoice = $ilance->db->query("
                                                SELECT *
                                                FROM " . DB_PREFIX . "invoices
                                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                                    AND user_id = '" . intval($userid) . "'
                                        ");
                                        if ($ilance->db->num_rows($sql_invoice) > 0)
                                        {
                                                $res_invoice = $ilance->db->fetch_array($sql_invoice);
                                                $sql_escrow = $ilance->db->query("
                                                        SELECT *
                                                        FROM " . DB_PREFIX . "projects_escrow
                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                            AND status = 'pending'
                                                ");
                                                if ($ilance->db->num_rows($sql_escrow) > 0)
                                                {
                                                        $res_escrow = $ilance->db->fetch_array($sql_escrow);
                                                        
                                                        // #### SERVICE BUYER FUNDING ESCROW ACCOUNT #######
                                                        
                                                        // first of all, if we are the buyer (owner) of a service auction,
                                                        // let's determine if we had any fees associated with this escrow deposit..
                                                        // if we do, generate a paid invoice showing the fee as complete just to allow
                                                        // admin and user to review from transaction history
                                                        if ($res_escrow['project_user_id'] == $userid)
                                                        {
                                                                // we are the actual service buyer (owner of auction) funding our escrow account
                                                                // buyer escrow fees
                                                                $fee = 0;
                                                                $bidamount = $res_escrow['bidamount'];
                                                                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                                                {
                                                                        // escrow commission fees to auction owner enabled
                                                                        // find out how the admin charges and generate unpaid invoice
                                                                        if ($ilconfig['escrowsystem_servicebuyerfixedprice'] > 0)
                                                                        {
                                                                                // fixed escrow cost to buyer
                                                                                $fee = $ilconfig['escrowsystem_servicebuyerfixedprice'];
                                                                        }
                                                                        else
                                                                        {
                                                                                if ($ilconfig['escrowsystem_servicebuyerpercentrate'] > 0)
                                                                                {
                                                                                        // percentage rate of total winning bid amount
                                                                                        // which would be the same as the amount being forwarded into escrow
                                                                                        $fee = ($bidamount * $ilconfig['escrowsystem_servicebuyerpercentrate'] / 100);
                                                                                }
                                                                        }
                                                                        
                                                                        if ($fee > 0)
                                                                        {
                                                                                // generate paid escrow fee invoice to the buyer of the project
                                                                                // this will allow the admin and buyer to see the commission fee invoice
                                                                                // separate for this escrow transaction and since we've already applied
                                                                                // any applicable taxes, we'll send a 0 to the ending of this accounting
                                                                                // function to disable any tax support
                                                                                $ilance->accounting = construct_object('api.accounting');
                                                                                $escrowfee_invoice_id = $ilance->accounting->insert_transaction(
                                                                                        0,
                                                                                        intval($res_escrow['project_id']),
                                                                                        0,
                                                                                        intval($res_escrow['project_user_id']),
                                                                                        0,
                                                                                        0,
                                                                                        0,
                                                                                        $phrase['_escrow_transaction_fee_securing_funds_for_auction'] . ' - #' . $res_escrow['project_id'] . ': ' . fetch_auction('project_title', $res_escrow['project_id']),
                                                                                        sprintf("%01.2f", $fee),
                                                                                        sprintf("%01.2f", $fee),
                                                                                        'paid',
                                                                                        'debit',
                                                                                        'account',
                                                                                        DATETIME24H,
                                                                                        DATEINVOICEDUE,
                                                                                        DATETIME24H,
                                                                                        '',
                                                                                        0,
                                                                                        0,
                                                                                        1
                                                                                );
                                                                                
                                                                                $ilance->db->query("
                                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                                        SET isescrowfee = '1'
                                                                                        WHERE invoiceid = '" . intval($escrowfee_invoice_id) . "'
                                                                                ", 0, null, __FILE__, __LINE__);
                                                                                
                                                                                // because we've inserted a separate fee for the escrow transaction
                                                                                // let's set our $amount to the bid amount value only
                                                                                // since both members will have access to this transaction
                                                                                // and it would appear confusing to the "provider" showing
                                                                                // total including the buyer fees.
                                                                                // (however the buyer will see his fee as a separate transaction)
                                                                                $amount = $res_escrow['bidamount'];
                                                                                
                                                                                // additionally we'll track this fee payment for the total spending amount for the user
                                                                                insert_income_spent(intval($userid), sprintf("%01.2f", $fee), 'credit');
                                                                        }
                                                                }
                                                        }
                                                        
                                                        // #### PRODUCT BIDDER FUNDING ESCROW ACCOUNT ######
                                                        else if ($res_escrow['user_id'] == $userid)
                                                        {
                                                                // we are the actual product bidder (winner of auction) funding our escrow account
                                                                // bidder escrow fees
                                                                $fee = 0;
                                                                $bidamount = $res_escrow['bidamount'];
                                                                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                                                {
                                                                        // escrow commission fees to bidder enabled
                                                                        // find out how the admin charges and generate unpaid invoice
                                                                        if ($ilconfig['escrowsystem_bidderfixedprice'] > 0)
                                                                        {
                                                                                // fixed escrow cost to buyer
                                                                                $fee = $ilconfig['escrowsystem_bidderfixedprice'];
                                                                        }
                                                                        else
                                                                        {
                                                                                if ($ilconfig['escrowsystem_bidderpercentrate'] > 0)
                                                                                {
                                                                                        // percentage rate of total winning bid amount
                                                                                        // which would be the same as the amount being forwarded into escrow
                                                                                        $fee = ($bidamount * $ilconfig['escrowsystem_bidderpercentrate'] / 100);
                                                                                }
                                                                        }
                                                                        
                                                                        if ($fee > 0)
                                                                        {
                                                                                // generate paid escrow fee invoice to the winner of the auction
                                                                                // this will allow the admin and winner to see the commission fee invoice
                                                                                // separate for this escrow transaction and since we've already applied
                                                                                // any applicable taxes, we'll send a 0 to the ending of this accounting
                                                                                // function to disable any tax support
                                                                                $ilance->accounting = construct_object('api.accounting');
                                                                                $escrowfee_invoice_id = $ilance->accounting->insert_transaction(
                                                                                        0,
                                                                                        intval($res_escrow['project_id']),
                                                                                        0,
                                                                                        intval($res_escrow['user_id']),
                                                                                        0,
                                                                                        0,
                                                                                        0,
                                                                                        $phrase['_escrow_transaction_fee_securing_funds_for_auction'] . ' - #' . $res_escrow['project_id'] . ': ' . fetch_auction('project_title', $res_escrow['project_id']),
                                                                                        sprintf("%01.2f", $fee),
                                                                                        sprintf("%01.2f", $fee),
                                                                                        'paid',
                                                                                        'debit',
                                                                                        'account',
                                                                                        DATETIME24H,
                                                                                        DATEINVOICEDUE,
                                                                                        DATETIME24H,
                                                                                        '',
                                                                                        0,
                                                                                        0,
                                                                                        1,
                                                                                        '',
                                                                                        0,
                                                                                        0,
                                                                                        1
                                                                                );
                                                                                
                                                                                $ilance->db->query("
                                                                                        UPDATE " . DB_PREFIX . "invoices
                                                                                        SET isescrowfee = '1'
                                                                                        WHERE invoiceid = '" . intval($escrowfee_invoice_id) . "'
                                                                                ", 0, null, __FILE__, __LINE__);
                                                                                
                                                                                // because we've inserted a separate fee for the escrow transaction
                                                                                // let's set our $amount to the bid amount + shipping value only
                                                                                // since both members will have access to this escrow transaction
                                                                                // and it would appear confusing to the "merchant" showing
                                                                                // total including the winning bidders fees.
                                                                                // (however the bidder will see his fee as a separate transaction)
                                                                                $amount = ($res_escrow['bidamount'] + $res_escrow['shipping']);
                                                                                
                                                                                // additionally we'll track this fee payment for the total spending amount for the user
                                                                                insert_income_spent(intval($userid), sprintf("%01.2f", $fee), 'credit');
                                                                        }
                                                                }
                                                        }
                                                        
                                                        // because we've generated a separate transaction fee for the bidder above (if applicable)
                                                        // let's adjust the main escrow transaction to amounts that make sense to both
                                                        // parties while viewing the transaction.  ultimately, the bidder will have another
                                                        // separate transaction to view the fee and related taxes
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "invoices
                                                                SET paid = '" . $amount . "',
                                                                amount = '" . $amount . "',
                                                                totalamount = '" . $amount . "',
                                                                status = 'paid',
                                                                paiddate = '" . DATETIME24H . "'
                                                                WHERE invoiceid = '" . intval($invoiceid) . "'
                                                                AND user_id = '" . intval($userid) . "'
                                                                LIMIT 1
                                                        ");
                                    
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET available_balance = '" . $avail_balance_after . "',
                                                                total_balance = '" . $total_balance_after . "'
                                                                WHERE user_id = '" . intval($userid) . "'
                                                        ");
                                    
                                                        // track income spent
                                                        insert_income_spent(intval($userid), sprintf("%01.2f", $amount), 'credit');
                                                        
                                                        $escrowprojectid = $ilance->db->fetch_field(DB_PREFIX . "projects_escrow", "escrow_id = '" . $res_escrow['escrow_id'] . "'", "project_id");
                                                        if (fetch_auction('project_state', $escrowprojectid) == 'service')
                                                        {
                                                                // remember - we update the "amount in escrow" as the original bid amount
                                                                // since the buyer had to pay the full amount + fee so if this was for example
                                                                // an awarded bid for $20.00 and the fee was $5.00 the amount the buyer pays is $25.00
                                                                // however the amount that the held in escrow is $20.00 to the provider
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "projects_escrow
                                                                        SET escrowamount = '" . $res_escrow['bidamount'] . "',
                                                                        status = 'confirmed',
                                                                        date_paid = '" . DATETIME24H . "'
                                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                                            AND escrow_id = '" . $res_escrow['escrow_id'] . "'
                                                                        LIMIT 1
                                                                ");
                                        
                                                                $ilance->email = construct_dm_object('email', $ilance);
                                                                
                                                                $ilance->email->mail = SITE_EMAIL;
                                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                                $ilance->email->get('escrow_payment_forward_admin');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                                
                                                                $ilance->email->mail = fetch_user('email', $userid);
                                                                $ilance->email->slng = fetch_user_slng($userid);
                                        
                                                                $ilance->email->get('escrow_payment_forward_provider');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                                
                                                                $ilance->email->mail = fetch_user('email', $res_escrow['user_id']);
                                                                $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                                                                
                                                                $ilance->email->get('escrow_payment_forward_buyer');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                        }
                                                        else if (fetch_auction('project_state', $escrowprojectid) == 'product')
                                                        {
                                                                // remember - we update the "amount in escrow" as the original bid amount + shipping
                                                                // since the bidder had to pay the full amount + fee (maybe) so if this was for example
                                                                // an winning bid for $20.00 and the fee was $5.00 the amount the bidder pays is $25.00
                                                                // however the amount that is held in escrow is $20.00 to the merchant
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "projects_escrow
                                                                        SET escrowamount = '" . ($res_escrow['bidamount'] + $res_escrow['shipping']) . "',
                                                                        status = 'started',
                                                                        date_paid = '" . DATETIME24H . "'
                                                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                                                        AND escrow_id = '" . $res_escrow['escrow_id'] . "'
                                                                        LIMIT 1
                                                                ");
                                        
                                                                $ilance->email = construct_dm_object('email', $ilance);
                                                                
                                                                $ilance->email->mail = SITE_EMAIL;
                                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                                $ilance->email->get('product_escrow_payment_foward_admin');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                                
                                                                $ilance->email->mail = fetch_user('email', $res_escrow['project_user_id']);
                                                                $ilance->email->slng = fetch_user_slng($res_escrow['project_user_id']);
                                        
                                                                $ilance->email->get('product_escrow_payment_forward_merchant');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                                
                                                                $ilance->email->mail = fetch_user('email', $res_escrow['user_id']);
                                                                $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                                                                
                                                                $ilance->email->get('product_escrow_payment_forward_bidders');		
                                                                $ilance->email->set(array(
                                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                                        '{{invoice_id}}' => intval($invoiceid),
                                                                        '{{biddername}}' => fetch_user('username', $res_escrow['user_id']),
                                                                        '{{ownername}}' => fetch_user('username', $res_escrow['project_user_id']),
                                                                        '{{project_title}}' => fetch_auction('project_title', $res_escrow['project_id']),
                                                                        '{{escrow_amount}}' => $ilance->currency->format(($res_escrow['bidamount']+$res_escrow['shipping'])),
                                                                ));
                                                                
                                                                $ilance->email->send();
                                                        }
                                                        
                                                        print_notice($phrase['_invoice_payment_complete'], $phrase['_your_invoice_has_been_paid_in_full'], $ilpage['accounting'], $phrase['_my_account']);
                                                        exit();
                                                }
                                                else
                                                {
                                                        $area_title = $phrase['_invoice_payment_menu_denied_payment_does_not_belong_to_user'];
                                                        $page_title = SITE_NAME . ' - ' . $phrase['_invoice_payment_menu_denied_payment'];
                                                        
                                                        print_notice($phrase['_invoice_error'], $phrase['_were_sorry_this_invoice_does_not_exist']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                                        exit();
                                                }
                                        }
                                        else
                                        {
                                                $area_title = $phrase['_funds_not_available'];
                                                $page_title = SITE_NAME . ' - ' . $phrase['_funds_not_available'];
                                                
                                                print_notice($phrase['_invoice_payment_warning_insufficient_funds'], $phrase['_were_sorry_this_invoice_can_not_be_paid_due_to_insufficient_funds']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                                exit();
                                        }                        
                                }
                                else
                                {
                                        $area_title = $phrase['_funds_not_available'];
                                        $page_title = SITE_NAME . ' - ' . $phrase['_funds_not_available'];
                                        
                                        print_notice($phrase['_invoice_payment_warning_insufficient_funds'], $phrase['_were_sorry_this_invoice_can_not_be_paid_due_to_insufficient_funds']."<br /><br />".$phrase['_please_contact_customer_support'], $ilpage['accounting'], $phrase['_my_account']);
                                        exit();   
                                }
                        }
                }                
        }
        
        /**
        * Function to process a service escrow payment refund
        *
        * @param       integer      bid id
        * @param       integer      project id
        * @param       string       project owner user id
        *
        * @return      bool         Returns true or false if refund could be completed
        */
        function service_escrow_refund($bidid = 0, $projectid = 0, $projectuserid = 0)
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig;
                
                $projectid = intval($projectid);
                $bidid = intval($bidid);
                $projectuserid = intval($projectuserid);
        
                $sql_rfp = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "projects
                        WHERE date_end <= '" . DATETODAY . " " . TIMENOW . "'
                            AND user_id = '" . $projectuserid . "'
                            AND project_id = '" . $projectid . "'
                        LIMIT 1
                ");
                if ($ilance->db->num_rows($sql_rfp) > 0)
                {
                        $new_project_status = 'expired';
                }
                else
                {
                        $new_project_status = 'open';
                }
        
                // ensure escrow payment is paid
                $escrow_check = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "projects_escrow
                        WHERE project_id = '" . $projectid . "'
                            AND project_user_id = '" . $projectuserid . "'
                            AND bid_id = '" . $bidid . "'
                ");
                if ($ilance->db->num_rows($escrow_check) > 0)
                {
                        // escrow account exists: (recredit) account & create invoice trail
                        $resultescrow = $ilance->db->fetch_array($escrow_check);
                        $escrowrefund = $resultescrow['escrowamount'];
                        
                        if ($escrowrefund > 0)
                        {
                                // update account balances and credit account service buyer for amount held in escrow
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "users
                                        SET total_balance = total_balance + $escrowrefund,
                                        available_balance = available_balance + $escrowrefund
                                        WHERE user_id = '" . $projectuserid . "'
                                ");
                                
                                // subtract escrow payment from total income spent
                                insert_income_spent($projectuserid, sprintf("%01.2f", $escrowrefund), 'debit');
                                
                                // created refund invoice trail
                                $ilance->accounting = construct_object('api.accounting');
                                $ilance->accounting->insert_transaction(
                                        0,
                                        0,
                                        0,
                                        intval($projectuserid),
                                        0,
                                        0,
                                        0,
                                        $phrase['_escrow_refund_credit'] . ": " . stripslashes($ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $resultescrow['invoiceid'] . "'", "description")),
                                        sprintf("%01.2f", $escrowrefund),
                                        sprintf("%01.2f", $escrowrefund),
                                        'paid',
                                        'credit',
                                        'account',
                                        DATETIME24H,
                                        DATEINVOICEDUE,
                                        DATETIME24H,
                                        '',
                                        0,
                                        0,
                                        0
                                );
                                
                                // reopen auction if time is left
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects
                                        SET escrow_id = '0',
                                        status = '" . $new_project_status . "'
                                        WHERE user_id = '" . $projectuserid . "'
                                            AND project_id = '" . $projectid . "'
                                        LIMIT 1
                                ");
                        }
                        
                        // remove the escrow account for this auction
                        $ilance->db->query("
                                DELETE FROM " . DB_PREFIX . "projects_escrow
                                WHERE project_user_id = '" . $projectuserid . "'
                                    AND project_id = '" . $projectid . "'
                                    AND bid_id = '" . $bidid . "'
                                LIMIT 1
                        ");
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('escrow_cancelled_autorefund_admin');		
                        $ilance->email->set(array(
                                '{{owner}}' => fetch_user('username', $projectuserid),
                                '{{provider}}' => fetch_user('username', $resultescrow['user_id']),
                                '{{project_title}}' => fetch_auction('project_title', $projectid),
                                '{{escrowamount}}' => $ilance->currency->format($resultescrow['escrowamount']),
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = fetch_user('email', $projectuserid);
                        $ilance->email->slng = fetch_user_slng($projectuserid);
                        
                        $ilance->email->get('service_escrow_cancelled_autorefund');		
                        $ilance->email->set(array(
                                '{{owner}}' => fetch_user('username', $projectuserid),
                                '{{provider}}' => fetch_user('username', $resultescrow['user_id']),
                                '{{project_title}}' => fetch_auction('project_title', $projectid),
                                '{{escrowamount}}' => $ilance->currency->format($resultescrow['escrowamount']),
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = fetch_user('email', $resultescrow['user_id']);
                        $ilance->email->slng = fetch_user_slng($resultescrow['user_id']);
                        
                        $ilance->email->get('service_escrow_cancelled_autorefund');		
                        $ilance->email->set(array(
                                '{{owner}}' => fetch_user('username', $projectuserid),
                                '{{provider}}' => fetch_user('username', $resultescrow['user_id']),
                                '{{project_title}}' => fetch_auction('project_title', $projectid),
                                '{{escrowamount}}' => $ilance->currency->format($resultescrow['escrowamount']),
                        ));
                        
                        $ilance->email->send();
                }
        }
        
        /**
        * Function to process a service escrow function within the admin control panel
        *
        * @param       string       mode
        * @param       integer      escrow id
        * @param       string       admin mode
        *
        * @return      bool         Returns true or false if refund could be completed
        */
        function service_escrow($mode, $escrowid = 0, $adminmode = '')
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                if (isset($mode) AND $mode == 'cancel' AND isset($escrowid) AND $escrowid > 0 AND ($ilconfig['escrowsystem_payercancancelfunds'] OR $adminmode))
                {
                        $sql_escrow_amount = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE escrow_id = '" . intval($escrowid) . "'
                                LIMIT 1
                        ");
                        if ($ilance->db->num_rows($sql_escrow_amount) > 0)
                        {
                                $res_escrow = $ilance->db->fetch_array($sql_escrow_amount);
                                $buyerid = $res_escrow['project_user_id'];
                                $amount = $res_escrow['escrowamount'];
                                if (empty($amount))
                                {
                                        $amount = 0;   
                                }
                                
                                // update escrow table and set as cancelled
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects_escrow
                                        SET status = 'cancelled'
                                        WHERE escrow_id = '" . intval($escrowid) . "'
                                            AND project_user_id = '" . intval($buyerid) . "'
                                        LIMIT 1
                                ");
                                
                                $sql_account = $ilance->db->query("
                                        SELECT available_balance, total_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . intval($buyerid) . "'
                                ");
                                if ($ilance->db->num_rows($sql_account) > 0)
                                {
                                        $res_account = $ilance->db->fetch_array($sql_account);
                                        $avail_balance = $res_account['available_balance'];
                                        $total_balance = $res_account['total_balance'];
                                        if ($amount > 0)
                                        {
                                                $new_avail_balance = ($avail_balance + $amount);
                                                $new_total_balance = ($total_balance + $amount);
                                                
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET total_balance = '" . $new_total_balance . "',
                                                        available_balance = '" . $new_avail_balance . "'
                                                        WHERE user_id = '" . intval($buyerid) . "'
                                                ");
                                                
                                                // track income spent
                                                insert_income_spent($buyerid, sprintf("%01.2f", $amount), 'debit');
                                        }
                                        
                                        $ilance->email = construct_dm_object('email', $ilance);
                
                                        $ilance->email->mail = fetch_user('email', $res_escrow['user_id']);
                                        $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                                        
                                        $ilance->email->get('service_escrow_release_cancelled_provider');		
                                        $ilance->email->set(array(
                                                '{{project_id}}' => $res_escrow['project_id'],
                                                '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                '{{escrowamount}}' => $ilance->currency->format($amount),
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        $ilance->email->mail = fetch_user('email', $res_escrow['project_user_id']);
                                        $ilance->email->slng = fetch_user_slng(intval($buyerid));
                                        
                                        $ilance->email->get('service_escrow_release_cancelled_buyer');		
                                        $ilance->email->set(array(
                                                '{{project_id}}' => $res_escrow['project_id'],
                                                '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                '{{escrowamount}}' => $ilance->currency->format($amount),
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        $ilance->email->mail = SITE_EMAIL;
                                        $ilance->email->slng = fetch_site_slng();
                                        
                                        $ilance->email->get('service_escrow_release_cancelled_admin');		
                                        $ilance->email->set(array(
                                                '{{project_id}}' => $res_escrow['project_id'],
                                                '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                '{{escrowamount}}' => $ilance->currency->format($amount),
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        if (isset($adminmode) AND $adminmode)
                                        {
                                                print_action_success($phrase['_funds_within_this_escrow_account_have_been_returned_to_the_buyer_all_parties_have_been_informed'], $ilpage['accounting'] . '?cmd=escrow');
                                                exit();
                                        }
                                        else
                                        {
                                                print_notice($area_title, $phrase['_you_have_cancelled_funds_within_your_escrow_account_for_a_particular_service_auction'], $ilpage['buying'].'?cmd=management&amp;sub=rfp-escrow', $phrase['_return_to_the_previous_menu']);
                                                exit();
                                        }
                                }
                        }
                }
                else if (isset($mode) AND $mode == 'refund' AND $escrowid > 0 AND $adminmode == 1)
                {
                        // refunding escrow allows the admin to take back the money originally forwarded
                        // into the service providers account back to the buyer
                        // this function assumes the escrowamount field is 0.00 since the funds should
                        // have been already released from this escrow account to the provider
                        $sql_escrow_amount = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE escrow_id = '" . intval($escrowid) . "'
                                    AND escrowamount = '0.00'
                                LIMIT 1
                        ");
                        if ($ilance->db->num_rows($sql_escrow_amount) > 0)
                        {
                                $res_escrow = $ilance->db->fetch_array($sql_escrow_amount);
                                
                                // service provider
                                $providerid = $res_escrow['user_id'];
                                
                                // service buyer
                                $buyerid = $res_escrow['project_user_id'];
                                
                                // bid amount entered into escrow
                                $amount = $res_escrow['bidamount'];
                                
                                $escrow['commissionfee'] = 0;
                                $escrow['totalamount'] = sprintf("%01.2f", $amount);
                                $escrow['accountamount'] = sprintf("%01.2f", $amount);
                                
                                // commission fee for escrow release (paid by service provider)
                                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                {
                                        $logic = $this->fetch_escrow_commission_logic('serviceprovider');
                                        if ($logic == 'fixed')
                                        {
                                                // fixed service commission logic
                                                $escrow['commissionfee'] = sprintf("%01.2f", $commissionfee);
                                                $escrow['totalamount']  = sprintf("%01.2f", $amount);
                                                
                                                // service provider paid a fee to obtain this escrow amount
                                                // so take the amount he made and add the fee he/she paid
                                                $escrow['accountamount'] = sprintf("%01.2f", $amount) - $escrow['commissionfee'];
                                        }
                                        else
                                        {
                                                // percentage of the overall cost logic
                                                $escrow['commissionfee'] = sprintf("%01.2f", ($amount * $this->fetch_escrow_commission('serviceprovider') / 100));
                                                $escrow['totalamount']  = sprintf("%01.2f", $amount);
                                                $escrow['accountamount'] = sprintf("%01.2f", $amount) - $escrow['commissionfee'];
                                        }
                                }
                        }
                        else
                        {
                                $amount = 0;
                                $buyerid = 0;
                        }
            
                        // let's finish the refund process	
                        $sql_account2 = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . intval($providerid) . "'
                        ");
                        if ($ilance->db->num_rows($sql_account2) > 0)
                        {
                                #################################################
                                ## DEBIT PROVIDER ACCOUNT #######################
                                #################################################
                                $res_account2 = $ilance->db->fetch_array($sql_account2);
                                $avail_balance2 = $res_account2['available_balance'];
                                $total_balance2 = $res_account2['total_balance'];
                                
                                if ($escrow['accountamount'] > 0)
                                {
                                        // subtract amount released to provider
                                        $new_avail_balance2 = ($avail_balance2 - $escrow['accountamount']);
                                        $new_total_balance2 = ($total_balance2 - $escrow['accountamount']);
                    
                                        // update account balances
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET total_balance = '" . $new_total_balance2 . "',
                                                available_balance = '" . $new_avail_balance2 . "'
                                                WHERE user_id = '" . intval($providerid) . "'
                                        ");
                    
                                        // track income spent
                                        insert_income_reported($providerid, sprintf("%01.2f", $escrow['accountamount']), 'debit');
                    
                                        // create transaction trail
                                        $ilance->accounting = construct_object('api.accounting');
                                        
                                        $ilance->accounting->insert_transaction(
                                                0,
                                                0,
                                                0,
                                                intval($providerid),
                                                0,
                                                0,
                                                0,
                                                $phrase['_refund_from_online_account'] . " - #" . $res_escrow['project_id'] . ": " . fetch_auction('project_title', $res_escrow['project_id']) . ": " . $phrase['_back_to'] . ": " . fetch_user('username', $buyerid),
                                                sprintf("%01.2f", $escrow['accountamount']),
                                                sprintf("%01.2f", $escrow['accountamount']),
                                                'paid',
                                                'debit',
                                                'account',
                                                DATETIME24H,
                                                DATEINVOICEDUE,
                                                DATETIME24H,
                                                $phrase['_refund_forced_by_administration_please_note_this_refund_would_have_included_any_fees_you_paid_during_the_escrow_process'],
                                                0,
                                                0,
                                                0
                                        );
                                }
                
                                #################################################
                                ## CREDIT BUYER ACCOUNT #########################
                                #################################################
                                $sql_account = $ilance->db->query("
                                        SELECT available_balance, total_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . intval($buyerid) . "'
                                ");
                                if ($ilance->db->num_rows($sql_account) > 0)
                                {
                                        $res_account = $ilance->db->fetch_array($sql_account);
                                        $avail_balance = $res_account['available_balance'];
                                        $total_balance = $res_account['total_balance'];
                                        if ($amount > 0)
                                        {
                                                $new_avail_balance = ($avail_balance + $amount);
                                                $new_total_balance = ($total_balance + $amount);
                        
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET total_balance = '" . $new_total_balance . "',
                                                        available_balance = '" . $new_avail_balance . "'
                                                        WHERE user_id = '" . intval($buyerid) . "'
                                                ");
                        
                                                // track income spent
                                                insert_income_spent(intval($buyerid), sprintf("%01.2f", $amount), 'debit');
                                
                                                ##########################################
                                                ## INSERT NEW TXN AS CREDDITED [FOR BUYER]
                                                $ilance->accounting = construct_object('api.accounting');
                                                
                                                $ilance->accounting->insert_transaction(
                                                        0,
                                                        0,
                                                        0,
                                                        intval($buyerid),
                                                        0,
                                                        0,
                                                        0,
                                                        $phrase['_refund_to_online_account'] . " - #" . $res_escrow['project_id'] . ": " . fetch_auction('project_title', $res_escrow['project_id']) . ": " . $phrase['_from'] . ": " . fetch_user('username', $providerid),
                                                        sprintf("%01.2f", $amount),
                                                        sprintf("%01.2f", $amount),
                                                        'paid',
                                                        'credit',
                                                        'account',
                                                        DATETIME24H,
                                                        DATEINVOICEDUE,
                                                        DATETIME24H,
                                                        $phrase['_refund_forced_by_administration_please_note_this_refund_would_have_included_any_fees_you_paid_during_the_escrow_process'],
                                                        0,
                                                        0,
                                                        0
                                                );
                        
                                                #######################################
                                                ## UPDATE ESCROW TABLE > SET AS PENDING
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "projects_escrow
                                                        SET status = 'pending'
                                                        WHERE escrow_id = '" . intval($escrowid) . "'
                                                            AND project_user_id = '" . intval($buyerid) . "'
                                                        LIMIT 1
                                                ");
                                
                                                $ilance->email = construct_dm_object('email', $ilance);
                
                                                $ilance->email->mail = $ilance->db->fetch_field(DB_PREFIX . 'users', 'user_id=' . $res_escrow['user_id'], 'email');
                                                $ilance->email->slng = fetch_user_slng($res_escrow['user_id']);
                                                
                                                $ilance->email->get('service_escrow_release_cancelled_provider');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->send();
                        
                                                $ilance->email->mail = $ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'email');
                                                $ilance->email->slng = fetch_user_slng((int)$buyerid);
                                                
                                                $ilance->email->get('service_escrow_release_cancelled_buyer');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                $ilance->email->get('service_escrow_release_cancelled_admin');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res_escrow['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['user_id'],'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res_escrow['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res_escrow['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                if (isset($adminmode) AND $adminmode)
                                                {
                                                        print_action_success($phrase['_funds_were_debitted_from_the_providers_account_back_into_the_buyers_account_the_escrow_status_for_this_auction_is_pending'], $ilpage['accounting'].'?cmd=escrow');
                                                        exit();
                                                }
                                                else
                                                {
                                                        print_notice($area_title, $phrase['_you_have_cancelled_funds_within_your_escrow_account_for_a_particular_service_auction'], $ilpage['buying'].'?cmd=management&amp;sub=rfp-escrow', $phrase['_return_to_the_previous_menu']);
                                                        exit();
                                                }
                                        }
                                        else
                                        {
                                                $area_title = $phrase['_rfp_escrow_management'] . ' - (' . $phrase['_cancel_release_of_funds_error'] . ')';
                                                $page_title = SITE_NAME . ' - ' . $phrase['_rfp_escrow_management'].' - ('.$phrase['_cancel_release_of_funds_error'].')';
                                                
                                                if (isset($adminmode) AND $adminmode)
                                                {
                                                        print_action_success($phrase['_were_sorry_there_was_an_error_while_cancelling_the_funds_within_escrow'], $ilpage['accounting'].'?cmd=escrow');
                                                        exit();
                                                }
                                                else
                                                {
                                                        print_notice($area_title, $phrase['_were_sorry_there_was_an_error_while_cancelling_the_funds_within_escrow'], $ilpage['buying'].'?cmd=management&amp;sub=rfp-escrow', $phrase['_return_to_the_previous_menu']);
                                                        exit();
                                                }
                                        }
                                }
                        }
                }
                else if (isset($mode) AND $mode == 'confirmrelease' AND $escrowid > 0)
                {
                        // admin (or buyer of auction) forces funds from buyer's account to provider's account
                        // escrow fees will be generated to service provider if applicable
                        // based on the amount held within the 'fee2' column
                        $sql = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE escrow_id = '" . intval($escrowid) . "'
                                    AND status = 'confirmed'
                        ");
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $escrow = array();
                                
                                // escrow buyer
                                $buyerid = $res['project_user_id'];
                                
                                // escrow provider
                                $providerid = $res['user_id'];
                                
                                // total amount in escrow
                                $amount = $res['escrowamount'];
                                
                                // does this escrow account hold a fee to the service provider?
                                if ($res['fee2'] > 0)
                                {
                                        $escrow['commissionfee'] = sprintf("%01.2f", $res['fee2']);
                                }
                                else
                                {
                                        $escrow['commissionfee'] = 0;    
                                }
                                $escrow['totalamount'] = sprintf("%01.2f", $amount);
                                
                                // release funds from escrow; set as finished
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects_escrow
                                        SET escrowamount = '0.00',
                                        status = 'finished',
                                        date_released = '" . DATETIME24H . "'
                                        WHERE escrow_id = '" . intval($escrowid) . "'
                                            AND project_user_id = '" . intval($buyerid) . "'
                                        LIMIT 1
                                ");
                                
                                $ilance->accounting = construct_object('api.accounting');
                                if ($escrow['commissionfee'] > 0)
                                {
                                        // create a paid escrow commission fee based on the release of this escrow payment
                                        // to the service providers online account balance
                                        // do not apply any tax because this fee already has tax applied!
                                        // the ending 1 tells the system to NOT calculate any tax for this entry!
                                        $ilance->accounting->insert_transaction(
                                                0,
                                                0,
                                                0,
                                                intval($providerid),
                                                0,
                                                0,
                                                0,
                                                $phrase['_commission_fee_for_escrow_deposit'] . ' - #' . $res['project_id'] . ': ' . $phrase['_from'] . ' ' . fetch_user('username', $res['project_user_id']),
                                                sprintf("%01.2f", $escrow['commissionfee']),
                                                sprintf("%01.2f", $escrow['commissionfee']),
                                                'paid',
                                                'debit',
                                                'account',
                                                DATETIME24H,
                                                DATEINVOICEDUE,
                                                DATETIME24H,
                                                '',
                                                0,
                                                0,
                                                0,
                                                '',
                                                0,
                                                0,
                                                1 
                                        );
                                        
                                        insert_income_spent($providerid, sprintf("%01.2f", $escrow['commissionfee']), 'credit');
                                        
                                        // since we've auto-paid the fee to the service provider let's re-adjust
                                        // the total amount we'll show as the release to the provider's online account balance
                                        $escrow['totalamount'] = ($escrow['totalamount'] - $escrow['commissionfee']);
                                }
                                
                                // release buyers funds from escrow into the providers online account and show transaction
                                if ($escrow['totalamount'] > 0)
                                {
                                        $ilance->accounting->insert_transaction(
                                                0,
                                                0,
                                                0,
                                                intval($providerid),
                                                0,
                                                0,
                                                0,
                                                $phrase['_escrow_payment_release_for'] . ' - #' . $res['project_id'] . ': ' . fetch_auction('project_title', $res['project_id']) . ': ' . $phrase['_from'] . ': ' . fetch_user('username', $res['project_user_id']),
                                                sprintf("%01.2f", $escrow['totalamount']),
                                                sprintf("%01.2f", $escrow['totalamount']),
                                                'paid',
                                                'credit',
                                                'account',
                                                DATETIME24H,
                                                DATEINVOICEDUE,
                                                DATETIME24H,
                                                '',
                                                0,
                                                0,
                                                0
                                        );
                                                           
                                        // track income reported for provider (release to account)
                                        insert_income_reported($providerid, sprintf("%01.2f", $escrow['totalamount']), 'credit');
                                        
                                        // fetch providers account balance and deposit escrow funds!
                                        $sqlacc = $ilance->db->query("
                                                SELECT available_balance, total_balance
                                                FROM " . DB_PREFIX . "users
                                                WHERE user_id = '" . $providerid . "'
                                        ");
                                        if ($ilance->db->num_rows($sqlacc) > 0)
                                        {
                                                $resacc = $ilance->db->fetch_array($sqlacc);
                                                
                                                $available_balance_before = $resacc['available_balance'];
                                                $total_balance_before = $resacc['total_balance'];
                                                
                                                $available_balance_after = ($available_balance_before + $escrow['totalamount']);
                                                $total_balance_after = ($total_balance_before + $escrow['totalamount']);
                                                
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET available_balance = '" . $available_balance_after . "',
                                                        total_balance = '" . $total_balance_after . "'
                                                        WHERE user_id = '" . $providerid . "'
                                                ");
                                                
                                                $ilance->email = construct_dm_object('email', $ilance);
                
                                                $ilance->email->mail = $ilance->db->fetch_field(DB_PREFIX . "users", "user_id = '" . $res['project_user_id'] . "'", "email");
                                                $ilance->email->slng = fetch_user_slng(intval($buyerid));
                                                
                                                $ilance->email->get('service_escrow_release_funds_buyer');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$providerid,'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                                        '{{commissionfee}}' => $ilance->currency->format($escrow['commissionfee']),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = $ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$providerid,'email');
                                                $ilance->email->slng = fetch_user_slng($res['user_id']);
                                                
                                                $ilance->email->get('service_escrow_release_funds_provider');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res['user_id'],'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($escrow['totalamount']),
                                                        '{{commission}}' => $ilance->currency->format($escrow['commissionfee']),
                                                        '{{commissionfee}}' => $ilance->currency->format($escrow['commissionfee']),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $ilance->email->mail = SITE_EMAIL;
                                                $ilance->email->slng = fetch_site_slng();
                                                
                                                $ilance->email->get('service_escrow_release_funds_admin');		
                                                $ilance->email->set(array(
                                                        '{{project_id}}' => $res['project_id'],
                                                        '{{biddername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res['user_id'],'username')),
                                                        '{{ownername}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'users','user_id='.$res['project_user_id'],'username')),
                                                        '{{project_title}}' => stripslashes($ilance->db->fetch_field(DB_PREFIX.'projects','project_id='.$res['project_id'],'project_title')),
                                                        '{{escrowamount}}' => $ilance->currency->format($escrow['totalamount']),
                                                        '{{commission}}' => $ilance->currency->format($escrow['commissionfee']),
                                                        '{{commissionfee}}' => $ilance->currency->format($escrow['commissionfee']),
                                                ));
                                                
                                                $ilance->email->send();
                                                
                                                $area_title = $phrase['_rfp_escrow_management'] . ' - (' . $phrase['_release_of_funds_complete'] . ')';
                                                $page_title = SITE_NAME . ' - ' . $phrase['_rfp_escrow_management'] . ' - (' . $phrase['_release_of_funds_complete'] . ')';
                                                
                                                if (isset($adminmode) AND $adminmode)
                                                {
                                                        print_action_success($phrase['_you_have_successfully_forwarded_funds_from_your_escrow_account_to_your_service_provider'], $ilpage['accounting'].'?cmd=escrows');
                                                        exit();
                                                }
                                                else
                                                {
                                                        print_notice($phrase['_release_of_funds_complete'], $phrase['_you_have_successfully_forwarded_funds_from_your_escrow_account_to_your_service_provider'], $ilpage['buying'].'?cmd=management&amp;sub=rfp-escrow', $phrase['_return_to_the_previous_menu']);
                                                        exit();
                                                }
                                        }
                                }
                        }
                        else
                        {
                                $area_title = $phrase['_rfp_escrow_management'] . ' - (' . $phrase['_release_of_funds_error'] . ')';
                                $page_title = SITE_NAME . ' - ' . $phrase['_rfp_escrow_management'] . ' - (' . $phrase['_release_of_funds_error'] . ')';
                                
                                if (isset($adminmode) AND $adminmode)
                                {
                                        print_action_success($phrase['_were_sorry_there_was_a_problem_releasing_the_funds_from_within_this_escrow_account_to_your_service_provider'], $ilpage['accounting'].'?cmd=escrows');
                                        exit();
                                }
                                else
                                {
                                        print_notice($area_title, $phrase['_were_sorry_there_was_a_problem_releasing_the_funds_from_within_this_escrow_account_to_your_service_provider'], $ilpage['buying'].'?cmd=management&amp;sub=rfp-escrow', $phrase['_return_to_the_previous_menu']);
                                        exit();
                                }
                        }
                }
        }
        
        /**
        * Function to process a purchase now escrow payment for a particular user for a specific amount.  This function takes final value fee and insertion
        * fee permission exemptions into consideration.
        *
        * @param       string       payment method (offline or account)
        * @param       integer      project id
        * @param       integer      order qty
        * @param       integer      order amount
        * @param       integer      order total amount
        * @param       integer      seller id
        * @param       integer      buyer id
        * @param       bool         is shipping address required?
        * @param       integer      shipping address id
        * @param       integer      account id
        *
        * @return      bool         Returns true or false if reversal could be completed
        */
        function instant_purchase_now($method = '', $projectid = 0, $qty = 0, $amount = 0, $total = 0, $seller_id = 0, $buyer_id = 0, $shipping_address_required, $shipping_address_id, $accountid)
        {
                global $ilance, $myapi, $phrase, $page_title, $area_title, $ilconfig, $ilpage;
                
                $ilance->subscription = construct_object('api.subscription');
                
                $shippingaddress = $phrase['_no_shipping_address_required_assuming_digital_item_delivery_please_communicate_with_customer'];
                if (isset($shipping_address_required) AND $shipping_address_required)
                {
                        $shippingaddress = print_shipping_address_text(intval($buyer_id));
                }
                
                // #### INSTANT PURCHASE VIA OFFLINE PAYMENT NO ESCROW #########
                if ($method == 'offline')
                {
                        // #### FINAL VALUE FEE ############################################
                        // calculate final value fee to merchant for the total amount passed to this function
                        // we will not include shipping in the final value fee amount to calculate
                        // try to debit funds, else create unpaid txn
                        $invoiceid = $isfvfpaid = $avail_bal = $total_bal = 0;
                        $paymentstatus = $phrase['_no_charge'];
                        $cid = fetch_auction('cid', $projectid);
                        $fvf = calculate_final_value_fee(($amount * $qty), $cid, 'product');
                        
                        // check if we're exempt from fvf fees
                        if (!empty($seller_id) AND $seller_id > 0 AND $ilance->subscription->check_access($seller_id, 'fvfexempt') == 'yes')
                        {
                                $fvf = 0;
                        }
                        
                        if ($fvf > 0)
                        {
                                $ilance->accounting = construct_object('api.accounting');
                                
                                // we have a fee!
                                $account = $ilance->db->query("
                                        SELECT available_balance, total_balance
                                        FROM " . DB_PREFIX . "users
                                        WHERE user_id = '" . intval($seller_id) . "'
                                ");
                                if ($ilance->db->num_rows($account) > 0)
                                {
                                        $res = $ilance->db->fetch_array($account);
                                        
                                        $avail_bal = $res['available_balance'];
                                        $total_bal = $res['total_balance'];
                                }
                                
                                $autopayments = fetch_user('autopayment', $seller_id);
                                
                                if ($avail_bal >= $fvf AND $autopayments)
                                {
                                        $paymentstatus = $phrase['_paid_and_processed_via_online_account_balance'];
                                        
                                        // we have enough funds to cover the fee!
                                        // pay fvf and generate paid transaction
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET available_balance = available_balance - $fvf,
                                                total_balance = total_balance - $fvf
                                                WHERE user_id = '" . intval($seller_id) . "'
                                        ");
                                        
                                        $invoiceid = $ilance->accounting->insert_transaction(
                                                0,
                                                intval($projectid),
                                                0,
                                                intval($seller_id),
                                                0,
                                                0,
                                                0,
                                                $phrase['_purchase_now_seller_final_value_fee'] . ' - ' . fetch_auction('project_title', intval($projectid)) . ' #' . intval($projectid),
                                                sprintf("%01.2f", $fvf),
                                                sprintf("%01.2f", $fvf),
                                                'paid',
                                                'debit',
                                                'account',
                                                DATETIME24H,
                                                DATETIME24H,
                                                DATETIME24H,
                                                '',
                                                0,
                                                0,
                                                1
                                        );
                                        
                                        insert_income_spent(intval($seller_id), $fvf, 'credit');
                                        
                                        $isfvfpaid = '1';
                                }
                                else
                                {
                                        // seller do not have enough funds to cover fee!
                                        // generate fvf unpaid transaction!
                                        // create an unpaid final value fee to the seller
                                        $paymentstatus = $phrase['_pending_immediate_payment'];
                                        
                                        $invoiceid = $ilance->accounting->insert_transaction(
                                                0,
                                                intval($projectid),
                                                0,
                                                intval($seller_id),
                                                0,
                                                0,
                                                0,
                                                $phrase['_purchase_now_seller_final_value_fee'] . ' - ' . fetch_auction('project_title', intval($projectid)) . ' #' . intval($projectid),
                                                sprintf("%01.2f", $fvf),
                                                '',
                                                'unpaid',
                                                'debit',
                                                'account',
                                                DATETIME24H,
                                                DATEINVOICEDUE,
                                                '',
                                                '',
                                                0,
                                                0,
                                                1
                                        );
                                }
                                
                                // update invoice to isfvf = 1
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "invoices
                                        SET isfvf = '1'
                                        WHERE invoiceid = '" . intval($invoiceid) . "'
                                ");
                        }
                        
                        // #### CREATE BUY NOW ORDER #######################################
                        // in this case, since escrow is disabled the fee column
                        // will still show a fee based on the "final value fee" vs the escrow commission.
                        // and the fee2 column will be just 0.00 (fee to buyer which should be nill)
                        $ilance->db->query("
                                INSERT INTO " . DB_PREFIX . "buynow_orders
                                (orderid, project_id, buyer_id, owner_id, qty, amount, fvf, fvfinvoiceid, isfvfpaid, ship_required, ship_location, orderdate, status)
                                VALUES(
                                NULL,
                                '" . intval($projectid) . "',
                                '" . intval($buyer_id) . "',
                                '" . intval($seller_id) . "',
                                '" . intval($qty) . "',
                                '" . sprintf("%01.2f", $total) . "',
                                '" . sprintf("%01.2f", $fvf) . "',
                                '" . intval($invoiceid) . "',
                                '" . intval($isfvfpaid) . "',
                                '" . intval($shipping_address_required) . "',
                                '" . $ilance->db->escape_string($shippingaddress) . "',
                                '" . DATETIME24H . "',
                                'offline')
                        ");
                        $neworderid = $ilance->db->insert_id();
                        
                        $bqty = fetch_auction('buynow_qty', $projectid);
                        $buynowqty = ($bqty - $qty);
                        if ($buynowqty < 0)
                        {
                                $buynowqty = 0;
                        }
                        
                        // update the qty available for this item
                        $ilance->db->query("
                                UPDATE " . DB_PREFIX . "projects
                                SET hasbuynowwinner = '1',
                                buynow_qty = " . intval($buynowqty) . "
                                WHERE project_id = '" . intval($projectid) . "'
                                LIMIT 1
                        ");
                        
                        $ship_costs = fetch_auction('ship_costs', $projectid);
                        
                        // if there are no qty available then change open status to expired
                        $buynow_qtyleft = $bqty;
                        $filtered_auctiontype = fetch_auction('filtered_auctiontype', $projectid);
                        $projectstatus = fetch_auction('status', $projectid);
                        
                        if ($buynow_qtyleft == 0 AND $filtered_auctiontype == 'fixed' AND $projectstatus == 'open')
                        {
                                // expire auction
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects
                                        SET status = 'expired',
                                        date_end = '" . DATETIME24H . "'
                                        WHERE project_id = '" . intval($projectid) . "'
                                ");
                                
                                // send email notification
                                // to be added..
                        }
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                
                        $ilance->email->mail = fetch_user('email', intval($buyer_id));
                        $ilance->email->slng = $_SESSION['ilancedata']['user']['slng'];
                        
                        $ilance->email->get('buynow_offline_purchase_buyer');		
                        $ilance->email->set(array(
                                '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $projectid)),
                                '{{project_id}}' => $projectid,
                                '{{qty}}' => $qty,
                                '{{amount}}' => $ilance->currency->format($amount),
                                '{{total}}' => $ilance->currency->format($total),
                                '{{ship_costs}}' => $ilance->currency->format($ship_costs),
                                '{{shippingaddress}}' => $shippingaddress,                                
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = fetch_user('email', intval($seller_id));
                        $ilance->email->slng = fetch_user_slng($seller_id);
                        
                        $ilance->email->get('buynow_offline_purchase_seller');		
                        $ilance->email->set(array(
                                '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{buyer_fullname}}' => $_SESSION['ilancedata']['user']['fullname'],
                                '{{buyer_email}}' => $_SESSION['ilancedata']['user']['email'],
                                '{{seller}}' => fetch_user('username', intval($seller_id)),
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $projectid)),
                                '{{project_id}}' => $projectid,
                                '{{qty}}' => $qty,
                                '{{amount}}' => $ilance->currency->format($amount),
                                '{{total}}' => $ilance->currency->format($total),
                                '{{ship_costs}}' => $ilance->currency->format($ship_costs),
                                '{{shippingaddress}}' => $shippingaddress,
                                '{{fvf}}' => $ilance->currency->format($fvf),
                                '{{fvftotal}}' => $ilance->currency->format($fvf),
                                '{{paymentstatus}}' => $paymentstatus,                                
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('buynow_offline_purchase_admin');		
                        $ilance->email->set(array(
                                '{{buyer}}' => $_SESSION['ilancedata']['user']['username'],
                                '{{buyer_fullname}}' => $_SESSION['ilancedata']['user']['fullname'],
                                '{{buyer_email}}' => $_SESSION['ilancedata']['user']['email'],
                                '{{seller}}' => fetch_user('username', intval($seller_id)),
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $projectid)),
                                '{{project_id}}' => $projectid,
                                '{{qty}}' => $qty,
                                '{{amount}}' => $ilance->currency->format($amount),
                                '{{total}}' => $ilance->currency->format($total),
                                '{{ship_costs}}' => $ilance->currency->format($ship_costs),
                                '{{shippingaddress}}' => $shippingaddress,
                                '{{fvf}}' => $ilance->currency->format($fvf),
                                '{{fvftotal}}' => $ilance->currency->format($fvf),
                                '{{paymentstatus}}' => $paymentstatus,                                
                        ));
                        
                        $ilance->email->send();
                        
                        return true;
                }
                    
                // #### INSTANT PURCHASE TO ESCROW VIA ACCOUNT BALANCE #########
                else if ($method == 'account')
                {
                        $sel_balance = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . intval($buyer_id) . "'
                        ");
                        if ($ilance->db->num_rows($sel_balance) > 0)
                        {
                                $res_balance = $ilance->db->fetch_array($sel_balance);
                                
                                if ($res_balance['available_balance'] >= $total)
                                {
                                        $area_title = $phrase['_instant_purchase_to_escrow_via_online_account'];
                                        $page_title = SITE_NAME . ' - ' . $phrase['_instant_purchase_to_escrow_via_online_account'];
                                        
                                        // the total amount the buyer forwarded would have included any escrow fees if applicable
                                        // because of this our total amount has changed so lets find out what the original amount
                                        // is including any shipping costs to forward into the buynow escrow account
                                        $ship_handling = fetch_auction('ship_handling', $projectid);
                                        $ship_costs = fetch_auction('ship_costs', $projectid);
                                        $item_costs = fetch_auction('buynow_price', $projectid);
                                        
                                        if ($ship_handling == 'flatrate' AND $ship_costs > 0)
                                        {
                                                $rawtotal = sprintf("%01.2f", ($item_costs * $qty) + $ship_costs);
                                        }
                                        else
                                        {
                                                $rawtotal = sprintf("%01.2f", ($item_costs * $qty));
                                        }                   
                        
                                        // #### ESCROW FEES ####################
                                        // we'll populate the fee fields we expect to be paid from the seller or buyer of this listing
                                        // $fee = seller escrow fee
                                        // $escrowfeebuyer = buyer escrow fee
                                        
                                        $escrowfee = $escrowfeebuyer = 0;
                                        $escrowfeebuyerinvoiceid = $isescrowfeebuyerpaid = $escrowfeeinvoiceid = $isescrowfeepaid = '0';
                                        
                                        if ($ilconfig['escrowsystem_escrowcommissionfees'])
                                        {
                                                $ilance->tax = construct_object('api.tax');
                                                
                                                // escrow commission fees to auction owner enabled
                                                if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                                {
                                                        // fixed escrow cost to provider for release of funds
                                                        $escrowfee = sprintf("%01.2f", $ilconfig['escrowsystem_merchantfixedprice']);
                                                }
                                                else
                                                {
                                                        if ($ilconfig['escrowsystem_merchantpercentrate'] > 0)
                                                        {
                                                                // percentage rate of total winning bid amount
                                                                // which would be the same as the amount being forwarded into escrow
                                                                $escrowfee = sprintf("%01.2f", ($rawtotal * $ilconfig['escrowsystem_merchantpercentrate'] / 100));
                                                        }
                                                }
                                                
                                                if ($escrowfee > 0)
                                                {
                                                        $taxamount = 0;
                                                        $istaxable = '0';
                                                        $taxinfo = '';
                                                        if ($ilance->tax->is_taxable(intval($seller_id), 'commission') AND $ilconfig['escrowsystem_feestaxable'])
                                                        {
                                                                // fetch tax amount to charge for this invoice type
                                                                $taxamount = $ilance->tax->fetch_amount(intval($seller_id), $escrowfee, 'commission', 0);
                                                                $taxinfo = $ilance->tax->fetch_amount(intval($seller_id), $escrowfee, 'commission', 1);
                                                                $istaxable = '1';
                                                        }
                                                        
                                                        // exact amount to charge buyer
                                                        $escrowfeenotax = $escrowfee;
                                                        $escrowfee = sprintf("%01.2f", ($escrowfee + $taxamount));
                                                        
                                                        // charge escrow fee to seller as a separate transaction
                                                        // this creates a transaction history item for the buyer of item
                                                        $txn = construct_transaction_id();
                                                        
                                                        $availablebalance = fetch_user('available_balance', $seller_id);
                                                        $totalbalance = fetch_user('total_balance', $seller_id);
                                                        if ($availablebalance >= $escrowfee)
                                                        {
                                                                $ilance->db->query("
                                                                        INSERT INTO " . DB_PREFIX . "invoices
                                                                        (invoiceid, projectid, user_id, description, amount, paid, totalamount, istaxable, taxamount, taxinfo, status, invoicetype, paymethod, ipaddress, createdate, duedate, paiddate, custommessage, transactionid, isescrowfee)
                                                                        VALUES(
                                                                        NULL,
                                                                        '" . intval($projectid) . "',
                                                                        '" . intval($seller_id) . "',
                                                                        '" . $ilance->db->escape_string($phrase['_purchase_now_seller_escrow_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                        '" . $escrowfeenotax . "',
                                                                        '" . $escrowfee . "',
                                                                        '" . $escrowfee . "',
                                                                        '" . $istaxable . "',
                                                                        '" . $taxamount . "',
                                                                        '" . $taxinfo . "',
                                                                        'paid',
                                                                        'debit',
                                                                        'account',
                                                                        '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                        '" . $txn . "',
                                                                        '1')
                                                                ");
                                                                $escrowfeeinvoiceid = $ilance->db->insert_id();
                                                                
                                                                // debit funds from seller's account
                                                                $new_total = ($totalbalance - $escrowfee);
                                                                $new_avail = ($availablebalance - $escrowfee);
                                            
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "users
                                                                        SET available_balance = '" . sprintf("%01.2f", $new_avail) . "',
                                                                        total_balance = '" . sprintf("%01.2f", $new_total) . "'
                                                                        WHERE user_id = '" . intval($seller_id) . "'
                                                                ");
                                            
                                                                insert_income_spent(intval($seller_id), $escrowfee, 'credit');
                                                                $isescrowfeepaid = '1';        
                                                        }
                                                        else
                                                        {
                                                                $ilance->db->query("
                                                                        INSERT INTO " . DB_PREFIX . "invoices
                                                                        (invoiceid, projectid, user_id, description, amount, totalamount, istaxable, taxamount, taxinfo, status, invoicetype, paymethod, ipaddress, createdate, duedate, custommessage, transactionid, isescrowfee)
                                                                        VALUES(
                                                                        NULL,
                                                                        '" . intval($projectid) . "',
                                                                        '" . intval($seller_id) . "',
                                                                        '" . $ilance->db->escape_string($phrase['_purchase_now_seller_escrow_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                        '" . $escrowfeenotax . "',
                                                                        '" . $escrowfee . "',
                                                                        '" . $istaxable . "',
                                                                        '" . $taxamount . "',
                                                                        '" . $taxinfo . "',
                                                                        'unpaid',
                                                                        'debit',
                                                                        'account',
                                                                        '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                        '" . $txn . "',
                                                                        '1')
                                                                ");
                                                                $escrowfeeinvoiceid = $ilance->db->insert_id();
                                                                $isescrowfeepaid = '0';         
                                                        }
                                                }
                        
                                                // #### PRODUCT BUYER ESCROW FEES ##########################
                                                // we'll populate the fee2 field which denotes any fees the buyer of this buynow purchase
                                                // must pay the site owner .. we'll also calculate any tax if applicable to ensure that the
                                                // fee to the buyer will include the full fee amount + any applicable taxes (for commission txns)
                                           
                                                // escrow commission fees to auction owner enabled
                                                if ($ilconfig['escrowsystem_bidderfixedprice'] > 0)
                                                {
                                                        // fixed escrow cost to buyer
                                                        $escrowfeebuyer = sprintf("%01.2f", $ilconfig['escrowsystem_bidderfixedprice']);
                                                }
                                                else
                                                {
                                                        if ($ilconfig['escrowsystem_bidderpercentrate'] > 0)
                                                        {
                                                                // percentage rate of total winning bid amount
                                                                // which would be the same as the amount being forwarded into escrow
                                                                $escrowfeebuyer = sprintf("%01.2f", ($rawtotal * $ilconfig['escrowsystem_bidderpercentrate'] / 100));
                                                        }
                                                }
                                                
                                                if ($escrowfeebuyer > 0)
                                                {
                                                        $taxamount = 0;
                                                        $istaxable = '0';
                                                        $taxinfo = '';
                                                        if ($ilance->tax->is_taxable(intval($buyer_id), 'commission') AND $ilconfig['escrowsystem_feestaxable'])
                                                        {
                                                                // fetch tax amount to charge for this invoice type
                                                                $taxamount = $ilance->tax->fetch_amount(intval($buyer_id), $escrowfeebuyer, 'commission', 0);
                                                                $taxinfo = $ilance->tax->fetch_amount(intval($buyer_id), $escrowfeebuyer, 'commission', 1);
                                                                $istaxable = '1';
                                                        }
                                                        
                                                        // exact amount to charge buyer
                                                        $escrowfeebuyernotax = $escrowfeebuyer;
                                                        $escrowfeebuyer = sprintf("%01.2f", ($escrowfeebuyer + $taxamount));
                                                        
                                                        // charge escrow fee to buyer as a separate transaction
                                                        // this creates a transaction history item for the buyer of item
                                                        $txn = construct_transaction_id();
                                                        
                                                        $availablebalance = fetch_user('available_balance', $buyer_id);
                                                        $totalbalance = fetch_user('total_balance', $buyer_id);
                                                        if ($availablebalance >= $escrowfeebuyer)
                                                        {
                                                                $ilance->db->query("
                                                                        INSERT INTO " . DB_PREFIX . "invoices
                                                                        (invoiceid, projectid, user_id, p2b_user_id, description, amount, paid, totalamount, istaxable, taxamount, taxinfo, status, invoicetype, paymethod, ipaddress, createdate, duedate, paiddate, custommessage, transactionid, isescrowfee)
                                                                        VALUES(
                                                                        NULL,
                                                                        '" . intval($projectid) . "',
                                                                        '" . intval($buyer_id) . "',
                                                                        '" . intval($seller_id) . "',
                                                                        '" . $ilance->db->escape_string($phrase['_purchase_now_buyer_escrow_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                        '" . $escrowfeebuyernotax . "',
                                                                        '" . $escrowfeebuyer . "',
                                                                        '" . $escrowfeebuyer . "',
                                                                        '" . $istaxable . "',
                                                                        '" . $taxamount . "',
                                                                        '" . $taxinfo . "',
                                                                        'paid',
                                                                        'debit',
                                                                        'account',
                                                                        '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                        '" . $txn . "',
                                                                        '1')
                                                                ");
                                                                $escrowfeebuyerinvoiceid = $ilance->db->insert_id();
                                                                $isescrowfeebuyerpaid = '1';
                                                                
                                                                // debit funds from buyer's account
                                                                $new_total = ($totalbalance - $escrowfeebuyer);
                                                                $new_avail = ($availablebalance - $escrowfeebuyer);
                                            
                                                                $ilance->db->query("
                                                                        UPDATE " . DB_PREFIX . "users
                                                                        SET available_balance = '" . sprintf("%01.2f", $new_avail) . "',
                                                                        total_balance = '" . sprintf("%01.2f", $new_total) . "'
                                                                        WHERE user_id = '" . intval($buyer_id) . "'
                                                                ");
                                            
                                                                insert_income_spent(intval($buyer_id), $escrowfeebuyer, 'credit');
                                                        }
                                                        else
                                                        {
                                                                $ilance->db->query("
                                                                        INSERT INTO " . DB_PREFIX . "invoices
                                                                        (invoiceid, projectid, user_id, description, amount, totalamount, istaxable, taxamount, taxinfo, status, invoicetype, paymethod, ipaddress, createdate, duedate, custommessage, transactionid, isescrowfee)
                                                                        VALUES(
                                                                        NULL,
                                                                        '" . intval($projectid) . "',
                                                                        '" . intval($buyer_id) . "',
                                                                        '" . $ilance->db->escape_string($phrase['_purchase_now_buyer_escrow_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                        '" . $escrowfeebuyernotax . "',
                                                                        '" . $escrowfeebuyer . "',
                                                                        '" . $istaxable . "',
                                                                        '" . $taxamount . "',
                                                                        '" . $taxinfo . "',
                                                                        'unpaid',
                                                                        'debit',
                                                                        'account',
                                                                        '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . DATETIME24H . "',
                                                                        '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                        '" . $txn . "',
                                                                        '1')
                                                                ");
                                                                $escrowfeebuyerinvoiceid = $ilance->db->insert_id();
                                                                $isescrowfeebuyerpaid = '0';
                                                        }
                                                }
                                        }
                                        
                                        // this creates a transaction history item for the buyer of item
                                        $transactionid = construct_transaction_id();
                                        
                                        $ilance->db->query("
                                                INSERT INTO " . DB_PREFIX . "invoices
                                                (invoiceid, projectid, user_id, p2b_user_id, description, amount, paid, totalamount, status, invoicetype, paymethod, ipaddress, createdate, duedate, paiddate, custommessage, transactionid)
                                                VALUES(
                                                NULL,
                                                '" . intval($projectid) . "',
                                                '" . intval($buyer_id) . "',
                                                '" . intval($seller_id) . "',
                                                '" . $ilance->db->escape_string($phrase['_purchase_now']) . " " . $ilance->db->escape_string($phrase['_escrow_payment_forward']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                '" . $rawtotal . "',
                                                '" . $rawtotal . "',
                                                '" . $rawtotal . "',
                                                'paid',
                                                'buynow',
                                                'account',
                                                '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                '" . DATETIME24H . "',
                                                '" . DATETIME24H . "',
                                                '" . DATETIME24H . "',
                                                '" . $ilance->db->escape_string($phrase['_funds_held_within_escrow_until_item_has_been_delivered']) . " - " . DATETIME24H . "',
                                                '" . $transactionid . "')
                                        ");
                                        $newinvoiceid = $ilance->db->insert_id();
                                        
                                        // debit customers account data
                                        $availablebalance = fetch_user('available_balance', $buyer_id);
                                        $totalbalance = fetch_user('total_balance', $buyer_id);
                                        
                                        $new_total = ($totalbalance - $rawtotal);
                                        $new_avail = ($availablebalance - $rawtotal);
                    
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "users
                                                SET available_balance = '" . sprintf("%01.2f", $new_avail) . "',
                                                total_balance = '" . sprintf("%01.2f", $new_total) . "'
                                                WHERE user_id = '" . intval($buyer_id) . "'
                                        ");
                    
                                        insert_income_spent(intval($buyer_id), $rawtotal, 'credit');
                                        
                                        // this creates a separate escrow fee transaction (which we can refund if seller refunds escrow amount in future)
                                        
                                        // create buynow escrow order
                                        // note: escrowfee will be charged to the seller
                                        // escrowfee is the fee charged to seller for using escrow
                                        // escrowfeebuyer is the fee charged to buyer for using escrow
                                        // fvf will be charged to the seller when the buyer releases payment in escrow to seller
                                        // fvfbuyer is currently unused
                                        // escrowfeeinvoiceid holds invoice id associated with escrow fee to seller
                                        // escrowfeebuyerinvoiceid holds invoice id associated with escrow fee to buyer
                                        // fvfinvoiceid holds invoice id associated with fee to seller
                                        // fvfbuyerinvoiceid holds invoice id associated with fee to buyer
                                        
                                        // #### CALCULATE FVF TO SELLER FOR THIS ORDER #####################################
                                        // this is a separate final value fee if applicable which is separate from the escrow fee logic above
                                        $cid = fetch_auction('cid', $projectid);
                                        $fvfbuyer = '0.00'; // not in use
                                        $fvfinvoiceid = '0';
                                        $isfvfpaid = '0';  
                                        $fvf = calculate_final_value_fee($rawtotal, $cid, 'product');
                                        
                                        // check if we're exempt from fvf fees
                                        if (!empty($seller_id) AND $seller_id > 0 AND $ilance->subscription->check_access($seller_id, 'fvfexempt') == 'yes')
                                        {
                                                $fvf = 0;
                                        }
                                        
                                        if ($fvf > 0)
                                        {
                                                $fvf = sprintf("%01.2f", $fvf);
                                                
                                                // charge escrow fee to seller as a separate transaction
                                                // this creates a transaction history item for the buyer of item
                                                $txn = construct_transaction_id();
                                                
                                                $availablebalance = fetch_user('available_balance', $seller_id);
                                                $totalbalance = fetch_user('total_balance', $seller_id);
                                                $autopayments = fetch_user('autopayment', $seller_id);
                                                
                                                if ($availablebalance >= $fvf AND $autopayments)
                                                {
                                                        $ilance->db->query("
                                                                INSERT INTO " . DB_PREFIX . "invoices
                                                                (invoiceid, projectid, user_id, description, amount, paid, totalamount, status, invoicetype, paymethod, ipaddress, createdate, duedate, paiddate, custommessage, transactionid, isfvf)
                                                                VALUES(
                                                                NULL,
                                                                '" . intval($projectid) . "',
                                                                '" . intval($seller_id) . "',
                                                                '" . $ilance->db->escape_string($phrase['_purchase_now_seller_final_value_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                '" . $fvf . "',
                                                                '" . $fvf . "',
                                                                '" . $fvf . "',
                                                                'paid',
                                                                'debit',
                                                                'account',
                                                                '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                '" . DATETIME24H . "',
                                                                '" . DATETIME24H . "',
                                                                '" . DATETIME24H . "',
                                                                '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                '" . $txn . "',
                                                                '1')
                                                        ");
                                                        $fvfinvoiceid = $ilance->db->insert_id();
                                                        
                                                        // debit funds from seller's account
                                                        $new_total = ($availablebalance - $fvf);
                                                        $new_avail = ($totalbalance - $fvf);
                                    
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET available_balance = '" . sprintf("%01.2f", $new_avail) . "',
                                                                total_balance = '" . sprintf("%01.2f", $new_total) . "'
                                                                WHERE user_id = '" . intval($seller_id) . "'
                                                        ");
                                    
                                                        insert_income_spent(intval($seller_id), $fvf, 'credit');
                                                        $isfvfpaid = '1';        
                                                }
                                                else
                                                {
                                                        $ilance->db->query("
                                                                INSERT INTO " . DB_PREFIX . "invoices
                                                                (invoiceid, projectid, user_id, description, amount, totalamount, status, invoicetype, paymethod, ipaddress, createdate, duedate, custommessage, transactionid, isfvf)
                                                                VALUES(
                                                                NULL,
                                                                '" . intval($projectid) . "',
                                                                '" . intval($seller_id) . "',
                                                                '" . $ilance->db->escape_string($phrase['_purchase_now_seller_final_value_fee']) . " - " . $ilance->db->escape_string(fetch_auction('project_title', $projectid)) . " #" . $projectid . "',
                                                                '" . $fvf . "',
                                                                '" . $fvf . "',
                                                                'unpaid',
                                                                'debit',
                                                                'account',
                                                                '" . $ilance->db->escape_string($_SERVER['REMOTE_ADDR']) . "',
                                                                '" . DATETIME24H . "',
                                                                '" . DATETIME24H . "',
                                                                '" . $ilance->db->escape_string($phrase['_may_include_applicable_taxes']) . "',
                                                                '" . $txn . "',
                                                                '1')
                                                        ");
                                                        $fvfinvoiceid = $ilance->db->insert_id();
                                                        $isfvfpaid = '0';
                                                }
                                                
                                                // update invoice to isfvf = 1
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET isfvf = '1'
                                                        WHERE invoiceid = '" . intval($fvfinvoiceid) . "'
                                                ");
                                        }
                                        
                                        // this creates the buy now order and any applicable invoicing logic
                                        $ilance->db->query("
                                                INSERT INTO " . DB_PREFIX . "buynow_orders
                                                (orderid, project_id, buyer_id, owner_id, invoiceid, qty, amount, escrowfee, isescrowfeepaid, escrowfeeinvoiceid, escrowfeebuyer, isescrowfeebuyerpaid, escrowfeebuyerinvoiceid, fvf, isfvfpaid, fvfinvoiceid, fvfbuyer, isfvfbuyerpaid, ship_required, ship_location, orderdate, paiddate, status)
                                                VALUES(
                                                NULL,
                                                '" . intval($projectid) . "',
                                                '" . intval($buyer_id) . "',
                                                '" . intval($seller_id) . "',
                                                '" . $newinvoiceid . "',
                                                '" . $qty . "',
                                                '" . $rawtotal . "',
                                                '" . $escrowfee . "',
                                                '" . $isescrowfeepaid . "',
                                                '" . $escrowfeeinvoiceid . "',
                                                '" . $escrowfeebuyer . "',
                                                '" . $isescrowfeebuyerpaid . "',
                                                '" . $escrowfeebuyerinvoiceid . "',
                                                '" . $fvf . "',
                                                '" . $isfvfpaid . "',
                                                '" . $fvfinvoiceid . "',
                                                '" . $fvfbuyer . "',
                                                '0',
                                                '" . $shipping_address_required . "',
                                                '" . $ilance->db->escape_string($shippingaddress) . "',
                                                '" . DATETIME24H . "',
                                                '" . DATETIME24H . "',
                                                'paid')
                                        ");                                        
                                        $neworderid = $ilance->db->insert_id();
                                        
                                        // tie buynowid to the associated invoice
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "invoices
                                                SET buynowid = '" . intval($neworderid) . "'
                                                WHERE invoiceid = '" . intval($newinvoiceid) . "'
                                        ");
                                        
                                        // tie fvf invoice to seller to this buy now order so we have some association
                                        if ($fvfinvoiceid > 0)
                                        {
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET buynowid = '" . intval($neworderid) . "'
                                                        WHERE invoiceid = '" . intval($fvfinvoiceid) . "'
                                                ");
                                        }
                                        
                                        // tie escrow fee invoice to seller so we can update isescrowfeepaid used by other cancel or paid invoice actions
                                        if ($escrowfeeinvoiceid > 0)
                                        {
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET buynowid = '" . intval($neworderid) . "'
                                                        WHERE invoiceid = '" . intval($escrowfeeinvoiceid) . "'
                                                ");        
                                        }
                                        
                                        if ($escrowfeebuyerinvoiceid > 0)
                                        {
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET buynowid = '" . intval($neworderid) . "'
                                                        WHERE invoiceid = '" . intval($escrowfeebuyerinvoiceid) . "'
                                                ");       
                                        }
                                        
                                        $bqty = fetch_auction('buynow_qty', $projectid);
                                        $buynowqty = $bqty - 1;
                                        if ($buynowqty < 0)
                                        {
                                                $buynowqty = 0;
                                        }
                                        
                                        // update the qty available for this item
                                        $ilance->db->query("
                                                UPDATE " . DB_PREFIX . "projects
                                                SET hasbuynowwinner = '1',
                                                buynow_qty = " . intval($buynowqty) . "
                                                WHERE project_id = '" . intval($projectid) . "'
                                                LIMIT 1
                                        ");
                                        
                                        // if there are no qty available then change open status to expired
                                        $buynow_qtyleft = fetch_auction('buynow_qty', $projectid);
                                        $filtered_auctiontype = fetch_auction('filtered_auctiontype', $projectid);
                                        $projectstatus = fetch_auction('status', $projectid);
                                        
                                        if ($buynow_qtyleft == 0 AND $filtered_auctiontype == 'fixed' AND $projectstatus == 'open')
                                        {
                                                // expire auction
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "projects
                                                        SET status = 'expired',
                                                        date_end = '" . DATETIME24H . "'
                                                        WHERE project_id = '" . intval($projectid) . "'
                                                ");
                                                
                                                // send email notification
                                                // to be added..
                                        }
                    
                                        $ilance->email = construct_dm_object('email', $ilance);
                
                                        $ilance->email->mail = fetch_user('email', intval($buyer_id));
                                        $ilance->email->slng = fetch_user_slng(intval($buyer_id));
                                        
                                        $ilance->email->get('escrow_buynow_payment_confirmation');		
                                        $ilance->email->set(array(
                                                '{{customer}}' => $_SESSION['ilancedata']['user']['username'],
                                                '{{transactionid}}' => $transactionid,
                                                '{{merchant}}' => fetch_user('username', intval($seller_id)),
                                                '{{fullname}}' => fetch_user('fullname', intval($buyer_id)),
                                                '{{email}}' => fetch_user('email', intval($buyer_id)),
                                                '{{phone}}' => fetch_user('phone', intval($buyer_id)),
                                                '{{projectid}}' => $projectid,
                                                '{{description}}' => $phrase['_escrow_payment_forward'].": (".$projectid.")",
                                                '{{qty}}' => $qty,
                                                '{{amount_formatted}}' => $ilance->currency->format($amount),
                                                '{{total_amount_formatted}}' => $ilance->currency->format($total),
                                                '{{invoiceid}}' => $newinvoiceid,
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        $ilance->email->mail = fetch_user('email', intval($seller_id));
                                        $ilance->email->slng = fetch_user_slng(intval($seller_id));
                                        
                                        $ilance->email->get('product_escrow_payment_foward_merchant');		
                                        $ilance->email->set(array(
                                                '{{customer}}' => $_SESSION['ilancedata']['user']['username'],
                                                '{{transactionid}}' => $transactionid,
                                                '{{merchant}}' => fetch_user('username', intval($seller_id)),
                                                '{{fullname}}' => fetch_user('fullname', intval($buyer_id)),
                                                '{{email}}' => fetch_user('email', intval($buyer_id)),
                                                '{{phone}}' => fetch_user('phone', intval($buyer_id)),
                                                '{{projectid}}' => $projectid,
                                                '{{description}}' => $phrase['_escrow_payment_forward'].": (".$projectid.")",
                                                '{{qty}}' => $qty,
                                                '{{amount_formatted}}' => $ilance->currency->format($rawtotal),
                                                '{{total_amount_formatted}}' => $ilance->currency->format($total),
                                                '{{invoiceid}}' => $newinvoiceid,
                                                '{{shippingaddress}}' => $shippingaddress,
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        $ilance->email->mail = SITE_EMAIL;
                                        $ilance->email->slng = fetch_site_slng();
                                        
                                        $ilance->email->get('escrow_buynow_payment_confirmation_admin');		
                                        $ilance->email->set(array(
                                                '{{customer}}' => $_SESSION['ilancedata']['user']['username'],
                                                '{{transactionid}}' => $transactionid,
                                                '{{merchant}}' => fetch_user('username', intval($seller_id)),
                                                '{{fullname}}' => fetch_user('fullname', intval($buyer_id)),
                                                '{{email}}' => fetch_user('email', intval($buyer_id)),
                                                '{{phone}}' => fetch_user('phone', intval($buyer_id)),
                                                '{{projectid}}' => $projectid,
                                                '{{description}}' => $phrase['_escrow_payment_forward'].": (".$projectid.")",
                                                '{{qty}}' => $qty,
                                                '{{amount_formatted}}' => $ilance->currency->format($rawtotal),
                                                '{{total_amount_formatted}}' => $ilance->currency->format($total),
                                                '{{invoiceid}}' => $newinvoiceid,
                                                '{{shippingaddress}}' => $shippingaddress,
                                        ));
                                        
                                        $ilance->email->send();
                                        
                                        return true;
                                }
                        }
                }
                
                return false;
        }
        
        /**
        * Function to process the logic when a seller confirms delivery on a purchase now item for the buyer
        *
        * @param       integer      order id
        *
        * @return      bool         Returns true or false if delivery confirmation could be completed
        */
        function instant_purchase_now_confirmation($orderid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase, $area_title, $page_title;
                
                // set escrow account to confirmed phase
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "buynow_orders 
                        SET status = 'pending_delivery'
                        WHERE orderid = '" . intval($orderid) . "'
                ", 0, null, __FILE__, __LINE__);
                
                $sql = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "buynow_orders
                        WHERE orderid = '" . intval($orderid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res_escrow = $ilance->db->fetch_array($sql);
                        
                        // update listing as shipped (listings table)
                        mark_listing_as_shipped($res_escrow['project_id']);
                        
                        $amount = $res_escrow['amount'];
                        
                        $ilance->GPC['countryname'] = isset($ilance->GPC['countryname']) ? $ilance->GPC['countryname'] : '';
                        
                        $ilance->email = construct_dm_object('email', $ilance);
                        
                        $ilance->email->mail = fetch_user('email', $res_escrow['buyer_id']);
                        $ilance->email->slng = fetch_user_slng($res_escrow['buyer_id']);
        
                        $ilance->email->get('purchase_now_delivery_confirmed_buyer');		
                        $ilance->email->set(array(
                                '{{project_id}}' => $res_escrow['project_id'],
                                '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                '{{escrowamount}}' => $ilance->currency->format($amount),
                                '{{countryname}}' => $ilance->GPC['countryname'],
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = fetch_user('email', $res_escrow['owner_id']);
                        $ilance->email->slng = $_SESSION['ilancedata']['user']['slng'];
                        
                        $ilance->email->get('purchase_now_delivery_confirmed_merchant');		
                        $ilance->email->set(array(
                                '{{project_id}}' => $res_escrow['project_id'],
                                '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                '{{escrowamount}}' => $ilance->currency->format($amount),
                        ));
                        
                        $ilance->email->send();
                        
                        $ilance->email->mail = SITE_EMAIL;
                        $ilance->email->slng = fetch_site_slng();
                        
                        $ilance->email->get('purchase_now_delivery_confirmed_admin');		
                        $ilance->email->set(array(
                                '{{project_id}}' => $res_escrow['project_id'],
                                '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                '{{escrowamount}}' => $ilance->currency->format($amount),
                        ));
                        
                        $ilance->email->send();
                        
                        // we should also check if any fees are pending, and remind everyone including admin!
                        
                        return true;
                }
                
                return false;
        }
        
        /**
        * Function to process the logic when a seller confirms delivery (for offline payment control) on a purchase now item for the buyer
        *
        * @param       integer      order id
        *
        * @return      bool         Returns true or false if delivery confirmation could be completed
        */
        function instant_purchase_now_confirmation_offline($orderid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase, $area_title, $page_title;
                
                // set buynow account to confirmed phase
                $ilance->db->query("
                        UPDATE " . DB_PREFIX . "buynow_orders 
                        SET status = 'offline_delivered'
                        WHERE orderid = '" . intval($orderid) . "'
                ", 0, null, __FILE__, __LINE__);
                
                $sql = $ilance->db->query("
                        SELECT project_id
                        FROM " . DB_PREFIX . "buynow_orders
                        WHERE orderid = '" . intval($orderid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res = $ilance->db->fetch_array($sql);
                        
                        // update listing as shipped (listings table)
                        mark_listing_as_shipped($res['project_id']);
                
                        // todo: send email advising buyer that seller has confirmed offline delivery
                }
                
                return true;
        }
        
        /**
        * Function to process a purchase now escrow order release (buyer releasing funds to seller from escrow)
        *
        * @param       integer      order id
        *
        * @return      bool         Returns true or false if release could be completed
        */
        function instant_purchase_now_release($orderid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase, $area_title, $page_title;
                
                $orderid = intval($orderid);
                        
                // bidder releasing funds to merchant for purchase now item
                $sql = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "buynow_orders
                        WHERE orderid = '" . $orderid . "'
                                AND invoiceid > 0
                                AND qty > 0
                                AND amount > 0
                                AND paiddate != '0000-00-00 00:00:00'
                                AND status = 'pending_delivery'
                ");
                if ($ilance->db->num_rows($sql) > 0)
                {
                        $res_escrow = $ilance->db->fetch_array($sql);
                        $amount = $res_escrow['amount'];
                        
                        // fetch current seller account balance
                        $sql_account = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . $res_escrow['owner_id'] . "'
                        ");
                        if ($ilance->db->num_rows($sql_account) > 0)
                        {
                                $res_account = $ilance->db->fetch_array($sql_account);
                                
                                $avail_balance = $res_account['available_balance'];
                                $total_balance = $res_account['total_balance'];
                                
                                // calculate new balances for seller
                                $new_avail_balance = ($avail_balance + $amount);
                                $new_total_balance = ($total_balance + $amount);
        
                                // update seller online account balance with funds from escrow
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "users
                                        SET total_balance = '" . $new_total_balance . "', available_balance = '" . $new_avail_balance . "'
                                        WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                ");
        
                                // adjust sellers total income received from buyers
                                insert_income_reported($res_escrow['owner_id'], sprintf("%01.2f", $amount), 'credit');
                    
                                // adjust buyers total amount spent to sellers
                                insert_income_spent($res_escrow['buyer_id'], sprintf("%01.2f", $amount), 'credit');
        
                                // set escrow status to finished and clear escrow amount field (as funds are now released)
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "buynow_orders
                                        SET status = 'delivered',
                                        amount = '0.00',
                                        arrivedate = '" . DATETIME24H . "'
                                        WHERE orderid = '" . $orderid . "'
                                                AND buyer_id = '" . $res_escrow['buyer_id'] . "'
                                ");
        
                                // create new invoice/receipt to seller (full amount from bidder)
                                // this allows the seller to review past history of any payments received from escrow
                                $ilance->accounting = construct_object('api.accounting');
                                $ilance->accounting->insert_transaction(
                                        0,
                                        $res_escrow['project_id'],
                                        0,
                                        $res_escrow['owner_id'],
                                        0,
                                        0,
                                        0,
                                        $phrase['_escrow_payment_release_for_purchase_now_item'] . ' ' . $res_escrow['project_id'] . ": " . fetch_auction('project_title', $res_escrow['project_id']) . ": " . $phrase['_from_bidder'] . ": " . fetch_user('username', $res_escrow['buyer_id']),
                                        sprintf("%01.2f", $amount),
                                        sprintf("%01.2f", $amount),
                                        'paid',
                                        'credit',
                                        'account',
                                        DATETIME24H,
                                        DATEINVOICEDUE,
                                        DATETIME24H,
                                        $phrase['_purchase_now_escrow_credit_to_online_account_balance'] . ' - ' . DATETIME24H,
                                        0,
                                        0,
                                        0
                                );
                                
                                $escrowfeeamount = $fvfamount = 0;
                                
                                // here we will detect if any fees for this escrow process are still unpaid by the seller.
                                // because we've just released funds, the seller now has some money to pay any pending invoices related to this escrow.
                                // let's do that now
                                
                                // #### REGULAR ESCROW FEE CHECKUP #############
                                if ($res_escrow['isescrowfeepaid'] == '0' AND $res_escrow['escrowfeeinvoiceid'] > 0)
                                {
                                        if ($res_escrow['escrowfee'] > 0)
                                        {
                                                // get total amount (because this fee might be taxable) depending on admin settings and tax logic
                                                $escrowfeeamount = $ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $res_escrow['escrowfeeinvoiceid'] . "'", "totalamount");
                                                
                                                // automatically pay the debt so the site doens't have to track anyone down for payment
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET total_balance = total_balance - $escrowfeeamount, available_balance = available_balance - $escrowfeeamount
                                                        WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                ");
                        
                                                insert_income_spent($res_escrow['owner_id'], sprintf("%01.2f", $escrowfeeamount), 'credit');
                                        }
                                }
                                
                                // #### FINAL VALUE FEE FOR ITEM POSTED IN THIS CATEGORY 
                                if ($res_escrow['isfvfpaid'] == '0' AND $res_escrow['fvfinvoiceid'] > 0)
                                {
                                        if ($res_escrow['fvf'] > 0)
                                        {
                                                // get total amount (because this fee might be taxable) depending on admin settings and tax logic
                                                $fvfamount = $ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $res_escrow['fvfinvoiceid'] . "'", "totalamount");
                                                
                                                // automatically pay the debt so the site doens't have to track anyone down for payment
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "users
                                                        SET total_balance = total_balance - $fvfamount, available_balance = available_balance - $fvfamount
                                                        WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                ");
                        
                                                insert_income_spent($res_escrow['owner_id'], sprintf("%01.2f", $fvfamount), 'credit');
                                        }
                                }
        
                                $ilance->email = construct_dm_object('email', $ilance);
                                
                                $ilance->email->mail = fetch_user('email', $res_escrow['buyer_id']);
                                $ilance->email->slng = fetch_user_slng($res_escrow['buyer_id']);
                                
                                // email user
                                $ilance->email->get('product_escrow_release_funds_bidder');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                ));
                                
                                $ilance->email->send();
                                
                                // email merchant
                                $ilance->email->mail = fetch_user('email', $res_escrow['owner_id']);
                                $ilance->email->slng = fetch_user_slng($res_escrow['owner_id']);
                                
                                $ilance->email->get('product_escrow_release_funds_merchant');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                        '{{commissionfee}}' => $ilance->currency->format($escrowfeeamount + $fvfamount),
                                ));
                                
                                $ilance->email->send();
        
                                // email admin
                                $ilance->email->mail = SITE_EMAIL;
                                $ilance->email->slng = fetch_site_slng();
                                
                                $ilance->email->get('product_escrow_release_funds_admin');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($amount),
                                        '{{commissionfee}}' => $ilance->currency->format($escrowfeeamount + $fvfamount),
                                ));
                                
                                $ilance->email->send();
        
                                return true;
                        }
                }
                
                return false;
        }
        
        /**
        * Function to process a purchase now escrow order cancellation (seller returns funds to buyer)
        *
        * @param       integer      order id
        *
        * @return      bool         Returns true or false if reversal could be completed
        */
        function instant_purchase_now_reversal($orderid = 0)
        {
                global $ilance, $ilconfig, $ilpage, $phrase, $area_title, $page_title;
                
                $sql_escrow_amount = $ilance->db->query("
                        SELECT *
                        FROM " . DB_PREFIX . "buynow_orders
                        WHERE orderid = '" . intval($orderid) . "'
                ", 0, null, __FILE__, __LINE__);
                if ($ilance->db->num_rows($sql_escrow_amount) > 0)
                { 
                        $res_escrow = $ilance->db->fetch_array($sql_escrow_amount);
                        
                        $escrowamount = $ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $res_escrow['invoiceid'] . "'", "totalamount");
                        
                        // buyer account details
                        $sql_account = $ilance->db->query("
                                SELECT available_balance, total_balance
                                FROM " . DB_PREFIX . "users
                                WHERE user_id = '" . $res_escrow['buyer_id'] . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql_account) > 0)
                        {
                                $res_account = $ilance->db->fetch_array($sql_account);
                                
                                $avail_balance = $res_account['available_balance'];
                                $total_balance = $res_account['total_balance'];
                                
                                $new_avail_balance = ($avail_balance + $escrowamount);
                                $new_total_balance = ($total_balance + $escrowamount);
                                
                                // update bidder account balance
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "users
                                        SET total_balance = '" . $new_total_balance . "',
                                        available_balance = '" . $new_avail_balance . "'
                                        WHERE user_id = '" . $res_escrow['buyer_id'] . "'
                                ", 0, null, __FILE__, __LINE__);
                                
                                // create transaction credit
                                $ilance->accounting = construct_object('api.accounting');
                                $ilance->accounting->insert_transaction(
                                        0,
                                        $res_escrow['project_id'],
                                        $res_escrow['orderid'],
                                        $res_escrow['buyer_id'],
                                        0,
                                        0,
                                        0,
                                        $phrase['_purchase_now_delivery_cancellation'] . ' - ' . $phrase['_order_id'] . ' #' . $res_escrow['orderid'] . ': ' . $phrase['_refund_from_escrow_to_online_account'],
                                        sprintf("%01.2f", $escrowamount),
                                        sprintf("%01.2f", $escrowamount),
                                        'paid',
                                        'credit',
                                        'account',
                                        DATETIME24H,
                                        DATEINVOICEDUE,
                                        DATETIME24H,
                                        $phrase['_merchant_cancelled_purchase_now_delivery_on'] . ' ' . DATETIME24H,
                                        0,
                                        0,
                                        0
                                );
                                
                                // adjust buyers escrow fund payment by debitting amount spent (so income_spent) goes back down due to refund
                                insert_income_spent($res_escrow['buyer_id'], sprintf("%01.2f", $escrowamount), 'debit');
                                
                                // #### REFUND BUYER ESCROW FEES ###############
                                // refund buyer any amount paid for escrow fees taken during purchase now process
                                // if the buyer escrow fee has not been paid, cancel it.
                                $escrowfeebuyeramount = 0;
                                if ($res_escrow['escrowfeebuyer'] > 0 AND $res_escrow['escrowfeebuyerinvoiceid'] > 0)
                                {
                                        // was the invoice paid?
                                        if ($res_escrow['isescrowfeebuyerpaid'])
                                        {
                                                // buyer already paid it - recredit buyer's account.
                                                $escrowfeebuyeramount = $ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $res_escrow['escrowfeebuyerinvoiceid'] . "'", "totalamount");
                                                        
                                                // buyer account details
                                                $sql_account = $ilance->db->query("
                                                        SELECT available_balance, total_balance
                                                        FROM " . DB_PREFIX . "users
                                                        WHERE user_id = '" . $res_escrow['buyer_id'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($sql_account) > 0)
                                                {
                                                        $res_account = $ilance->db->fetch_array($sql_account);
                                                        
                                                        $avail_balance = $res_account['available_balance'];
                                                        $total_balance = $res_account['total_balance'];
                                                        
                                                        $new_avail_balance = ($avail_balance + $escrowfeebuyeramount);
                                                        $new_total_balance = ($total_balance + $escrowfeebuyeramount);
                                                        
                                                        // update bidder account balance
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET total_balance = '" . $new_total_balance . "',
                                                                available_balance = '" . $new_avail_balance . "'
                                                                WHERE user_id = '" . $res_escrow['buyer_id'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                        
                                                        // adjust bidders total amount spent to this merchant > debit the spendings for this refund
                                                        insert_income_spent($res_escrow['buyer_id'], sprintf("%01.2f", $escrowfeebuyeramount), 'debit');
                                                        
                                                        $ilance->accounting = construct_object('api.accounting');
                                                        $ilance->accounting->insert_transaction(
                                                                0,
                                                                $res_escrow['project_id'],
                                                                $res_escrow['orderid'],
                                                                $res_escrow['buyer_id'],
                                                                0,
                                                                0,
                                                                0,
                                                                $phrase['_purchase_now_delivery_cancellation'] . " - " . $phrase['_order_id'] . " #" . $res_escrow['orderid'] . ": " . $phrase['_purchase_now_buyer_escrow_fee_refund'],
                                                                sprintf("%01.2f", $escrowfeebuyeramount),
                                                                sprintf("%01.2f", $escrowfeebuyeramount),
                                                                'paid',
                                                                'credit',
                                                                'account',
                                                                DATETIME24H,
                                                                DATEINVOICEDUE,
                                                                DATETIME24H,
                                                                $phrase['_purchase_now_escrow_fee_refund_completed_on'] . " " . DATETIME24H,
                                                                0,
                                                                0,
                                                                0
                                                        );
                                                }
                                        }
                                        else
                                        {
                                                // buyer hasn't paid it - cancel it!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'cancelled',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_fee_was_cancelled_due_to_buy_now_order_cancellation']) . "'
                                                        WHERE invoiceid = '" . $res_escrow['escrowfeebuyerinvoiceid'] . "'
                                                        LIMIT 1
                                                ");
                                        }
                                }
                                
                                
                                // #### REFUND SELLER ESCROW FEES ##############
                                // refund seller any amount paid for escrow fees taken during purchase now process
                                $escrowfeeselleramount = 0;
                                if ($res_escrow['escrowfee'] > 0 AND $res_escrow['escrowfeeinvoiceid'] > 0)
                                {
                                        // was the invoice paid?
                                        if ($res_escrow['isescrowfeepaid'])
                                        {
                                                // seller already paid it - recredit seller's account.
                                                $escrowfeeselleramount = $ilance->db->fetch_field(DB_PREFIX."invoices", "invoiceid = '" . $res_escrow['escrowfeeinvoiceid'] . "'", "totalamount");
                                                        
                                                // seller account details
                                                $sql_account = $ilance->db->query("
                                                        SELECT available_balance, total_balance
                                                        FROM " . DB_PREFIX . "users
                                                        WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($sql_account) > 0)
                                                {
                                                        $res_account = $ilance->db->fetch_array($sql_account);
                                                        
                                                        $avail_balance = $res_account['available_balance'];
                                                        $total_balance = $res_account['total_balance'];
                                                        
                                                        $new_avail_balance = ($avail_balance + $escrowfeeselleramount);
                                                        $new_total_balance = ($total_balance + $escrowfeeselleramount);
                                                        
                                                        // update bidder account balance
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET total_balance = '" . $new_total_balance . "',
                                                                available_balance = '" . $new_avail_balance . "'
                                                                WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                        
                                                        // adjust bidders total amount spent to this merchant > debit the spendings for this refund
                                                        insert_income_spent($res_escrow['owner_id'], sprintf("%01.2f", $escrowfeeselleramount), 'debit');
                                                        
                                                        $ilance->accounting = construct_object('api.accounting');
                                                        $ilance->accounting->insert_transaction(
                                                                0,
                                                                $res_escrow['project_id'],
                                                                $res_escrow['orderid'],
                                                                $res_escrow['owner_id'],
                                                                0,
                                                                0,
                                                                0,
                                                                $phrase['_purchase_now_delivery_cancellation'] . " - " . $phrase['_order_id'] . " #" . $res_escrow['orderid'] . ": " . $phrase['_purchase_now_seller_escrow_fee_refund'],
                                                                sprintf("%01.2f", $escrowfeeselleramount),
                                                                sprintf("%01.2f", $escrowfeeselleramount),
                                                                'paid',
                                                                'credit',
                                                                'account',
                                                                DATETIME24H,
                                                                DATEINVOICEDUE,
                                                                DATETIME24H,
                                                                $phrase['_purchase_now_escrow_fee_refund_completed_on'] . " " . DATETIME24H,
                                                                0,
                                                                0,
                                                                0
                                                        );
                                                }
                                        }
                                        else
                                        {
                                                // seller hasn't paid it - cancel it!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'cancelled',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_fee_was_cancelled_due_to_buy_now_order_cancellation']) . "'
                                                        WHERE invoiceid = '" . $res_escrow['escrowfeeinvoiceid'] . "'
                                                        LIMIT 1
                                                ");
                                        }

                                }
                                
                                // #### REFUND SELLER FINAL VALUE FEES #########
                                $fvfselleramount = 0;
                                if ($res_escrow['fvf'] > 0 AND $res_escrow['fvfinvoiceid'] > 0)
                                {
                                        // was the invoice paid?
                                        if ($res_escrow['isfvfpaid'])
                                        {
                                                // marketplace took a fvf from seller, let's refund it
                                                $fvfselleramount = $ilance->db->fetch_field(DB_PREFIX . "invoices", "invoiceid = '" . $res_escrow['fvfinvoiceid'] . "'", "totalamount");
                                                        
                                                // seller account details
                                                $sql_account = $ilance->db->query("
                                                        SELECT available_balance, total_balance
                                                        FROM " . DB_PREFIX . "users
                                                        WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                                if ($ilance->db->num_rows($sql_account) > 0)
                                                {
                                                        $res_account = $ilance->db->fetch_array($sql_account);
                                                        
                                                        $avail_balance = $res_account['available_balance'];
                                                        $total_balance = $res_account['total_balance'];
                                                        
                                                        $new_avail_balance = ($avail_balance + $fvfselleramount);
                                                        $new_total_balance = ($total_balance + $fvfselleramount);
                                                        
                                                        // update bidder account balance
                                                        $ilance->db->query("
                                                                UPDATE " . DB_PREFIX . "users
                                                                SET total_balance = '" . $new_total_balance . "',
                                                                available_balance = '" . $new_avail_balance . "'
                                                                WHERE user_id = '" . $res_escrow['owner_id'] . "'
                                                        ", 0, null, __FILE__, __LINE__);
                                                        
                                                        // adjust bidders total amount spent to this merchant > debit the spendings for this refund
                                                        insert_income_spent($res_escrow['owner_id'], sprintf("%01.2f", $fvfselleramount), 'debit');
                                                        
                                                        $ilance->accounting = construct_object('api.accounting');
                                                        $ilance->accounting->insert_transaction(
                                                                0,
                                                                $res_escrow['project_id'],
                                                                $res_escrow['orderid'],
                                                                $res_escrow['owner_id'],
                                                                0,
                                                                0,
                                                                0,
                                                                $phrase['_purchase_now_delivery_cancellation'] . " - " . $phrase['_order_id'] . " #" . $res_escrow['orderid'] . ": Purchase Now Seller Final Value Fee Refund",
                                                                sprintf("%01.2f", $fvfselleramount),
                                                                sprintf("%01.2f", $fvfselleramount),
                                                                'paid',
                                                                'credit',
                                                                'account',
                                                                DATETIME24H,
                                                                DATEINVOICEDUE,
                                                                DATETIME24H,
                                                                $phrase['_purchase_now_final_value_fee_refund_completed_on'] . " " . DATETIME24H,
                                                                0,
                                                                0,
                                                                0
                                                        );
                                                }        
                                        }
                                        else
                                        {
                                                // seller hasn't paid it - cancel it!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'cancelled',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_fee_was_cancelled_due_to_buy_now_order_cancellation']) . "'
                                                        WHERE invoiceid = '" . $res_escrow['fvfinvoiceid'] . "'
                                                        LIMIT 1
                                                ");        
                                        }

                                }
                                
                                // set escrow account to cancelled
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "buynow_orders
                                        SET status = 'cancelled',
                                        amount = '0.00',
                                        escrowfee = '0.00',
                                        escrowfeebuyer = '0.00',
                                        fvf = '0.00',
                                        fvfbuyer = '0.00',
                                        isescrowfeepaid = '0',
                                        isescrowfeebuyerpaid = '0',
                                        isfvfpaid = '0',
                                        isfvfbuyerpaid = '0',
                                        canceldate = '" . DATETIME24H . "'
                                        WHERE orderid = '" . intval($orderid) . "'
                                                AND buyer_id = '" . intval($res_escrow['buyer_id']) . "'
                                ");
                                
                                // increase qty back to auction listing
                                $ilance->db->query("
                                        UPDATE " . DB_PREFIX . "projects
                                        SET buynow_qty = buynow_qty + " . $res_escrow['qty'] . "
                                        WHERE project_id = '" . $res_escrow['project_id'] . "'
                                ");
                                
                                // mark as unshipped (listing table)
                                mark_listing_as_unshipped($res_escrow['project_id']);
                                
                                $ilance->email = construct_dm_object('email', $ilance);
                                
                                $ilance->email->mail = fetch_user('email', $res_escrow['buyer_id']);
                                $ilance->email->slng = fetch_user_slng($res_escrow['buyer_id']);
        
                                $ilance->email->get('purchase_now_delivery_cancelled_buyer');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($escrowamount),
                                        '{{escrowfeeamount}}' => $ilance->currency->format($escrowfeebuyeramount),
                                ));
                                
                                $ilance->email->send();
                                
                                $ilance->email->mail = fetch_user('email', $res_escrow['owner_id']);
                                $ilance->email->slng = fetch_user_slng($res_escrow['owner_id']);
                                
                                $ilance->email->get('purchase_now_delivery_cancelled_merchant');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($escrowamount),
                                        '{{escrowfeeamount}}' => $ilance->currency->format($escrowfeeselleramount),
                                        '{{fvfamount}}' => $ilance->currency->format($fvfselleramount),
                                ));
                                
                                $ilance->email->send();
                                
                                $lostfees = 'Seller FVF: ' . $ilance->currency->format($fvfselleramount) . ', Seller Escrow Fee: ' . $ilance->currency->format($escrowfeeselleramount) . ', Buyer Escrow Fee: ' . $ilance->currency->format($escrowfeebuyeramount) . ', Total Loss: ' . $ilance->currency->format($fvfselleramount + $escrowfeeselleramount + $escrowfeebuyeramount);
                                
                                $ilance->email->mail = SITE_EMAIL;
                                $ilance->email->slng = fetch_site_slng();
                                
                                $ilance->email->get('purchase_now_delivery_cancelled_admin');		
                                $ilance->email->set(array(
                                        '{{project_id}}' => $res_escrow['project_id'],
                                        '{{biddername}}' => fetch_user('username', $res_escrow['buyer_id']),
                                        '{{ownername}}' => fetch_user('username', $res_escrow['owner_id']),
                                        '{{project_title}}' => stripslashes(fetch_auction('project_title', $res_escrow['project_id'])),
                                        '{{escrowamount}}' => $ilance->currency->format($escrowamount),
                                        '{{lostfees}}' => $lostfees,
                                ));
                                
                                $ilance->email->send();
                                
                                // here we could also send email to any bidders previously that bid on this item
                                
                                return true;
                        }
                }
                
                return false;
        }
        
        /**
        * Function to fetch the escrow status
        *
        * @param       integer      project id
        *
        * @return      bool         Returns the status of the escrow account
        */
        function status($projectid = 0)
        {
                global $ilance, $myapi, $ilconfig, $phrase, $ilconfig;
                
                if ($ilconfig['escrowsystem_enabled'])
                {
                        $filter_escrow = fetch_auction('filter_escrow', intval($projectid));
                        
                        if ($filter_escrow)
                        {
                                $sql = $ilance->db->query("
                                        SELECT escrow_id
                                        FROM " . DB_PREFIX . "projects
                                        WHERE project_id = '" . intval($projectid) . "'
                                ");
                                if ($ilance->db->num_rows($sql) > 0)
                                {
                                        $result = $ilance->db->fetch_array($sql);
                                        if ($result['escrow_id'] > 0)
                                        {
                                                $sql2 = $ilance->db->query("
                                                        SELECT *
                                                        FROM " . DB_PREFIX . "projects_escrow
                                                        WHERE escrow_id = '" . $result['escrow_id'] . "'
                                                ");
                                                if ($ilance->db->num_rows($sql2) > 0)
                                                {
                                                        $result2 = $ilance->db->fetch_array($sql2);
                                                        $status = ucfirst($result2['status']);
                                                }
                                                else
                                                {
                                                        $status = '-';
                                                }
                                        }
                                        else
                                        {
                                                $status = '-';
                                        }
                                }
                                else
                                {
                                        $status = '-';
                                }
                        }
                        else
                        {
                                $status = '-';
                        }
                }
                else
                {
                        $status = '-';
                }
                
                return $status;
        }
        
        /**
        * Function to obtain the owner of an escrow account.
        *
        * @param       integer      auction id
        * @param       integer      invoice id
        * @param       string       mode (service or product)
        *
        * @return      string       username of escrow account owner
        */
        function fetch_escrow_owner($projectid = 0, $invoiceid = 0, $mode = '')
        {
                global $ilance, $myapi;
                
                if ($mode == 'service')
                {
                        // project_user_id = the buyer of this service auction
                        // user_id = the service provider who wants payment from escrow
                        $sql = $ilance->db->query("
                                SELECT project_user_id
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND invoiceid = '" . intval($invoiceid) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $value = fetch_user('username', $res['project_user_id']);
                        }
                        else
                        {
                                $value = '--';
                        }
                }
                else if ($mode == 'product')
                {
                        // project_user_id = the merchant of this product auction
                        // user_id = the bidder/winner who must pay this escrow invoice
                        $sql = $ilance->db->query("
                                SELECT project_user_id
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND invoiceid = '" . intval($invoiceid) . "'
                                LIMIT 1
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $value = fetch_user('username', $res['project_user_id']);
                        }
                        else
                        {
                                $value = '--';
                        }
                }
                
                return $value;
        }
        
        /**
        * Function to obtain the opponent's username of an escrow account between a buyer and seller.
        *
        * @param       integer      auction id
        * @param       integer      invoice id
        * @param       string       mode (service or product)
        *
        * @return      string       username of escrow opponent
        */
        function fetch_escrow_opponent($projectid = 0, $invoiceid = 0, $mode = '')
        {
                global $ilance, $myapi;
                
                if ($mode == 'service')
                {
                        // project_user_id = the buyer of this service auction
                        // user_id = the service provider who wants payment from escrow
                        $sql = $ilance->db->query("
                                SELECT user_id
                                FROM " . DB_PREFIX . "projects_escrow
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND invoiceid = '" . intval($invoiceid) . "'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $value = fetch_user('username', $res['user_id']);
                        }
                        else
                        {
                                $value = '--';
                        }
                }
                else if ($mode == 'product')
                {
                        // project_user_id = the merchant of this product auction
                        // user_id = the bidder/winner who must pay this escrow invoice
                        $sql = $ilance->db->query("
                                SELECT user_id FROM " . DB_PREFIX . "projects_escrow
                                WHERE project_id = '" . intval($projectid) . "'
                                    AND invoiceid = '" . intval($invoiceid) . "'
                                LIMIT 1"
                        , 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $res = $ilance->db->fetch_array($sql);
                                $value = fetch_user('username', $res['user_id']);
                        }
                        else
                        {
                                $value = '--';
                        }
                }
                
                return $value;
        }
        
        /**
        * Function to fetch the escrow commission amount defined by the admin based on a custom commission argument.
        *
        * @param       string       commission type
        *
        * @return      string       calculated amount
        */
        function fetch_escrow_commission($ctype = '')
        {
                global $ilance, $myapi, $ilconfig;
                
                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                {
                        switch ($ctype)
                        {
                                case 'merchantbuynow':
                                {
                                        if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_merchantfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_merchantpercentrate'];
                                        }
                                        break;
                                }                            
                                case 'bidderbuynow':
                                {
                                        if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_merchantfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_merchantpercentrate'];
                                        }
                                        break;
                                }                            
                                case 'servicebuyer':
                                {
                                        if ($ilconfig['escrowsystem_servicebuyerfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_servicebuyerfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_servicebuyerpercentrate'];
                                        }
                                        break;
                                }                            
                                case 'serviceprovider':
                                {
                                        if ($ilconfig['escrowsystem_providerfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_providerfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_providerpercentrate'];
                                        }
                                        break;
                                }                            
                                case 'productmerchant':
                                {
                                        if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_merchantfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_merchantpercentrate'];
                                        }
                                        break;
                                }                            
                                case 'productbidder':
                                {
                                        if ($ilconfig['escrowsystem_bidderfixedprice'] > 0)
                                        {
                                                return $ilconfig['escrowsystem_bidderfixedprice'];
                                        }
                                        else 
                                        {
                                                return $ilconfig['escrowsystem_bidderpercentrate'];
                                        }
                                        break;
                                }
                        }
                }
                
                return 0;
        }
        
        /**
        * Function to fetch the escrow commission logic defined by the admin based on a custom commission argument.
        *
        * @param       string       commission type
        *
        * @return      string       returns a string (fixed/percentage)
        */
        function fetch_escrow_commission_logic($ctype = '')
        {
                global $ilconfig;
                
                if ($ilconfig['escrowsystem_escrowcommissionfees'])
                {
                        if (isset($ctype) AND !empty($ctype))
                        {
                                switch ($ctype)
                                {
                                        case 'merchantbuynow':
                                        {
                                                if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }
                                        case 'bidderbuynow':
                                        {
                                                if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }                                    
                                        case 'servicebuyer':
                                        {
                                                if ($ilconfig['escrowsystem_servicebuyerfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }                                    
                                        case 'serviceprovider':
                                        {
                                                if ($ilconfig['escrowsystem_providerfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }                                    
                                        case 'productmerchant':
                                        {
                                                if ($ilconfig['escrowsystem_merchantfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }                                    
                                        case 'productbidder':
                                        {
                                                if ($ilconfig['escrowsystem_bidderfixedprice'] > 0)
                                                {
                                                        return 'fixed';
                                                }
                                                else 
                                                {
                                                        return 'percentage';
                                                }
                                                break;
                                        }
                                }
                        }
                }
        }
        
        /**
        * Function to find all escrow related invoices and check the escrow table for a match if we do not have a match,
        * chances are the escrow account was not created or the admin removed the invoiceid # in the escrow table tied to the
        * auction .. so in these situations we'll set that invoice to 'cancelled'
        */
        function cancel_unlinked_escrow_invoices()
        {
                global $ilance, $phrase, $ilconfig, $ilpage;
                
                $cronlog = '';
                
                if ($ilconfig['escrowsystem_enabled'])
                {
                        $sql = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "invoices
                                WHERE invoicetype = 'escrow'
                                    AND status = 'unpaid'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $sent = 0;
                                while ($resms = $ilance->db->fetch_array($sql))
                                {
                                        $sqlec = $ilance->db->query("
                                                SELECT *
                                                FROM " . DB_PREFIX . "projects_escrow
                                                WHERE project_id = '" . $resms['projectid'] . "'
                                                    AND invoiceid = '" . $resms['invoiceid'] . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        if ($ilance->db->num_rows($sqlec) == 0)
                                        {
                                                $sent++;
                                                // cannot find escrow tied to invoice
                                                // cancel this invoice as we're not sure what happened!?!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'cancelled',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_transaction_cancelled_due_to_unlinked_and_unpaid_escrow_account_for_this_specific_invoice']) . "'
                                                        WHERE invoiceid = '" . $resms['invoiceid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                }
                                $cronlog .= 'Removed ' . $sent . ' unlinked/unpaid escrow transactions (reason: auction not found), ';
                                unset($sent);
                        }
                    
                        // we'll use the same logic for unlinked escrows as above for buynow escrow orders below
                        $sql = $ilance->db->query("
                                SELECT *
                                FROM " . DB_PREFIX . "invoices
                                WHERE invoicetype = 'buynow'
                                    AND status = 'unpaid'
                        ", 0, null, __FILE__, __LINE__);
                        if ($ilance->db->num_rows($sql) > 0)
                        {
                                $sent = 0;
                                while ($resin = $ilance->db->fetch_array($sql))
                                {
                                        $sqlbn = $ilance->db->query("
                                                SELECT *
                                                FROM " . DB_PREFIX . "buynow_orders
                                                WHERE project_id = '" . $resin['projectid'] . "'
                                                    AND invoiceid = '" . $resin['invoiceid'] . "'
                                        ", 0, null, __FILE__, __LINE__);
                                        if ($ilance->db->num_rows($sqlbn) == 0)
                                        {
                                                $sent++;
                                                // cannot find buynow escrow tied to invoice
                                                // cancel this invoice as we're not sure what happened!?!
                                                $ilance->db->query("
                                                        UPDATE " . DB_PREFIX . "invoices
                                                        SET status = 'cancelled',
                                                        custommessage = '" . $ilance->db->escape_string($phrase['_transaction_cancelled_due_to_unlinked_and_unpaid_escrow_account_for_this_specific_invoice']) . "'
                                                        WHERE invoiceid = '" . $resin['invoiceid'] . "'
                                                ", 0, null, __FILE__, __LINE__);
                                        }
                                }
                                $cronlog .= 'Removed ' . $sent . ' unlinked/unpaid buy now escrow transactions (reason: auction not found), ';
                                unset($sent);
                        }
                }
                
                return $cronlog;
        }
}

/*======================================================================*\
|| ####################################################################
|| # CVS: $RCSfile: class.escrow.inc.php,v $ - $Revision: 1.00 $
|| ####################################################################
\*======================================================================*/
?>